! MODULE       : function_library
!
! CONTACT      : orchidee-help _at_ ipsl.jussieu.fr
!
! LICENCE      : IPSL (2006)
! This software is governed by the CeCILL licence see ORCHIDEE/ORCHIDEE_CeCILL.LIC
!
!>\BRIEF       Collection of functions that are used throughout the ORCHIDEE code
!!
!!\n DESCRIPTION: Collection of modules to : (1) convert one variable into another i.e. basal area
!! to diameter, diamter to tree height, diameter to crown area, etc. (2) ...
!!
!! RECENT CHANGE(S): None
!!
!! REFERENCE(S)	:
!!
!! SVN          :
!! $HeadURL: svn://forge.ipsl.jussieu.fr/orchidee/branches/ORCHIDEE-DOFOCO/ORCHIDEE/src_stomate/stomate_prescribe.f90 $
!! $Date: 2013-01-04 16:50:56 +0100 (Fri, 04 Jan 2013) $
!! $Revision: 1126 $
!! \n
!_ ================================================================================================================================

MODULE function_library

  ! modules used:

!$  USE ioipsl_para
  USE pft_parameters
  USE constantes

  IMPLICIT NONE

  ! private & public routines

  PRIVATE
  PUBLIC calculate_c0_alloc, &
       wood_to_ba_old, &
       wood_to_ba_eff, &
       wood_to_ba, &
       wood_to_height, &
       wood_to_qmheight, &
       wood_to_height_eff, &
       wood_to_volume, &
       wood_to_dia, &
       wood_to_qmdia,  &
       wood_to_dia_eff, &
       wood_to_circ, &
       wood_to_cv_eff, &
       wood_to_cn,&
       wood_to_cn_eff, &
       cc_to_biomass, &
       biomass_to_cc, &
       cc_to_lai, &
       biomass_to_lai, &
       lai_to_biomass, &
       Nmax, &
       Nmaxyield, &
       Nminyield, &
       calculate_rdi, &
       check_biomass_sync, &
       check_surface_area, &
       check_mass_balance, &
       biomass_to_coupled_lai, &
       sort_circ_class_biomass

  INTERFACE biomass_to_lai
     MODULE PROCEDURE biomass_to_lai_0d, biomass_to_lai_1d
  END INTERFACE

  INTERFACE lai_to_biomass
     MODULE PROCEDURE lai_to_biomass_0d,  lai_to_biomass_1d
  END INTERFACE

  INTERFACE cc_to_biomass
     MODULE PROCEDURE cc_to_biomass_2d, cc_to_biomass_4d
  END INTERFACE cc_to_biomass

  INTERFACE biomass_to_cc
     MODULE PROCEDURE biomass_to_cc_1d, biomass_to_cc_3d
  END INTERFACE biomass_to_cc

  INTERFACE cc_to_lai
     MODULE PROCEDURE cc_to_lai_0d, cc_to_lai_1d
  END INTERFACE

  CONTAINS


!! ================================================================================================================================
!! FUNCTION    : biomass_to_coupled_lai
!!
!>\BRIEF        Calculate the coupled_LAI based on biomass and veget of each pft
!!
!! DESCRIPTION : Calculates the lai that is coupled with the atmosphere  
!! 
!!             
!! 
!! RECENT CHANGE(S): None
!!
!! RETURN VALUE : ::coupled_lai (m2/m2)
!!
!! REFERENCE(S)	: 
!!
!! FLOWCHART	: None
!! \n
!_ ================================================================================================================================
   
  FUNCTION biomass_to_coupled_lai(biomass_leaf, veget, pft)

 !! 0. Variable and parameter declaration

    !! 0.1 Input variables
    INTEGER(i_std)                                       :: pft                    !! PFT number (-)
    REAL(r_std)                                          :: veget                  !! 1-Pgap or the vegetation cover
    REAL(r_std)                                          :: biomass_leaf           !! Biomass of an individual tree within a circ 
                                                                                   !! class @tex $(m^{2} ind^{-1})$ @endtex 

    !! 0.2 Output variables         
    REAL(r_std)                                          :: biomass_to_coupled_lai !! The fraction of the LAI that is assumed to interact
                                                                                   !! with the atmosphere @tex $(m^{2} m^{-2})$ @endtex
    !! 0.3 Modified variables

    !! 0.4 Local variables
    REAL(r_std)                                          :: lai                    !! Leaf area index 
                                                                                   !! @tex $(m^{2} m^{-2})$ @endtex               
!_ ================================================================================================================================
 
    lai = biomass_to_lai(biomass_leaf, pft) 
   
    !+++CHECK+++
    ! In a closed canopy not all the leaves fully interact with the 
    ! atmosphere because leaves can shelter each other. In a more 
    ! open canopy most leaves can interact with the atmosphere.
    ! For the moment we are not clear how to calculate the fraction of
    ! the canopy that is coupled to the atmosphere. Also, based on the
    ! simulated evapotranspiration we need the entire canopy to be
    ! coupled to the atmosphere to obtain simulations which are close
    ! to the observations (Jung's upscaled product).
    IF (veget .GT. min_sechiba) THEN
     
       biomass_to_coupled_lai = lai   

    ELSE
   
       ! There are so many gaps that veget is extremely small. It is fair to
       ! assume that the whole canopy is coupled to the atmosphere.
       biomass_to_coupled_lai = lai
      
    ENDIF

  END FUNCTION biomass_to_coupled_lai
 

!! ================================================================================================================================
!! FUNCTION     : biomass_to_lai_0d
!!
!>\BRIEF        Calculate the LAI based on the leaf biomass
!!
!! DESCRIPTION : Calculates the LAI of a PFT/grid square based on the leaf biomass
!! 
!! RECENT CHANGE(S): None
!!
!! RETURN VALUE : ::LAI [m**2 m**{-2}]
!!
!! REFERENCE(S)	: 
!!
!! FLOWCHART	: None
!! \n
!_ ================================================================================================================================
   
  FUNCTION biomass_to_lai_0d(leaf_biomass, pft)

 !! 0. Variable and parameter declaration

    !! 0.1 Input variables

    INTEGER(i_std)                                          :: pft               !! PFT number (-)
    REAL(r_std)                                             :: leaf_biomass      !! Biomass of the leaves
                                                                                 !! @tex $(gC m^{-2})$ @endtex 


    !! 0.2 Output variables
                
    REAL(r_std)                                             :: biomass_to_lai_0d !! Leaf area index 
                                                                                 !! @tex $(m^{2} m^{-2})$ @endtex 

    !! 0.3 Modified variables

    !! 0.4 Local variables
    REAL(r_std)                                             :: impose_lai         !! LAI read from run.def
!_ ================================================================================================================================
 
    !! 1. Calculate the LAI from the leaf biomass
    IF(sla_dyn) THEN
       biomass_to_lai_0d = log(1.+ ext_coeff_N(pft)* leaf_biomass * slainit(pft))/(ext_coeff_N(pft)) 
    ELSE
       biomass_to_lai_0d = leaf_biomass * sla(pft)
    ENDIF
     
!!$    !+++++++++ TEMP ++++++++++
!!$    ! This is a perfect place to hack the code to make it run with
!!$    ! constant lai
!!$    WRITE(numout,*) 'WARNING ERROR: Using fake lai values for testing!'
!!$    biomass_to_lai=3.79052
!!$    !+++++++++++++++++++++++++

            !+++++++ TEMP ++++++++++
            ! This code is only used for evaluation of the performance of the multi-layer energy budget. 
            ! To reduce the complexity of the tests we want to impose the LAI and its vertical distribution. 
            ! The solution is not very elegant but it works. 
            !  IF (ld_fake_height) THEN
            ! In order to imposed lai, we read the TOTAL_LAI from run.def
            !  CALL getin_p('TOTAL_LAI', impose_lai)
            ! This part of code reset the sla vale to match which alow modeled LAI equal to TOTAL LAI. 
            ! Althought this is ugly way to match the modeled LAI and impose LAI. 
            ! You probably need to go to your ORCHIDEE out put file to find out the suitable SLA value 
            ! and reset it agin in the run.def.
            ! So, we impose LAI & structure for a quick testing the performance of multilayer energy budget
            ! without changing the leaf_biomass.
            !     IF ( leaf_biomass .GT. 0.0) THEN
            !        sla(pft)=impose_lai/leaf_biomass
            !          WRITE(numout,'(A,F20.8)') 'USE A FAKE SLA BASED ON imposed LAI/LEAFMASS=', sla(pft)
            !     ENDIF 
            !        biomass_to_lai=leaf_biomass*sla(pft) 
            !  ENDIF 
            !++++++++++++++++++++++++

  END FUNCTION biomass_to_lai_0d


!! ================================================================================================================================
!! FUNCTION     : biomass_to_lai_1d
!!
!>\BRIEF        Calculate the LAI based on the leaf biomass
!!
!! DESCRIPTION : Calculates the LAI of a PFT/grid square based on the leaf biomass
!! 
!! RECENT CHANGE(S): None
!!
!! RETURN VALUE : ::LAI [m**2 m**{-2}]
!!
!! REFERENCE(S)	: 
!!
!! FLOWCHART	: None
!! \n
!_ ================================================================================================================================
   
  FUNCTION biomass_to_lai_1d(leaf_biomass, npts, pft)

 !! 0. Variable and parameter declaration

    !! 0.1 Input variables
    INTEGER(i_std)                                          :: npts
    INTEGER(i_std)                                          :: pft               !! PFT number (-)
    REAL(r_std), DIMENSION(npts)                            :: leaf_biomass      !! Biomass of the leaves
                                                                                 !! @tex $(gC m^{-2})$ @endtex 

    !! 0.2 Output variables
                
    REAL(r_std), DIMENSION(npts)                            :: biomass_to_lai_1d !! Leaf area index 
                                                                                 !! @tex $(m^{2} m^{-2})$ @endtex 
!_ ================================================================================================================================
 
    !! 1. Calculate the LAI from the leaf biomass
    IF(sla_dyn) THEN
       biomass_to_lai_1d = log(1.+ ext_coeff_N(pft)* leaf_biomass * slainit(pft))/(ext_coeff_N(pft)) 
    ELSE
       biomass_to_lai_1d = leaf_biomass * sla(pft)
    ENDIF

  END FUNCTION biomass_to_lai_1d


!! ================================================================================================================================
!! FUNCTION     : lai_to_biomass_0d
!!
!>\BRIEF        Calculate leaf biomass from lai
!!
!! DESCRIPTION : Calculates leaf biomass (m2 m-2) from leaf biomass (gC m-2)
!! 
!! RECENT CHANGE(S): None
!!
!! RETURN VALUE : ::Leaf biomass [gC m^{-2}]
!!
!! REFERENCE(S)	: 
!!
!! FLOWCHART	: None
!! \n
!_ ================================================================================================================================

  FUNCTION lai_to_biomass_0d(lai, pft)

 !! 0. Variable and parameter declaration

    !! 0.1 Input variables
    INTEGER(i_std)                                          :: pft               !! PFT number (-)
    REAL(r_std)                                             :: lai               !! Leaf Area Index
                                                                                 !! @tex $(m^{2} m^{-2})$ @endtex


    !! 0.2 Output variables
                
    REAL(r_std)                                             :: lai_to_biomass_0d !! Leaf area index
                                                                                 !! @tex $(gC m^{-2})$ @endtex  
                                                                                  
!_ ================================================================================================================================
 
    !! 1. Calculate the LAI from the leaf biomass
    IF(sla_dyn) THEN
       lai_to_biomass_0d = ( exp(lai*ext_coeff_N(pft)) - 1.) / &
               (ext_coeff_N(pft) * slainit(pft))       
    ELSE
       lai_to_biomass_0d = lai / sla(pft)
    ENDIF

END FUNCTION lai_to_biomass_0d



!! ================================================================================================================================
!! FUNCTION     : lai_to_biomass_1d
!!
!>\BRIEF        Calculate leaf biomass from lai
!!
!! DESCRIPTION : Calculates leaf biomass (m2 m-2) from leaf biomass (gC m-2)
!! 
!! RECENT CHANGE(S): None
!!
!! RETURN VALUE : ::Leaf biomass [gC m^{-2}]
!!
!! REFERENCE(S)	: 
!!
!! FLOWCHART	: None
!! \n
!_ ================================================================================================================================

  FUNCTION lai_to_biomass_1d(lai, npts, pft)

 !! 0. Variable and parameter declaration

    !! 0.1 Input variables
    INTEGER(i_std)                                          :: npts
    INTEGER(i_std)                                          :: pft               !! PFT number (-)
    REAL(r_std), DIMENSION(npts)                            :: lai               !! Leaf Area Index
                                                                                 !! @tex $(m^{2} m^{-2})$ @endtex


    !! 0.2 Output variables
                
    REAL(r_std), DIMENSION(npts)                            :: lai_to_biomass_1d !! Leaf area index
                                                                                 !! @tex $(gC m^{-2})$ @endtex  
                                                                                 
!_ ================================================================================================================================
 
    !! 1. Calculate the LAI from the leaf biomass
    IF(sla_dyn) THEN
       lai_to_biomass_1d = ( exp(lai*ext_coeff_N(pft)) - 1.) / &
               (ext_coeff_N(pft) * slainit(pft))       
    ELSE
       lai_to_biomass_1d = lai / sla(pft)
    ENDIF

  END FUNCTION lai_to_biomass_1d
  
  
!! ================================================================================================================================
!! FUNCTION     : calculate_c0_alloc
!!
!>\BRIEF        Calculate the baseline root vs sapwood allocation
!!
!! DESCRIPTION : Calculates the baseline root vs sapwood allocation based on the 
!! parameters of the pipe model (hydraulic conductivities) and the
!! turnover of the different components              
!! 
!! RECENT CHANGE(S): None
!!
!! RETURN VALUE : ::c0_alloc (m)
!!
!! REFERENCE(S)	: 
!!
!! FLOWCHART	: None
!! \n
!_ ================================================================================================================================
   
  FUNCTION calculate_c0_alloc(pts, pft, tau_eff_root, tau_eff_sap)

 !! 0. Variable and parameter declaration

    !! 0.1 Input variables

    INTEGER(i_std)                             :: pts               !! Pixel number (-)
    INTEGER(i_std)                             :: pft               !! PFT number (-)
    REAL(r_std)                                :: tau_eff_root      !! Effective longivety for leaves (days)
    REAL(r_std)                                :: tau_eff_sap       !! Effective longivety for leaves (days)
    
    !! 0.2 Output variables
                
    REAL(r_std)                                :: calculate_c0_alloc  !! quadratic mean height (m) 

    !! 0.3 Modified variables

    !! 0.4 Local variables
    REAL(r_std)                                :: sapwood_density
    REAL(r_std)                                :: qm_dia            !! quadratic mean diameter (m)

!_ ================================================================================================================================
 
    !! 1. Calculate c0_alloc
    IF ( is_tree(pft) ) THEN

       sapwood_density = deux * pipe_density(pft) / kilo_to_unit
       calculate_c0_alloc = sqrt(k_root(pft)/k_sap(pft)*tau_eff_sap/tau_eff_root*sapwood_density)

    ! Grasses and croplands   
    ELSE

       !+++CHECK+++
       ! Simply copied the same formulation as for trees but note
       ! that the sapwood in trees vs grasses and crops has a very
       ! meaning. In grasses and crops is structural carbon to ensure
       ! that the allocation works. In trees it really is the sapwood
       sapwood_density = deux * pipe_density(pft) / kilo_to_unit
       calculate_c0_alloc = sqrt(k_root(pft)/k_sap(pft)*tau_eff_sap/tau_eff_root*sapwood_density)
       !+++++++++++

    ENDIF ! is_tree(j)

 END FUNCTION calculate_c0_alloc


!! ================================================================================================================================
!! FUNCTION     : wood_to_ba_eff
!!
!>\BRIEF        Calculate effective basal area from woody biomass making use of allometric relationships
!!
!! DESCRIPTION :  Calculate basal area of an individual tree from the woody biomass of that tree making 
!! use of allometric relationships. Effective basal area accounts for both above and below ground carbon
!! and is the basis for the application of the rule of Deleuze and Dhote.
!! (i) woodmass = tree_ff * pipe_density*ba*height
!! (ii) height = pipe_tune2 * sqrt(4/pi*ba) ** pipe_tune_3  
!! 
!! RECENT CHANGE(S): None
!!
!! RETURN VALUE : effective basal area (m2 ind-1)
!!
!! REFERENCE(S)	: 
!!
!! FLOWCHART	: None
!! \n
!_ ================================================================================================================================
 
 FUNCTION wood_to_ba_eff(biomass_temp, pft)

 !! 0. Variable and parameter declaration

    !! 0.1 Input variables

    INTEGER(i_std)                                          :: pft                  !! PFT number (-)
    REAL(r_std), DIMENSION(:,:)                             :: biomass_temp         !! Biomass of an individual tree within a circ 
                                                                                    !! class @tex $(m^{2} ind^{-1})$ @endtex 

    !! 0.2 Output variables
                
    REAL(r_std), DIMENSION(ncirc)                           :: wood_to_ba_eff       !! Effective basal area of an individual tree within a circ
                                                                                    !! class @tex $(m^{2} ind^{-1})$ @endtex

    !! 0.3 Modified variables

    !! 0.4 Local variables
    INTEGER(i_std)                                          :: l                    !! Index
    REAL(r_std)                                             :: woodmass_ind         !! Woodmass of an individual tree
                                                                                    !! @tex $(gC ind^{-1})$ @endtex

!_ ================================================================================================================================
 
 !! 1. Calculate basal area from woodmass

    IF ( is_tree(pft) ) THEN
       
       DO l = 1,ncirc
         
          ! Woodmass of an individual tree
          woodmass_ind = biomass_temp(l,isapabove) + biomass_temp(l,isapbelow) + &
             biomass_temp(l,iheartabove) + biomass_temp(l,iheartbelow)

          ! Basal area of that individual (m2 ind-1)  
          wood_to_ba_eff(l) = (pi/4*(woodmass_ind/(tree_ff(pft)*pipe_density(pft)*pipe_tune2(pft))) &
                  **(2./pipe_tune3(pft)))**(pipe_tune3(pft)/(pipe_tune3(pft)+2))
                 
       ENDDO

    ELSE

       WRITE(numout,*) 'pft ',pft
       CALL ipslerr (3,'wood_to_ba_eff', &
            'wood_to_ba_eff is not defined for this PFT.', &
            'See the output file for more details.','')

    ENDIF

 END FUNCTION wood_to_ba_eff

!! ================================================================================================================================
!! FUNCTION     : wood_to_ba_old
!!
!>\BRIEF        Calculate basal area from woody biomass making use of allometric relationships
!!
!! DESCRIPTION : Calculate basal area of an individual tree from the woody biomass of that tree making 
!! use of allometric relationships given below. Here basal area is defined in line with its classical 
!! forestry meaning.
!! (i) woodmass = tree_ff * pipe_density*ba*height
!! (ii) height = pipe_tune2 * sqrt(4/pi*ba) ** pipe_tune_3  
!! 
!! RECENT CHANGE(S): None
!!
!! RETURN VALUE : basal area (m2 ind-1)
!!
!! REFERENCE(S)	: 
!!
!! FLOWCHART	: None
!! \n
!_ ================================================================================================================================
 
 FUNCTION wood_to_ba_old(biomass_temp, pft)

 !! 0. Variable and parameter declaration

    !! 0.1 Input variables

    INTEGER(i_std)                                          :: pft               !! PFT number (-)
    REAL(r_std), DIMENSION(:)                             :: biomass_temp        !! Biomass of an individual tree within a circ 
                                                                                 !! class @tex $(m^{2} ind^{-1})$ @endtex 

    !! 0.2 Output variables
                
    REAL(r_std)                                            :: wood_to_ba_old     !! Basal area of an individual tree within a circ
                                                                                 !! class @tex $(m^{2} ind^{-1})$ @endtex

    !! 0.3 Modified variables

    !! 0.4 Local variables
    REAL(r_std)                                             :: woodmass_ind      !! Woodmass of an individual tree
                                                                                 !! @tex $(gC ind^{-1})$ @endtex

!_ ================================================================================================================================
 
 !! 1. Calculate basal area from woodmass

    IF ( is_tree(pft) ) THEN

             
          ! Woodmass of an individual tree
          woodmass_ind = biomass_temp(iheartabove) + biomass_temp(isapabove)


          ! Basal area of that individual (m2 ind-1)  
          wood_to_ba_old = (pi/4*(woodmass_ind/(tree_ff(pft)*pipe_density(pft)*pipe_tune2(pft))) &
                  **(2./pipe_tune3(pft)))**(pipe_tune3(pft)/(pipe_tune3(pft)+2))


    ELSE

       WRITE(numout,*) 'pft ',pft
       CALL ipslerr (3,'wood_to_ba', &
            'wood_to_ba is not defined for this PFT.', &
            'See the output file for more details.','')

    ENDIF

 END FUNCTION wood_to_ba_old


!! ================================================================================================================================
!! FUNCTION     : ccbio_to_biomass
!!
!>\BRIEF        Calculate total biomass from circ_class_biomass
!!
!! DESCRIPTION : circ_class_biomass is expressed in gC tree-1 and thus needs to
!!               be multiplied by the number of trees per circumference class
!!               circ_class_n expressed in tree m-2 to calculate the total
!!               biomass in gC m-2. 
!! 
!! RECENT CHANGE(S): None
!!
!! RETURN VALUE : total biomass (gC(N) m-2)
!!
!! REFERENCE(S)	: 
!!
!! FLOWCHART	: None
!! \n
!_ ================================================================================================================================
 
 FUNCTION cc_to_biomass_2d(npts,ivm,ccbio,ccind)

 !! 0. Variable and parameter declaration

    !! 0.1 Input variables

    INTEGER(i_std), INTENT(in)                        :: npts              !! Domain size (unitless)
    INTEGER(i_std), INTENT(in)                        :: ivm               !! number of the specific PFT (unitless)
    REAL(r_std), DIMENSION(:,:,:)                     :: ccbio             !! circ_class_biomass. Biomass of an individual 
                                                                           !! tree @tex $(gC(N) ind^{-1})$ @endtex 
    REAL(r_std), DIMENSION(:)                         :: ccind             !! circ_class_n. Number of trees per circumference
                                                                           !! class

    !! 0.2 Output variables
                
    REAL(r_std), DIMENSION(nparts,nelements)          :: cc_to_biomass_2d  !! biomass @tex $(gC(N) m^{2})$ @endtex

    !! 0.3 Modified variables

    !! 0.4 Local variables
    INTEGER(i_std)                                    :: iele,icir,ipar    !! Index

!_ ================================================================================================================================
    
    cc_to_biomass_2d(:,:) = zero

    DO iele = 1,nelements

       ! Note that when used in 2d ivm is a single PFT
       IF ( is_tree(ivm) ) THEN

          DO ipar = 1,nparts

             DO icir = 1,ncirc
                
                cc_to_biomass_2d(ipar,iele) = cc_to_biomass_2d(ipar,iele) + &
                     ccbio(icir,ipar,iele) * ccind(icir)
                                        
             ENDDO
      
          ENDDO

       ELSE

          ! Grasses and cropland only have one circumference class
          DO ipar = 1,nparts
             
             cc_to_biomass_2d(ipar,iele) = cc_to_biomass_2d(ipar,iele) + &
                  ccbio(1,ipar,iele) * ccind(1)
             
          ENDDO

       ENDIF

    ENDDO

 END FUNCTION cc_to_biomass_2d

 
!! ================================================================================================================================

 FUNCTION cc_to_biomass_4d(npts,nvm,ccbio,ccind)

 !! 0. Variable and parameter declaration

    !! 0.1 Input variables

    INTEGER(i_std), INTENT(in)                        :: npts               !! Domain size (unitless)
    INTEGER(i_std), INTENT(in)                        :: nvm                !! Total number of PFTs (unitless)
    REAL(r_std), DIMENSION(:,:,:,:,:)                 :: ccbio              !! circ_class_biomass. Biomass of an individual 
                                                                            !! tree @tex $(gC(N) ind^{-1})$ @endtex 
    REAL(r_std), DIMENSION(:,:,:)                     :: ccind              !! circ_class_n. Number of trees per circumference
                                                                            !! class

    !! 0.2 Output variables
                
    REAL(r_std), DIMENSION(npts,nvm,nparts,nelements) :: cc_to_biomass_4d   !! biomass @tex $(gC(N) m^{2})$ @endtex

    !! 0.3 Modified variables

    !! 0.4 Local variables
    INTEGER(i_std)                                    :: iele,icir,ipar,ivm !! Index

!_ ================================================================================================================================
    
    cc_to_biomass_4d(:,:,:,:) = zero

    DO iele = 1,nelements

       ! Note that when used in 4d, nvm rather than ivm
       ! should be passed. Now the function will loop over all
       ! PFTs instead of just 1 PFT for the 2d version of this
       ! function.
       DO ivm = 2,nvm

          IF ( is_tree(ivm) ) THEN

             DO ipar = 1,nparts

                DO icir = 1,ncirc

                   cc_to_biomass_4d(:,ivm,ipar,iele) = cc_to_biomass_4d(:,ivm,ipar,iele) + &
                      ccbio(:,ivm,icir,ipar,iele) * ccind(:,ivm,icir)
                                        
                ENDDO
      
             ENDDO

          ELSE

            ! Grasses and cropland only have one circumference class
            DO ipar = 1,nparts

               cc_to_biomass_4d(:,ivm,ipar,iele) = cc_to_biomass_4d(:,ivm,ipar,iele) + &
                   ccbio(:,ivm,1,ipar,iele) * ccind(:,ivm,1)

            ENDDO

          ENDIF

       ENDDO

    ENDDO

 END FUNCTION cc_to_biomass_4d


!! ================================================================================================================================
!! FUNCTION     : biomass_to_cc
!!
!>\BRIEF        Calculate circ_class_biomass from biomass
!!
!! DESCRIPTION : The labile and carbres pools are often dealt with at the stand
!!               level for simplicity but then need to be stored at the plant 
!!               level. This is needed because labile and carbres C is lost when
!!               plants are dying. Also, all plant C and N should be stored in a
!!               single variable. 
!! 
!! RECENT CHANGE(S): None
!!
!! RETURN VALUE : total biomass (gC(N) m-2)
!!
!! REFERENCE(S)	: 
!!
!! FLOWCHART	: None
!! \n
!_ ================================================================================================================================
 
 FUNCTION biomass_to_cc_1d(bio,ccbio,ccind)

 !! 0. Variable and parameter declaration

    !! 0.1 Input variables

    REAL(r_std)                                       :: bio               !! biomass. Biomass of a specific pool of 
                                                                           !! a whole stand 
                                                                           !! @tex $(gC(N) m^{-2})$ @endtex
    REAL(r_std), DIMENSION(:)                         :: ccbio             !! circ_class_biomass. Biomass of a specif pool of 
                                                                           !! the individual trees in each circumference class 
                                                                           !! @tex $(gC(N) ind^{-1})$ @endtex 
    REAL(r_std), DIMENSION(:)                         :: ccind             !! circ_class_n. Number of trees per circumference
                                                                           !! class

    !! 0.2 Output variables
                
    REAL(r_std), DIMENSION(ncirc)                     :: biomass_to_cc_1d  !! biomass @tex $(gC(N) m^{2})$ @endtex

    !! 0.3 Modified variables

    !! 0.4 Local variables

    INTEGER(i_std)                                    :: icir              !! Index
    REAL(r_std), DIMENSION(ncirc)                     :: share_ncirc       !! ratio of specific pool
                                                                           !! across circumference classes

!_ ================================================================================================================================
    
    biomass_to_cc_1d(:) = zero

    ! Convert the pools
    IF (SUM(ccbio(:)*ccind(:)) .GT. zero) THEN
       
       ! Proportional to the present pools
       share_ncirc(:) = (ccbio(:)*ccind(:))/SUM(ccbio(:)*ccind(:))

    ELSEIF (SUM(ccind(:)) .GT. zero) THEN

       ! There are no pools at present. Make it proportional
       ! to the number of trees
       share_ncirc(:) = ccind(:)/SUM(ccind(:))

    ELSE 

       ! There is no biomass in circ_class_biomass or individuals
       share_ncirc(:) = zero


    ENDIF

    share_ncirc(1) = zero
    share_ncirc(1) = 1 - SUM(share_ncirc(:))

    ! Distribute bio over the ncirc circumference classes of biomass_to_cc
    ! biomass_to_cc will eventually be assigned to circ_class_biomass
    DO icir = 1,ncirc

       IF (ccind(icir) .GT. zero) THEN

          biomass_to_cc_1d(icir) = bio / ccind(icir) * share_ncirc(icir)

       ELSE

          biomass_to_cc_1d(icir) = zero

       ENDIF

    ENDDO

 END FUNCTION biomass_to_cc_1d

!! ================================================================================================================================
FUNCTION biomass_to_cc_3d(bio,ccbio,ccind,npts,nvm)

 !! 0. Variable and parameter declaration

    !! 0.1 Input variables
    INTEGER(i_std), INTENT(in)                        :: npts              !! Domain size (unitless)
    INTEGER(i_std), INTENT(in)                        :: nvm               !! Total number of PFTs (unitless)
    REAL(r_std),DIMENSION(:,:)                        :: bio               !! biomass. Biomass of a specific pool of 
                                                                           !! a whole stand. dimensions are npts and nvm 
                                                                           !! @tex $(gC(N) m^{-2})$ @endtex
    REAL(r_std), DIMENSION(:,:,:)                     :: ccbio             !! circ_class_biomass. Biomass of a specif pool of 
                                                                           !! the individual trees in each circumference class 
                                                                           !! @tex $(gC(N) ind^{-1})$ @endtex 
    REAL(r_std), DIMENSION(:,:,:)                     :: ccind             !! circ_class_n. Number of trees per circumference
                                                                           !! class
   
    !! 0.2 Output variables
                
    REAL(r_std), DIMENSION(npts,nvm,ncirc)            :: biomass_to_cc_3d  !! biomass @tex $(gC(N) m^{2})$ @endtex

    !! 0.3 Modified variables

    !! 0.4 Local variables

    INTEGER(i_std)                                    :: icir,ipts,ivm     !! Index
    REAL(r_std), DIMENSION(ncirc)                     :: share_ncirc       !! ratio of specific pool
                                                                           !! across circumference classes

!_ ================================================================================================================================
    
    biomass_to_cc_3d(:,:,:) = zero

    DO ipts = 1,npts

       DO ivm = 1,nvm

          ! Convert the pools
          IF (is_tree(ivm)) THEN

             ! Forests have ncirc circ classes
             IF (SUM(ccbio(ipts,ivm,:)*ccind(ipts,ivm,:)) .GT. zero) THEN
       
                share_ncirc(:) = (ccbio(ipts,ivm,:)*ccind(ipts,ivm,:)) / &
                     SUM(ccbio(ipts,ivm,:)*ccind(ipts,ivm,:))

             ELSE

                share_ncirc(:) = zero
                
             ENDIF

          ELSE
            
             ! Grass and crops
             share_ncirc(:) = zero

          ENDIF

          ! Make sure that the shares add up to one
          ! and that grass and croplands are properly
          ! dealt with
          share_ncirc(1) = zero
          share_ncirc(1) = 1 - SUM(share_ncirc(:))
                   
          DO icir = 1,ncirc

             IF (ccind(ipts,ivm,icir) .GT. zero) THEN

                biomass_to_cc_3d(ipts,ivm,icir) = bio(ipts,ivm) / &
                     ccind(ipts,ivm,icir) * share_ncirc(icir)

             ELSE

                biomass_to_cc_3d(ipts,ivm,icir) = zero

             ENDIF ! plants are present in this circ_class

          ENDDO ! icir

       ENDDO ! ivm
       
    ENDDO ! ipts

 END FUNCTION biomass_to_cc_3d

!! ================================================================================================================================







!! ================================================================================================================================
!! FUNCTION     : cc_to_lai_0d
!!
!>\BRIEF        Calculate the LAI based on the circ_class_biomass and circ_class_n
!!
!! DESCRIPTION : Calculates the LAI of a PFT/grid square based on the leaf biomass
!! 
!! RECENT CHANGE(S): None
!!
!! RETURN VALUE : ::LAI [m**2 m**{-2}]
!!
!! REFERENCE(S)	: 
!!
!! FLOWCHART	: None
!! \n
!_ ================================================================================================================================
   
  FUNCTION cc_to_lai_0d(ccbio,ccind,pft)

 !! 0. Variable and parameter declaration

    !! 0.1 Input variables
    INTEGER(i_std), INTENT(in)                        :: pft                !! Number of the PFT under study (unitless)
    REAL(r_std), DIMENSION(:)                         :: ccbio              !! circ_class_biomass. Biomass of an individual 
                                                                            !! tree @tex $(gC(N) ind^{-1})$ @endtex 
    REAL(r_std), DIMENSION(:)                         :: ccind              !! circ_class_n. Number of trees per circumference
                                                                            !! class

    !! 0.2 Output variables
    REAL(r_std)                                       :: cc_to_lai_0d       !! leaf area index @tex $(m^{2} m^{-2})$ @endtex

    !! 0.3 Modified variables

    !! 0.4 Local variables
    INTEGER(i_std)                                    :: icir               !! Index
    REAL(r_std)                                       :: leaf_biomass       !! Leaf mass for carbon calculated from 
                                                                            !! circ_class_biomass and circ_class_n.
                                                                            !! @tex $(gC m^{-2})$ @endtex
!_ ================================================================================================================================
 
    !! 1. Calculate leaf biomass
    cc_to_lai_0d = zero
    leaf_biomass = zero

    IF ( is_tree(pft) ) THEN

       DO icir = 1,ncirc
                
          leaf_biomass = leaf_biomass + ccbio(icir) * ccind(icir)
                                        
       ENDDO

    ELSE

       ! Grasses and cropland only have one circumference class
       leaf_biomass = ccbio(1) * ccind(1)
        
    ENDIF

    !! 2. Calculate the LAI from the leaf biomass
    IF(sla_dyn) THEN
       
       cc_to_lai_0d = log(1.+ ext_coeff_N(pft) * leaf_biomass * &
            slainit(pft))/(ext_coeff_N(pft)) 
        
    ELSE

       cc_to_lai_0d = leaf_biomass * sla(pft)

    ENDIF

 END FUNCTION cc_to_lai_0d


!! ================================================================================================================================
!! FUNCTION     : cc_to_lai_1d
!!
!>\BRIEF        Calculate the LAI based on the circ_class_biomass and circ_class_n
!!
!! DESCRIPTION : Calculates the LAI of a PFT/grid square based on the leaf biomass
!! 
!! RECENT CHANGE(S): None
!!
!! RETURN VALUE : ::LAI [m**2 m**{-2}]
!!
!! REFERENCE(S)	: 
!!
!! FLOWCHART	: None
!! \n
!_ ================================================================================================================================
   
  FUNCTION cc_to_lai_1d(npts,ccbio,ccind,pft)

 !! 0. Variable and parameter declaration

    !! 0.1 Input variables
    INTEGER(i_std), INTENT(in)                        :: npts               !! Domain size (unitless)
    INTEGER(i_std), INTENT(in)                        :: pft                !! number of PFT under study (unitless)
    REAL(r_std), DIMENSION(:,:,:,:)                   :: ccbio              !! circ_class_biomass. Biomass of an individual 
                                                                            !! tree @tex $(gC(N) ind^{-1})$ @endtex 
    REAL(r_std), DIMENSION(:,:)                       :: ccind              !! circ_class_n. Number of trees per circumference
                                                                            !! class

    !! 0.2 Output variables
                
    REAL(r_std), DIMENSION(npts)                      :: cc_to_lai_1d       !! leaf area index @tex $(m^{2} m^{-2})$ @endtex

    !! 0.3 Modified variables

    !! 0.4 Local variables
    INTEGER(i_std)                                    :: icir               !! Index
    REAL(r_std), DIMENSION(npts)                      :: leaf_biomass       !! Leaf mass for carbon calculated from 
                                                                            !! circ_class_biomass and circ_class_n.
                                                                            !! @tex $(gC m^{-2})$ @endtex
!_ ================================================================================================================================
 
    !! 1. Calculate leaf biomass for a specific PFT
    cc_to_lai_1d(:) = zero
    leaf_biomass(:) = zero

    ! Note that when used in 2d ivm is a single PFT
    IF ( is_tree(pft) ) THEN

       DO icir = 1,ncirc
                
          leaf_biomass(:) = leaf_biomass(:) + &
               ccbio(:,icir,ileaf,icarbon) * ccind(:,icir)
                                        
       ENDDO

    ELSE

       ! Grasses and cropland only have one circumference class
             
        leaf_biomass(:) = ccbio(:,1,ileaf,icarbon) * ccind(:,1)

     ENDIF

     !! 2. Calculate the LAI from the leaf biomass
     IF(sla_dyn) THEN

       cc_to_lai_1d(:) = log(1.+ ext_coeff_N(pft)* leaf_biomass(:) * &
            slainit(pft))/(ext_coeff_N(pft)) 

    ELSE

       cc_to_lai_1d(:) = leaf_biomass(:) * sla(pft)

    ENDIF

 END FUNCTION cc_to_lai_1d


!! ================================================================================================================================
!! FUNCTION     : wood_to_ba
!!
!>\BRIEF        Calculate basal area from woody biomass making use of allometric relationships
!!
!! DESCRIPTION : Calculate basal area of an individual tree from the woody biomass of that tree making 
!! use of allometric relationships given below. Here basal area is defined in line with its classical 
!! forestry meaning.
!! (i) woodmass = tree_ff * pipe_density*ba*height
!! (ii) height = pipe_tune2 * sqrt(4/pi*ba) ** pipe_tune_3  
!! 
!! RECENT CHANGE(S): None
!!
!! RETURN VALUE : basal area (m2 ind-1)
!!
!! REFERENCE(S)	: 
!!
!! FLOWCHART	: None
!! \n
!_ ================================================================================================================================
 
 FUNCTION wood_to_ba(biomass_temp, pft)

 !! 0. Variable and parameter declaration

    !! 0.1 Input variables

    INTEGER(i_std)                                          :: pft               !! PFT number (-)
    REAL(r_std), DIMENSION(:,:)                             :: biomass_temp      !! Biomass of an individual tree within a circ 
                                                                                 !! class @tex $(m^{2} ind^{-1})$ @endtex 

    !! 0.2 Output variables
                
    REAL(r_std), DIMENSION(ncirc)                           :: wood_to_ba        !! Basal area of an individual tree within a circ
                                                                                 !! class @tex $(m^{2} ind^{-1})$ @endtex

    !! 0.3 Modified variables

    !! 0.4 Local variables
    INTEGER(i_std)                                          :: l                 !! Index
    REAL(r_std)                                             :: woodmass_ind      !! Woodmass of an individual tree
                                                                                 !! @tex $(gC ind^{-1})$ @endtex

!_ ================================================================================================================================
 
 !! 1. Calculate basal area from woodmass

    IF ( is_tree(pft) ) THEN

       DO l = 1,ncirc
             
          ! Woodmass of an individual tree
          woodmass_ind = biomass_temp(l,iheartabove) + biomass_temp(l,isapabove)


          ! Basal area of that individual (m2 ind-1)  
          wood_to_ba(l) = (pi/4*(woodmass_ind/(tree_ff(pft)*pipe_density(pft)*pipe_tune2(pft))) &
                  **(2./pipe_tune3(pft)))**(pipe_tune3(pft)/(pipe_tune3(pft)+2))

       ENDDO

    ELSE

       WRITE(numout,*) 'pft ',pft
       CALL ipslerr (3,'wood_to_ba', &
            'wood_to_ba is not defined for this PFT.', &
            'See the output file for more details.','')

    ENDIF

 END FUNCTION wood_to_ba



!! ================================================================================================================================
!! FUNCTION     : wood_to_qmheight
!!
!>\BRIEF        Calculate the quadratic mean height from the biomass
!!
!! DESCRIPTION : Calculates the quadratic mean height from the biomass
!! 
!! RECENT CHANGE(S): None
!!
!! RETURN VALUE : ::qm_height (m)
!!
!! REFERENCE(S)	: 
!!
!! FLOWCHART	: None
!! \n
!_ ================================================================================================================================
   
  FUNCTION wood_to_qmheight(biomass_temp, ind, pft)

 !! 0. Variable and parameter declaration

    !! 0.1 Input variables

    INTEGER(i_std)                             :: pft               !! PFT number (-)
    REAL(r_std), DIMENSION(ncirc,nparts)       :: biomass_temp      !! Biomass of the leaves @tex $(gC m^{-2})$ @endtex
    REAL(r_std), DIMENSION(ncirc)              :: ind               !! Number of individuals @tex $(m^{-2})$ @endtex


    !! 0.2 Output variables
                
    REAL(r_std)                                :: wood_to_qmheight  !! quadratic mean height (m) 

    !! 0.3 Modified variables

    !! 0.4 Local variables
    REAL(r_std), DIMENSION(ncirc)              :: circ_class_ba     !! basal area for each circ_class @tex $(m^{2})$ @endtex
    REAL(r_std)                                :: qm_dia            !! quadratic mean diameter (m)

!_ ================================================================================================================================
 
    !! 1. Calculate qm_height from the biomass
    IF ( is_tree(pft) ) THEN

       ! Basal area at the tree level (m2 tree-1)
       circ_class_ba(:) = wood_to_ba(biomass_temp(:,:),pft) 
       
       IF (SUM(ind(:)) .NE. zero) THEN

          qm_dia = SQRT( 4/pi*SUM(circ_class_ba(:)*ind(:))/SUM(ind(:)) )
          
       ELSE
          
          qm_dia = zero

       ENDIF
       
       wood_to_qmheight = pipe_tune2(pft)*(qm_dia**pipe_tune3(pft))
             

    ! Grasses and croplands   
    ELSE

       ! Calculate height as a function of the leaf and structural biomass. Use structural
       ! biomass to make sure that the grasslands have a roughness length during the winter
       ! If the biomass increases, vegetation height will increase as well. Divide by 
       ! ind(ipts,j) to obtain the height of the individual. biomass(ileaf) is in gC m-2 
       ! whereas qm is the height of the individual.
       IF (SUM(ind(:)) .NE. zero) THEN

          wood_to_qmheight = SUM(biomass_temp(:,ileaf) + biomass_temp(:,isapabove)) / &
               SUM(ind(:)) * sla(pft) * lai_to_height(pft)

       ELSE

          wood_to_qmheight = zero

       ENDIF

    ENDIF ! is_tree(j)

 END FUNCTION wood_to_qmheight


!! ================================================================================================================================
!! FUNCTION     : wood_to_qmdia
!!
!>\BRIEF        Calculate the quadratic mean diameter from the biomass
!!
!! DESCRIPTION : Calculates the quadratic mean diameter from the aboveground biomass
!! 
!! RECENT CHANGE(S): None
!!
!! RETURN VALUE : ::qm_dia (m)
!!
!! REFERENCE(S)	: 
!!
!! FLOWCHART	: None
!! \n
!_ ================================================================================================================================
   
  FUNCTION wood_to_qmdia(biomass_temp, ind, pft)

 !! 0. Variable and parameter declaration

    !! 0.1 Input variables

    INTEGER(i_std)                             :: pft               !! PFT number (-)
    REAL(r_std), DIMENSION(ncirc,nparts)       :: biomass_temp      !! Biomass of the leaves @tex $(gC m^{-2})$ @endtex 
    REAL(r_std), DIMENSION(ncirc)              :: ind               !! Number of individuals @tex $(m^{-2})$ @endtex

    !! 0.2 Output variables
                
    REAL(r_std)                                :: wood_to_qmdia     !! quadratic mean diameter (m) 

    !! 0.3 Modified variables

    !! 0.4 Local variables
    REAL(r_std), DIMENSION(ncirc)              :: circ_class_ba     !! basal area for each circ_class @tex $(m^{2})$ @endtex

!_ ================================================================================================================================
 
    !! 1. Calculate qm_dia from the biomass
    IF ( is_tree(pft) ) THEN

       ! Basal area at the tree level (m2 tree-1)
       circ_class_ba(:) = wood_to_ba(biomass_temp(:,:),pft) 
       
       IF (SUM(ind(:)) .NE. zero) THEN

          wood_to_qmdia = SQRT( 4/pi*SUM(circ_class_ba(:)*ind(:))/SUM(ind(:)) )
          
       ELSE
          
          wood_to_qmdia = zero

       ENDIF


    ! Grasses and croplands   
    ELSE

       wood_to_qmdia = zero   
       
    ENDIF ! is_tree(pft)

 END FUNCTION wood_to_qmdia


!! ================================================================================================================================
!! FUNCTION     : wood_to_volume
!!
!>\BRIEF        This allometric function computes volume as a function of 
!! biomass at stand scale. Volume \f$(m^3 m^{-2}) = f(biomass (gC m^{-2}))\f$
!!
!! DESCRIPTION : None
!!
!! RECENT CHANGE(S): None
!! 
!! RETURN VALUE : biomass_to_volume
!!
!! REFERENCE(S)	: See above, module description.
!!
!! FLOWCHART	: None
!! \n
!_ ================================================================================================================================

  FUNCTION wood_to_volume(npts,ccbio,ccind,pft,branch_ratio,inc_branches)

 !! 0. Variable and parameter declaration

    !! 0.1 Input variables
    INTEGER(i_std), INTENT(in)                :: npts           !! Domain size (unitless)
    REAL(r_std), INTENT(in), DIMENSION(:,:,:) :: ccbio          !! circ_class_biomass or biomass of an 
                                                                !! individual tree in each circumference 
                                                                !! class @tex $(gC tree^{-1})$ @endtex  
    REAL(r_std), INTENT(in), DIMENSION(:)     :: ccind          !! Number of trees per circumference class
    REAL(r_std), INTENT(in)                   :: branch_ratio   !! Branch ratio of sap and heartwood biomass
                                                                !! unitless
    INTEGER(i_std), INTENT(in)                :: pft            !! Plant functional type (unitless)
    INTEGER(i_std), INTENT(in)                :: inc_branches   !! Include the branches in the volume calculation?
                                                                !! 0: exclude the branches from the volume calculation 
                                                                !! (thus correct the biomass for the branch ratio)
                                                                !! 1: include the branches in the volume calculation  
                                                                !! (thus use all aboveground biomass)
                                    
    

    !! 0.2 Output variables

    REAL(r_std)                               :: wood_to_volume !! The volume of wood per square meter 
                                                                !!  @tex $(m^3 m^{-2})$ @endtex

    !! 0.3 Modified variables

    !! 0.4 Local variables
    REAL(r_std), DIMENSION(nparts,nelements)  :: biomass        !! Total biomass at the stand level
                                                                !! @tex $(gC m^{-2})$ @endtex
    REAL(r_std)                               :: woody_biomass  !! Woody biomass at the stand level 
                                                                !! @tex $(gC m^{-2})$ @endtex

!_ ================================================================================================================================

 !! 1. Volume to biomass

    ! Calculate stand biomass from circ_class_biomass. In cc_to_biomass_2d
    ! which is used here, npts is not used but we need to pass it to have
    ! a consistent call structure for cc_to_biomass_2d and cc_to_biomass_4d
    biomass = cc_to_biomass(npts,pft,ccbio,ccind)

    ! Woody biomass used in the calculation
    IF (inc_branches .EQ. 0) THEN

       woody_biomass = (biomass(isapabove,icarbon)+biomass(iheartabove,icarbon)) * &
            (un - branch_ratio)

    ELSEIF (inc_branches .EQ. 1) THEN

       woody_biomass = (biomass(isapabove,icarbon)+biomass(iheartabove,icarbon))

    ELSE

       CALL ipslerr(3,'ERROR in wood_to_volume',&
            'Specify how the branches should be accounted for','','')

    ENDIF

    ! Wood volume expressed in m**3 / m**2
    wood_to_volume = woody_biomass/(pipe_density(pft))

  END FUNCTION wood_to_volume


!! ================================================================================================================================
!! FUNCTION     : wood_to_height
!!
!>\BRIEF        Calculate tree height from woody biomass making use of allometric relationships
!!
!! DESCRIPTION : Calculate height of an individual tree from the woody biomass of that tree making 
!! use of allometric relationships. This is the height used in forestry and for calculating the aerodynamic
!! interactions 
!! (i) height(:) = pipe_tune2(j)*(4/pi*ba(:))**(pipe_tune3(j)/2) and
!! (ii) woodmass_ind = tree_ff*pipe_density*ba*height   
!! 
!! RECENT CHANGE(S): None
!!
!! RETURN VALUE : height (m)
!!
!! REFERENCE(S)	: 
!!
!! FLOWCHART	: None
!! \n
!_ ================================================================================================================================
   
  FUNCTION wood_to_height(biomass_temp, pft)

 !! 0. Variable and parameter declaration

    !! 0.1 Input variables

    INTEGER(i_std)                                          :: pft                !! PFT number (-)
    REAL(r_std), DIMENSION(:,:)                             :: biomass_temp       !! Biomass of an individual tree within a circ 
                                                                                  !! class @tex $(m^{2} ind^{-1})$ @endtex 

    !! 0.2 Output variables
                
    REAL(r_std), DIMENSION(ncirc)                           :: wood_to_height     !! Height of an individual tree within a circ
                                                                                  !! class (m)

    !! 0.3 Modified variables

    !! 0.4 Local variables
    INTEGER(i_std)                                          :: l                  !! Index
    REAL(r_std)                                             :: woodmass_ind       !! Woodmass of an individual tree
                                                              !! @tex $(gC ind{-1})$ @endtex

!_ ================================================================================================================================
 
 !! 1. Calculate height from woodmass

    IF ( is_tree(pft) ) THEN
       
       DO l = 1,ncirc
         
          ! Woodmass of an individual tree (only the aboveground component)
          woodmass_ind = biomass_temp(l,isapabove) + biomass_temp(l,iheartabove)

          ! Height of that individual
          wood_to_height(l) = (((woodmass_ind/(tree_ff(pft)*&
               pipe_density(pft))*4/pi)**(pipe_tune3(pft)/2))*&
               pipe_tune2(pft))**(1/(pipe_tune3(pft)/2+1))
            
       ENDDO

    ELSE

       WRITE(numout,*) 'ERROR: wood_to_height, pft ',pft
       CALL ipslerr (3,'wood_to_height', &
            'wood_to_height is not defined for this PFT.', &
            'See the output file for more details.','')

    ENDIF

 END FUNCTION wood_to_height


!! ================================================================================================================================
!! FUNCTION     : wood_to_height_eff
!!
!>\BRIEF        Calculate the effective tree height from woody biomass making use of allometric relationships
!!
!! DESCRIPTION : Calculate the effective height of an individual tree from the woody biomass of that tree making 
!! use of allometric relationships. Effective height makes use of both above and belowground biomass and is
!! used in the calculation of the allocation according to deleuze and dhote, hydraulic architecture because also
!! the height of the belowground part should be included.
!! (i) height(:) = pipe_tune2(j)*(4/pi*ba(:))**(pipe_tune3(j)/2) and
!! (ii) woodmass_ind = tree_ff*pipe_density*ba*height   
!! 
!! RECENT CHANGE(S): None
!!
!! RETURN VALUE : height (m)
!!
!! REFERENCE(S)	: 
!!
!! FLOWCHART	: None
!! \n
!_ ================================================================================================================================
   
  FUNCTION wood_to_height_eff(biomass_temp, pft)

 !! 0. Variable and parameter declaration

    !! 0.1 Input variables

    INTEGER(i_std)                                          :: pft                !! PFT number (-)
    REAL(r_std), DIMENSION(:,:)                             :: biomass_temp       !! Biomass of an individual tree within a circ 
                                                                                  !! class @tex $(m^{2} ind^{-1})$ @endtex 

    !! 0.2 Output variables
                
    REAL(r_std), DIMENSION(ncirc)                           :: wood_to_height_eff !! Effective height of an individual tree within a circ
                                                                                  !! class (m)

    !! 0.3 Modified variables

    !! 0.4 Local variables
    INTEGER(i_std)                                          :: l                  !! Index
    REAL(r_std)                                             :: woodmass_ind       !! Woodmass of an individual tree
                                                                                  !! @tex $(gC ind{-1})$ @endtex
                                                                                  
!_ ================================================================================================================================
 
 !! 1. Calculate height from woodmass

    IF ( is_tree(pft) ) THEN
       
       DO l = 1,ncirc
         
          ! Woodmass of an individual tree. Both above and belowground biomass are used
          woodmass_ind = biomass_temp(l,isapabove) + biomass_temp(l,isapbelow) + &
             biomass_temp(l,iheartabove) + biomass_temp(l,iheartbelow)

          ! Height of that individual
          wood_to_height_eff(l) = (((woodmass_ind/(tree_ff(pft)*pipe_density(pft))*4/pi)**(pipe_tune3(pft)/2))&
               *pipe_tune2(pft))**(1/(pipe_tune3(pft)/2+1))
          
       ENDDO

    ELSE

       WRITE(numout,*) 'pft ',pft
       CALL ipslerr (3,'wood_to_height_eff', &
            'wood_to_height_eff is not defined for this PFT.', &
            'See the output file for more details.','')

    ENDIF

 END FUNCTION wood_to_height_eff


!! ================================================================================================================================
!! FUNCTION     : wood_to_dia
!!
!>\BRIEF        Calculate diameter from woody biomass making use of allometric relationships
!!
!! DESCRIPTION : Calculate diameter of an individual tree from the woody biomass of that tree making 
!! use of allometric relationships. Makes only use of the aboveground biomass and relates to the
!! typical forestry diameter (but not normalized at 1.3 m)
!! (i) woodmass_ind = tree_ff * pipe_density * height * pi/4*dia**2
!! (ii) height = pipe_tune2 * dia * pipe_tune3 
!! 
!! RECENT CHANGE(S): None
!!
!! RETURN VALUE : diameter (m)
!!
!! REFERENCE(S)	: 
!!
!! FLOWCHART	: None
!! \n
!_ ================================================================================================================================
   
  FUNCTION wood_to_dia(biomass_temp, pft)


 !! 0. Variable and parameter declaration


    !! 0.1 Input variables

    INTEGER(i_std)                                          :: pft               !! PFT number (-)
    REAL(r_std), DIMENSION(:,:)                             :: biomass_temp      !! Biomass of an individual tree within a circ 
                                                                                 !! class @tex $(m^{2} ind^{-1})$ @endtex 
    !! 0.2 Output variables
                
    REAL(r_std), DIMENSION(ncirc)                           :: wood_to_dia       !! Diameter of an individual tree within a circ
                                                                                 !! class (m)

    !! 0.3 Modified variables

    !! 0.4 Local variables
    INTEGER(i_std)                                          :: l                 !! Index
    REAL(r_std)                                             :: woodmass_ind      !! Woodmass of an individual tree
                                                                                 !! @tex $(gC ind^{-1})$ @endtex

!_ ================================================================================================================================
 
 !! 1. Calculate basal area from woodmass

    IF ( is_tree(pft) ) THEN
       
       DO l = 1,ncirc
         
          ! Woodmass of an individual tree
          woodmass_ind = biomass_temp(l,isapabove) + biomass_temp(l,iheartabove)

          ! Basal area of that individual (m2 ind-1)  
          wood_to_dia(l) = (4/pi*woodmass_ind/(tree_ff(pft)*pipe_density(pft)*pipe_tune2(pft))) ** &
                  (1./(2+pipe_tune3(pft)))
                 
       ENDDO

    ELSE

       WRITE(numout,*) 'pft ',pft
       CALL ipslerr (3,'wood_to_dia', &
            'wood_to_dia is not defined for this PFT.', &
            'See the output file for more details.','')

    ENDIF

 END FUNCTION wood_to_dia


!! ================================================================================================================================
!! FUNCTION     : wood_to_dia_eff
!!
!>\BRIEF        Calculate effective diameter from woody biomass making use of allometric relationships
!!
!! DESCRIPTION : Calculate the effective diameter of an individual tree from the woody biomass of that tree making 
!! use of allometric relationships. Effective diameter accounts for both above and belowground biomass. 
!! (i) woodmass_ind = tree_ff * pipe_density * height * pi/4*dia**2
!! (ii) height = pipe_tune2 * dia * pipe_tune3 
!! 
!! RECENT CHANGE(S): None
!!
!! RETURN VALUE : diameter (m)
!!
!! REFERENCE(S)	: 
!!
!! FLOWCHART	: None
!! \n
!_ ================================================================================================================================
   
  FUNCTION wood_to_dia_eff(biomass_temp, pft)

 !! 0. Variable and parameter declaration

    !! 0.1 Input variables

    INTEGER(i_std)                                          :: pft               !! PFT number (-)
    REAL(r_std), DIMENSION(:,:)                             :: biomass_temp      !! Biomass of an individual tree within a circ 
                                                                                 !! class @tex $(m^{2} ind^{-1})$ @endtex 

    !! 0.2 Output variables
                
    REAL(r_std), DIMENSION(ncirc)                           :: wood_to_dia_eff   !! Diameter of an individual tree within a circ
                                                                                 !! class (m)

    !! 0.3 Modified variables

    !! 0.4 Local variables
    INTEGER(i_std)                                          :: l                 !! Index
    REAL(r_std)                                             :: woodmass_ind      !! Woodmass of an individual tree
                                                                                 !! @tex $(gC ind^{-1})$ @endtex

!_ ================================================================================================================================
 
 !! 1. Calculate basal area from woodmass

    IF ( is_tree(pft) ) THEN
       
       DO l = 1,ncirc
         
          ! Woodmass of an individual tree
          woodmass_ind = biomass_temp(l,isapabove) + biomass_temp(l,isapbelow) + &
             biomass_temp(l,iheartabove) + biomass_temp(l,iheartbelow)

          ! Basal area of that individual (m2 ind-1)  
          wood_to_dia_eff(l) = (4/pi*woodmass_ind / &
               (tree_ff(pft)*pipe_density(pft)*pipe_tune2(pft))) ** &
               (1./(2+pipe_tune3(pft)))
                 
       ENDDO

    ELSE

       WRITE(numout,*) 'pft ',pft
       CALL ipslerr (3,'wood_to_dia_eff', &
            'wood_to_dia_eff is not defined for this PFT.', &
            'See the output file for more details.','')

    ENDIF

 END FUNCTION wood_to_dia_eff


!! ================================================================================================================================
!! FUNCTION     : wood_to_circ
!!
!>\BRIEF        Calculate circumference from woody biomass making use of allometric relationships
!!
!! DESCRIPTION : All this does it computer the diameter using a different routine, and then
!!               convert that into a circumference.  
!! 
!! RECENT CHANGE(S): None
!!
!! RETURN VALUE : circumference (m)
!!
!! REFERENCE(S)	: 
!!
!! FLOWCHART	: None
!! \n
!_ ================================================================================================================================
   
  FUNCTION wood_to_circ(biomass_temp, pft)

 !! 0. Variable and parameter declaration

    !! 0.1 Input variables

    INTEGER(i_std)                                          :: pft               !! PFT number (-)
    REAL(r_std), DIMENSION(:,:)                             :: biomass_temp      !! Biomass of an individual tree within a circ 
                                                                                 !! class @tex $(m^{2} ind^{-1})$ @endtex 

    !! 0.2 Output variables
                
    REAL(r_std), DIMENSION(ncirc)                           :: wood_to_circ      !! Circumference of an individual tree within a circ
                                                                                 !! class (m)

    !! 0.3 Modified variables

    !! 0.4 Local variables

!_ ================================================================================================================================
 
    !! 1. Calculate diameter from woodmass

    wood_to_circ(:)=val_exp

    wood_to_circ(:)=wood_to_dia(biomass_temp(:,:),pft)

    ! convert to a circumference (m)
    wood_to_circ(:) =  wood_to_circ(:)*pi

 END FUNCTION wood_to_circ


   !! ================================================================================================================================
!! FUNCTION     : wood_to_cv_eff
!!
!>\BRIEF        Calculate crown volume from woody biomass making use of allometric relationships
!!
!! DESCRIPTION : Calculate basal areadiameter of an individual tree from the woody biomass of that tree making 
!! use of allometric relationships:
!! (i) Basal area is written as a function of wood_mass: woodmass_ind = tree_ff*pipe_density*ba*height
!! (ii) height = pipe_tune2 * sqrt( 4/pi*ba ) ** pipe_tune3
!! (iii) cn = pipe_tune1 * sqrt( 4/pi*ba ) ** pipe_tune_exp_coeff  
!! (iv) cv = 4.0 / 3.0 * pi * ( SQRT( cn/pi ) ) ** 3
!! 
!! NOTE: the difference between this and wood_to_cv is that this include aboveground and belowground
!!       biomass in the calculation
!!
!! RECENT CHANGE(S): None
!!
!! RETURN VALUE : diameter (m3 ind-1)
!!
!! REFERENCE(S)	: 
!!
!! FLOWCHART	: None
!! \n
!_ ================================================================================================================================
   
  FUNCTION wood_to_cv_eff(biomass_temp, pft)

 !! 0. Variable and parameter declaration

    !! 0.1 Input variables

    INTEGER(i_std)                                          :: pft               !! PFT number (-)
    REAL(r_std), DIMENSION(:,:)                             :: biomass_temp      !! Biomass of an individual tree within a circ 
                                                                                 !! class @tex $(m^{2} ind^{-1})$ @endtex 

    !! 0.2 Output variables
                
    REAL(r_std), DIMENSION(ncirc)                           :: wood_to_cv_eff    !! Crown volume of an individual tree 
                                                                                 !! @tex $(m^{2} ind{-1})$ @endtex

    !! 0.3 Modified variables

    !! 0.4 Local variables
    INTEGER(i_std)                                          :: l                 !! Index
    REAL(r_std)                                             :: woodmass_ind      !! Woodmass of an individual tree
                                                                                 !! @tex $(gC ind^{-1})$ @endtex
    REAL(r_std), DIMENSION(ncirc)                           :: dia               !! Diameter of an individual tree (m)

!_ ================================================================================================================================
 
 !! 1. Calculate crown volume from woodmass

    IF ( is_tree(pft) ) THEN

       ! Crown diameter of the individual tree (m2 ind-1)  
       dia(:) = (4./pi*wood_to_cn_eff(biomass_temp(:,:),pft))**(1./2.)
       wood_to_cv_eff(:) = pi/6.*dia(:)**3.  
       ! WRITE(numout,*) 'wood_to_cv_eff, ', wood_to_cv_eff(:)

    ELSE

       WRITE(numout,*) 'pft ',pft
       CALL ipslerr (3,'wood_to_cv_eff', &
            'wood_to_cv_eff is not defined for this PFT.', &
            'See the output file for more details.','')

    ENDIF

  END FUNCTION wood_to_cv_eff

!! ================================================================================================================================
!! FUNCTION     : wood_to_cn
!!
!>\BRIEF        Calculate crown area from woody biomass making use of allometric relationships
!!
!! DESCRIPTION : Calculate crown area of an individual tree from the woody biomass of that tree 
!! making use of allometric relationship that relates crown area (cn) to diameter (dia) as 
!! pipe_tune1*dia**pipe_tune_exp_coeff where the pipe_tune parameters are pft-specific. 
!! (i) Basal area is written as a function of wood_mass: woodmass_ind = tree_ff*pipe_density*ba*height
!! (ii) height = pipe_tune2*sqrt(4/pi*ba)**pipe_tune3
!! (iii) cn = pipe_tune1*sqrt(4/pi*ba)**pipe_tune_exp_coeff   
!! 
!! RECENT CHANGE(S): None
!!
!! RETURN VALUE : crown area (m2 ind-1)
!!
!! REFERENCE(S)	: 
!!
!! FLOWCHART	: None
!! \n
!_ ================================================================================================================================
   
  FUNCTION wood_to_cn(biomass_temp, pft)

 !! 0. Variable and parameter declaration

    !! 0.1 Input variables

    INTEGER(i_std)                                          :: pft               !! PFT number (-)
    REAL(r_std), DIMENSION(:,:)                             :: biomass_temp      !! biomass of an individual tree within a circ 
                                                                                 !! class @tex $(m^{2} ind^{-1})$ @endtex 

    !! 0.2 Output variables
                
    REAL(r_std), DIMENSION(ncirc)                           :: wood_to_cn        !! Crown area of an individual tree within a circ
                                                                                 !! class @tex $(m^{2} ind-1)$ @endtex

    !! 0.3 Modified variables

    !! 0.4 Local variables
    INTEGER(i_std)                                          :: l                 !! Index
    REAL(r_std)                                             :: woodmass_ind      !! Woodmass of an individual tree
                                                                                 !! @tex $(gC ind{-1})$ @endtex

!_ ================================================================================================================================
 
 !! 1. Calculate crown area from woodmass

    IF ( is_tree(pft) ) THEN
       
       DO l = 1,ncirc
         
          ! Woodmass of an individual tree
          woodmass_ind = biomass_temp(l,isapabove) + biomass_temp(l,iheartabove)
 
          ! Crown area of that individual
          ! cn = pipe_tune1*sqrt(4/pi*ba)**pipe_tune_exp_coeff 
          wood_to_cn(l) = pipe_tune1(pft) * SQRT( 4/pi*(pi/4*(woodmass_ind/&
               (tree_ff(pft)*pipe_density(pft)*pipe_tune2(pft)))**&
               (2./pipe_tune3(pft)))**(pipe_tune3(pft)/&
               (pipe_tune3(pft)+2)) ) ** pipe_tune_exp_coeff(pft)
                
       ENDDO

    ELSE

       WRITE(numout,*) 'ERROR: wood_to_cn, pft ',pft
       CALL ipslerr (3,'ERROR: wood_to_cn', &
            'wood_to_cn is not defined for this PFT.', &
            'See the output file for more details.','')

    ENDIF

 END FUNCTION wood_to_cn

!! ================================================================================================================================
!! FUNCTION     : wood_to_cn_eff
!!
!>\BRIEF        Calculate crown area from woody biomass making use of allometric relationships
!!
!! DESCRIPTION : Calculate crown area of an individual tree from the woody biomass of that tree 
!! making use of allometric relationship that relates crown area (cn) to diameter (dia) as 
!! pipe_tune1*dia**pipe_tune_exp_coeff where the pipe_tune parameters are pft-specific. 
!! (i) Basal area is written as a function of wood_mass: woodmass_ind = tree_ff*pipe_density*ba*height
!! (ii) height = pipe_tune2*sqrt(4/pi*ba)**pipe_tune3
!! (iii) cn = pipe_tune1*sqrt(4/pi*ba)**pipe_tune_exp_coeff   
!!
!! NOTE: This is different from wood_to_cn because this uses both the aboveground and
!!       belowground wood mass.
!! 
!! RECENT CHANGE(S): None
!!
!! RETURN VALUE : crown area (m2 ind-1)
!!
!! REFERENCE(S)	: 
!!
!! FLOWCHART	: None
!! \n
!_ ================================================================================================================================
   
  FUNCTION wood_to_cn_eff(biomass_temp, pft)

 !! 0. Variable and parameter declaration

    !! 0.1 Input variables

    INTEGER(i_std)                                          :: pft               !! PFT number (-)
    REAL(r_std), DIMENSION(:,:)                             :: biomass_temp      !! biomass of an individual tree within a circ 
                                                                                 !! class @tex $(m^{2} ind^{-1})$ @endtex 

    !! 0.2 Output variables
                
    REAL(r_std), DIMENSION(ncirc)                           :: wood_to_cn_eff    !! Crown area of an individual tree within a circ
                                                                                 !! class @tex $(m^{2} ind-1)$ @endtex

    !! 0.3 Modified variables

    !! 0.4 Local variables
    INTEGER(i_std)                                          :: l                 !! Index
    REAL(r_std)                                             :: woodmass_ind      !! Woodmass of an individual tree
                                                                                 !! @tex $(gC ind{-1})$ @endtex

!_ ================================================================================================================================
 
 !! 1. Calculate crown area from woodmass

    IF ( is_tree(pft) ) THEN
       
       DO l = 1,ncirc
         
          ! Woodmass of an individual tree
          woodmass_ind = biomass_temp(l,isapabove) + biomass_temp(l,iheartabove) + &
               biomass_temp(l,isapbelow) + biomass_temp(l,iheartbelow)
 
          ! Crown area of that individual
          ! cn = pipe_tune1*sqrt(4/pi*ba)**pipe_tune_exp_coeff 
          wood_to_cn_eff(l) = pipe_tune1(pft) * SQRT( 4/pi*(pi/4*(woodmass_ind/&
               (tree_ff(pft)*pipe_density(pft)*pipe_tune2(pft)))**(2./pipe_tune3(pft)))**&
               (pipe_tune3(pft)&
               /(pipe_tune3(pft)+2)) ) ** pipe_tune_exp_coeff(pft)
                
       ENDDO

    ELSE

       WRITE(numout,*) 'pft ',pft
       CALL ipslerr (3,'wood_to_cn_eff', &
            'wood_to_cn_eff is not defined for this PFT.', &
            'See the output file for more details.','')

    ENDIF

  END FUNCTION wood_to_cn_eff


!! ================================================================================================================================
!! FUNCTION     : Nmax
!!
!>\BRIEF        This function determines the maximum number of trees per hectare for 
!! a given quadratic mean diameter (Dg). It applies the self-thinning principle 
!! of Reineke (1933), with Dg instead of mean diameter (Dhote 1999).
!! Parameterization: Dhote (1999) for broad-leaved and Vacchiano (2008)
!! for needle-leaved.
!!
!! DESCRIPTION : None
!!
!! RECENT CHANGE(S): None
!!
!! RETURN VALUE : Nmax
!!
!! REFERENCE(S)   : See above, module description.
!!
!! FLOWCHART   : None
!! \n
!_ ================================================================================================================================
   
  FUNCTION Nmax(Dg,no_pft)

 !! 0. Variable and parameter declaration

    !! 0.1 Input variables

    REAL(r_std)      :: Dg              !! Quadratic mean diameter (cm)
    INTEGER(i_std)   :: no_pft          !! Plant functional type (unitless) 

    !! 0.2 Output variables

    REAL(r_std)      :: Nmax            !! Maximum number of trees according to the self-thinning model in (trees ha-1)

    !! 0.3 Modified variables

    !! 0.4 Local variables

!_ ================================================================================================================================
 !! 1. maximum number of trees per hectare for a given quadratic mean diameter

    IF (is_tree(no_pft)) THEN

       ! thinning curve of the MTC
       Nmax = (Dg/alpha_self_thinning(no_pft))**(un/beta_self_thinning(no_pft))

       ! Truncate the range to avoid huge numbers due the exponental model used to describe self-thinning
       ! Don't set to ntreesmax else stomate_mark_kill will assume we are at self-thinning and
       ! won't apply background mortality. 
       ! Nmax = MIN( Nmax, ntreesmax(no_pft))
       Nmax = MAX( Nmax, dens_target(no_pft) )

    ELSE

       WRITE(numout,*) 'Self thinning is not defined for PFT, ', no_pft
       CALL ipslerr (3,'nmax', &
            'Self thinning is not defined for this PFT.', &
            'See the output file for more details.','')

    ENDIF

  END FUNCTION Nmax


!! ================================================================================================================================
!! FUNCTION     : Nmaxyield
!!
!>\BRIEF        This function determines the maximum number of trees per hectare for 
!! a given quadratic mean diameter (Dg). It applies the 75 percentile  of the European 
!! yield tables
!!
!! DESCRIPTION : None
!!
!! RECENT CHANGE(S): None
!!
!! RETURN VALUE : Nmaxyield
!!
!! REFERENCE(S)   : See above, module description.
!!
!! FLOWCHART   : None
!! \n
!_ ================================================================================================================================
   
  FUNCTION Nmaxyield(Dg,no_pft)

 !! 0. Variable and parameter declaration

    !! 0.1 Input variables

    REAL(r_std)      :: Dg              !! Quadratic mean diameter (cm)
    INTEGER(i_std)   :: no_pft          !! Plant functional type (unitless) 

    !! 0.2 Output variables

    REAL(r_std)      :: Nmaxyield       !! Maximum number of trees according to the self-thinning model

    !! 0.3 Modified variables

    !! 0.4 Local variables

!_ ================================================================================================================================
 !! 1. maximum number of trees per hectare for a given quadratic mean diameter

    IF (is_tree(no_pft)) THEN

       ! thinning curve of the MTC
       Nmaxyield = (Dg/alpha_rdi_upper(no_pft))**(un/beta_rdi_upper(no_pft))

       ! Truncate the range to avoid huge numbers due the exponental 
       ! model used to describe self-thinning
       Nmaxyield = MIN( Nmaxyield, nmaxtrees(no_pft) )
       Nmaxyield = MAX( Nmaxyield, dens_target(no_pft) )

    ELSE

       WRITE(numout,*) 'Self thinning is not defined for PFT, ', no_pft
       CALL ipslerr (3,'ERROR: Nmaxyield', &
            'Self thinning is not defined for this PFT.', &
            'See the output file for more details.','')

    ENDIF

  END FUNCTION Nmaxyield


!! ================================================================================================================================
!! FUNCTION     : Nminyield
!!
!>\BRIEF        This function determines the minimum number of trees per hectare for 
!! a given quadratic mean diameter (Dg). It applies the 25 percentile of the European 
!! yield tables
!!
!! DESCRIPTION : None
!!
!! RECENT CHANGE(S): None
!!
!! RETURN VALUE : Nminyield
!!
!! REFERENCE(S)   : See above, module description.
!!
!! FLOWCHART   : None
!! \n
!_ ================================================================================================================================
   
  FUNCTION Nminyield(Dg,no_pft)

 !! 0. Variable and parameter declaration

    !! 0.1 Input variables

    REAL(r_std)      :: Dg              !! Quadratic mean diameter (cm)
    INTEGER(i_std)   :: no_pft          !! Plant functional type (unitless) 

    !! 0.2 Output variables

    REAL(r_std)      :: Nminyield       !! Minimum number of trees according to the self-thinning model

    !! 0.3 Modified variables

    !! 0.4 Local variables

!_ ================================================================================================================================
 !! 1. minimum number of trees per hectare for a given quadratic mean diameter

    IF (is_tree(no_pft)) THEN

       ! thinning curve of the MTC
       Nminyield = (Dg/alpha_rdi_lower(no_pft))**(un/beta_rdi_lower(no_pft))

       ! Truncate the range to avoid huge numbers due the exponental 
       ! model used to describe self-thinning
       Nminyield = MIN( Nminyield, nmaxtrees(no_pft) )
       Nminyield = MAX( Nminyield, dens_target(no_pft) )

    ELSE

       WRITE(numout,*) 'Self thinning is not defined for PFT, ', no_pft
       CALL ipslerr (3,'ERROR: Nminyield', &
            'Self thinning is not defined for this PFT.', &
            'See the output file for more details.','')


    ENDIF

  END FUNCTION Nminyield


!! ================================================================================================================================
!! FUNCTION     : calculate_rdi
!!
!>\BRIEF        Calculate the relative density index of a stand
!!
!! DESCRIPTION   : None
!!
!! RECENT CHANGE(S): None
!!
!! RETURN VALUE : calculate_rdi
!!
!! REFERENCE(S)   : Bellassen, V., Le Maire, G., Dhote, J.F., Viovy, N., Ciais, P., 2010. 
!! Modeling forest management within a global vegetation model – Part 1: 
!! model structure and general behaviour. Ecological Modelling 221, 2458–2474.
!!
!! FLOWCHART   : None
!! \n
!_ ================================================================================================================================
  
  FUNCTION calculate_rdi(diameters_temp,circ_class_n_temp,ivm)

 !! 0. Variable and parameter declaration

    IMPLICIT NONE

    !! 0.1 Input variables
    REAL(r_std),DIMENSION(ncirc),INTENT(in)            :: circ_class_n_temp   !! The distribution of individuals
                                                                              !! for a given grid square and PFT  
    REAL(r_std),DIMENSION(ncirc),INTENT(in)            :: diameters_temp      !! The diameters of the trunks
                                                                              !! for each circ class (m)
    INTEGER(i_std),INTENT(IN)                          :: ivm                 !! The number of the PFT
                                                                              !! (unitless)

    !! 0.2 Output variables
    REAL(r_std)                                        :: calculate_rdi       !! The RDI of the stand 
                                                                              !! (unitless, always positive)

    !! 0.3 Modified variables

    !! 0.4 Local variables
    REAL(r_std)                                        :: Dg                  !! The quadratic mean diameter
    REAL(r_std)                                        :: ind_loc
    INTEGER(i_std)                                     :: icir                !! Indices

!_ ================================================================================================================================

 !! 1. Calculate the relative density index for a stand, based on
 !!    the number of trees in each circumference class and the
 !!    the diameter of a model tree in each class. 
    ind_loc=SUM(circ_class_n_temp(:))

    Dg=zero

    DO icir=1,ncirc
       Dg=Dg+circ_class_n_temp(icir)*m2_to_ha*(diameters_temp(icir)*m_to_cm)**2
    ENDDO
    Dg=SQRT(Dg/(ind_loc*m2_to_ha))
   
    calculate_rdi=(ind_loc*m2_to_ha)/Nmax(Dg,ivm)

  END FUNCTION calculate_rdi


!! ================================================================================================================================
!! SUBROUTINE  : check_biomass_sync
!!
!>\BRIEF       
!!
!! DESCRIPTION : 
!! RECENT CHANGE(S): None
!!
!! MAIN OUTPUT VARIABLE(S):
!!
!! REFERENCE(S) : None
!!
!! FLOWCHART    : None
!! \n
!_ ================================================================================================================================
  SUBROUTINE check_biomass_sync ( check_point, npts, biomass, &
       circ_class_biomass, circ_class_n , ind)

 !! 0. Variable and parameter description

    !! 0.1 Input variables
    INTEGER(i_std), INTENT(in)                                 :: npts                 !! Domain size (unitless)
    REAL(r_std), DIMENSION(:,:,:,:,:), INTENT(in)              :: circ_class_biomass   !! Biomass of the componets of the model  
                                                                                       !! tree within a circumference
                                                                                       !! class @tex $(gC ind^{-1})$ @endtex  
    REAL(r_std), DIMENSION(:,:,:), INTENT(in)                  :: circ_class_n         !! Number of individuals in each circ class
                                                                                       !! @tex $(m^{-2})$ @endtex
    REAL(r_std), DIMENSION(:,:,:,:), INTENT(in)                :: biomass              !! Stand level biomass 
                                                                                       !! @tex $(gC m^{-2})$ @endtex
    CHARACTER(*),INTENT(in)                                    :: check_point          !! A flag to indicate at which
                                                                                       !! point in the code we're doing
                                                                                       !! this check
    REAL(r_std), DIMENSION(:,:), INTENT(in)                    :: ind                  !! Density of individuals 
                                                                                       !! @tex $(m^{-2})$ @endtex 

    !! 0.2 Output variables

    !! 0.3 Modified variables

    !! 0.4 Local variables
    INTEGER                                                    :: iele,ipts,ivm,ipar,icir
    REAL(r_std)                                                :: total_circ_class_biomass
    REAL(r_std),DIMENSION(ncirc)                               :: tree_size
    LOGICAL                                                    :: lnegative
    LOGICAL                                                    :: lsync
    LOGICAL                                                    :: lindividuals
    REAL(r_std), DIMENSION(npts,nvm,nelements)                 :: bm_sync              !! The difference betweeen the
                                                                                       !! biomass in the circ_classes and
                                                                                       !! the total biomass 
                                                                                       !! @tex $(gC m^{-2})$ @endtex
!_ ================================================================================================================================

    lsync=.TRUE.
    lnegative=.FALSE.
    lindividuals=.TRUE.

    bm_sync(:,:,:)=zero

    !+++NOTE+++
    ! 5-10% is gained in speed by skipping this routine
    !++++++++++

    ! Check to see if the biomass is not equal to the total biomass
    ! in circ_class_biomass anywhere.
    DO ipts=1,npts

       DO ivm=1,nvm

          ! Only woody PFTs have circumference classes therefore
          ! only woody PFTs need to be syncronized
          IF(.NOT.hack_circ_class)THEN
             IF(is_tree(ivm)) THEN
                tree_size(:)=zero
                DO icir=1,ncirc
                   tree_size(icir) = &
                        SUM(circ_class_biomass(ipts,ivm,icir,:,icarbon))
                ENDDO
                DO icir=2,ncirc
                   IF(tree_size(icir) .LT. tree_size(icir-1)-min_stomate)THEN
                      WRITE(numout,*) 'ERROR: stopping in sync'
                      WRITE(numout,*) check_point
                      WRITE(numout,*) 'ipts,ivm: ',ipts,ivm,icir,icir-1
                      WRITE(numout,*) 'tree_size(icir), tree_size(icir-1), ',&
                           tree_size(icir), tree_size(icir-1), tree_size(icir) - &
                           tree_size(icir-1)  
                      WRITE(numout,*) 'icir, tree_size: ',icir, tree_size(:)
                      CALL ipslerr (3,'check_biomass_sync', &
                           'The size of the trees in the circ class are ',&
                           'not monotonically increasing!',&
                           'Look in the output file for more details.')   
                   ENDIF
                ENDDO
             ENDIF
          ENDIF
          
          DO iele=1,nelements
                
             DO ipar=1,nparts
                
                total_circ_class_biomass=zero
           
                DO icir=1,ncirc
                   
                   total_circ_class_biomass=total_circ_class_biomass + &
                        circ_class_biomass(ipts,ivm,icir,ipar,iele) * &
                        circ_class_n(ipts,ivm,icir)

                   ! Check as well to see if our biomass is ever negative.
                   ! It really should not be.
                   IF(circ_class_biomass(ipts,ivm,icir,ipar,iele) .LT. &
                        -min_stomate)THEN

                      lnegative=.TRUE.
                      WRITE(numout,*) '!***********************************'
                      WRITE(numout,*) 'Error: Negative biomass component!'
                      WRITE(numout,*) 'Check point: ',TRIM(check_point)
                      WRITE(numout,*) 'circ_class_biomass(ipts,ivm,icir,ipar,iele) ',&
                           circ_class_biomass(ipts,ivm,icir,ipar,iele)
                      WRITE(numout,'(A,5I5)') 'ipts,ivm,icir,ipar,iele', &
                           ipts,ivm,icir,ipar,iele
                      WRITE(numout,*) '!***********************************'

                   ENDIF
                ENDDO

                IF(ABS(biomass(ipts,ivm,ipar,iele) -  &
                     total_circ_class_biomass) .GT. sync_threshold)THEN

                   WRITE(numout,*) '!***********************************'
                   WRITE(numout,*) 'Biomass and circ_class_biomass are not equal!'
                   WRITE(numout,*) 'Check point: ',TRIM(check_point)
                   WRITE(numout,100) 'biomass(ipts,ivm,ipar,iele) ',&
                        biomass(ipts,ivm,ipar,iele)
                   WRITE(numout,100) 'total_circ_class_biomass ',&
                        total_circ_class_biomass
                   WRITE(numout,100) 'Difference: ',&
                        ABS(biomass(ipts,ivm,ipar,iele) - total_circ_class_biomass)
                   WRITE(numout,*) 'ipts,ivm,ipar,iele',ipts,ivm,ipar,iele
                   WRITE(numout,*) '!***********************************'
100                FORMAT(A,E20.10)
                   lsync=.FALSE.

                ENDIF

             ENDDO

             ! we are not going to save the biomass for every component 
             ! right now, just the total
             bm_sync(ipts,ivm,iele)=zero

             DO ipar=1,nparts
                   
                DO icir=1,ncirc

                   bm_sync(ipts,ivm,iele)=bm_sync(ipts,ivm,iele) + &
                        circ_class_biomass(ipts,ivm,icir,ipar,iele) * &
                        circ_class_n(ipts,ivm,icir)

                ENDDO ! ncirc

             ENDDO ! nparts

             bm_sync(ipts,ivm,iele)=ABS(bm_sync(ipts,ivm,iele)-&
                  SUM(biomass(ipts,ivm,:,iele)))

          ENDDO ! nelements

          IF(ABS(ind(ipts,ivm) - SUM(circ_class_n(ipts,ivm,:))) &
               .GT. min_stomate)THEN
             
             WRITE(numout,*) '!***********************************'
             WRITE(numout,*) 'ind and circ_class_n are not equal!'
             WRITE(numout,*) 'Check point: ',TRIM(check_point)
             WRITE(numout,100) 'ind(ipts,ivm) ',&
                  ind(ipts,ivm)
             WRITE(numout,100) 'circ_class_n ',&
                  SUM(circ_class_n(ipts,ivm,:))
             WRITE(numout,100) 'Difference: ',&
                  ABS(ind(ipts,ivm) - SUM(circ_class_n(ipts,ivm,:)))
             WRITE(numout,*) 'ipts,ivm',ipts,ivm
             WRITE(numout,*) '!***********************************'
             lindividuals = .FALSE.

          ENDIF

       ENDDO ! loop over PFTs

    ENDDO ! loop over points
 
    IF(.NOT. lsync) THEN
       WRITE(numout,*) 'ERROR: stopping in sync #2'
       WRITE(numout,*) 'Stopping'
       CALL ipslerr (3,'check_biomass_sync', &
            'circ_class_biomass*circ_class_n is not equal to the total biomass',&
            'Look in the output file for more details.',&
            '')
    ENDIF

    If(lnegative) THEN
       WRITE(numout,*) 'ERROR: negative biomass'
       WRITE(numout,*) 'Stopping'
       CALL ipslerr (3,'check_biomass_sync', &
            'One of the biomass pools is negative!',&
            'Look in the output file for more details.',&
            '')
    ENDIF

    IF(.NOT. lindividuals) THEN
       WRITE(numout,*) 'ERROR: number of individuals started to diverge'
       WRITE(numout,*) 'Stopping'
       CALL ipslerr (3,'check_biomass_sync', &
            'ind and circ_class_n are not similar',&
            'Look in the output file for more details.',&
            '')
    ENDIF
    

  END SUBROUTINE check_biomass_sync

!! ================================================================================================================================
!! SUBROUTINE  : check_surface_area
!!
!>\BRIEF       
!!
!! DESCRIPTION : 
!! RECENT CHANGE(S): None
!!
!! MAIN OUTPUT VARIABLE(S):
!!
!! REFERENCE(S) : None
!!
!! FLOWCHART    : None
!! \n
!_ ================================================================================================================================
  SUBROUTINE check_surface_area(check_point, npts, veget_max_begin, &
       veget_max, test_type)

  !! 0. Variable and parameter description

    !! 0.1 Input variables
    INTEGER(i_std), INTENT(in)                        :: npts                 !! Domain size (unitless)
    CHARACTER(*),INTENT(in)                           :: check_point          !! A flag to indicate at which
                                                                              !! point in the code we're doing
                                                                              !! this check
    CHARACTER(*), INTENT(in)                          :: test_type            !! Which test do we want to run on veget_max
    REAL(r_std), DIMENSION(:,:), INTENT(in)           :: veget_max            !! "maximal" coverage fraction of a PFT 
                                                                              !! (LAI -> infinity) on ground. May sum to
                                                                              !! less than unity if the pixel has
                                                                              !! nobio area. (unitless; 0-1)
    REAL(r_std), DIMENSION(:,:), INTENT(in)           :: veget_max_begin     !! temporary storage of veget_max to check area conservation

    !! 0.2 Output variables

    !! 0.3 Modified variables

    !! 0.4 Local variables
    INTEGER(i_std)                                    :: iele, ipts, ivm      !! Indices
    INTEGER(i_std)                                    :: ipar, icir           !! Indices
    INTEGER(i_std)                                    :: ivma,iagec,ipft     !! Counter
    REAL(r_std)                                       :: temp_begin, temp_end !! temporary variables to accumulate veget_max of
                                                                              !! different age classes of the same species  

!_ ================================================================================================================================

    SELECT CASE (test_type)
       
    CASE ('pft')
       
       ! In all subroutines where veget_max should be preserved at the
       ! PFT level, the most strict test can be performed
       DO ipts=1,npts

          DO ivm = 1,nvm

             IF (veget_max_begin(ipts,ivm) - veget_max(ipts,ivm) .GT. &
                  min_stomate ) THEN
                WRITE(numout,*) 'Surface area is not preserved in ', &
                     TRIM(check_point)
                WRITE(numout,*) 'Area has changed for PFT, ',ipts, ivm
                WRITE(numout,*) 'Change from, ',veget_max_begin(ipts,ivm), &
                     ' to ', veget_max(ipts,ivm)
                IF(err_act.GT.1) CALL ipslerr (plev, TRIM(check_point), &
                        'Surface area not preserved','','')
             ENDIF
             
          ENDDO !nvm

          ! One message per pixel
!!$          IF (err_act.GT.1) WRITE(numout,*) 'Surface area preserved in ',  &
!!$               TRIM(check_point), ipts
             
       ENDDO ! npts

    CASE ('ageclass')

       ! In subroutines where veget_max can move from one pft to 
       ! another within the same species. This is the test to use
       DO ipts=1,npts
          
          DO ivma=1,nvmap

             ! get pft number
             ivm=start_index(ivma)

             ! If we only have a single age class for this
             ! PFT, we can simplify the test
             IF(nagec_pft(ivma) .EQ. 1) THEN

                IF (veget_max_begin(ipts,ivm) - veget_max(ipts,ivm) .GT. &
                     min_stomate ) THEN
                   WRITE(numout,*) 'Surface area is not preserved in ', &
                        TRIM(check_point)
                   WRITE(numout,*) 'Area has changed for PFT, ',ipts, ivm
                   WRITE(numout,*) 'Change from, ',veget_max_begin(ipts,ivm), &
                        ' to ', veget_max(ipts,ivm)
                   IF(err_act.GT.1) CALL ipslerr (plev, TRIM(check_point), &
                        'Surface area not preserved','','')
                ENDIF

             ELSE ! more than 1 age class

                temp_begin = 0
                temp_end = 0
                DO iagec = nagec_pft(ivma),1,-1
                   
                   ! Accumulate veget_max for all pfts/age classes
                   ! that belong to the same species
                   ipft = ivm+iagec-1
                   temp_begin = temp_begin + veget_max_begin(ipts,ipft)
                   temp_end = temp_end + veget_max(ipts,ipft)

                ENDDO
                
                IF (temp_begin - temp_end .GT. min_stomate ) THEN
                   WRITE(numout,*) 'Surface area is not preserved in ', &
                        TRIM(check_point)
                   WRITE(numout,*) 'Area has changed for PFT, ',ipts, ivm
                   WRITE(numout,*) 'Change from, ',veget_max_begin(ipts,ivm), &
                        ' to ', veget_max(ipts,ivm)
                   IF(err_act.GT.1) CALL ipslerr (plev, TRIM(check_point), &
                        'Surface area not preserved','','')
                ENDIF

             ENDIF ! number of age classes

          ENDDO ! nvmap

          ! One message per pixel
          !IF (err_act.GT.1) WRITE(numout,*) 'Surface area preserved in ',  &
          !     TRIM(check_point), ipts
          
       ENDDO ! ipts

    CASE('pixel')

       ! Use this test in subroutines where veget_max can be moved
       ! between species. This is the least strict test and only 
       ! test wheter the total veget_max was preserved.
       DO ipts=1,npts
             
          IF (ABS(SUM(veget_max_begin(ipts,:)) - SUM(veget_max(ipts,:))) .GT. &
               min_stomate ) THEN
             WRITE(numout,*) 'Surface area is not preserved in ', &
                  TRIM(check_point)
             WRITE(numout,*) 'Area in pixel ', ipts,' has changed form, ', & 
             SUM(veget_max_begin(ipts,:)), 'to,', SUM(veget_max(ipts,:))
             IF(err_act.GT.1) CALL ipslerr (plev, TRIM(check_point), &
                  'Surface area not preserved','','')
          ENDIF
          
          ! One message per pixel
          !IF (err_act.GT.1) WRITE(numout,*) 'Surface area preserved in ',  &
          !     TRIM(check_point), ipts

       ENDDO ! npts
       
    CASE DEFAULT

        WRITE(numout,*) 'Error: an unknown type/resolution was requested'
        WRITE(numout,*) ' for the surface area check. Check the cases'
        WRITE(numout,*) ' defined in src_parameters/function_library or'
        WRITE(numout,*) ' the spelling of the type in the CALL argument',&
            test_type
        CALL ipslerr (3,'surface_area_check in function_library',&
             'unknown type', 'Check *_out_orchidee for more details','')        
       
    END SELECT

  END SUBROUTINE check_surface_area

!! ================================================================================================================================
!! SUBROUTINE  : check_mass balance
!!
!>\BRIEF       
!!
!! DESCRIPTION : 
!! RECENT CHANGE(S): None
!!
!! MAIN OUTPUT VARIABLE(S):
!!
!! REFERENCE(S) : None
!!
!! FLOWCHART    : None
!! \n
!_ ================================================================================================================================
  SUBROUTINE check_mass_balance(check_point, closure_intern, npts, pool_end, &
            pool_start, veget_max, test_type)

 !! 0. Variable and parameter description

    !! 0.1 Input variables
    INTEGER(i_std), INTENT(in)                        :: npts                 !! Domain size (unitless)
    CHARACTER(*),INTENT(in)                           :: check_point          !! A flag to indicate at which
                                                                              !! point in the code we're doing
                                                                              !! this check
    REAL(r_std), DIMENSION(:,:,:), INTENT(in)         :: closure_intern       !! Check closure of internal mass balance
                                                                              !! @tex $(gC pixel^{-1} dt^{-1})$ @endtex
    REAL(r_std), DIMENSION(:,:,:), INTENT(in)         :: pool_start           !! Start and end pool of this routine 
                                                                              !! @tex $(gC pixel^{-1} dt^{-1})$ @endtex
    REAL(r_std), DIMENSION(:,:,:), INTENT(in)         :: pool_end             !! Start and end pool of this routine 
    CHARACTER(*), INTENT(in)                          :: test_type            !! Which test do we want to run on veget_max
    REAL(r_std), DIMENSION(:,:), INTENT(in)           :: veget_max            !! "maximal" coverage fraction of a PFT 
                                                                              !! (LAI -> infinity) on ground. May sum to
                                                                              !! less than unity if the pixel has
                                                                              !! nobio area. (unitless; 0-1) 

    !! 0.2 Output variables

    !! 0.3 Modified variables

    !! 0.4 Local variables
    INTEGER                                           :: iele, ipts, ivm      !! Indices
    INTEGER                                           :: ipar, icir, err      !! Indices
    INTEGER(i_std)                                    :: ivma,iagec,ipft      !! Counter
    REAL(r_std)                                       :: temp_begin, temp_end !! temporary variables to accumulate veget_max of

    CHARACTER(LEN=10)                                 :: var_name             !! Name of element

!_ ================================================================================================================================

    SELECT CASE (test_type)
       
    CASE ('ipts')

       ! Close the mass balance for one specific pixel and pft. 
       ! This :test is used for subroutines that are called within 
       ! a DO ipts = 1,npts and DO ivm = 1,nvm loop. The mass
       ! balance is checked for a single pft at a single pixel. 

       DO iele=1,nelements
          
          IF (iele.EQ.icarbon) var_name = 'carbon'
          IF (iele.EQ.initrogen) var_name = 'nitrogen'
          err = zero

          ! Mass balance closure          
          IF(ABS(closure_intern(1,1,iele)) .GT. min_stomate)THEN

             ! Count the number of errors
             err = err + 1

             ! Give extra details of where the mass balance
             ! problem occurs. Stop the model if the user
             ! asked for it
             IF(err_act.GT.1) THEN
                WRITE(numout,*) 'Error: mass balance is not closed in ', &
                     TRIM(check_point), ' for ', var_name
                WRITE(numout,*) ' Difference is, ', &
                     closure_intern(1,1,iele)
                WRITE(numout,*) ' pool_end,pool_start: ', &
                     pool_end(1,1,iele), &
                     pool_start(1,1,iele)
                CALL ipslerr (plev, TRIM(check_point), &
                     'Mass balance error','','')
             ENDIF

          ENDIF

          !+++CHECK+++
          ! A lot of output - only write a message if their is a problem
!!$          ! One message per pixel. The err condition is needed because under
!!$          ! err_act = 2 the model will write an error to the out_orchidee file
!!$          ! but will not be stopped. Therefore it should be checked whether we
!!$          ! have an error or not
!!$          IF (err_act.GT.1 .AND. err.EQ.0) THEN
!!$             WRITE(numout,*) 'Mass balance closure in ', &
!!$                  TRIM(check_point), ' for ',var_name
!!$          ENDIF
          !++++++++++

       ENDDO ! nelements

    CASE ('pft')
    
       ! Close mass balance at the pixel and pft level.
       DO iele=1,nelements

          IF (iele.EQ.icarbon) var_name = 'carbon'
          IF (iele.EQ.initrogen) var_name = 'nitrogen'

          DO ipts=1,npts

             err = zero

             DO ivm=1,nvm

                !check for NaN
                IF (isnan(pool_start(ipts,ivm,iele))) THEN
                    WRITE(numout,*)'At the pixel',ipts,' for element',iele,&
                        ' and for PFT',ivm,' pool_start is NaN'
                    WRITE(numout,*) ' pool_start: ', pool_start(ipts,:,iele)
                    CALL ipslerr(plev,TRIM(check_point),'NaN problem','','')
                ENDIF
                IF (isnan(pool_end(ipts,ivm,iele))) THEN
                    WRITE(numout,*) 'At the pixel',ipts,' for element',iele,&
                    ' and for PFT',ivm,' pool_end is NaN'
                    WRITE(numout,*) ' pool_end: ', pool_end(ipts,:,iele)
                    CALL ipslerr(plev,TRIM(check_point),'NaN problem','','')
                ENDIF
                IF (isnan(closure_intern(ipts,ivm,iele))) THEN
                   WRITE(numout,*) 'At the pixel',ipts,' for element',iele,&
                    ' and for PFT',ivm,' closure_intern is NaN'
                   WRITE(numout,*) ' closure_intern: ', closure_intern(ipts,:,iele)
                    CALL ipslerr(plev,TRIM(check_point), 'NaN problem','','')
                ENDIF


                ! Mass balance closure          
                IF(ABS(closure_intern(ipts,ivm,iele)) .GT. min_stomate)THEN
              
                   err = err+1
      
                   ! If the model is under development or being tested we will
                   ! output extra information to facilitate debugging.
                   IF(err_act.GT.1)THEN
                      WRITE(numout,*) 'Error: mass balance is not closed in ', &
                           TRIM(check_point), ' for ', var_name
                      WRITE(numout,*) ' ipts, ivm; ', ipts,ivm
                      WRITE(numout,*) ' Difference is, ', &
                           closure_intern(ipts,ivm,iele)
                      WRITE(numout,*) ' pool_end,pool_start: ', &
                           pool_end(ipts,ivm,iele), &
                           pool_start(ipts,ivm,iele)
                      IF (err_act.GT.1) CALL ipslerr (plev, TRIM(check_point), &
                           'Mass balance error','','')
                   ENDIF
                ENDIF
             ENDDO ! nvm

             !+++CHECK+++
             ! A lot of output - only write a message if their is a problem
!!$             ! One message per pixel
!!$             IF (err_act.GT.1 .AND. err.EQ.zero) THEN
!!$                WRITE(numout,*) 'Mass balance closure in ', &
!!$                     TRIM(check_point), ' for ',var_name
!!$             ENDIF
             !++++++++++

          ENDDO ! npts


       ENDDO ! nelements

    CASE ('ageclass')
       
       ! This option is only used when err_act.GE.2

       ! In the subroutine under test, biomass can move
       ! from one age class to another age class in the
       ! same species. Test across all age classes for
       ! the same species.
       DO iele=1,nelements

          IF (iele.EQ.icarbon) var_name = 'carbon'
          IF (iele.EQ.initrogen) var_name = 'nitrogen'
        
          DO ipts=1,npts

             err = zero

             DO ivma=1,nvmap

                ! get pft number
                ivm=start_index(ivma)

                !check for NaN
                IF (isnan(pool_start(ipts,ivm,iele))) THEN
                    WRITE(numout,*)'At the pixel',ipts,' for element',iele,&
                        ' and for PFT',ivm,' pool_start is NaN'
                    WRITE(numout,*) ' pool_start: ', pool_start(ipts,:,iele)
                    CALL ipslerr(plev,TRIM(check_point),'NaN problem','','')
                ENDIF
                IF (isnan(pool_end(ipts,ivm,iele))) THEN
                    WRITE(numout,*) 'At the pixel',ipts,' for element',iele,&
                    ' and for PFT',ivm,' pool_end is NaN'
                    WRITE(numout,*) ' pool_end: ', pool_end(ipts,:,iele)
                    CALL ipslerr(plev,TRIM(check_point),'NaN problem','','')
                ENDIF
                IF (isnan(closure_intern(ipts,ivm,iele))) THEN
                   WRITE(numout,*) 'At the pixel',ipts,' for element',iele,&
                    ' and for PFT',ivm,' closure_intern is NaN'
                   WRITE(numout,*) ' closure_intern: ', closure_intern(ipts,:,iele)
                    CALL ipslerr(plev,TRIM(check_point), 'NaN problem','','')
                ENDIF

                ! If we only have a single age class for this
                ! PFT, we can simplify the test
                IF(nagec_pft(ivma) .EQ. 1) THEN

                   ! Mass balance closure          
                   IF(ABS(closure_intern(ipts,ivm,iele)) .GT. min_stomate)THEN
                      err=err+1
                      WRITE(numout,*) 'Error: mass balance is not closed in ', &
                           TRIM(check_point), ' for ', var_name
                      WRITE(numout,*) ' ipts, ivm; ', ipts,ivm
                      WRITE(numout,*) ' Difference is, ', &
                           closure_intern(ipts,ivm,iele)
                      WRITE(numout,*) ' pool_end,pool_start: ', &
                           pool_end(ipts,ivm,iele), &
                           pool_start(ipts,ivm,iele)
                      IF (err_act.GT.1) CALL ipslerr (plev, TRIM(check_point), &
                           'Mass balance error','','')

                   ENDIF ! test for closure

                ELSE ! more than 1 age class

                   temp_begin = 0
                   DO iagec = nagec_pft(ivma),1,-1
                      
                      ! Accumulate check_intern for all pfts/age classes
                      ! that belong to the same species
                      ipft = ivm+iagec-1
                      temp_begin = temp_begin + closure_intern(ipts,ipft,iele)
                      
                   ENDDO ! nagec_pft
                   
                   IF (temp_begin .GT. min_stomate) THEN

                      ! mass balance
                      err=err+1
                      WRITE(numout,*) 'Error: mass balance is not closed in ', &
                           TRIM(check_point), ' for ', var_name
                      WRITE(numout,*) ' ipts, ivma; ', ipts,ivma
                      WRITE(numout,*) ' first and last pft, ', &
                           start_index(ivma), ipft
                      WRITE(numout,*) ' Difference is, ', &
                           temp_begin
                      IF (err_act.GT.1) CALL ipslerr (plev, TRIM(check_point), &
                           'Mass balance error','','')

                   ENDIF ! test for closure

                ENDIF ! number of age classes

             ENDDO ! ivmap

             
             !+++CHECK+++
             ! A lot of output - only write a message if their is a problem
!!$             ! One message per pixel
!!$             IF (err_act.GT.1 .AND. err.EQ.zero) THEN
!!$                WRITE(numout,*) 'Mass balance closure in ', &
!!$                     TRIM(check_point), ' for ',var_name
!!$             ENDIF
             !++++++++++

          ENDDO ! npts

       ENDDO ! iele

    CASE ('pixel')

       ! For testing subroutines where biomass can also move
       ! from one species to another.
       DO iele = 1, nelements

          IF (iele.EQ.icarbon) var_name = 'carbon'
          IF (iele.EQ.initrogen) var_name = 'nitrogen'
        
          DO ipts=1,npts

             err = zero
         
                !check for NaN
                IF (isnan(SUM(pool_start(ipts,:,iele)))) THEN
                    WRITE(numout,*)'At the pixel',ipts,' for element',iele,&
                    ' pool_start is NaN'
                    WRITE(numout,*) ' pool_start: ', pool_start(ipts,:,iele)
                    CALL ipslerr(plev,TRIM(check_point),'NaN problem','','')
                ENDIF
                IF (isnan(SUM(pool_end(ipts,:,iele)))) THEN
                    WRITE(numout,*) 'At the pixel',ipts,' for element',iele,&
                    ' pool_end is NaN'
                    WRITE(numout,*) ' pool_end: ', pool_end(ipts,:,iele)
                    CALL ipslerr(plev,TRIM(check_point),'NaN problem','','')
                ENDIF
                IF (isnan(SUM(closure_intern(ipts,:,iele)))) THEN
                   WRITE(numout,*) 'At the pixel',ipts,' for element',iele,&
                   ' closure_intern is NaN'
                   WRITE(numout,*) ' closure_intern: ', closure_intern(ipts,:,iele)
                    CALL ipslerr(plev,TRIM(check_point), 'NaN problem','','')
                ENDIF
 
             ! Mass balance closure          
             IF(ABS(SUM(closure_intern(ipts,:,iele))) .GT. min_stomate)THEN

                err = err+1

                IF(err_act.GT.1)THEN
                   WRITE(numout,*) 'Error: mass balance is not closed in ', &
                        TRIM(check_point), ' for ', var_name
                   WRITE(numout,*) ' ipts, ', ipts
                   WRITE(numout,*) ' Difference is, ', &
                        SUM(closure_intern(ipts,:,iele))
                   WRITE(numout,*) ' pool_end,pool_start: ', &
                        SUM(pool_end(ipts,:,iele)), &
                        SUM(pool_start(ipts,:,iele))
                   IF (err_act.GT.1) CALL ipslerr (plev, TRIM(check_point), &
                        'Mass balance error','','')
                ENDIF
                
             ENDIF

             !+++CHECK+++
             ! A lot of output - only write a message if their is a problem
!!$             ! One message per pixel
!!$             IF (err_act.GT.1 .AND. err.EQ.zero) THEN
!!$                WRITE(numout,*) 'Mass balance closure in ', &
!!$                     TRIM(check_point), ' for ',var_name
!!$             ENDIF
             !++++++++++
             
          ENDDO ! npts
          
       ENDDO ! iele

    CASE ('products')

       ! This option is only used when err_act.GE.2
      
       ! testing mass balance for the product pool. Veget_max is not used
       ! ivm is not defined for closure_intern 
       DO iele=1,nelements

          IF (iele.EQ.icarbon) var_name = 'carbon'
          IF (iele.EQ.initrogen) var_name = 'nitrogen'
          
          DO ipts=1,npts

             err = zero

             IF(ABS(closure_intern(ipts,1,iele)) .GT. min_stomate)THEN

                err = err+1 
                WRITE(numout,*) 'Error: mass balance is not closed'//&
                     'in basic_product_use', var_name
                WRITE(numout,*) 'ipts,ivm,iele ', ipts
                WRITE(numout,*) 'Difference is, ', &
                     closure_intern(ipts,1,iele)
                WRITE(numout,*) 'pool_end, pool_start: ', &
                        pool_end(ipts,1,iele), pool_start(ipts,1,iele)
                IF(err_act.GT.1) CALL ipslerr (plev,'basic_product_use', &
                     'Mass balance error.','','')
             ENDIF
             
             !+++CHECK+++
             ! A lot of output - only write a message if their is a problem
!!$             ! One message per pixel
!!$             IF (err_act.GT.1 .AND. err.EQ.zero) THEN
!!$                WRITE(numout,*) 'Mass balance closure in ', &
!!$                     TRIM(check_point), ' for ',var_name
!!$             ENDIF
             !++++++++++  

          ENDDO
       ENDDO
    
    CASE DEFAULT

        WRITE(numout,*) 'Error: an unknown type/resolution was requested'
        WRITE(numout,*) ' for the mass balance check. Check the cases'
        WRITE(numout,*) ' defined in src_parameters/function_library or'
        WRITE(numout,*) ' the spelling of the type in the CALL argument',&
            test_type
        CALL ipslerr (3,'mass_balance_check in function_library',&
        'unknown type', 'Check *_out_orchidee for more details','')        
       
    END SELECT

 END SUBROUTINE check_mass_balance

!! ================================================================================================================================
!! FUNCTION     : sort_circ_class_biomass
!!
!>\BRIEF        
!!
!! DESCRIPTION : sort the diameters in circ_class_biomass from low to high and
!!               change the order in circ_class_n accordingly
!! 
!! RECENT CHANGE(S): None
!!
!! RETURN VALUE : ::circ_class_biomass (gC tree-1),circ_class_n (tree m-2)
!!
!! REFERENCE(S)	: 
!!
!! FLOWCHART	: None
!! \n
!_ ================================================================================================================================
   
  SUBROUTINE sort_circ_class_biomass(circ_class_biomass_new,circ_class_n_new)

 !! 0. Variable and parameter declaration

    !! 0.1 Input variables

    !! 0.2 Output variables

    !! 0.3 Modified variables
    REAL(r_std), DIMENSION(:), INTENT(inout)           :: circ_class_n_new
    REAL(r_std), DIMENSION(:,:,:), INTENT(inout)       :: circ_class_biomass_new

    !! 0.4 Local variables
    INTEGER(i_std)                                     :: inc,icir,jcir        !! Indices
    REAL(r_std), DIMENSION(nparts,nelements)           :: v1                   !! temporay variable for sorting circ_class_biomass
    REAL(r_std)                                        :: v2                   !! temporay variables for sorting circ_class_n
    REAL(r_std)                                        :: sort                 !! flag triggering sorting routine (0 or 1)
    REAL(r_std), DIMENSION(ncirc)                      :: tree_size            !! temporay variables for checking the biomass
                                                                               !! for each circ class @tex ($gC tree^{-1}$) @endtex
    
!_ ================================================================================================================================

    ! Calculate the total biomass in each circumference class
    tree_size(:)=zero
    DO icir=1,ncirc

       ! Only check the order of the carbon pools
       tree_size(icir)=SUM(circ_class_biomass_new(icir,:,icarbon))

    ENDDO

    ! Check whether the order was preserved 
    sort = zero
    DO icir=2,ncirc

       IF(tree_size(icir) .LT. tree_size(icir-1))THEN

          ! The order is not preserved
          sort = un
          EXIT

       ENDIF

    ENDDO

    ! Sort the biomasses. Note that at the same time we also 
    ! have to sort circ_class_n. We made use of the Shell sort
    ! method
    IF (sort==1) THEN

       ! Based on nummerical recipies in Fortran
       inc = 1

       DO
          inc=3*inc+1
          IF (inc .GT. ncirc) EXIT

       ENDDO

       DO
          inc = inc/3

          DO icir=inc+1,ncirc

             v1(:,:) = circ_class_biomass_new(icir,:,:)
             v2 = circ_class_n_new(icir)
             jcir=icir

             DO

                ! Use the carbon pool to sort the circumference classes
                ! but when moving circ classes around, also move the
                ! other elements, i.e. nitrogen.
                IF (SUM(circ_class_biomass_new(jcir-inc,:,icarbon)) .LE. &
                     SUM(v1(:,icarbon))) EXIT
                circ_class_biomass_new(jcir,:,:) = &
                     circ_class_biomass_new(jcir-inc,:,:)
                circ_class_n_new(jcir) = circ_class_n_new(jcir-inc) 
                jcir = jcir-inc
                IF (jcir .LE. inc) EXIT

             ENDDO

             circ_class_biomass_new(jcir,:,:)=v1(:,:)
             circ_class_n_new(jcir)=v2

          ENDDO

          IF (inc .LE. 1) EXIT

       ENDDO

       ! Debug
       WRITE(numout,*) 'sort end - carbon', &
            SUM(circ_class_biomass_new(:,:,icarbon),2),&
            circ_class_n_new(:)
       WRITE(numout,*) 'sort end - nitrogen', &
            SUM(circ_class_biomass_new(:,:,initrogen),2)
       !-

    ENDIF

  END SUBROUTINE sort_circ_class_biomass

END MODULE function_library

