! =================================================================================================================================
! MODULE       : stomate
!
! CONTACT      : orchidee-help _at_ ipsl.jussieu.fr
!
! LICENCE      : IPSL (2006)
! This software is governed by the CeCILL licence see ORCHIDEE/ORCHIDEE_CeCILL.LIC
!
!>\BRIEF       Groups the subroutines that: (1) initialize all variables in 
!! stomate, (2) read and write forcing files of stomate and the soil component,
!! (3) aggregates and convert variables to handle the different time steps 
!! between sechiba and stomate, (4) call subroutines that govern major stomate
!! processes (litter,\ soil, and vegetation dynamics) and (5) structures these tasks 
!! in stomate_main
!!
!!\n DESCRIPTION : None
!!
!! RECENT CHANGE(S) : None
!!
!! REFERENCE(S)	: None
!!
!! SVN :
!! $HeadURL$
!! $Date$
!! $Revision$
!! \n
!_ ================================================================================================================================

MODULE stomate

  ! Modules used:
  USE netcdf
  USE defprec
  USE grid
  USE constantes
  USE constantes_soil
  USE vertical_soil_var
  USE pft_parameters
  USE structures
  USE sapiens_forestry,   ONLY : read_in_fm_map, read_in_litter_map, &
                                 read_in_species_change_map, &
                                 read_in_desired_fm_map
  USE stomate_io
  USE stomate_data
  USE stomate_season
  USE stomate_lpj
  USE stomate_litter
  USE stomate_vmax
  USE stomate_som_dynamics
  USE stomate_resp
  USE mod_orchidee_para
  USE ioipsl_para 
  USE xios_orchidee
  USE function_library,   ONLY : check_biomass_sync, cc_to_lai, &
                                 check_surface_area, check_mass_balance
  USE matrix_resolution
  
  IMPLICIT NONE

  ! Private & public routines

  PRIVATE
  PUBLIC stomate_main,stomate_clear,init_forcing, stomate_forcing_read, stomate_initialize, stomate_finalize, stomate_veget_update

  INTERFACE stomate_accu 
     MODULE PROCEDURE stomate_accu_r1d, stomate_accu_r2d, stomate_accu_r3d, stomate_accu_r4d
  END INTERFACE

  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:)    :: veget_cov_max        !! Maximal fractional coverage: maximum share of a pixel
                                                                         !! taken by a PFT. Excluding no_bio fraction 
!$OMP THREADPRIVATE(veget_cov_max)

  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:)    :: age                  !! Age of PFT it normalized by biomass - can increase and
                                                                         !! decrease - (years)
!$OMP THREADPRIVATE(age)
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:)    :: adapted              !! Winter too cold for PFT to survive (0-1, unitless)
!$OMP THREADPRIVATE(adapted)
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:)    :: regenerate           !! Winter sufficiently cold to produce viable seeds 
                                                                         !! (0-1, unitless)
!$OMP THREADPRIVATE(regenerate)
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:)    :: everywhere           !! Is the PFT everywhere in the grid box or very localized 
                                                                         !! (after its intoduction)
!$OMP THREADPRIVATE(everywhere)
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:)    :: fireindex            !! Probability of fire (unitless)
!$OMP THREADPRIVATE(fireindex)
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:)    :: veget_lastlight      !! Vegetation fractions (on ground) after last light 
                                                                         !! competition (unitless) 
!$OMP THREADPRIVATE(veget_lastlight)
  REAL(r_std), ALLOCATABLE,SAVE,DIMENSION(:,:)   :: fpc_max              !! "maximal" coverage fraction of a grid box (LAI -> 
                                                                         !! infinity) on ground. [??CHECK??] It's set to zero here, 
                                                                         !! and then is used once in lpj_light.f90 to test if 
                                                                         !! fpc_nat is greater than it. Something seems missing
!$OMP THREADPRIVATE(fpc_max)
  LOGICAL,ALLOCATABLE,SAVE,DIMENSION(:,:)        :: PFTpresent           !! PFT exists (equivalent to veget > 0 for natural PFTs)
!$OMP THREADPRIVA\TE(PFTpresent)
  REAL,ALLOCATABLE,SAVE,DIMENSION(:,:)           :: plant_status         !! Growth and phenological status of the plant
                                                                         !! Different stati defined in constantes
!$OMP THREADPRIVATE(plant_status)
  LOGICAL,ALLOCATABLE,SAVE,DIMENSION(:,:)        :: need_adjacent        !! This PFT needs to be in present in an adjacent gridbox 
                                                                         !! if it is to be introduced in a new gridbox
!$OMP THREADPRIVATE(need_adjacent)
!--
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:)    :: humrel_daily         !! Daily plant available water -root profile weighted 
                                                                         !! (0-1, unitless)
!$OMP THREADPRIVATE(humrel_daily)

  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:)    :: vir_humrel_daily     !! Virtual daily plant available water -root profile weighted 
                                                                         !! (0-1, unitless)
!$OMP THREADPRIVATE(vir_humrel_daily)

  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:)    :: stressed_daily       !! Accumulated proxy for stressed ecosystem functioning
                                                                         !! see variable stressed defined in sechiba
!$OMP THREADPRIVATE(stressed_daily)

  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:)    :: unstressed_daily     !! Accumulated proxy for unstressed ecosystem functioning
                                                                         !! see variable stressed defined in sechiba 
!$OMP THREADPRIVATE(unstressed_daily)
 
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:)    :: daylight             !! Time steps dt_radia during daylight
!$OMP THREADPRIVATE(daylight)

 REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:)     :: daylight_count    !! Time steps dt_radia during daylight and when there is growth (gpp>0)
!$OMP THREADPRIVATE(daylight_count)

  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:)    :: transpir_supply_daily !! Daily supply of water for transpiration @tex $(mm dt^{-1})$ @endtex
! $OMP THREADPRIVATE(transpir_supply_daily)

  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:)    :: vir_transpir_supply_daily !! Daily supply of water for transpiration 
                                                                         !! @tex $(mm dt^{-1})$ @endtex
! $OMP THREADPRIVATE(vir_transpir_supply_daily)                                                                        
  
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:)    :: transpir_daily       !! Daily demand of water for transpiration @tex $(mm dt^{-1})$ @endtex
! $OMP THREADPRIVATE(transpir_daily) 


  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:)    :: humrel_week          !! "Weekly" plant available water -root profile weighted
                                                                         !! (0-1, unitless)
!$OMP THREADPRIVATE(humrel_week)
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:)    :: humrel_month         !! "Monthly" plant available water -root profile weighted
                                                                         !! (0-1, unitless)
!$OMP THREADPRIVATE(humrel_month)
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:)    :: humrel_growingseason !! Mean growing season moisture availability (used for 
                                                                         !! allocation response)
!$OMP THREADPRIVATE(humrel_growingseason)
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:)    :: vir_humrel_growingseason !! Mean growing season moisture availability (used for 
                                                                         !! allocation response)
!$OMP THREADPRIVATE(vir_humrel_growingseason)
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:)    :: maxhumrel_lastyear   !! Last year's max plant available water -root profile 
                                                                         !! weighted (0-1, unitless)
!$OMP THREADPRIVATE(maxhumrel_lastyear)
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:)    :: maxhumrel_thisyear   !! This year's max plant available water -root profile 
                                                                         !! weighted (0-1, unitless) 
!$OMP THREADPRIVATE(maxhumrel_thisyear)
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:)    :: minhumrel_lastyear   !! Last year's min plant available water -root profile 
                                                                         !! weighted (0-1, unitless)  
!$OMP THREADPRIVATE(minhumrel_lastyear)
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:)    :: minhumrel_thisyear   !! This year's minimum plant available water -root profile
                                                                         !! weighted (0-1, unitless)
!$OMP THREADPRIVATE(minhumrel_thisyear)
!---  
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:)      :: t2m_daily            !! Daily air temperature at 2 meter (K)
!$OMP THREADPRIVATE(t2m_daily)

  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:)      :: Tseason              !! "seasonal" 2 meter temperatures (K)
!$OMP THREADPRIVATE(Tseason)
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:)      :: Tseason_length       !! temporary variable to calculate Tseason
!$OMP THREADPRIVATE(Tseason_length)
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:)      :: Tseason_tmp          !! temporary variable to calculate Tseason
!$OMP THREADPRIVATE(Tseason_tmp)
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:)    :: Tmin_spring_time
!$OMP THREADPRIVATE(Tmin_spring_time)
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:,:)  :: onset_date           !! Date in the year at when the leaves started to grow(begin_leaves), only for diagnostics.
!$OMP THREADPRIVATE(onset_date)
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:)      :: t2m_week             !! Mean "weekly" (default 7 days) air temperature at 2 
                                                                         !! meter (K)  
!$OMP THREADPRIVATE(t2m_week)
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:)      :: t2m_month            !! Mean "monthly" (default 20 days) air temperature at 2 
                                                                         !! meter (K)
!$OMP THREADPRIVATE(t2m_month)
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:)      :: t2m_longterm         !! Mean "Long term" (default 3 years) air temperature at 
                                                                         !! 2 meter (K) 
!$OMP THREADPRIVATE(t2m_longterm)
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:)      :: t2m_min_daily        !! Daily minimum air temperature at 2 meter (K)
!$OMP THREADPRIVATE(t2m_min_daily)
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:)      :: tsurf_daily          !! Daily surface temperatures (K)
!$OMP THREADPRIVATE(tsurf_daily)

!---
! variables added for windthrow module  ---
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:)       :: wind_speed_daily    !! Daily maximum wind speed at 2 meter (ms-1)
!$OMP THREADPRIVATE(wind_speed_daily)
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:)       :: wind_max_daily      !! Temporary daily maximum speed used to calculate wind_speed_daily (ms-1)
!$OMP THREADPRIVATE(wind_max_daily)
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:)       :: soil_temp_daily     !! Daily maximum soil temperature at 0.8 meter below ground(K)
!$OMP THREADPRIVATE(soil_temp_daily)
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:)       :: soil_max_daily      !! Temporary daily maximum soil temperature used to calculate soil_temp_speed_daily (ms-1)
!$OMP THREADPRIVATE(soil_max_daily)
!---
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:)      :: precip_daily         !! Daily precipitations sum @tex $(mm day^{-1})$ @endtex
!$OMP THREADPRIVATE(precip_daily)
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:)      :: precip_lastyear      !! Last year's annual precipitation sum 
                                                                         !! @tex $??(mm year^{-1})$ @endtex
!$OMP THREADPRIVATE(precip_lastyear)
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:)      :: precip_thisyear      !! This year's annual precipitation sum 
                                                                         !! @tex $??(mm year^{-1})$ @endtex 
!$OMP THREADPRIVATE(precip_thisyear)
!---
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:)    :: soilhum_daily        !! Daily soil humidity (0-1, unitless)
!$OMP THREADPRIVATE(soilhum_daily)
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:)    :: soilhum_month        !! Soil humidity - integrated over a month (0-1, unitless) 
!$OMP THREADPRIVATE(soilhum_month)
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:)    :: tsoil_daily          !! Daily soil temperatures (K)
!$OMP THREADPRIVATE(tsoil_daily)
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:)    :: tsoil_month          !! Soil temperatures at each soil layer integrated over a
                                                                         !! month (K) 
!$OMP THREADPRIVATE(tsoil_month)
!--- 
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:)      :: litterhum_daily      !! Daily litter humidity (0-1, unitless)
!$OMP THREADPRIVATE(litterhum_daily)
!---
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:,:)  :: control_moist        !! Moisture control of heterotrophic respiration 
                                                                         !! (0-1, unitless)
!$OMP THREADPRIVATE(control_moist)
 REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:,:)   :: drainage             !! Fraction of water lost from the soil column by leaching (-)  
!$OMP THREADPRIVATE(drainage) 
 REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:)     :: drainage_daily       !! Daily Fraction of water lost from the soil column by leaching (-)  
!$OMP THREADPRIVATE(drainage_daily) 
 REAL(r_std),ALLOCATABLE,SAVE, DIMENSION(:,:)    :: n_mineralisation_d   !! net nitrogen mineralisation of decomposing SOM 	                                                                                         !! (gN/m**2/day), supposed to be NH4  
!$OMP THREADPRIVATE(n_mineralisation_d) 
 REAL(r_std),ALLOCATABLE,SAVE, DIMENSION(:,:,:)  :: plant_n_uptake_daily   !! Uptake of soil N by plants    
                                                                         !! (gN/m**2/day)  
!$OMP THREADPRIVATE(plant_n_uptake_daily) 
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:,:)  :: control_temp         !! Temperature control of heterotrophic respiration at the
                                                                         !! different soil levels (0-1, unitless)
!$OMP THREADPRIVATE(control_temp)
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:)    :: control_moist_daily  !! Moisture control of heterotrophic respiration daily 
                                                                         !! (0-1, unitless)
!$OMP THREADPRIVATE(control_moist_daily)
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:)    :: control_temp_daily   !! Temperature control of heterotrophic respiration, above
                                                                         !! and below daily (0-1, unitless)
!$OMP THREADPRIVATE(control_temp_daily)
!---
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:)    :: gdd_init_date        !! inital date for gdd count 
!$OMP THREADPRIVATE(gdd_init_date)

  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:)    :: gdd_from_growthinit  !! gdd from beginning of season (C)
!$OMP THREADPRIVATE(gdd_from_growthinit)
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:)      :: gdd0_lastyear        !! Last year's annual Growing Degree Days,
                                                                         !! threshold 0 deg C (K) 
!$OMP THREADPRIVATE(gdd0_lastyear)
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:)      :: gdd0_thisyear        !! This year's annual Growing Degree Days,
                                                                         !! threshold 0 deg C (K)
!$OMP THREADPRIVATE(gdd0_thisyear)
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:)    :: gdd_m5_dormance      !! Growing degree days for onset of growing season, 
                                                                         !! threshold -5 deg C (K)
!$OMP THREADPRIVATE(gdd_m5_dormance)
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:)    :: gdd_midwinter        !! Growing degree days for onset of growing season, 
                                                                         !! since midwinter (K)
!$OMP THREADPRIVATE(gdd_midwinter)
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:)    :: ncd_dormance         !! Number of chilling days since leaves were lost (days) 
!$OMP THREADPRIVATE(ncd_dormance)
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:)    :: ngd_minus5           !! Number of growing days, threshold -5 deg C (days)
!$OMP THREADPRIVATE(ngd_minus5)
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:)    :: hum_min_dormance     !! Minimum moisture during dormance (0-1, unitless) 
!$OMP THREADPRIVATE(hum_min_dormance)
!---
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:)    :: gpp_daily            !! Daily gross primary productivity per ground area 
                                                                         !! @tex $(gC m^{-2} dt_stomate^{-1})$ @endtex
!$OMP THREADPRIVATE(gpp_daily) 
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:)    :: gpp_week             !! Mean "weekly" (default 7 days) GPP  
                                                                         !! @tex $(gC m^{-2} dt_stomate^{-1})$ @endtex
!$OMP THREADPRIVATE(gpp_week)
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:)    :: maxgppweek_lastyear  !! Last year's maximum "weekly" GPP  
                                                                         !! @tex $(gC m^{-2} dt_stomate^{-1})$ @endtex 
!$OMP THREADPRIVATE(maxgppweek_lastyear)
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:)    :: maxgppweek_thisyear  !! This year's maximum "weekly" GPP  
                                                                         !! @tex $(gC m^{-2} dt_stomate^{-1})$ @endtex  
!$OMP THREADPRIVATE(maxgppweek_thisyear)
!---
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:)    :: npp_daily            !! Daily net primary productivity per ground area 
                                                                         !! @tex $(gC m^{-2} dt_stomate^{-1})$ @endtex 
!$OMP THREADPRIVATE(npp_daily)
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:)    :: npp_longterm         !! "Long term" (default 3 years) net primary productivity 
                                                                         !! per ground area  
                                                                         !! @tex $(gC m^{-2} year^{-1})$ @endtex   
!$OMP THREADPRIVATE(npp_longterm)
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:)    :: npp_equil            !! Equilibrium NPP written to forcesoil 
                                                                         !! @tex $(gC m^{-2} year^{-1})$ @endtex
!$OMP THREADPRIVATE(npp_equil)
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:)      :: npp_tot              !! Total NPP written to forcesoil 
                                                                         !! @tex $(gC m^{-2} year^{-1})$ @endtex 
!$OMP THREADPRIVATE(npp_tot)
!---
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:,:)  :: resp_maint_part_radia!! Maintenance respiration of different plant parts per 
                                                                         !! total ground area at Sechiba time step  
                                                                         !! @tex $(gC m^{-2} dt_sechiba^{-1})$ @endtex
!$OMP THREADPRIVATE(resp_maint_part_radia)
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:,:)  :: resp_maint_part      !! Maintenance respiration of different plant parts per
                                                                         !! total ground area at Stomate time step 
                                                                         !! @tex $(gC m^{-2} dt_stomate^{-1})$ @endtex
!$OMP THREADPRIVATE(resp_maint_part)
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:)    :: resp_maint_radia     !! Maintenance respiration per ground area at Sechiba time
                                                                         !! step   
                                                                         !! @tex $(gC m^{-2} dt_sechiba^{-1})$ @endtex
!$OMP THREADPRIVATE(resp_maint_radia)
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:)    :: resp_maint_d         !! Maintenance respiration per ground area at Stomate time 
                                                                         !! step  
                                                                         !! @tex $(gC m^{-2} dt_stomate^{-1})$ @endtex
!$OMP THREADPRIVATE(resp_maint_d)
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:)    :: resp_growth_d        !! Growth respiration per ground area 
                                                                         !! @tex $(gC m^{-2} dt_stomate^{-1})$ @endtex
!$OMP THREADPRIVATE(resp_growth_d)
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:)    :: resp_hetero_d        !! Heterotrophic respiration per ground area 
                                                                         !! @tex $(gC m^{-2} dt_stomate^{-1})$ @endtex
!$OMP THREADPRIVATE(resp_hetero_d)
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:)    :: resp_hetero_radia    !! Heterothrophic respiration per ground area at Sechiba
                                                                         !! time step 
                                                                         !! @tex $(gC m^{-2} dt_sechiba^{-1})$ @endtex 
!$OMP THREADPRIVATE(resp_hetero_radia)
!---
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:,:)     :: turnover_time       !! Turnover time of grasses 
                                                                         !! @tex $(dt_stomate^{-1})$ @endtex 
!$OMP THREADPRIVATE(turnover_time)
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:,:,:) :: turnover_daily      !! Senescence-driven turnover (better: mortality) of 
                                                                         !! leaves and roots  
                                                                         !! @tex $(gC m^{-2} dt_stomate^{-1})$ @endtex
!$OMP THREADPRIVATE(turnover_daily)
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:,:,:) :: turnover_littercalc !! Senescence-driven turnover (better: mortality) of 
                                                                         !! leaves and roots at Sechiba time step 
                                                                         !! @tex $(gC m^{-2} dt_sechiba^{-1})$ @endtex 
!$OMP THREADPRIVATE(turnover_littercalc)
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:,:,:) :: turnover_longterm   !! "Long term" (default 3 years) senescence-driven 
                                                                         !! turnover (better: mortality) of leaves and roots 
                                                                         !! @tex $(gC m^{-2} year^{-1})$ @endtex
!$OMP THREADPRIVATE(turnover_longterm)
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:,:,:) :: bm_to_litter        !! Background (not senescence-driven) mortality of biomass
                                                                         !! @tex $(gC m^{-2} dt_stomate^{-1})$ @endtex
!$OMP THREADPRIVATE(bm_to_litter)
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:,:,:) :: bm_to_littercalc    !! conversion of biomass to litter per ground area at 
                                                                         !! Sechiba time step 
                                                                         !! @tex $(gC m^{-2} dt_sechiba^{-1})$ @endtex 
!$OMP THREADPRIVATE(bm_to_littercalc)
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:,:)  :: dead_leaves          !! Metabolic and structural pools of dead leaves on ground
                                                                         !! per PFT @tex $(gC m^{-2})$ @endtex 
!$OMP THREADPRIVATE(dead_leaves)
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:,:,:,:):: litter             !! Above and below ground metabolic and structural litter 
                                                                         !! per ground area 
                                                                         !! @tex $(gC m^{-2})$ @endtex 
!$OMP THREADPRIVATE(litter)
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:)    :: firelitter           !! Total litter above the ground that could potentially 
                                                                         !! burn @tex $(gC m^{-2})$ @endtex 
!$OMP THREADPRIVATE(firelitter)
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:,:,:):: carbon_input     !! Quantity of carbon going into carbon pools from litter
                                                                         !! decomposition per ground area  at Sechiba time step 
                                                                         !! @tex $(gC m^{-2} dt_sechiba^{-1})$ @endtex 
!$OMP THREADPRIVATE(carbon_input)
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:,:,:):: nitrogen_input       !! Quantity of nitrogen going into nitrogen pools from litter 
                                                                         !! decomposition per ground area  at Sechiba time step  
                                                                         !! @tex $(gC m^{-2} dtradia^{-1})$ @endtex  
!$OMP THREADPRIVATE(nitrogen_input) 
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:,:,:)  :: som_input_daily !! Daily quantity of carbon going into carbon pools from
                                                                           !! litter decomposition per ground area 
                                                                           !! @tex $(gC m^{-2} day^{-1})$ @endtex 
!$OMP THREADPRIVATE(som_input_daily)
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:,:,:)  :: som                !! Soil organic matter  pools per ground area: active, slow, or 
                                                                         !! passive, @tex $(gC or N m^{-2})$ @endtex 
!$OMP THREADPRIVATE(som)
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:,:)  :: lignin_struc         !! Ratio Lignine/Carbon in structural litter for above and
                                                                         !! below ground compartments (unitless)
!$OMP THREADPRIVATE(lignin_struc)
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:,:)  :: lignin_wood          !! Ratio Lignine/Carbon in woody litter for above and
                                                                         !! below ground compartments (unitless)	
!$OMP THREADPRIVATE(lignin_wood)
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:)    :: lm_lastyearmax       !! Last year's maximum leaf mass per ground area for each
                                                                         !! PFT @tex $(gC m^{-2})$ @endtex  
!$OMP THREADPRIVATE(lm_lastyearmax)
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:)    :: lm_thisyearmax       !! This year's maximum leaf mass per ground area for each
                                                                         !! PFT @tex $(gC m^{-2})$ @endtex  
!$OMP THREADPRIVATE(lm_thisyearmax)
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:)    :: maxfpc_lastyear      !! Last year's maximum fpc for each natural PFT, on ground
                                                                         !! [??CHECK] fpc but this ones look ok (computed in 
                                                                         !! season, used in light)?? 
!$OMP THREADPRIVATE(maxfpc_lastyear)
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:)    :: maxfpc_thisyear      !! This year's maximum fpc for each PFT, on ground (see 
                                                                         !! stomate_season), [??CHECK] fpc but this ones look ok 
                                                                         !! (computed in season, used in light)??
!$OMP THREADPRIVATE(maxfpc_thisyear)
!---
  REAL(r_std), ALLOCATABLE,SAVE,DIMENSION(:,:,:)  :: leaf_age             !! Age of different leaf classes (days)
!$OMP THREADPRIVATE(leaf_age)
  REAL(r_std), ALLOCATABLE,SAVE,DIMENSION(:,:,:)  :: leaf_frac            !! PFT fraction of leaf mass in leaf age class (0-1, 
                                                                         !! unitless) 
!$OMP THREADPRIVATE(leaf_frac)
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:)    :: when_growthinit      !! Days since beginning of growing season (days)
!$OMP THREADPRIVATE(when_growthinit)
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:)    :: herbivores           !! Time constant of probability of a leaf to be eaten by a
                                                                         !! herbivore (days)
!$OMP THREADPRIVATE(herbivores)
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:)    :: RIP_time             !! How much time ago was the PFT eliminated for the last 
                                                                         !! time (year)
!$OMP THREADPRIVATE(RIP_time)
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:)    :: time_hum_min         !! Time elapsed since strongest moisture limitation (days) 
!$OMP THREADPRIVATE(time_hum_min)
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:)    :: drain_daily          !! daily fraction of water lost from the soil column by leaching (-)  
!$OMP THREADPRIVATE(drain_daily)

!---
  REAL(r_std),ALLOCATABLE,DIMENSION(:,:)         :: clay_fm              !! Soil clay content (0-1, unitless), parallel computing
  REAL(r_std),ALLOCATABLE,DIMENSION(:,:)         :: clay_fm_g            !! Soil clay content (0-1, unitless), parallel computing
  REAL(r_std),ALLOCATABLE,DIMENSION(:,:)         :: silt_fm              !! Soil silt content (0-1, unitless), parallel computing 
  REAL(r_std),ALLOCATABLE,DIMENSION(:,:)         :: silt_fm_g            !! Soil silt content (0-1, unitless), parallel computing 
  REAL(r_std),ALLOCATABLE,DIMENSION(:,:)         :: bulk_fm              !! Bulk density (kg/m**3), parallel computing    
  REAL(r_std),ALLOCATABLE,DIMENSION(:,:)         :: bulk_fm_g            !! Bulk density (kg/m**3), parallel computing 

  REAL(r_std),ALLOCATABLE,DIMENSION(:,:)         :: precip_fm            !! Daily precipitations sum @tex $(mm day^{-1})$ @endtex,
                                                                         !! parallel computing
  REAL(r_std),ALLOCATABLE,DIMENSION(:,:)         :: precip_fm_g          !! Daily precipitations sum @tex $(mm day^{-1})$ @endtex,
                                                                         !! parallel computing
  REAL(r_std),ALLOCATABLE,DIMENSION(:,:)         :: litterhum_daily_fm   !! Daily relative humidity of litter (0-1, unitless), 
                                                                         !! parallel computing
  REAL(r_std),ALLOCATABLE,DIMENSION(:,:)         :: litterhum_daily_fm_g !! Daily relative humidity of litter (0-1, unitless), 
                                                                         !! parallel computing
  REAL(r_std),ALLOCATABLE,DIMENSION(:,:)         :: t2m_daily_fm         !! Daily air temperature at 2 meter (K), parallel 
                                                                         !! computing
  REAL(r_std),ALLOCATABLE,DIMENSION(:,:)         :: t2m_daily_fm_g       !! Daily air temperature at 2 meter (K), parallel 
                                                                         !! computing
  REAL(r_std),ALLOCATABLE,DIMENSION(:,:)         :: t2m_min_daily_fm     !! Daily minimum air temperature at 2 meter (K), 
                                                                         !! parallel computing
  REAL(r_std),ALLOCATABLE,DIMENSION(:,:)         :: t2m_min_daily_fm_g   !! Daily minimum air temperature at 2 meter (K), 
                                                                         !! parallel computing
  REAL(r_std),ALLOCATABLE,DIMENSION(:,:)         :: tsurf_daily_fm       !! Daily surface temperatures (K), parallel 
                                                                         !! computing
  REAL(r_std),ALLOCATABLE,DIMENSION(:,:)         :: tsurf_daily_fm_g     !! Daily surface temperatures (K), parallel 
                                                                         !! computing 
  REAL(r_std),ALLOCATABLE,DIMENSION(:,:,:)       :: tsoil_daily_fm       !! Daily soil temperatures (K), parallel computing 
  REAL(r_std),ALLOCATABLE,DIMENSION(:,:,:)       :: tsoil_daily_fm_g     !! Daily soil temperatures (K), parallel computing
  REAL(r_std),ALLOCATABLE,DIMENSION(:,:,:)       :: soilhum_daily_fm     !! Daily soil humidity (0-1, unitless), parallel computing
  REAL(r_std),ALLOCATABLE,DIMENSION(:,:,:)       :: soilhum_daily_fm_g   !! Daily soil humidity (0-1, unitless), parallel computing
  REAL(r_std),ALLOCATABLE,DIMENSION(:,:,:)       :: humrel_daily_fm      !! Daily relative humidity of atmosphere (0-1, unitless), 
                                                                         !! parallel computing
  REAL(r_std),ALLOCATABLE,DIMENSION(:,:,:)       :: humrel_daily_fm_g    !! Daily relative humidity of atmosphere (0-1, unitless), 
                                                                         !! parallel computing
  REAL(r_std),ALLOCATABLE,DIMENSION(:,:,:)       :: gpp_daily_fm         !! Daily gross primary productivity per ground area 
                                                                         !! @tex $(gC m^{-2} dt_stomate^{-1})$ @endtex, 
                                                                         !! parallel computing
  REAL(r_std),ALLOCATABLE,DIMENSION(:,:,:)       :: gpp_daily_fm_g       !! Daily gross primary productivity per ground area 
                                                                         !! @tex $(gC m^{-2} day^{-1})$ @endtex, 
                                                                         !! parallel computing
  REAL(r_std),ALLOCATABLE,DIMENSION(:,:,:)       :: veget_fm             !! Vegetation coverage taking into account non-biological
                                                                         !! coverage (unitless), parallel computing
  REAL(r_std),ALLOCATABLE,DIMENSION(:,:,:)       :: veget_fm_g           !! Vegetation coverage taking into account non-biological
                                                                         !! coverage (unitless), parallel computing
  REAL(r_std),ALLOCATABLE,DIMENSION(:,:,:)       :: veget_max_fm         !! Maximum vegetation coverage taking into account 
                                                                         !! non-biological coverage (unitless), parallel computing
  REAL(r_std),ALLOCATABLE,DIMENSION(:,:,:)       :: veget_max_fm_g       !! Maximum vegetation coverage taking into account none 
                                                                         !! biological coverage (unitless), parallel computing
  REAL(r_std),ALLOCATABLE,DIMENSION(:,:,:)       :: lai_fm               !! Leaf area index @tex $@tex $(m^2 m^{-2})$ @endtex$ @endtex, 
                                                                         !! parallel computing
  REAL(r_std),ALLOCATABLE,DIMENSION(:,:,:)       :: lai_fm_g             !! Leaf area index @tex $@tex $(m^2 m^{-2})$ @endtex$ @endtex, 
                                                                         !! parallel computing
  REAL(r_std),ALLOCATABLE,DIMENSION(:,:,:)      :: drainage_fm           !! daily fraction of water lost from the soil column by leaching (-)                                                                           
  REAL(r_std),ALLOCATABLE,DIMENSION(:,:,:)      :: drainage_fm_g         !! drain 
                                                                         !! parallel computing 
 REAL(r_std), ALLOCATABLE,SAVE,DIMENSION(:,:)   :: cn_leaf_min_season    !! Seasonal min CN ratio of leaves 
!$OMP THREADPRIVATE(cn_leaf_min_season)
 REAL(r_std), ALLOCATABLE,SAVE,DIMENSION(:,:)   :: nstress_season        !! N-related seasonal stress (used for allocation) 
!$OMP THREADPRIVATE(nstress_season)
 REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:,:)  :: soil_n_min            !! mineral nitrogen in the soil (gN/m**2)   
                                                                         !! (first index=kjpindex, second index=nvm, third index=nnspec)   
!$OMP THREADPRIVATE(soil_n_min)
 REAL(r_std), ALLOCATABLE,SAVE,DIMENSION(:,:) :: p_O2                  !! partial pressure of oxigen in the soil (hPa)(first index=kjpindex, second index=nvm)
!$OMP THREADPRIVATE(p_O2)
                      
 REAL(r_std), ALLOCATABLE,SAVE,DIMENSION(:,:) :: bact                  !! denitrifier biomass (gC/m**2)
                                                                         !! (first index=kjpindex, second index=nvm)
!$OMP THREADPRIVATE(bact)

  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:)   :: co2_fire              !! Carbon emitted to the atmosphere by burning living 
                                                                         !! and dead biomass 
                                                                         !! @tex $(gC m^{-2} dt_stomate^{-1})$ @endtex 
!$OMP THREADPRIVATE(co2_fire)
!!$  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:)    :: co2_to_bm_dgvm       !! Psuedo-photosynthesis,C used to provide seedlings with
!!$                                                                         !! an initial biomass, arbitrarily removed from the 
!!$                                                                         !! atmosphere  
!!$                                                                         !! @tex $(gC m^{-2} dt_stomate^{-1})$ @endtex 
!!$!$OMP THREADPRIVATE(co2_to_bm_dgvm)
!!$  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:)    :: n_to_bm              !! N taken from ?? to provide seedlings with
!!$                                                                         !! an initial N biomass
!!$                                                                         !! @tex $(gN m^{-2} dt_stomate^{-1})$ @endtex 
!!$!$OMP THREADPRIVATE(n_to_bm)
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:,:)  :: atm_to_bm            !! C and N taken from the atmosphere to provide seedlings 
                                                                         !! with an initial N biomass
                                                                         !! @tex $(gN m^{-2} dt_stomate^{-1})$ @endtex 
!$OMP THREADPRIVATE(atm_to_bm)

!!$  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:)    :: co2_flux_daily       !! Daily net CO2 flux between atmosphere and biosphere 
!!$                                                                         !! @tex $(gC m^{-2} dt_stomate^{-1})$ @endtex
!!$                                                                         !! [??CHECK] sign convention?
!!$!$OMP THREADPRIVATE(co2_flux_daily)
!!$  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:)    :: co2_flux_monthly     !! Monthly net CO2 flux between atmosphere and biosphere 
!!$                                                                         !! @tex $(gC m^{-2} dt_stomate^{-1})$ @endtex
!!$                                                                         !! [??CHECK] sign convention? 
!!$!$OMP THREADPRIVATE(co2_flux_monthly)

  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:,:)  :: prod_s               !! Wood products remaining in the 1 year-turnover pool 
                                                                         !! after the annual release for each compartment 
                                                                         !! @tex $(gC m^{-2})$ @endtex    
                                                                         !! (0:1 input from year of land cover change),
                                                                         !! dimension(#pixels,0:1 years
!$OMP THREADPRIVATE(prod_s)
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:,:)  :: prod_m               !! Wood products remaining in the 10 year-turnover pool 
                                                                         !! after the annual release for each compartment 
                                                                         !! @tex $(gC m^{-2})$ @endtex    
                                                                         !! (0:10 input from year of land cover change),
                                                                         !! dimension(#pixels,0:10 years
!$OMP THREADPRIVATE(prod_m)
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:,:)  :: prod_l               !! Wood products remaining in the 100 year-turnover pool
                                                                         !! after the annual release for each compartment
                                                                         !! @tex $(gC m^{-2})$ @endtex  
                                                                         !! (0:100 input from year of land cover change), 
                                                                         !! dimension(#pixels,0:100 years)
!$OMP THREADPRIVATE(prod_l)
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:,:)  :: flux_s               !! Wood decomposition from the 1 year-turnover pool 
                                                                         !! compartments 
                                                                         !! @tex $(gC m^{-2} year^{-1})$ @endtex 
                                                                         !! dimension(#pixels,0:1)  
!$OMP THREADPRIVATE(flux_s)
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:,:)  :: flux_m               !! Wood decomposition from the 10 year-turnover pool 
                                                                         !! compartments 
                                                                         !! @tex $(gC m^{-2} year^{-1})$ @endtex 
                                                                         !! dimension(#pixels,0:10)  
!$OMP THREADPRIVATE(flux_m)
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:,:)  :: flux_l              !! Wood decomposition from the 100 year-turnover pool 
                                                                         !! compartments 
                                                                         !! @tex $(gC m^{-2} year^{-1})$ @endtex
                                                                         !! dimension(#pixels,0:100)
!$OMP THREADPRIVATE(flux_l)
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:)    :: flux_prod_s         !! Release during first year following land cover change 
                                                                         !! (paper, burned, etc...) 
                                                                         !! @tex $(gC m^{-2} year^{-1})$ @endtex  
!$OMP THREADPRIVATE(flux_prod_s)
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:)    :: flux_prod_m         !! Total annual release from the 10 year-turnover pool
                                                                         !! sum of flux_m  
                                                                         !! @tex $(gC m^{-2} year^{-1})$ @endtex
!$OMP THREADPRIVATE(flux_prod_m)
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:)    :: flux_prod_l         !! Total annual release from the 100 year-turnover pool 
                                                                         !! sum of flux_l 
                                                                         !! @tex $(gC m^{-2} year^{-1})$ @endtex
!$OMP THREADPRIVATE(flux_prod_l)
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:)      :: carb_mass_total      !! Total on-site and off-site C pool 
                                                                         !! @tex $(??gC m^{-2})$ @endtex                        
!$OMP THREADPRIVATE(carb_mass_total)
!---
  REAL(r_std), SAVE                              :: tau_longterm
!$OMP THREADPRIVATE(tau_longterm)
  REAL(r_std),SAVE                               :: dt_days=zero         !! Time step of STOMATE (days) 
!$OMP THREADPRIVATE(dt_days)
  INTEGER(i_std),SAVE                            :: date=0               !! Date (days) 
!$OMP THREADPRIVATE(date)
  INTEGER(i_std),ALLOCATABLE,SAVE,DIMENSION(:)   :: nforce               !! Number of states calculated for the soil forcing 
                                                                         !! variables (unitless), dimension(::nparan*::nbyear) both 
                                                                         !! given in the run definition file    
!$OMP THREADPRIVATE(nforce)
  INTEGER(i_std),ALLOCATABLE,SAVE,DIMENSION(:)   :: isf                  !! Index for number of time steps that can be stored in 
                                                                         !! memory (unitless), dimension (#nsfm)
!$OMP THREADPRIVATE(isf)
  INTEGER(i_std),ALLOCATABLE,SAVE,DIMENSION(:)   :: nf_cumul             !! Number of years over which the average is calculated in
                                                                         !! forcesoil when cumul flag is set, dimension (#nsft)
                                                                         !! [??CHECK] definition the dimension is number of 
                                                                         !! timesteps in a year?
!$OMP THREADPRIVATE(nf_cumul)
  INTEGER(i_std), SAVE                           :: spinup_period        !! Period of years used to calculate the resolution of the system for spinup analytic. 
                                                                         !! This period correspond in most cases to the period of years of forcing data used
  INTEGER,PARAMETER                              :: r_typ = nf90_real4   !! Specify data format (server dependent)
  LOGICAL,ALLOCATABLE,SAVE,DIMENSION(:)          :: nf_written           !! Flag indicating whether the forcing data have been 
                                                                         !! written
!$OMP THREADPRIVATE(nf_written)
!---
  LOGICAL, SAVE                                  :: do_slow=.FALSE.      !! Flag that determines whether stomate_accu calculates
                                                                         !! the sum(do_slow=.FALSE.) or the mean 
                                                                         !! (do_slow=.TRUE.)
!$OMP THREADPRIVATE(do_slow)
  LOGICAL, SAVE                                  :: EndOfYear=.FALSE.    !! Update annual variables? This variable must be 
                                                                         !! .TRUE. once a year
!$OMP THREADPRIVATE(EndOfYear)
  LOGICAL, SAVE                                  :: EndOfMonth=.FALSE.   !! Update monthly variables? This variable must be 
                                                                         !!.TRUE. once a month 
!$OMP THREADPRIVATE(EndOfMonth)
  LOGICAL, SAVE                                  :: l_first_stomate = .TRUE.!! Is this the first call of stomate?
!$OMP THREADPRIVATE(l_first_stomate)
  LOGICAL, SAVE                                  :: cumul_forcing=.FALSE.!! flag for cumul of forcing if teststomate
!$OMP THREADPRIVATE(cumul_forcing)
  LOGICAL, SAVE                                  :: cumul_Cforcing=.FALSE.  !! Flag, if internal parameter cumul_Cforcing is 
                                                                            !! TRUE then ::nbyear (defined in run definition 
                                                                            !! file will be forced to 1 later in this module. If 
                                                                            !! FALSE the mean over ::nbyear is written in forcesoil
!$OMP THREADPRIVATE(cumul_Cforcing)
!--- 

  REAL(r_std),DIMENSION(:),ALLOCATABLE,SAVE      :: circ_class_dist   !! When the circumference class distribution
                                                                      !! is redone due to empty classes, this is the
                                                                      !! tree distribution used.  Notice that this distribution
                                                                      !! is normalized after being read in.
!$OMP THREADPRIVATE(circ_class_dist)
  REAL(r_std),DIMENSION(:,:,:),ALLOCATABLE,SAVE  :: store_sum_delta_ba!! Sum of increase of basal area (m^2) which does not pass
                                                                      !! through gap clean
!$OMP THREADPRIVATE(store_sum_delta_ba)
  REAL(r_std),DIMENSION(:),ALLOCATABLE,SAVE      :: st_dist           !! During self-thinning, we need to decide which
                                                                      !! circumference classes to kill trees in.  This
                                                                      !! is the distribution that tells us this.  Notice that 
                                                                      !! it is normalized after being read in.
!$OMP THREADPRIVATE(st_dist)
  
!!$  !+++CHECK+++
!!$  ! replaced by harvest_xxx. Seems like post processing for xios
!!$  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:)      :: harvest_above_monthly   !! [??CHECK] post-processing - should be removed?
!!$!$OMP THREADPRIVATE(harvest_above_monthly)
!!$  !+++++++++++
!!$
!!$  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:)      :: flux_prod_monthly      !! [??CHECK] post-processing - should be removed?
!!$!$OMP THREADPRIVATE(flux_prod_monthly)
!---
  INTEGER(i_std), SAVE                               :: global_years        !! Global counter of years (year)
!$OMP THREADPRIVATE(global_years)
  LOGICAL, ALLOCATABLE, SAVE, DIMENSION(:)           :: ok_equilibrium      !! Logical array marking the points where the resolution is ok 
                                                                            !! (true/false)
!$OMP THREADPRIVATE(ok_equilibrium)
  LOGICAL, ALLOCATABLE, SAVE, DIMENSION(:)           :: carbon_eq           !! Logical array to mark the carbon pools at equilibrium ? 
                                                                            !! If true, the job stops. (true/false)
!$OMP THREADPRIVATE(carbon_eq)
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION(:)       :: nbp_accu            !! Accumulated Net Biospheric Production over the year (gC.m^2 )
!$OMP THREADPRIVATE(nbp_accu)
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION(:,:)     :: nbp_accu_pool       !! Accumulated pool-basaed Net Biospheric Production over the year (gC.m^2 )
!$OMP THREADPRIVATE(nbp_accu_pool)
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION(:)       :: nbp_flux            !! Net Biospheric Production (gC.m^2.day^{-1})
!$OMP THREADPRIVATE(nbp_flux)
  REAL(r_std), ALLOCATABLE, SAVE,DIMENSION(:,:)      :: nbp_pool            !! Pool-based Net Biospheric Production (gC.m^2.day^{-1})
!$OMP THREADPRIVATE(nbp_pool)
  REAL(r_std), ALLOCATABLE, SAVe,DIMENSION(:,:)      :: nbp_pool_start      !! Biomass pool as calculated from the 
                                                                            !! previous time step
!$OMP THREADPRIVATE(nbp_pool_start)
  REAL(r_std), ALLOCATABLE, SAVE,DIMENSION(:,:)      :: nbp_pool_end        !! Biomass pool calculated at the end 
                                                                            !! current time step
!$OMP THREADPRIVATE(nbp_pool_end)
  REAL(r_std), ALLOCATABLE, DIMENSION(:,:,:,:)       :: matrixA             !! matrix containing the fluxes between the carbon pools
                                                                            !! per sechiba time step 
                                                                            !! @tex $(gC.m^2.day^{-1})$ @endtex
!$OMP THREADPRIVATE(matrixA)
  REAL(r_std), ALLOCATABLE, DIMENSION(:,:,:)         :: vectorB             !! vector containing the litter increase per sechiba time step
                                                                            !! @tex $(gC m^{-2})$ @endtex
!$OMP THREADPRIVATE(vectorB)
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION(:,:,:,:) :: matrixV             !! matrix containing the accumulated values of matrixA 
!$OMP THREADPRIVATE(matrixV)
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION(:,:,:)   :: vectorU             !! matrix containing the accumulated values of vectorB
!$OMP THREADPRIVATE(vectorU)
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION(:,:,:,:) :: matrixW             !! matrix containing the opposite of matrixA
!$OMP THREADPRIVATE(matrixW)
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION(:,:,:)   :: previous_stock      !! Array containing the carbon stock calculated by the analytical
                                                                            !! method in the previous resolution
!$OMP THREADPRIVATE(previous_stock)
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION(:,:,:)   :: current_stock       !! Array containing the carbon stock calculated by the analytical
                                                                            !! method in the current resolution 
!$OMP THREADPRIVATE(current_stock)
  REAL(r_std), SAVE                                  :: eps_carbon          !! Stopping criterion for carbon pools (unitless,0-1)
!$OMP THREADPRIVATE(eps_carbon)
    REAL(r_std), ALLOCATABLE, DIMENSION(:,:)           :: sigma            !! Threshold for indivudal tree growth (m, trees whose
                                                                         !! circumference is smaller than sigma don't grow much)
!$OMP THREADPRIVATE(sigma)

  INTEGER(i_std), ALLOCATABLE, SAVE, DIMENSION(:,:)  :: age_stand        !! Age of stand (years)
!$OMP THREADPRIVATE(age_stand)
  
  INTEGER(i_std), ALLOCATABLE, SAVE, DIMENSION(:,:)  :: rotation_n       !! Rotation number (number of rotation since pft is managed)
!$OMP THREADPRIVATE(rotation_n)
  
  INTEGER(i_std), ALLOCATABLE, SAVE, DIMENSION(:,:)  :: last_cut         !! Years since last thinning (years)
!$OMP THREADPRIVATE(last_cut)

  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION(:,:,:)  :: CN_som_litter_longterm !! Longterm CN ratio of litter and som pools (gC/gN)

  REAL(r_std), SAVE                                 :: tau_CN_longterm      !! Counter used for calculating the longterm CN ratio of SOM and litter pools (seconds)    

  ! Functional Allocation

  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION(:,:)     :: KF               !! Scaling factor to convert sapwood mass
                                                                         !! into leaf mass (m). The initial value is calculated
                                                                         !! in prescribe and updated during allocation
!$OMP THREADPRIVATE(KF)

  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION(:,:)     :: k_latosa_adapt   !! Leaf to sapwood area adapted for waterstress. 
                                                                         !! Adaptation takes place at the end of the year
                                                                         !! (m)
!$OMP THREADPRIVATE(k_latosa_adapt)

  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION(:,:,:,:) :: harvest_pool     !! The pool which records the quantity of
                                                                         !! wood harvested and thinned due to forest
                                                                         !! management and LCC.  
                                                                         !! @tex $(gC m^{-2})$ @endtex 
!$OMP THREADPRIVATE(harvest_pool)

  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION(:,:)     :: harvest_type     !! Type of management that resulted
                                                                         !! in the harvest (unitless)
!$OMP THREADPRIVATE(harvest_type)  

  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION(:,:)     :: harvest_cut      !! Type of cutting that was used for the harvest
                                                                         !! (unitless)
!$OMP THREADPRIVATE(harvest_cut)

  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION(:,:)     :: harvest_area     !! Harvested area (m^{2})
!$OMP THREADPRIVATE(harvest_area)  

  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION(:,:,:)   :: harvest_5y_area  !!Total harvested area in the last 5 years (m^{2})
!$OMP THREADPRIVATE(harvest_5y_area)

  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION(:)       :: harvest_pool_bound       !! The boundaries of the diameter classes
                                                                                 !! in the wood harvest pools
                                                                                 !! @tex $(m)$ @endtex 
!$OMP THREADPRIVATE(harvest_pool_bound)

  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION(:,:)     :: mai              !! The mean annual increment used in
                                                                         !! forestry.  It is the average change
                                                                         !! in the wood volume of of the trunk
                                                                         !! over the lifetime of the forest.
                                                                         !! @tex $(m**3 / m**2 / year)$ @endtex 
!$OMP THREADPRIVATE(mai)

  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION(:,:)     :: pai              !! The period annual increment used in
                                                                         !! forestry.  It is the average change
                                                                         !! in the wood volume of of the trunk
                                                                         !! over the past n_pai years of the forest,
                                                                         !! where n_pai is defined in constants.f90.
                                                                         !! @tex $(m**3 / m**2 / year)$ @endtex 
!$OMP THREADPRIVATE(pai)

  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION(:,:)     :: previous_wood_volume !! The volume of the tree trunks
                                                                         !! in a stand for the previous year.
                                                                         !! @tex $(m**3 / m**2 )$ @endtex 
!$OMP THREADPRIVATE(previous_wood_volume)

  INTEGER(i_std), ALLOCATABLE, SAVE, DIMENSION(:,:)  :: mai_count        !! The number of times we've
                                                                         !! calculated the volume increment
                                                                         !! for a stand
!$OMP THREADPRIVATE(mai_count)

  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION(:,:)     :: coppice_dens     !! The density of a coppice at the first
                                                                         !! cutting.
                                                                         !! @tex $( 1 / m**2 )$ @endtex 
!$OMP THREADPRIVATE(coppice_dens)

  REAL(r_std), ALLOCATABLE,SAVE,DIMENSION(:,:)   :: rue_longterm         !! Longterm radiation use efficiency (??units??)
!$OMP THREADPRIVATE(rue_longterm)

  REAL(r_std), ALLOCATABLE,SAVE,DIMENSION(:,:)   :: lab_fac              !! Activity of labile pool factor (??units??)

  REAL(r_std),SAVE                                   :: dt_forcesoil        !! Time step of soil forcing file (days)
!$OMP THREADPRIVATE(dt_forcesoil)
  INTEGER(i_std),PARAMETER                           :: nparanmax=366       !! Maximum number of time steps per year for forcesoil
  INTEGER(i_std),SAVE                                :: nparan              !! Number of time steps per year for forcesoil read from run definition (unitless) 
!$OMP THREADPRIVATE(nparan)
  INTEGER(i_std),SAVE                                :: nbyear=1            !! Number of years saved for forcesoil (unitless) 
!$OMP THREADPRIVATE(nbyear)
  INTEGER(i_std),SAVE                                :: iatt                !! Time step of forcing of soil processes (iatt = 1 to ::nparan*::nbyear) 
!$OMP THREADPRIVATE(iatt)
  INTEGER(i_std),SAVE                                :: iatt_old=1          !! Previous ::iatt
!$OMP THREADPRIVATE(iatt_old)
  INTEGER(i_std),SAVE                                :: nsfm                !! Number of time steps that can be stored in memory (unitless) 
!$OMP THREADPRIVATE(nsfm)
  INTEGER(i_std),SAVE                                :: nsft                !! Number of time steps in a year (unitless)
!$OMP THREADPRIVATE(nsft)
  INTEGER(i_std),SAVE                                :: iisf                !! Current pointer for teststomate (unitless)
!$OMP THREADPRIVATE(iisf)
  CHARACTER(LEN=100), SAVE                           :: forcing_name        !! Name of forcing file 1
!$OMP THREADPRIVATE(forcing_name)
  CHARACTER(LEN=100), SAVE                           :: Cforcing_name       !! Name of forcing file 2
!$OMP THREADPRIVATE(Cforcing_name)
  INTEGER(i_std),SAVE                                :: Cforcing_id         !! File identifer of file 2
!$OMP THREADPRIVATE(Cforcing_id)    
  INTEGER(i_std),PARAMETER                           :: ndm = 10            !! Maximum number of dimensions (unitless)

INTEGER(i_std), ALLOCATABLE, SAVE, DIMENSION(:,:)  :: forest_managed   !! forest management flag (is the forest being managed?)
                                                                         !! (0-4,unitless)
!$OMP THREADPRIVATE(forest_managed)
  
  INTEGER(i_std), ALLOCATABLE, SAVE, DIMENSION(:,:)  :: forest_managed_lastyear !! forest management flag for the previous year
                                                                         !! (0-4,unitless)
!$OMP THREADPRIVATE(forest_managed_lastyear)
  INTEGER(i_std), ALLOCATABLE, SAVE, DIMENSION(:,:)  :: species_change_map   !! A map which gives the PFT number that each
                                                                         !! PFT will be replanted as in case of a clearcut.
                                                                         !! (1-nvm,unitless)
!$OMP THREADPRIVATE(species_change_map)
  INTEGER(i_std), ALLOCATABLE, SAVE, DIMENSION(:,:)  :: fm_change_map    !! A map which gives the desired FM strategy when
                                                                         !! the PFT will be replanted after a clearcut.
                                                                         !! (1-nvm,unitless)
!$OMP THREADPRIVATE(fm_change_map)
  LOGICAL, ALLOCATABLE, SAVE, DIMENSION(:,:)         :: lpft_replant     !! Indicates if this PFT has either died this year
                                                                         !! or been clearcut/coppiced.  If it has, it is not
                                                                         !! replanted until the end of the year.
!$OMP THREADPRIVATE(lpft_replant)
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION(:)       :: litter_demand    !! The amount of litter which will
                                                                         !! be moved from forest to crop pools
                                                                         !! at the end of the year.
                                                                         !! @tex $( gC / year )$ @endtex 
!$OMP THREADPRIVATE(litter_demand)

REAL(r_std), ALLOCATABLE, SAVE,DIMENSION(:,:)      :: wstress_season   !! Water stress factor, based on hum_rel_daily
                                                                         !! (unitless, 0-1)
!$OMP THREADPRIVATE(wstress_season)

  REAL(r_std), ALLOCATABLE, SAVE,DIMENSION(:,:)     :: wstress_month    !! Water stress factor, based on hum_rel_daily
                                                                         !! (unitless, 0-1)
!$OMP THREADPRIVATE(wstress_month)

REAL(r_std), ALLOCATABLE, SAVE, DIMENSION(:,:,:)    :: light_tran_to_level_season  !! Mean seasonal fraction of light transmitted 
                                                                                  !! to canopy levels
!$OMP THREADPRIVATE(light_tran_to_level_season)

 
PUBLIC clay_fm, silt_fm, bulk_fm, humrel_daily_fm, litterhum_daily_fm, t2m_daily_fm, &
   & t2m_min_daily_fm, tsurf_daily_fm, tsoil_daily_fm, soilhum_daily_fm, &
   & precip_fm, gpp_daily_fm, veget_fm, veget_max_fm, lai_fm
PUBLIC  dt_days, date, do_slow, EndOfYear
PUBLIC isf, nf_written

CONTAINS
  

!! ================================================================================================================================
!! SUBROUTINE 	: stomate_initialize
!!
!>\BRIEF        Initialization routine for stomate module. 
!!
!! DESCRIPTION  : Initialization routine for stomate module. Read options from parameter file, allocate variables, read variables 
!!                from restart file and initialize variables if necessary. 
!!                
!! \n
!_ ================================================================================================================================

SUBROUTINE stomate_initialize &
        (kjit,           kjpij,             kjpindex,                        &
         rest_id_stom,   hist_id_stom,      hist_id_stom_IPCC,               &
         index,          lalo,              neighbours,   resolution,        &
         contfrac,       totfrac_nobio,     clay,         silt,              &
         bulk,           t2m,                                                &
         veget,          veget_max,                                          &
         co2_flux,       fco2_lu,           deadleaf_cover,  assim_param,    &
         circ_class_biomass, circ_class_n,&
         lai_per_level,  laieff_fit)

    IMPLICIT NONE
    !! 0. Variable and parameter declaration
    !! 0.1 Input variables
    INTEGER(i_std),INTENT(in)                       :: kjit              !! Time step number (unitless)
    INTEGER(i_std),INTENT(in)                       :: kjpij             !! Total size of the un-compressed grid (unitless)
    INTEGER(i_std),INTENT(in)                       :: kjpindex          !! Domain size - terrestrial pixels only (unitless)
    INTEGER(i_std),INTENT(in)                       :: rest_id_stom      !! STOMATE's _Restart_ file identifier (unitless)
    INTEGER(i_std),INTENT(in)                       :: hist_id_stom      !! STOMATE's _history_ file identifier (unitless)
    INTEGER(i_std),INTENT(in)                       :: hist_id_stom_IPCC !! STOMATE's IPCC _history_ file identifier(unitless) 
    INTEGER(i_std),DIMENSION(kjpindex),INTENT(in)   :: index             !! The indices of the terrestrial pixels only (unitless) 
    REAL(r_std),DIMENSION(kjpindex,2),INTENT(in)    :: lalo              !! Geographical coordinates (latitude,longitude) for pixels (degrees) 
    INTEGER(i_std),DIMENSION(kjpindex,8),INTENT(in) :: neighbours        !! Neighoring grid points if land for the DGVM (unitless) 
    REAL(r_std),DIMENSION(kjpindex,2),INTENT(in)    :: resolution        !! Size in x an y of the grid (m) - surface area of the gridbox 
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)   :: contfrac          !! Fraction of continent in the grid cell (unitless)
    REAL(r_std),DIMENSION(kjpindex),INTENT(in)      :: totfrac_nobio     !! Fraction of grid cell covered by lakes, land ice, cities, ... (unitless) 
    REAL(r_std),DIMENSION(kjpindex),INTENT(in)      :: clay              !! Clay fraction of soil (0-1, unitless)
    REAL(r_std),DIMENSION(kjpindex),INTENT(in)      :: silt              !! Silt fraction of soil (0-1, unitless) 
    REAL(r_std),DIMENSION(kjpindex),INTENT(in)      :: bulk              !! Bulk density (kg/m**3) 
    REAL(r_std),DIMENSION(kjpindex),INTENT(in)      :: t2m               !! 2 m air temperature (K)
    REAL(r_std),DIMENSION(kjpindex,nvm),INTENT(in)  :: veget             !! Fraction of vegetation type including 
                                                                         !! non-biological fraction (unitless) 
    REAL(r_std),DIMENSION(kjpindex,nvm),INTENT(in)  :: veget_max         !! Maximum fraction of vegetation type including 
                                                                         !! non-biological fraction (unitless) 

    !! 0.2 Output variables
    REAL(r_std),DIMENSION(kjpindex,nvm),INTENT(out) :: co2_flux          !! CO2 flux between atmosphere and biosphere
    REAL(r_std),DIMENSION(kjpindex),INTENT(out)     :: fco2_lu           !! CO2 flux between atmosphere and biosphere from land-use (without forest management)  
    REAL(r_std),DIMENSION(kjpindex),INTENT(out)     :: deadleaf_cover    !! Fraction of soil covered by dead leaves (unitless)
    REAL(r_std),DIMENSION(kjpindex,nvm,npco2),INTENT(out) :: assim_param !! min+max+opt temperatures (K) & vmax for photosynthesis  
    
    !! 0.3 Modified variables
    REAL(r_std),DIMENSION(:,:,:,:,:),INTENT(inout)  :: circ_class_biomass!! Biomass per circumference class @tex $(gC tree^{-1})$ @endtex
    REAL(r_std),DIMENSION(:,:,:),INTENT(inout)      :: circ_class_n      !! Number of trees within each circumference
    REAL(r_std), DIMENSION(:,:,:), INTENT(inout)    :: lai_per_level     !! This is the LAI per vertical level
                                                                         !! @tex $(m^{2} m^{-2})$ @endtex
    TYPE(laieff_type),DIMENSION (:,:,:),INTENT(inout) &
                                                    :: laieff_fit        !! Fitted parameters for the effective LAI

    !! 0.4 Local variables
    REAL(r_std)                                   :: dt_days_read         !! STOMATE time step read in restart file (days)
    INTEGER(i_std)                                :: l,k,ji, jv, i, j, m  !! indices     
    REAL(r_std),PARAMETER                         :: max_dt_days = 5.     !! Maximum STOMATE time step (days)
    REAL(r_std),DIMENSION(kjpindex,nvm)           :: rprof                !! Coefficient of the exponential functions that 
                                                                          !! relates root density to soil depth (unitless) 
    REAL(r_std),DIMENSION(kjpindex,nvm)           :: gpp_daily_x          !! "Daily" gpp for teststomate  
                                                                          !! @tex $(??gC m^{-2} dt_stomate^{-1})$ @endtex 
    REAL(r_std),DIMENSION(kjpindex,nvm)           :: veget_cov            !! Fractional coverage: actually share of the pixel 
                                                                          !! covered by a PFT (fraction of ground area covered), 
                                                                          !! taking into account LAI
    INTEGER(i_std)                                :: ier                  !! Check errors in netcdf call (unitless)

    INTEGER(i_std)                                :: max_totsize          !! Memory management - maximum memory size (Mb)
    INTEGER(i_std)                                :: totsize_1step        !! Memory management - memory required to store one 
                                                                          !! time step on one processor (Mb) 
    INTEGER(i_std)                                :: totsize_tmp          !! Memory management - memory required to store one 
                                                                          !! time step on all processors(Mb) 
    INTEGER(i_std)                                :: vid                  !! Variable identifer of netCDF (unitless)
    INTEGER(i_std)                                :: nneigh               !! Number of neighbouring pixels
    INTEGER(i_std)                                :: direct               !! 
    INTEGER(i_std),DIMENSION(ndm)                 :: d_id                 !! 
    LOGICAL                                       :: l_error              !! error flag
    REAL(r_std)                                   :: temp_total           !! Used for renormalizing
    INTEGER(i_std),ALLOCATABLE, DIMENSION(:,:)    :: fm_map_temp          !! A temporary variable to hold the forest
                                                                          !! management map which is read in from a file
                                                                          !! (0-4,unitless)
    INTEGER(i_std), ALLOCATABLE, DIMENSION(:,:)   :: species_map_temp     !! A temporary variable to hold the species
                                                                          !! change map which is read in from a file
                                                                          !! (1-nvm,unitless)
    INTEGER(i_std)                                :: forest_managed_temp  !! Temporary variable to affect forest_managed(1,6)
 !================================================================================================================================
    
    !! 1. Initialize variable

    ! Initialize local printlev
    !printlev_loc=get_printlev('stomate')

    !! Update flag
    l_first_stomate = .FALSE.
    
    !! 1.1 Store current time step in a common variable
    itime = kjit
    
    !![DISPENSABLE] 1.2 Copy the depth of the different soil layers from diaglev specified in slow_proc
    
    !! 1.3 PFT rooting depth across pixels, humescte is pre-defined 
    ! (constantes_veg.f90). It is defined as the coefficient of an exponential 
    ! function relating root density to depth 
    DO j=1,nvm
       rprof(:,j) = 1./humcste(j)
    ENDDO
    
!!$    !! 1.3.1 Set lai 
!!$    lai(:,ibare_sechiba) = zero
!!$    DO i = 1, kjpindex
!!$       DO j = 2,nvm
!!$          lai(i,j) = cc_to_lai(circ_class_biomass(i,j,:,ileaf,icarbon),circ_class_n(i,j,:),j)
!!$       ENDDO
!!$    ENDDO

    !! 1.4.0 Parameters for spinup
    !
    eps_carbon = 0.01
    !Config Key   = EPS_CARBON
    !Config Desc  = Allowed error on carbon stock
    !Config If    = SPINUP_ANALYTIC
    !Config Def   = 0.01
    !Config Help  = 
    !Config Units = [%]   
    CALL getin_p('EPS_CARBON',eps_carbon)       
    
    
    !Config Key   = SPINUP_PERIOD
    !Config Desc  = Period to calulcate equilibrium during spinup analytic
    !Config If    = SPINUP_ANALYTIC
    !Config Def   = -1
    !Config Help  = Period corresponds in most cases to the number of years of forcing data used in the spinup.
    !Config Units = [years]   
    spinup_period = -1
    CALL getin_p('SPINUP_PERIOD',spinup_period)       
    
    ! Check spinup_period values. 
    ! For periods uptil 6 years, to obtain equilibrium, a bigger period have to be used 
    ! and therefore spinup_period is adjusted to 10 years. 
    IF (spinup_analytic) THEN
       IF (spinup_period <= 0) THEN
          WRITE(numout,*) 'Error in parameter spinup_period. This parameter must be > 0 : spinup_period=',spinup_period
          CALL ipslerr_p (3,'stomate_initialize', &
               'Parameter spinup_period must be set to a positive integer.', &
               'Set this parameter to the number of years of forcing data used for the spinup.', &
               '')
       ELSE IF (spinup_period <= 6) THEN
          ! Adjust to bigger period. The period must be a multiple of the original period.
          WRITE(numout,*) 'Initial spinup_period =',spinup_period,' will be adjusted.'
          spinup_period = spinup_period*(INT(9/spinup_period)+1)
       END IF
       WRITE(numout,*) 'Spinup analytic is activated using eps_carbon=',eps_carbon, ' and spinup_period=',spinup_period
    END IF
    
    !! 1.4.0 Initialization of PFT specific parameters 
    ! Initialization of PFT specific parameters that have no value
    ! for the bare soil PFT i.e. fire resistance, flamability, maximum lai,
    ! settings for growing degree days (GDD), settings for senescence, 
    ! respiration coefficients, photosynthesis, etc.
    ! [DISPENSABLE]
    
    !! 1.4.1 Allocate memory for all variables in stomate
    ! Allocate memory for all variables in stomate, build new index
    ! tables accounting for the PFTs, read and check flags and set file
    ! identifier for restart and history files.
    CALL stomate_init (kjpij, kjpindex, index, lalo, &
         rest_id_stom, hist_id_stom, hist_id_stom_IPCC)
    
    !! 1.4.2 Initialization of PFT specific parameters
    ! Initialization of PFT specific parameters i.e. sla from leaf life, 
    ! sapling characteristics (biomass), migration speed, critical diameter,
    ! coldest tolerable temperature, critical values for phenology, maximum
    ! life time of leaves, respiration coefficients and photosynthesis.
    ! The subroutine also communicates settings read by stomate_constant_init.
    CALL data (kjpindex, lalo)
    
    !! 1.4.3 Initial conditions
    
    !! 1.4.3.1 Read initial values for STOMATE's variables from 
    !  the _restart_ file ??Shouldn't this be included in stomate_init?? 
    !  Looks like an initialization!
    co2_flux(:,:) = zero
    fco2_lu(:) = zero
    
    !! 1.4.3.2 Read the management status   
    !  This is the first call in this loop, It should just
    !  be initialised.
    forest_managed_lastyear(:,:) = 0
    
    !Config Key   = FOREST_MANAGED
    !Config Desc  = Forest management flag
    !Config If    = OK_STOMATE
    !Config Def   = 1: unmanaged, 2: high stand, 3: coppice, 
    !               4: short rotation coppice
    !Config Help  =
    !Config Units = [-]
    ! forest management is always activated but a setting 
    ! of ifm_none means don't do any human management.
    forest_managed(:,:) = ifm_none
    ! We need to things this was, instead of passing the whole array,
    ! since the domains of different processers will sometimes have
    ! different sizes, and calling MPI_BCAST with arrays of different
    ! sizes causes an error.  So we just read a single integer, BCAST
    ! that, and then change the arrays here.
    CALL getin_p('FOREST_MANAGED_FORCED',forest_managed_temp)
    IF(forest_managed_temp .NE. ifm_none)THEN
       forest_managed(:,:)=forest_managed_temp
    ENDIF
    
    ! We need to have the option to read the forest management
    ! strategy from a map (NetCDF file).  If this option is
    ! equal to Y, we will overwrite the forest_managed_forced 
    ! option above, so you should be careful to only use one
    ! or the other.
    ! If we prescribe a species change we will also overwrite
    ! forest_managed so we don't want to read it from a file
    ! but we want to use the restart values
    IF(ok_read_fm_map)THEN

       ! If we are using age classes, we read in the map in the same
       ! way but then we change it a bit to account for age classes.
       l_error = .FALSE.
       ALLOCATE(fm_map_temp(kjpindex,nvmap),stat=ier)
       l_error = l_error .OR. (ier /= 0)
       IF (l_error) THEN
          WRITE(numout,*) 'Problem with memory allocation: ' ,&
               'temporary FM map ',kjpindex,nvmap
          CALL ipslerr_p (3,'stomate_main', &
               'Problem with memory allocation','','')
       ENDIF
       
       CALL read_in_fm_map(kjpindex, lalo, neighbours, resolution, &
            contfrac, fm_map_temp)
       IF(nagec .GT. 1)THEN
          ! All age classes of the same PFT will have the same
          ! management
          DO jv = 1,nvm
             forest_managed(:,jv)=fm_map_temp(:,agec_group(jv))
          ENDDO
          
       ELSE
          forest_managed(:,:)=fm_map_temp(:,:)
          
       ENDIF
       
       DEALLOCATE(fm_map_temp)
       
    ENDIF
   
    ! Get values from _restart_ file. Note that only ::kjpindex, ::index, ::lalo 
    ! and ::resolution are input variables, all others are output variables.
    CALL readstart &
         (kjpindex, index, lalo, resolution, t2m, &
         dt_days_read, date, &
         adapted, regenerate, &
         humrel_daily, gdd_init_date, litterhum_daily, &
         t2m_daily, t2m_min_daily, tsurf_daily, tsoil_daily, &
         soilhum_daily, precip_daily, &
         gpp_daily, npp_daily, turnover_daily, &
         humrel_month, humrel_week, humrel_growingseason,&
         t2m_longterm, tau_longterm, t2m_month, t2m_week, &
         tsoil_month, soilhum_month, fireindex, firelitter, &
         maxhumrel_lastyear, maxhumrel_thisyear, &
         minhumrel_lastyear, minhumrel_thisyear, &
         maxgppweek_lastyear, maxgppweek_thisyear, &
         gdd0_lastyear, gdd0_thisyear, &
         precip_lastyear, precip_thisyear, &
         gdd_m5_dormance,  gdd_from_growthinit, gdd_midwinter, &
         ncd_dormance, ngd_minus5, &
         PFTpresent, npp_longterm, lm_lastyearmax, lm_thisyearmax, &
         maxfpc_lastyear, maxfpc_thisyear, &
         turnover_longterm, gpp_week, resp_maint_part, &
         leaf_age, leaf_frac, plant_status, when_growthinit, age, &
         resp_hetero_d, resp_maint_d, resp_growth_d, co2_fire, atm_to_bm, &
         veget_lastlight, everywhere, need_adjacent, RIP_time, &
         time_hum_min, hum_min_dormance, &
         litter, dead_leaves, &
         som, lignin_struc, lignin_wood, turnover_time,&
         prod_s, prod_m, prod_l, flux_s, flux_m, flux_l, &
         flux_prod_s, flux_prod_m, flux_prod_l, &
         bm_to_litter, carb_mass_total, &
         Tseason, Tseason_length, Tseason_tmp, &
         Tmin_spring_time, onset_date, &
         global_years, ok_equilibrium, nbp_accu, nbp_flux, &
         nbp_accu_pool, nbp_pool, nbp_pool_start, &
         matrixV, vectorU, previous_stock, current_stock, &
         assim_param, CN_som_litter_longterm, &
         tau_CN_longterm, KF, k_latosa_adapt, &
         rue_longterm, cn_leaf_min_season, nstress_season, &
         soil_n_min, p_O2, bact, &
         circ_class_biomass, circ_class_n, store_sum_delta_ba, &
         forest_managed, forest_managed_lastyear, &
         species_change_map, fm_change_map, lpft_replant, lai_per_level, &
         laieff_fit, wstress_season, wstress_month, &
         age_stand, rotation_n, last_cut, mai, pai, &
         previous_wood_volume, mai_count, coppice_dens, &
         light_tran_to_level_season,veget_max, harvest_5y_area)

    !! 1.4.3.4 Read litter raking map
    !  If we are doing litter raking, we need a map which
    !  tells us how much litter is required at every pixel.
    !  This number is independent of the PFTs.  
    IF(ok_litter_raking)THEN
       
       CALL read_in_litter_map(kjpindex, lalo, neighbours, resolution, &
            contfrac, litter_demand)
       
    ENDIF

    
    !! 1.4.3.5 Read species with which to replant
    !  If we are doing species change, we need to read in the map or force
    !  the PFT values.  We do this after the restart because otherwise our
    !  values are overwritten. When we implement the species change we
    !  usually want to change FM as well (and sometime we even need it, i.e.,
    !  when the current FM strategy is to coppice and we change the species
    !  to a conifer tree than we will need to change FM because conifers are
    !  typically not coppiced). Species change and FM change were implemented
    !  separatly because that was easier to test and debug but not all 
    !  combinations were tested. The recommended setting is read de FM_desired
    !  map when using species change.
    IF(ok_change_species)THEN
       
       IF(printlev_loc>=4) WRITE(numout,*) 'Use the species change code'
       
       ! Read species change map
       IF(ok_read_species_change_map)THEN
          
          WRITE(numout,*) 'Reading species change map'
          
          ! Allocate
          l_error = .FALSE.
          ALLOCATE(species_map_temp(kjpindex,nvmap),stat=ier)
          l_error = l_error .OR. (ier /= 0)
          IF (l_error) THEN
             WRITE(numout,*) 'Problem with memory allocation: ', &
                  kjpindex,nvmap
             CALL ipslerr_p (3,'stomate_main', &
                  'Problem with memory allocation',&
                  'temporary species change map','')
          ENDIF
          
          CALL read_in_species_change_map(kjpindex, lalo, neighbours, &
               resolution, contfrac, species_map_temp)
             
          ! If we are using age classes, we read in the map in the same
          ! way but then we change it a bit to account for age classes.
          IF(nagec .GT. 1)THEN
             
             ! All age classes of the same PFT will have the same
             ! management
             DO jv = 1,nvm
                species_change_map(:,jv)=species_map_temp(:,agec_group(jv))
             ENDDO
             
          ELSE

             ! The number of pfts on the map is identical
             ! to the number of PFTs used in the simulation
             WRITE(numout,*) 'Reading single value for species change'
             species_change_map(:,:)=species_map_temp(:,:)
             
          ENDIF

          DEALLOCATE(species_map_temp)
          
       ELSE
          
          IF(printlev_loc>=4) WRITE(numout,*) 'Species change map NOT read'
          IF(printlev_loc>=4) WRITE(numout,*) 'species_change_force, ', &
               species_change_force
          IF (species_change_force .EQ. -9999) THEN

             ! If we end-up here the user has set-up the model
             ! such that we use the species_change code but only
             ! to change the management. To keep the code simple
             ! we will use a dummy species_change_map
             IF(printlev_loc>=4) WRITE(numout,*) 'Use veget_cov_max instead'
             
             ! All age classes of the same PFT will have the same
             ! management
             DO jv = 1,nvm
                species_change_map(:,jv) = agec_group(jv)
             ENDDO
             
          ELSE
             
             ! Use a single value (::species_change_force) instead 
             ! of the information from a map. This was implemented
             ! as a feature for testing and/or debugging. It allows
             ! to test on a single pixel without reading maps.
             IF(printlev_loc>=4) WRITE(numout,*) 'Use ::species_change_force'
             species_change_map(:,:)=species_change_force
             
          ENDIF
          
       ENDIF

       ! If we apply a species change we will also prescribe a new 
       ! management strategy but we need to decide whether we will
       ! read the desired management from a map or use a single 
       ! prescribed value instead
       
       ! Read species change map
       IF(ok_read_desired_fm_map)THEN
          
          WRITE(numout,*) 'Reading desired FM map'
          ! Allocate
          l_error = .FALSE.
          ALLOCATE(fm_map_temp(kjpindex,nvmap),stat=ier)
          l_error = l_error .OR. (ier /= 0)
          IF (l_error) THEN
             WRITE(numout,*) 'Problem with memory allocation: ', &
                  kjpindex,nvmap
             CALL ipslerr_p (3,'stomate_main', &
                  'Problem with memory allocation',&
                  'temporary fm change map','')
          ENDIF
          
          ! If we are using age classes, we read in the map in the same
          ! way but then we change it a bit to account for age classes.
          CALL read_in_desired_fm_map(kjpindex, lalo, neighbours, &
               resolution, contfrac, fm_map_temp)
          
          IF(nagec .GT. 1)THEN
             
             ! All age classes of the same PFT will have the same
             ! management
             DO jv = 1,nvm
                fm_change_map(:,jv)=fm_map_temp(:,agec_group(jv))
             ENDDO
             
          ELSE
             
             ! The number of pfts on the map is identical
             ! to the number of PFTs used in the simulation
             fm_change_map(:,:)=fm_map_temp(:,:)
             
          ENDIF
          
          DEALLOCATE(fm_map_temp)
          
       ELSE
          
          IF(printlev_loc>=4) WRITE(numout,*) 'fm change map NOT read'
          IF(printlev_loc>=4) WRITE(numout,*) 'fm_change_force, ',fm_change_force
          IF(fm_change_force .EQ. -9999) THEN
             
             ! If we end-up here the user has set-up the model
             ! such that we use the species_change code but only
             ! to change the species while keeping the management
             ! constant. To keep the code simple we will use a 
             ! dummy fm_change_map
             IF(printlev_loc>=4) WRITE(numout,*) 'Use forest_managed instead'
             fm_change_map(:,:)=forest_managed(:,:)
             
          ELSE
             
             ! Use a single value (::species_change_force) instead 
             ! of the information from a map. This was implemented
             ! as a feature for testing and/or debugging. It allows
             ! to test on a single pixel without reading maps.
             IF(printlev_loc>=4) WRITE(numout,*) 'Use ::fm_change_force'
             fm_change_map(:,:)=fm_change_force
             
          ENDIF
          
       ENDIF
       
    ENDIF
    
    !! 1.4.5 Check time step
       
    !! 1.4.5.1 Allow STOMATE's time step to change although this is dangerous
    IF (dt_days /= dt_days_read) THEN
       WRITE(numout,*) 'slow_processes: STOMATE time step changes:', &
            & dt_days_read,' -> ',dt_days
    ENDIF
    
    !! 1.4.5.2 Time step has to be a multiple of a full day
    IF ( ( dt_days-REAL(NINT(dt_days),r_std) ) > min_stomate ) THEN
       WRITE(numout,*) 'slow_processes: STOMATE time step is not a mutiple of a full day:', &
            & dt_days,' days.'
       STOP
    ENDIF
    
    !! 1.4.5.3 upper limit to STOMATE's time step
    IF ( dt_days > max_dt_days ) THEN
       WRITE(numout,*) 'slow_processes: STOMATE time step exceeds the maximum value:', &
            & dt_days,' days > ', max_dt_days, ' days.'  
       STOP
    ENDIF
    
    !! 1.4.5.4 STOMATE time step must not be less than the forcing time step
    IF ( dt_sechiba > dt_days*one_day ) THEN
       WRITE(numout,*) &
            & 'slow_processes: STOMATE time step ::dt_days smaller than forcing time step ::dt_sechiba'
       STOP
    ENDIF
    
    !! 1.4.5.6 Final message on time step
    WRITE(numout,*) 'Slow_processes, STOMATE time step (days): ', dt_days
    
    !! 1.4.6 Write forcing file for teststomate
    IF (allow_forcing_write) THEN
       
       !Config Key   = STOMATE_FORCING_NAME
       !Config Desc  = Name of STOMATE's forcing file
       !Config If    = OK_STOMATE
       !Config Def   = NONE
       !Config Help  = Name that will be given
       !Config         to STOMATE's offline forcing file
       !Config         Compatible with Nicolas Viovy's driver
       !Config Units = [FILE]
       forcing_name = stomate_forcing_name
       CALL getin_p('STOMATE_FORCING_NAME',forcing_name)
       
       IF ( TRIM(forcing_name) /= 'NONE' ) THEN
          
          !! 1.4.6.1 Calculate steps that can be stored in memory
          ! Action for the root processor only (parallel computing)  
          IF (is_root_prc) CALL SYSTEM ('rm -f '//TRIM(forcing_name))
          WRITE(numout,*) 'writing a forcing file for STOMATE.'
          
          !Config Key   = STOMATE_FORCING_MEMSIZE
          !Config Desc  = Size of STOMATE forcing data in memory 
          !Config If    = OK_STOMATE
          !Config Def   = 50
          !Config Help  = This variable determines how many
          !Config         forcing states will be kept in memory.
          !Config         Must be a compromise between memory
          !Config         use and frequeny of disk access.
          !Config Units = [MegaBytes]
          max_totsize = 50
          CALL getin_p('STOMATE_FORCING_MEMSIZE', max_totsize)      
          max_totsize = max_totsize*1000000
          
          totsize_1step = &
               SIZE(clay)*KIND(clay) &
               +SIZE(silt)*KIND(silt) &
               +SIZE(bulk)*KIND(bulk) &
               +SIZE(humrel_daily)*KIND(humrel_daily) &
               +SIZE(litterhum_daily)*KIND(litterhum_daily) &
               +SIZE(t2m_daily)*KIND(t2m_daily) &
               +SIZE(t2m_min_daily)*KIND(t2m_min_daily) &
               +SIZE(tsurf_daily)*KIND(tsurf_daily) &
               +SIZE(tsoil_daily)*KIND(tsoil_daily) &
               +SIZE(soilhum_daily)*KIND(soilhum_daily) &
               +SIZE(precip_daily)*KIND(precip_daily) &
               +SIZE(gpp_daily_x)*KIND(gpp_daily_x) &
               +SIZE(veget)*KIND(veget) &
               +SIZE(veget_max)*KIND(veget_max) &
!!$               +SIZE(lai)*KIND(lai) &    
               +SIZE(drainage_daily)*KIND(drainage_daily) 
          
          ! Totsize_1step is the size on a single processor, sum
          ! all processors and send to all processors
          CALL reduce_sum(totsize_1step,totsize_tmp)
          CALL bcast(totsize_tmp)
          totsize_1step=totsize_tmp
          
          ! Total number of forcing steps
          nsft = INT(one_year/(dt_stomate/one_day))
          
          ! Number of forcing steps in memory
          nsfm = MIN(nsft, &
               MAX(1,NINT( REAL(max_totsize,r_std) &
               /REAL(totsize_1step,r_std))))
            
             
          !! 1.6.4.2 Allocate memory for variables containing forcing data  
          ! and initialize variables (set to zero).
          CALL init_forcing (kjpindex,nsfm,nsft)
          
          ! Indexing for writing forcing file
          isf(:) = (/ (i,i=1,nsfm) /)
          nf_written(:) = .FALSE.
          nf_cumul(:) = 0
          iisf = 0
          
          !! 1.6.4.3 Create netcdf file
          ! Create, define and populate a netcdf file containing the forcing data.
          ! For the root processor only (parallel computing). NF90_ are functions
          ! from and external library.  
          IF (is_root_prc) THEN
             
             ! Create new netCDF dataset
             ier = NF90_CREATE (TRIM(forcing_name),NF90_SHARE,forcing_id)
             
             ! Add variable attribute
             ! Note ::iim_g and ::jjm_g are dimensions of the global field and 
             ! ::nbp_glo is the number of global continental points
             ier = NF90_PUT_ATT (forcing_id,NF90_GLOBAL,'dt_sechiba',dt_sechiba)
             ier = NF90_PUT_ATT (forcing_id,NF90_GLOBAL,'dt_stomate',dt_stomate)
             ier = NF90_PUT_ATT (forcing_id,NF90_GLOBAL, &
                  'nsft',REAL(nsft,r_std))
             ier = NF90_PUT_ATT (forcing_id,NF90_GLOBAL, &
                  'kjpij',REAL(iim_g*jjm_g,r_std))
             ier = NF90_PUT_ATT (forcing_id,NF90_GLOBAL, &
                  'kjpindex',REAL(nbp_glo,r_std))
             
             ! Add new dimension
             ier = NF90_DEF_DIM (forcing_id,'points',nbp_glo,d_id(1))
             ier = NF90_DEF_DIM (forcing_id,'layers',nbdl,d_id(2))
             ier = NF90_DEF_DIM (forcing_id,'pft',nvm,d_id(3))
             direct=2
             ier = NF90_DEF_DIM (forcing_id,'direction',direct,d_id(4))
             nneigh=8
             ier = NF90_DEF_DIM (forcing_id,'nneigh',nneigh,d_id(5))
             ier = NF90_DEF_DIM (forcing_id,'time',NF90_UNLIMITED,d_id(6))
             ier = NF90_DEF_DIM (forcing_id,'nbparts',nparts,d_id(7))
             
             ! Add new variable
             ier = NF90_DEF_VAR (forcing_id,'points',    r_typ,d_id(1),vid)
             ier = NF90_DEF_VAR (forcing_id,'layers',    r_typ,d_id(2),vid)
             ier = NF90_DEF_VAR (forcing_id,'pft',       r_typ,d_id(3),vid)
             ier = NF90_DEF_VAR (forcing_id,'direction', r_typ,d_id(4),vid)
             ier = NF90_DEF_VAR (forcing_id,'nneigh',    r_typ,d_id(5),vid)
             ier = NF90_DEF_VAR (forcing_id,'time',      r_typ,d_id(6),vid)
             ier = NF90_DEF_VAR (forcing_id,'nbparts',   r_typ,d_id(7),vid)
             ier = NF90_DEF_VAR (forcing_id,'index',     r_typ,d_id(1),vid)
             ier = NF90_DEF_VAR (forcing_id,'contfrac',  r_typ,d_id(1),vid) 
             ier = NF90_DEF_VAR (forcing_id,'lalo', &
                  r_typ,(/ d_id(1),d_id(4) /),vid)
             ier = NF90_DEF_VAR (forcing_id,'neighbours', &
                  r_typ,(/ d_id(1),d_id(5) /),vid)
             ier = NF90_DEF_VAR (forcing_id,'resolution', &
                  r_typ,(/ d_id(1),d_id(4) /),vid)
             ier = NF90_DEF_VAR (forcing_id,'clay', &
                  r_typ,(/ d_id(1),d_id(6) /),vid)
             ier = NF90_DEF_VAR (forcing_id,'silt', & 
                  r_typ,(/ d_id(1),d_id(6) /),vid) 
             ier = NF90_DEF_VAR (forcing_id,'bulk', & 
                  r_typ,(/ d_id(1),d_id(6) /),vid) 
             ier = NF90_DEF_VAR (forcing_id,'humrel', &
                  r_typ,(/ d_id(1),d_id(3),d_id(6) /),vid)
             ier = NF90_DEF_VAR (forcing_id,'litterhum', &
                  r_typ,(/ d_id(1),d_id(6) /),vid)
             ier = NF90_DEF_VAR (forcing_id,'t2m', &
                  r_typ,(/ d_id(1),d_id(6) /),vid)
             ier = NF90_DEF_VAR (forcing_id,'t2m_min', &
                  r_typ,(/ d_id(1),d_id(6) /),vid)
             ier = NF90_DEF_VAR (forcing_id,'tsurf', &
                  r_typ,(/ d_id(1),d_id(6) /),vid)
             ier = NF90_DEF_VAR (forcing_id,'tsoil', &
                  r_typ,(/ d_id(1),d_id(2),d_id(6) /),vid)
             ier = NF90_DEF_VAR (forcing_id,'soilhum', &
                  r_typ,(/ d_id(1),d_id(2),d_id(6) /),vid)
             ier = NF90_DEF_VAR (forcing_id,'precip', &
                  r_typ,(/ d_id(1),d_id(6) /),vid)
             ier = NF90_DEF_VAR (forcing_id,'gpp', &
                  r_typ,(/ d_id(1),d_id(3),d_id(6) /),vid)
             ier = NF90_DEF_VAR (forcing_id,'veget', &
                  r_typ,(/ d_id(1),d_id(3),d_id(6) /),vid)
             ier = NF90_DEF_VAR (forcing_id,'veget_max', &
                  r_typ,(/ d_id(1),d_id(3),d_id(6) /),vid)
!!$             ier = NF90_DEF_VAR (forcing_id,'lai', &
!!$                  r_typ,(/ d_id(1),d_id(3),d_id(6) /),vid)
             ier = NF90_DEF_VAR (forcing_id,'drainage', & 
                  r_typ,(/ d_id(1),d_id(6),d_id(6) /),vid) 
             ier = NF90_ENDDEF (forcing_id)
             
             ! Given the name of a varaible, nf90_inq_varid finds the variable 
             ! ID (::vid). Put data value(s) into variable ::vid
             ier = NF90_INQ_VARID (forcing_id,'points',vid)
             ier = NF90_PUT_VAR (forcing_id,vid, &
                  (/(REAL(i,r_std),i=1,nbp_glo) /))
             ier = NF90_INQ_VARID (forcing_id,'layers',vid)
             ier = NF90_PUT_VAR (forcing_id,vid,(/(REAL(i,r_std),i=1,nbdl)/))
             ier = NF90_INQ_VARID (forcing_id,'pft',vid)
             ier = NF90_PUT_VAR (forcing_id,vid,(/(REAL(i,r_std),i=1,nvm)/))
             ier = NF90_INQ_VARID (forcing_id,'direction',vid)
             ier = NF90_PUT_VAR (forcing_id,vid,(/(REAL(i,r_std),i=1,2)/))
             ier = NF90_INQ_VARID (forcing_id,'nneigh',vid)
             ier = NF90_PUT_VAR (forcing_id,vid,(/(REAL(i,r_std),i=1,8)/))
             ier = NF90_INQ_VARID (forcing_id,'time',vid)
             ier = NF90_PUT_VAR (forcing_id,vid,(/(REAL(i,r_std),i=1,nsft)/))
             ier = NF90_INQ_VARID (forcing_id,'nbparts',vid)
             ier = NF90_PUT_VAR (forcing_id,vid,(/(REAL(i,r_std),i=1,nparts)/))
             ier = NF90_INQ_VARID (forcing_id,'index',vid)  
             ier = NF90_PUT_VAR (forcing_id,vid,REAL(index_g,r_std))
             ier = NF90_INQ_VARID (forcing_id,'contfrac',vid)
             ier = NF90_PUT_VAR (forcing_id,vid,REAL(contfrac_g,r_std))
             ier = NF90_INQ_VARID (forcing_id,'lalo',vid)
             ier = NF90_PUT_VAR (forcing_id,vid,lalo_g)
             !ym attention a neighbours, a modifier plus tard      
             ier = NF90_INQ_VARID (forcing_id,'neighbours',vid)
             ier = NF90_PUT_VAR (forcing_id,vid,REAL(neighbours_g,r_std))
             ier = NF90_INQ_VARID (forcing_id,'resolution',vid)
             ier = NF90_PUT_VAR (forcing_id,vid,resolution_g)
          ENDIF ! is_root_prc
       ENDIF ! (forcing_name) /= 'NONE'
    ENDIF 
    
    !! 1.4.7 write forcing file for forcesoil
    !! 1.4.7.1 Initialize
    !Config Key   = STOMATE_CFORCING_NAME
    !Config Desc  = Name of STOMATE's carbon forcing file
    !Config If    = OK_STOMATE
    !Config Def   = NONE
    !Config Help  = Name that will be given to STOMATE's carbon
    !Config         offline forcing file
    !Config         Compatible with Nicolas Viovy's driver
    !Config Units = [FILE]
    Cforcing_name = stomate_Cforcing_name
    CALL getin_p('STOMATE_CFORCING_NAME',Cforcing_name)
    
    IF ( TRIM(Cforcing_name) /= 'NONE' ) THEN
       
       ! Time step of forcesoil
       !Config Key   = FORCESOIL_STEP_PER_YEAR
       !Config Desc  = Number of time steps per year for carbon spinup.
       !Config If    = OK_STOMATE
       !Config Def   = 365
       !Config Help  = Number of time steps per year for carbon spinup.
       !Config Units = [days, months, year]
       nparan = 365
       CALL getin_p('FORCESOIL_STEP_PER_YEAR', nparan)
       
       ! Correct if setting is out of bounds 
       IF ( nparan < 1 ) nparan = 1
       
       !Config Key   = FORCESOIL_NB_YEAR
       !Config Desc  = Number of years saved for carbon spinup.
       !Config If    = OK_STOMATE
       !Config Def   = 1
       !Config Help  = Number of years saved for carbon spinup. If internal parameter cumul_Cforcing is TRUE in stomate.f90
       !Config         Then this parameter is forced to one.
       !Config Units = [years]
       CALL getin_p('FORCESOIL_NB_YEAR', nbyear)
       
       ! Set ::nbyear to 1. if ::cumul_Cforcing=.TRUE.
       IF ( cumul_Cforcing ) THEN
          CALL ipslerr_p (1,'stomate', &
               'Internal parameter cumul_Cforcing is TRUE in stomate.f90', &
               'Parameter FORCESOIL_NB_YEAR is therefore forced to 1.', &
               '::nbyear is thus set to 1.')
          nbyear=1
       ENDIF
       
       ! Make use of ::nparan to calculate ::dt_forcesoil
       dt_forcesoil = zero
       nparan = nparan+1
       DO WHILE ( dt_forcesoil < dt_stomate/one_day )
          nparan = nparan-1
          IF ( nparan < 1 ) THEN
             STOP 'Problem with number of soil forcing time steps ::nparan < 1.'
          ENDIF
          dt_forcesoil = one_year/REAL(nparan,r_std)
       ENDDO
       IF ( nparan > nparanmax ) THEN
          STOP 'Problem with number of soil forcing time steps ::nparan > ::nparanmax'
       ENDIF
       WRITE(numout,*) 'Time step of soil forcing (d): ',dt_forcesoil
       
       ! Allocate memory for the forcing variables of soil dynamics
       ALLOCATE( nforce(nparan*nbyear))
       nforce(:) = 0
       ALLOCATE(control_moist(kjpindex,nlevs,nparan*nbyear))
       ALLOCATE(npp_equil(kjpindex,nparan*nbyear))
       ALLOCATE(npp_tot(kjpindex))
       ALLOCATE(control_temp(kjpindex,nlevs,nparan*nbyear))
       ALLOCATE(carbon_input(kjpindex,ncarb,nvm,nparan*nbyear)) 
       ALLOCATE(nitrogen_input(kjpindex,ncarb,nvm,nparan*nbyear)) 
       ALLOCATE(drainage(kjpindex,nvm,nparan*nbyear))         

       ! Initialize variables, set to zero
       control_moist(:,:,:) = zero
       npp_equil(:,:) = zero
       npp_tot(:) = zero
       control_temp(:,:,:) = zero
       carbon_input(:,:,:,:) = zero
       nitrogen_input(:,:,:,:) = zero
       drainage(:,:,:) = zero 
    ENDIF ! Cforcing_name) /= 'NONE'
    
    !! 1.4.8 Calculate STOMATE's vegetation fractions from veget, veget_max
    DO j=1,nvm
       WHERE ((1.-totfrac_nobio(:)) > min_sechiba)       
          ! Pixels with vegetation
          veget_cov(:,j) = veget(:,j)/( 1.-totfrac_nobio(:) )
          veget_cov_max(:,j) = veget_max(:,j)/( 1.-totfrac_nobio(:) )
       ELSEWHERE
          ! Pixels without vegetation
          veget_cov(:,j) = zero
          veget_cov_max(:,j) = zero
       ENDWHERE
    ENDDO ! Loop over PFTs

    !! 1.4.9 Initialize non-zero variables
    CALL stomate_var_init &
         (kjpindex, veget_cov_max, leaf_age, leaf_frac, &
         dead_leaves, &
         veget_cov, deadleaf_cover, assim_param, &
         circ_class_biomass, circ_class_n)
    
    l_error =.FALSE.
    ALLOCATE(circ_class_dist(ncirc),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) ' Memory allocation error for circ_class_dist. ' // &
            'We need ncirc words = ',ncirc
       CALL ipslerr_p (3,'stomate_main', &
            'Memory allocation error for circ_class_dist','','')
    END IF
    !Config Key   = CIRC_CLASS_DIST
    !Config Desc  = The tree distribution to target if we have to redistribute.
    !Config if    = OK_STOMATE
    !Config Def   = 1
    !Config Help  =
    !Config Units = [-]
    circ_class_dist(:)=un
    CALL getin_p('CIRC_CLASS_DIST',circ_class_dist)
    
    ! Now we normalize the distribution
    temp_total=SUM(circ_class_dist(:))
    circ_class_dist(:)=circ_class_dist(:)/temp_total
    WRITE(numout,*) 'Target distribution for renormalization: ',circ_class_dist(:)

    ALLOCATE(st_dist(ncirc),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) ' Memory allocation error for st_dist. ' // &
            'We need ncirc words = ',ncirc
       CALL ipslerr_p (3,'stomate_main', &
            ' Memory allocation error for st_dist.','','')
    END IF
    !Config Key   = ST_DIST
    !Config Desc  = The distribution for killing trees in self-thinning.
    !Config if    = OK_STOMATE
    !Config Def   = circ_class_dist
    !Config Help  =
    !Config Units = [-]
    st_dist(:)=circ_class_dist(:)
    CALL getin_p('ST_DIST',st_dist)
    
    ! Now we normalize the distribution
    temp_total=SUM(st_dist(:))
    st_dist(:)=st_dist(:)/temp_total
    WRITE(numout,*) 'Target distribution for self-thinning: ',st_dist(:)
    
  END SUBROUTINE stomate_initialize
  

!! ================================================================================================================================
!! SUBROUTINE 	: stomate_main
!!
!>\BRIEF        Manages variable initialisation, reading and writing forcing 
!! files, aggregating data at stomate's time step (dt_stomate), aggregating data
!! at longer time scale (i.e. for phenology) and uses these forcing to calculate
!! CO2 fluxes (NPP and respirations) and C-pools (litter, soil, biomass, ...)
!!
!! DESCRIPTION  : The subroutine manages 
!! divers tasks:
!! (1) Initializing all variables of stomate (first call)
!! (2) Reading and writing forcing data (last call)
!! (3) Adding CO2 fluxes to the IPCC history files
!! (4) Converting the time steps of variables to maintain consistency between
!! sechiba and stomate
!! (5) Use these variables to call stomate_lpj, maint_respiration, littercalc,
!! som. The called subroutines handle: climate constraints 
!! for PFTs, PFT dynamics, Phenology, Allocation, NPP (based on GPP and
!! authothropic respiration), fire, mortality, vmax, assimilation temperatures,
!! all turnover processes, light competition, sapling establishment, lai,  
!! land cover change and litter and soil dynamics.
!! (6) Use the spin-up method developed by Lardy (2011)(only if SPINUP_ANALYTIC 
!! is set to TRUE).
!!
!! RECENT CHANGE(S) : None
!!
!! MAIN OUTPUT VARIABLE(S): deadleaf_cover, assim_param, lai, height, veget_cov, 
!! veget_cov_max, resp_maint, 
!! resp_hetero,resp_growth, co2_flux, fco2_lu.
!!
!! REFERENCES	: 
!! - Lardy, R, et al., A new method to determine soil organic carbon equilibrium,
!! Environmental Modelling & Software (2011), doi:10.1016|j.envsoft.2011.05.016
!!
!! FLOWCHART    : 
!! \latexonly 
!! \includegraphics[scale=0.5]{stomatemainflow.png}
!! \endlatexonly
!! \n
!_ ================================================================================================================================
  
  SUBROUTINE stomate_main &
       & (kjit, kjpij, kjpindex, &
       &  index, lalo, neighbours, resolution, contfrac, &
       &  totfrac_nobio, clay, silt, bulk, &
       &  t2m, temp_sol, stempdiag, &
       &  humrel, shumdiag, litterhumdiag, precip_rain, precip_snow, &
       &  tmc_pft, drainage_pft, swc_pft, gpp, &
       &  deadleaf_cover, assim_param, &
       &  frac_age, height, veget, veget_max, &
       &  veget_max_new, totfrac_nobio_new, &
       &  hist_id, hist2_id, rest_id_stom, hist_id_stom, hist_id_stom_IPCC, &
       &  co2_flux, fco2_lu, resp_maint, resp_hetero, &
       &  resp_growth, temp_growth, &
       &  soil_pH, pb, n_input, circ_class_biomass, &
       &  circ_class_n, lai_per_level, &
       &  laieff_fit, h_array_out, z_array_out, max_height_store, &
       &  transpir, transpir_mod, transpir_supply, vir_transpir_supply, &
       &  coszang, stressed, unstressed, Isotrop_Tran_Tot_p, &
       &  u, v)  

    IMPLICIT NONE


    !! 0. Variable and parameter declaration

    !! 0.1 Input variables

    INTEGER(i_std),INTENT(in)                       :: kjit              !! Time step number (unitless)
    INTEGER(i_std),INTENT(in)                       :: kjpindex          !! Domain size - terrestrial pixels only (unitless)
    INTEGER(i_std),INTENT(in)                       :: kjpij             !! Total size of the un-compressed grid (unitless)
    INTEGER(i_std),INTENT(in)                       :: rest_id_stom      !! STOMATE's _Restart_ file identifier (unitless)
    INTEGER(i_std),INTENT(in)                       :: hist_id_stom      !! STOMATE's _history_ file identifier (unitless)
    INTEGER(i_std),INTENT(in)                       :: hist_id_stom_IPCC !! STOMATE's IPCC _history_ file identifier 
    !! (unitless) 
    INTEGER(i_std),DIMENSION(kjpindex),INTENT(in)   :: index             !! Indices of the pixels on the map. Stomate uses a 
    !! reduced grid excluding oceans. ::index contains 
    !! the indices of the terrestrial pixels only 
    !! (unitless) 
    INTEGER(i_std),DIMENSION(kjpindex,8),INTENT(in) :: neighbours        !! Neighoring grid points if land for the DGVM 
    !! (unitless) 
    REAL(r_std),DIMENSION(kjpindex,2),INTENT(in)    :: lalo              !! Geographical coordinates (latitude,longitude) 
    !! for pixels (degrees) 
    REAL(r_std),DIMENSION(kjpindex,2),INTENT(in)    :: resolution        !! Size in x an y of the grid (m) - surface area of 
    !! the gridbox 
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)   :: contfrac          !! Fraction of continent in the grid cell (unitless)
    REAL(r_std),DIMENSION(kjpindex),INTENT(in)      :: totfrac_nobio     !! Fraction of grid cell covered by lakes, land 
    !! ice, cities, ... (unitless) 
    REAL(r_std),DIMENSION(kjpindex),INTENT(in)      :: clay              !! Clay fraction of soil (0-1, unitless)
    REAL(r_std),DIMENSION(kjpindex),INTENT(in)      :: silt              !! Silt fraction of soil (0-1, unitless) 
    REAL(r_std),DIMENSION(kjpindex),INTENT(in)      :: bulk              !! Bulk density (kg/m**3) 
    REAL(r_std),DIMENSION(kjpindex,nvm),INTENT(in)  :: humrel            !! Relative humidity ("moisture availability") 
    !! (0-1, unitless) 
    REAL(r_std),DIMENSION(kjpindex),INTENT(in)      :: t2m               !! 2 m air temperature (K)
    REAL(r_std),DIMENSION(kjpindex),INTENT(in)      :: temp_sol          !! Surface temperature (K)
    REAL(r_std),DIMENSION(kjpindex,nbdl),INTENT(in) :: stempdiag         !! Soil temperature (K)
    REAL(r_std),DIMENSION(kjpindex,nbdl),INTENT(in) :: shumdiag          !! Relative soil moisture (0-1, unitless)
    REAL(r_std),DIMENSION(kjpindex),INTENT(in)      :: litterhumdiag     !! Litter humidity (0-1, unitless)
    REAL(r_std),DIMENSION(kjpindex,nvm),INTENT(in)  :: transpir          !! transpiration @tex $(kg m^{-2} timestep^{-1})$
    REAL(r_std),DIMENSION(kjpindex,nvm),INTENT(in)  :: transpir_mod      !! transpir divided by veget_cov_max
    REAL(r_std),DIMENSION(kjpindex,nvm),INTENT(in)  :: transpir_supply   !! Supply of water for transpiration @tex $(mm dt^{-1})$ @endtex
    REAL(r_std),DIMENSION(:,:),INTENT(in)           :: vir_transpir_supply !! Virtual supply of water for transpiration to deal
    !! with water stress when PFT1 becomes vegetated in LCC
    !! @tex $(mm dt^{-1})$ @endtex
    REAL(r_std),DIMENSION(kjpindex),INTENT(in)      :: precip_rain       !! Rain precipitation  
    !! @tex $(mm dt_stomate^{-1})$ @endtex 
    REAL(r_std),DIMENSION(kjpindex),INTENT(in)      :: precip_snow       !! Snow precipitation  
    !! @tex $(mm dt_stomate^{-1})$ @endtex 
    REAL(r_std),DIMENSION(kjpindex,nvm),INTENT(in)  :: tmc_pft           !! Total soil water per PFT (mm/m2) 
    REAL(r_std),DIMENSION(kjpindex,nvm),INTENT(in)  :: drainage_pft      !! Drainage per PFT (mm/m2)   
    REAL(r_std),DIMENSION(kjpindex,nvm),INTENT(in)  :: swc_pft           !! Relative Soil water content [tmcr:tmcs] per pft (-)   
    REAL(r_std),DIMENSION(kjpindex,nvm),INTENT(in)  :: gpp               !! GPP of total ground area  
    
! Import wind speed from stomate_main via slowproc
    REAL(r_std),DIMENSION(kjpindex),INTENT (in)     :: u                 !! Lowest level wind speed in direction u (m/s)
    REAL(r_std),DIMENSION(kjpindex),INTENT (in)     :: v                 !! Lowest level wind speed in direction v (m/s)
  

!! @tex $(gC m^{-2} time step^{-1})$ @endtex 
    !! Calculated in sechiba, account for vegetation 
    !! cover and effective time step to obtain ::gpp_d 

    REAL(r_std),DIMENSION(kjpindex,nvm),INTENT(inout)  :: veget_max_new  !! New "maximal" coverage fraction of a PFT: only if 
    !! vegetation is updated in slowproc

    REAL(r_std),DIMENSION(kjpindex),INTENT(in)      :: totfrac_nobio_new !! Old fraction of nobio per gridcell
    REAL(r_std),DIMENSION(kjpindex),INTENT(in)      :: soil_pH           !! soil pH 	 
    REAL(r_std),DIMENSION(kjpindex), INTENT(in)     :: pb                !! Air pressure (hPa) 
    REAL(r_std),DIMENSION(:,:,:),INTENT(inout)      :: n_input           !! Nitrogen inputs into the soil  (gN/m**2/timestep) 

    INTEGER(i_std),INTENT(in)                       :: hist_id           !! ?? [DISPENSABLE] SECHIBA's _history_ file 
    !! identifier 
    INTEGER(i_std),INTENT(in)                       :: hist2_id          !! ?? [DISPENSABLE] SECHIBA's _history_ file 2 
    !! identifier 
    REAL(r_std),DIMENSION(kjpindex), INTENT(in)     :: coszang           !! the cosine of the zenith angle   
    REAL(r_std),DIMENSION (:,:,:), INTENT (in)      :: Isotrop_Tran_Tot_p!! Transmitted radiation per level (unitless)


    !! 0.2 Output variables

    REAL(r_std),DIMENSION(kjpindex,nvm),INTENT(out) :: co2_flux          !! CO2 flux between atmosphere and biosphere per 
    !! average ground area 
    !! @tex $(gC m^{-2} dt_stomate^{-1})$ @endtex  
    !! [??CHECK] sign convention? 
    REAL(r_std),DIMENSION(kjpindex),INTENT(out)     :: fco2_lu           !! CO2 flux between atmosphere and biosphere from 
    !! land-use (without forest management)  
    !! @tex $(gC m^{-2} dt_stomate^{-1})$ @endtex  
    !! [??CHECK] sign convention? 
    REAL(r_std),DIMENSION(kjpindex,nvm),INTENT(out) :: resp_maint        !! Maitenance component of autotrophic respiration in 
    !! @tex $(gC m^{-2} dt_stomate^{-1})$ @endtex 
    REAL(r_std),DIMENSION(kjpindex,nvm),INTENT(out) :: resp_growth       !! Growth component of autotrophic respiration in 
    !! @tex ($gC m^{-2} dt_stomate^{-1}$) @endtex
    REAL(r_std),DIMENSION(kjpindex,nvm),INTENT(out) :: resp_hetero       !! Heterotrophic respiration in  
    !! @tex $(gC m^{-2} dt_stomate^{-1})$ @endtex  
    REAL(r_std),DIMENSION(kjpindex),INTENT(out)     :: temp_growth       !! Growth temperature (Â°C)  
    !! Is equal to t2m_month 
    REAL(r_std),DIMENSION(kjpindex,nvm),INTENT(out) :: max_height_store  !! ???


    !! 0.3 Modified

!!$    REAL(r_std),DIMENSION(kjpindex,nvm),INTENT(inout)       :: lai            !! Leaf area inex @tex $(m^2 m^{-2})$ @endtex
    REAL(r_std),DIMENSION(kjpindex,nvm),INTENT(in)          :: veget          !! Fraction of vegetation type including 
    !! non-biological fraction (unitless) 
    REAL(r_std),DIMENSION(kjpindex,nvm),INTENT(inout)       :: veget_max      !! Maximum fraction of vegetation type including 
    !! non-biological fraction (unitless) 
    REAL(r_std),DIMENSION(kjpindex,nvm),INTENT(inout)       :: height         !! Height of vegetation (m)
    REAL(r_std),DIMENSION(kjpindex,nvm,npco2),INTENT(inout) :: assim_param    !! min+max+opt temperatures (K) & vmax, nue and leaf N for 
    !! photosynthesis  
    !! @tex $(\mu mol m^{-2}s^{-1})$ @endtex  
    REAL(r_std),DIMENSION(kjpindex),INTENT(inout)           :: deadleaf_cover !! Fraction of soil covered by dead leaves 
    !! (unitless) 
    REAL(r_std),DIMENSION(kjpindex,nvm,nleafages), &                          !! Age efficacity from STOMATE
         INTENT(inout)         :: frac_age        
    REAL(r_std), DIMENSION(kjpindex,nvm,ncirc,nparts,nelements), &            !! Biomass per circumference class @tex $(gC tree^{-1})$ @endtex
         INTENT(inout)                                      :: circ_class_biomass  
    REAL(r_std), DIMENSION(kjpindex,nvm,ncirc), &                             !! Number of trees within each circumference
         INTENT(inout)         :: circ_class_n   !! Biomass per PFT
    REAL(r_std), DIMENSION(:,:,:), INTENT(inout)            :: lai_per_level  !! This is the LAI per vertical level
    !! @tex $(m^{2} m^{-2})$ @endtex
    TYPE(laieff_type),DIMENSION (:,:,:), INTENT(inout)      :: laieff_fit     !! Fitted parameters for the effective LAI

    REAL(r_std),DIMENSION(kjpindex,nvm),INTENT(inout)       :: stressed       !! adjusted ecosystem functioning. Takes the unit of the variable
    !! used as a proxy for waterstress (assigned in sechiba).
    REAL(r_std),DIMENSION(kjpindex,nvm),INTENT(inout)       :: unstressed     !! initial ecosystem functioning after the first calculation and 
    !! before any recalculations. Takes the unit of the variable used
    !! as a proxy for unstressed (assigned in sechiba).
    REAL(r_std),DIMENSION(:,:,:,:), INTENT(inout)           :: h_array_out    !! An output of h_array, to use in sechiba
    REAL(r_std),DIMENSION(:,:,:,:), INTENT(inout)           :: z_array_out    !! An output of h_array, to use in sechiba


    !! 0.4 local variables

    CHARACTER(LEN=2), DIMENSION(nelements)        :: element_str              !! string suffix indicating element    
    REAL(r_std)                                   :: dt_days_read             !! STOMATE time step read in restart file (days)
    INTEGER(i_std)                                :: l,k,ji, jv, i, j, m      !! indices    
    REAL(r_std),PARAMETER                         :: max_dt_days = 5.         !! Maximum STOMATE time step (days)
    REAL(r_std)                                   :: hist_days                !! Writing frequency for history file (days)
    REAL(r_std),DIMENSION(0:nbdl)                 :: z_soil                   !! Variable to store depth of the different soil 
    !! layers (m) 
    REAL(r_std),DIMENSION(kjpindex,nvm)           :: rprof                    !! Coefficient of the exponential functions that 
    !! relates root density to soil depth (unitless) 
    REAL(r_std),DIMENSION(kjpindex)               :: cvegtot                  !! Total "vegetation" cover (unitless)
    REAL(r_std),DIMENSION(kjpindex)               :: precip                   !! Total liquid and solid precipitation  
    !! @tex $(??mm dt_stomate^{-1})$ @endtex 
    REAL(r_std),DIMENSION(kjpindex,nvm)           :: gpp_d                    !! Gross primary productivity per ground area 
    !! @tex $(??gC m^{-2} dt_stomate^{-1})$ @endtex  
    REAL(r_std),DIMENSION(kjpindex,nvm)           :: gpp_daily_x              !! "Daily" gpp for teststomate  
    !! @tex $(??gC m^{-2} dt_stomate^{-1})$ @endtex 
    REAL(r_std),DIMENSION(kjpindex,nvm)           :: resp_hetero_litter       !! Litter heterotrophic respiration per ground area 
    !! @tex $(gC m^{-2} day^{-1})$ @endtex  
    !! ??Same variable is also used to 
    !! store heterotrophic respiration per ground area 
    !! over ::dt_sechiba?? 
    REAL(r_std),DIMENSION(nvm)                    :: ld_redistribute          !! logical set to redistribute som and litter
    REAL(r_std),DIMENSION(kjpindex,nvm)           :: resp_hetero_soil         !! soil heterotrophic respiration  
    !! @tex $(gC m^{-2} day^{-1})$ @endtex
    REAL(r_std),DIMENSION(kjpindex,nvm)           :: veget_cov                !! Fractional coverage: actually share of the pixel 
    !! covered by a PFT (fraction of ground area), 
    !! taking into account LAI ??(= grid scale fpc)?? 
    REAL(r_std),DIMENSION(kjpindex,nvm)           :: veget_cov_max_new        !! Old value for maximal fractional coverage (unitless)
    REAL(r_std),DIMENSION(kjpindex,nvm)           :: vcmax                    !! Maximum rate of carboxylation
    REAL(r_std),DIMENSION(kjpindex,nvm)           :: nue                      !! Nitrogen use Efficiency with impact of leaf age (umol CO2 (gN)-1 s-1)
    !! @tex $(\mumol m^{-2} s^{-1})$ @endtex
    REAL(r_std),DIMENSION(kjpindex,nlevs)         :: control_moist_inst       !! Moisture control of heterotrophic respiration 
    !! (0-1, unitless) 
    REAL(r_std),DIMENSION(kjpindex,nlevs)         :: control_temp_inst        !! Temperature control of heterotrophic 
    !! respiration, above and below (0-1, unitless) 
    REAL(r_std),DIMENSION(kjpindex,ncarb,nvm,nelements) :: som_input_inst     !! Quantity of carbon going into carbon pools from 
    !! litter decomposition 
    !! @tex $(gC m^{-2} day^{-1})$ @endtex
    !    REAL(r_std),DIMENSION(kjpindex,nvm,nnspec)    :: soil_n_min               !! Mineral nitrogen in the soil 
    !                                                                              !! @tex $(gN m^{-2})$ @endtex
    INTEGER(i_std)                                :: ier                      !! Check errors in netcdf call (unitless)
    REAL(r_std)                                   :: sf_time                  !! Intermediate variable to calculate current time 
    !! step 
    INTEGER(i_std)                                :: max_totsize              !! Memory management - maximum memory size (Mb)
    INTEGER(i_std)                                :: totsize_1step            !! Memory management - memory required to store one 
    !! time step on one processor (Mb) 
    INTEGER(i_std)                                :: totsize_tmp              !! Memory management - memory required to store one 
    !! time step on all processors(Mb) 
    REAL(r_std)                                   :: xn                       !! How many times have we treated in this forcing 
    REAL(r_std), DIMENSION(kjpindex)              :: vartmp                   !! Temporary variable
    INTEGER(i_std)                                :: vid                      !! Variable identifer of netCDF (unitless)
    INTEGER(i_std)                                :: nneigh                   !! Number of neighbouring pixels
    INTEGER(i_std)                                :: direct                   !! ??
    INTEGER(i_std),DIMENSION(ndm)                 :: d_id                     !! ??
!!$    REAL(r_std)                                   :: net_co2_flux_monthly     !! ??[DISPENSABLE]
!!$    REAL(r_std)                                   :: net_co2_flux_monthly_sum !! ??[DISPENSABLE]
!!$    REAL(r_std)                                   :: net_cflux_prod_monthly_sum    !! AR5 output?? gC m2 month-1 (one variable for 
!!$                                                                                   !! reduce_sum and one for bcast??), parallel 
!!$                                                                                   !! computing 
!!$    REAL(r_std)                                   :: net_cflux_prod_monthly_tot    !! AR5 output?? gC m2 month-1 (one variable for 
!!$                                                                                   !! reduce_sum and one for bcast??), parallel 
!!$                                                                                   !! computing 
!!$    REAL(r_std)                                   :: net_harvest_above_monthly_sum !! AR5 output?? gC m2 month-1 (one variable for 
!!$                                                                                   !! reduce_sum and one for bcast??), parallel 
!!$                                                                                   !! computing 
!!$    REAL(r_std)                                   :: net_harvest_above_monthly_tot !! AR5 output?? gC m2 month-1 (one variable for 
!!$                                                                                   !! reduce_sum and one for bcast??), parallel 
!!$                                                                                   !! computing 
!!$    REAL(r_std)                                   :: net_biosp_prod_monthly_sum    !! AR5 output?? gC m2 month-1 (one variable for 
!!$                                                                                   !! reduce_sum and one for bcast??), parallel 
!!$                                                                                   !! computing 
!!$    REAL(r_std)                                   :: net_biosp_prod_monthly_tot    !! AR5 output?? gC m2 month-1 (one variable for 
!!$                                                                                   !! reduce_sum and one for bcast??), parallel 
!!$                                                                                   !! computing 
    REAL(r_std), DIMENSION(kjpindex,nvm,nbpools)   :: carbon_stock                 !! Array containing the carbon stock for each pool
    !! used by ORCHIDEE
    REAL(r_std), DIMENSION(kjpindex,nvm,nionspec)  :: plant_n_uptake                 !! Uptake of soil N by plants  
    !! (gN/m**2/timestep)  
    REAL(r_std), DIMENSION(kjpindex,nvm)           :: n_mineralisation             !! net nitrogen mineralisation of decomposing SOM 
    !! (gN/m**2/day), supposed to be NH4
    REAL(r_std), DIMENSION(kjpindex,nvm)           :: n_fungivores                 !! Fraction of N released for plant uptake due to 
    !! fungivore consumption.
    REAL(r_std), DIMENSION(kjpindex,nvm)           :: resp_total_soil              !! soil heterotrophic respiration (gC/day/m**2 by PFT) 
    REAL(r_std), DIMENSION(kjpindex,nvm,ncarb)     :: CN_target                    !! C to N ratio of SOM flux from one pool to another (gN m-2 dt-1)
    REAL(r_std), DIMENSION(kjpindex,nvm)           :: tau_eff_root                 !! Effective root turnover time that accounts
    !! waterstress (days)
    REAL(r_std), DIMENSION(kjpindex,nvm)           :: tau_eff_sap                  !! Effective sapwood turnover time that accounts
    !! waterstress (days)
    REAL(r_std), DIMENSION(kjpindex,nvm)           :: tau_eff_leaf                 !! Effective leaf turnover time that accounts
    !! waterstress (days)
    REAL(r_std), DIMENSION(kjpindex,nvm)           :: wstress_adapt                !! Factor to account for a long acclimation of
    !! of the PFT to the long-term waterstress in
    !! the pixel
    REAL(r_std), DIMENSION(kjpindex,nvm,ninput)    :: input_dt_sechiba             !! Used to add pft index to N inputs
    REAL(r_std), DIMENSION(kjpindex,nvm,nionspec)  :: leaching                     !! mineral nitrogen leached from the soil
    REAL(r_std), DIMENSION(kjpindex,nvm,nnspec)    :: emission                     !! volatile losses of nitrogen (gN/m**2/timestep)
    REAL(r_std), DIMENSION(kjpindex,nvm,nmbcomp,nelements) &
         :: check_intern                 !! Contains the components of the internal
    !! mass balance chech for this routine
    !! @tex $(gC pixel^{-1} dt^{-1})$ @endtex
    REAL(r_std), DIMENSION(kjpindex,nvm,nelements) :: closure_intern               !! Check closure of internal mass balance
                                                                                   !! @tex $(gC pixel^{-1} dt^{-1})$ @endtex
    REAL(r_std), DIMENSION(kjpindex,nvm,nelements) :: pool_start                   !! Start and end pool of this routine 
                                                                                   !! @tex $(gC pixel^{-1} dt^{-1})$ @endtex
    REAL(r_std), DIMENSION(kjpindex,nvm,nelements) :: pool_end                     !! Start and end pool of this routine 
                                                                                   !! @tex $(gC pixel^{-1} dt^{-1})$ @endtex
    REAL(r_std), DIMENSION(kjpindex)               :: pixel_area                   !! surface area of the pixel @tex ($m^{2}$)
    REAL(r_std), DIMENSION(kjpindex,nvm)           :: veget_cov_max_begin              !! veget_max at the start of the routine.
                                                                                   !! Used for consistency checks
    REAL(r_std), DIMENSION(kjpindex,nelements)     :: mass_balance_closure         !! Missing mass in the carbon balance 
                                                                                   !! @tex $(gC(N) pixel^{-1} dt^{-1})$ @endtex
    REAL(r_std), DIMENSION(kjpindex,nvm)           :: nbp                          !! Pool-based Net Biome production for each 
                                                                                   !! time step
    INTEGER(i_std)                                 :: inspec, ininput, inionspec   !! Indices
    INTEGER(i_std)                                 :: ipts, ivm, ilitt, ilev, icir !! Indices
    INTEGER(i_std)                                 :: icarb, ipar, iele, imbc      !! Indices

    ! Light for recruitment
    REAL(r_std),DIMENSION (kjpindex,nvm,nlevels_tot) &
         :: light_tran_to_level  !! Fraction of light transmitted to a given canopy level
    REAL(r_std),DIMENSION(kjpindex,nvm)            :: count_daylight       !! Time steps dt_radia during daylight
    
    ! Harvest area and wind speed for Windthrow 
    REAL(r_std), DIMENSION(kjpindex,nvm,wind_years)    :: harvest_5y_area      !! total harvested area in the last 5 years
    REAL(r_std), DIMENSION(kjpindex)                   :: wind_speed_actual    !! Actualwind speed calculated from the actual half hourly
                                                                               !! values u and v as given in the driver (ms-1)
  
    !_ ================================================================================================================================

    !! 1. Initialize variables

    !! 1.1 Store current time step in a common variable
    itime = kjit

    !![DISPENSABLE] 1.2 Copy the depth of the different soil layers from diaglev specified in slow_proc

    !printlev=get_printlev('stomate')

    !! 1.3 PFT rooting depth across pixels, humescte is pre-defined 
    ! (constantes_veg.f90). It is defined as the coefficient of an exponential 
    ! function relating root density to depth 
    DO j=1,nvm
       rprof(:,j) = 1./humcste(j)
    ENDDO

    !! 1.4 Initialize first call
    ! Set growth respiration to zero
    resp_growth(:,:) = zero
    atm_to_bm(:,:,:) = zero
    plant_n_uptake(:,:,:) = zero
    n_mineralisation(:,:) = zero
    mass_balance_closure(:,:) = zero 


    ! Check that initialization is done
    IF (l_first_stomate) CALL ipslerr_p(3,'stomate_main','Initialization not yet done.','','')

     IF (printlev_loc >= 2) THEN
       IF(do_slow)THEN
          WRITE(numout,*) 'stomate_main: date=',date,' ymds=', year, month, day, sec, ' itime=', itime, ' do_slow=',do_slow
       ENDIF
     ENDIF

    !! 3. Special treatment for some input arrays.

    !! 3.1 Sum of liquid and solid precipitation
    precip(:) = ( precip_rain(:) + precip_snow(:) )*one_day/dt_sechiba

    !! 3.2 Calculate STOMATE's vegetation fractions from veget and veget_max
    DO j=1,nvm 
       WHERE ((1.-totfrac_nobio(:)) > min_sechiba)
          ! Pixels with vegetation
          veget_cov(:,j) = veget(:,j)/( 1.-totfrac_nobio(:) )
          veget_cov_max(:,j) = veget_max(:,j)/( 1.-totfrac_nobio(:) )
       ELSEWHERE
          ! Pixels without vegetation
          veget_cov(:,j) = zero
          veget_cov_max(:,j) = zero
       ENDWHERE
    ENDDO

    IF ( do_now_stomate_lcchange ) THEN
       DO j=1,nvm
          WHERE ((1.-totfrac_nobio_new(:)) > min_sechiba)
             ! Pixels with vegetation
             veget_cov_max_new(:,j) = veget_max_new(:,j)/( 1.-totfrac_nobio_new(:) )
          ELSEWHERE
             ! Pixels without vegetation
             veget_cov_max_new(:,j) = zero
          ENDWHERE
       ENDDO
    ENDIF

    !! 3.3 Adjust time step of GPP 
    ! No GPP for bare soil
    gpp_d(:,1) = zero
    ! GPP per PFT
    DO j = 2,nvm   
       WHERE (veget_cov_max(:,j) > min_stomate)
          ! The PFT is available on the pixel
          gpp_d(:,j) =  gpp(:,j)/ veget_cov_max(:,j)* one_day/dt_sechiba  
       ELSEWHERE
          ! The PFT is absent on the pixel
          gpp_d(:,j) = zero
       ENDWHERE
    ENDDO

    !! 3.4 The first time step of the first day of the month  
    ! implies that the month is over
    IF ( day == 1 .AND. sec .LT. dt_sechiba ) THEN
       EndOfMonth=.TRUE.
    ELSE
       EndOfMonth=.FALSE.
    ENDIF


    !! 4. Calculate variables for dt_stomate (i.e. "daily")

    ! Note: If dt_days /= 1, then variables 'xx_daily' (eg. half-daily or bi-daily) are by definition
    ! not expressed on a daily basis. This is not a problem but could be
    ! confusing

    !! 4.1. Calculate water stress accounting for hydraulic architecture
    !  If hydraulic architecture is used, humrel_daily (water stress) for use in stomate is 
    !  not determined from humrel as calculated in hydrol.f90, but it is calculated as the 
    !  ratio between a proxy for stressed and unstressed ecosystem functioning. 
    !  Nightvalues are exclude. On the first day of the simulation (no biomass)
    !  the stressed and unstressed proxies are probably both equal to the initilazed value (=zero, 
    !  initialized in sechiba.f90), to avoid numerical issues in this case we set humrel_daily 
    !  to 1. If hydraulic architecture is not used humrel_daily is calculated from humrel as 
    !  determined in hydrol. 
    IF (ok_hydrol_arch) THEN
       ! Accumulate the half hourly values into a daily value. Accumulate the
       ! stressed and unstressed proxy first and then take the average. We 
       ! first accumulate and then take the ratio because that way we better 
       ! account for the night time values. If we do it in the other order 
       ! we need to assign a value to the ratio during the night. Whether we
       ! take zero or one, this will bias our waterstress number because 
       ! the number of half hours during the night is different for the 
       ! different pixels. So although water stress could be higher in the 
       ! south than in the north. During the growing season, day are shorter
       ! in the south so if we set the ratio to 1 during the night, our daily 
       ! water stress in the south may be less than in the north because we
       ! more 1's in the daily time series.

       ! The order of the calculation may depend on the proxy used. For example, 
       ! Accumulate transpir_supply and transpir first and then calculate ratio
       ! By doing this water stress is buffered (it is assumed that if 
       ! ::transpir_supply is larger than ::transpir at one timestep it can buffer 
       ! potential water stress in the next timestep. In reality this is not the 
       ! case: transpir_supply is a potential value not a realized one.
       DO ipts = 1,kjpindex

          ! The cosine of the zenith angle, is used to 
          ! identify night values.
          IF (coszang(ipts) .LT. min_stomate) THEN

             ! Redundant because the value will never be used
             ! For all pixels
             stressed(ipts,:) = zero
             unstressed(ipts,:) = zero

             ! For PFT1 under LCC
             ! vir_stressed = zero
             ! vir_unstressed = zero

          ELSE

             ! No vegetation present so ecosystem functioning
             ! was not defined
             WHERE (veget_cov_max(ipts,:) .LT. min_stomate)

                ! To avoid uninitialized values
                stressed(ipts,:) = zero
                unstressed(ipts,:) = zero

                ! Update the values to avoid uninitialized fields
                ! in ::stressed_daily and ::unstressed_daily
                stressed_daily(ipts,:) = stressed_daily(ipts,:) + &
                     stressed(ipts,:)
                unstressed_daily(ipts,:) = unstressed_daily(ipts,:) + &
                     unstressed(ipts,:)
                daylight(ipts,:) = daylight(ipts,:) + 1

                ! For PFT1 under LCC
                ! vir_stressed = zero
                ! vir_unstressed = zero

             ELSEWHERE

                ! The pixel and pft contain vegetation and their
                ! is daylight, store values to calculate the stress
                stressed_daily(ipts,:) = stressed_daily(ipts,:) + &
                     stressed(ipts,:)
                unstressed_daily(ipts,:) = unstressed_daily(ipts,:) + &
                     unstressed(ipts,:)
                daylight(ipts,:) = daylight(ipts,:) + 1


                ! For PFT1 under LCC
                ! vir_stressed = zero
                ! vir_unstressed = zero

             ENDWHERE

          ENDIF

       ENDDO


       ! Calculate the mean waterstress value at the end of each day
       IF (do_slow) THEN

          ! Calculate the ratio
          WHERE (unstressed_daily(:,:) .LT. min_stomate &
               .OR. stressed_daily(:,:) .LT. min_stomate)

             ! No ecosystem function thus no data, we assume
             ! there is no water stress
             humrel_daily(:,:) = un

          ELSEWHERE

             ! Calculate water stress. If we first calculate
             ! the daily sum and then the ratio there is no need
             ! to divide by daylight. If we accumulate ratios
             ! we will have to divide by daylight. This is
             ! arbitrary. To do this properly we should calculate
             ! the turgor in the cells and calculate growth
             ! based on that see i.e. Fatichi et al 2013, New
             ! Phytologist. We use a simple numerical construct
             ! (the sqrt of the ratio) to overcome that complexity.
             humrel_daily(:,:) = stressed_daily(:,:) / &
                  unstressed_daily(:,:)

          ENDWHERE

          !+++CHECK+++
          ! This needs to be done outside the where loop so that
          ! all PFTs have vir_humrel_daily set.
          ! vir_humrel_daily will be needed when accounting for the 
          ! water columns following a land cover change. It is partly 
          ! calculated in sechiba hydrol_arch but it is not passed all
          ! the way to here. Check and fix. For the moment LCC does
          ! not account for the water columns so ::humrel_daily and
          ! ::vir_humrel_daily are equal to each other
          vir_humrel_daily(:,:) = humrel_daily(:,:)
          ! We have a problem.  wstress is no longer calculated in a way
          ! that allows for a virtual quantity, since it's the ratio of the
          ! stressed and unstressed GPPs...which cannot be calculated if there
          ! is no vegetation.  Therefore, for any PFTs which are not
          ! currently present we give the daily virtual waterstress to be
          ! equal to one.
          !+++++++++++
          !---TEMP---
          IF(printlev_loc>=4)THEN 
             DO ipts=1,kjpindex
                DO ivm=1,nvm
                   IF( humrel_daily(ipts,ivm) .lt. 1.) THEN
                      WRITE(numout,*)'ivm,stressed_daily,unstressed_daily,diff,humrel_daily'

                      WRITE(numout,*) ivm, stressed_daily(ipts,ivm),unstressed_daily(ipts,ivm),&
                           & stressed_daily(ipts,ivm)-unstressed_daily(ipts,ivm), humrel_daily(ipts,ivm) 
                   ENDIF
                ENDDO
             ENDDO
          ENDIF
          !----------
          ! Set to zero to start accumulating for the next day
          stressed_daily(:,:) = zero
          unstressed_daily(:,:) = zero
          daylight(:,:) = zero

       ELSE

          ! Set to a large value so that it is easy to detect problems
          ! This value should never be used. It should only be passed
          ! to other routine at the end of the day when do_slow is false
          humrel_daily(:,:) = large_value
          vir_humrel_daily(:,:) = large_value

       ENDIF !(do_slow)

    ELSE
       ! No hydrological architecture
       CALL stomate_accu (do_slow, humrel, humrel_daily)

       ! The variable ::vir_humrel_arch was not defined for this case
       ! Assume no stress
       vir_humrel_daily = un

    ENDIF ! (ok_hydrol_arch)


    ! Here we add some calculations for daily max/min for windthrow module 
    IF (ok_windthrow) THEN

       ! Accumulate the half hourly values into a daily value. Accumulate the
       ! daily wind speed first and then take the average. grnd_80 is the index 
       ! of ngrnd closest to 80 cm depth.
       wind_speed_actual(:) = SQRT (u(:) * u(:) + v(:) * v(:))
       wind_max_daily(:) = MAX (wind_speed_actual(:),wind_max_daily(:) )
       soil_max_daily(:) = MAX (stempdiag(:,grnd_80),soil_max_daily(:) )

       IF (do_slow) THEN

          ! wind_speed_daily/soil_temp_daily will be passed to the wind_damage
          ! module. wind_max_daily/soil_max_daily reset to zero so they can be 
          ! used again for the next day
          wind_speed_daily(:) = wind_max_daily(:)
          wind_max_daily(:) = zero
          soil_temp_daily(:) = soil_max_daily(:)
          soil_max_daily(:) = zero

       ENDIF

    ENDIF


    ! Debug
    IF (printlev_loc>=4) THEN
       IF(do_slow) THEN
          WRITE(numout,*) 'CHECK: humrel_daily after stomate_accu',&
               humrel_daily(:,:)
          WRITE(numout,*) 'CHECK: vir_humrel_daily after stomate_accu',&
               vir_humrel_daily(:,:)
       ENDIF
    ENDIF
    !-

    !! Calculate the light that reaches each canopy layer
    !  Compute seasonal daytime transmitted light to canopy levels 
    !  This quantity is used to calculate how much recruitment can occur
    !  underneath the canopy. Recruitment is simulated in stomate_prescribe.f90
    light_tran_to_level(:,:,:) = un

    DO ipts=1,kjpindex

       DO ivm=1,nvm

          ! If we are at a daytime time step and there is growth (gpp>0) then
          ! accumulate instantaneous transmitted light to each canopy level
          IF ( coszang(ipts) .GT. min_stomate .AND. &
               gpp(ipts,ivm) .GT. min_stomate ) THEN

             daylight_count(ipts,ivm) = daylight_count(ipts,ivm) + 1

             DO ilev = nlevels_tot-1,1,-1

                light_tran_to_level(ipts,ivm,ilev) = &
                     light_tran_to_level(ipts,ivm,ilev+1) * &
                     Isotrop_Tran_Tot_p(ipts,ivm,ilev+1)

             ENDDO

             ! Accumulate over the growing season
             light_tran_to_level_season(ipts,ivm,:) = &
                  light_tran_to_level_season(ipts,ivm,:) + &
                  light_tran_to_level(ipts,ivm,:)

             ! Debug
             IF (printlev_loc.GT.4)THEN
                WRITE(numout,*) 'It is daytime and growth occurs, '&
                     &'daylight_count(ipts,ivm)= ', daylight_count(ipts,ivm)
                WRITE(numout,*) 'gpp(ipts,ivm)= ', gpp(ipts,ivm)
                WRITE(numout,*) 'fine resolution Absolute transmitted light '&
                     &'to each level (ivm,ipts,nlevels_tot) ',&
                     ivm,ipts,nlevels_tot
                DO ilev = nlevels_tot,1,-1
                   WRITE(numout,*) ilev,light_tran_to_level(ipts,ivm,ilev),&
                        Isotrop_Tran_Tot_p(ipts,ivm,ilev)
                ENDDO
             ENDIF
             !-

          ENDIF   ! daytime and growing season

       ENDDO

       ! Calculate average transmitted light at the end of the year
       IF (EndOfYear) THEN

          ! Calculate average transmitted light at the end of the year
          ! Only account for days during the growing season
          DO ivm=2,nvm

             IF (daylight_count(ipts,ivm) .GT. zero) THEN

                DO ilev=1,nlevels_tot

                   light_tran_to_level_season(ipts,ivm,ilev) = &
                        light_tran_to_level_season(ipts,ivm,ilev) / &
                        daylight_count(ipts,ivm)

                ENDDO

             ELSE

                ! There was no GPP during this year in this PFT. Most
                ! likely this implies that there is no vegetation. So,
                ! all the light is transmitted to the ground level.
                ! The fraction of transmitted light is thus set to 1.
                light_tran_to_level_season(ipts,ivm,:) = un

             ENDIF

             ! Debug
             IF(printlev_loc>=4)THEN
                WRITE(numout,*) 'DEBUG in stomate.f90 it is end of the ', &
                     'year in stomate.f90'
                WRITE(numout,*) 'daylight_count to divide by here is, ', &
                     daylight_count(ipts,:)
                WRITE(numout,*) 'transmitted light, ',&
                     light_tran_to_level_season(ipts,ivm,1),&
                     light_tran_to_level_season(ipts,ivm,10)
             ENDIF
             !- 
          ENDDO
          ! Reset the counter for the next year
          daylight_count(:,:)=zero

       ENDIF ! EndOfYear

    ENDDO ! ipts=1,kjpindex

    ! Debug
    IF(printlev_loc>=5)THEN
       IF (EndOfYear) THEN
          DO ipts=1,kjpindex              
             DO ivm=1,nvm
                WRITE(numout,*) 'Average absolute transmitted light &
                     & during the growing season to each canopy &
                     & level in stomate.f90 ', &
                     ivm,ipts,nlevels_tot
                DO ilev = nlevels_tot,1,-1
                   WRITE(numout,*) ilev,light_tran_to_level_season(ipts,ivm,ilev)
                ENDDO
             ENDDO
          ENDDO
       ENDIF
    ENDIF
    !-
    ! Write trans_to_light to output file. It is calculated at each subdaily
    ! timestep
    CALL xios_orchidee_send_field("LIGHT_TRAN_TO_LEVEL",light_tran_to_level(:,:,:))

    !! 4.1 Accumulate instantaneous variables (do_slow=.FALSE.) 
    ! Accumulate instantaneous variables (do_slow=.FALSE.) and eventually 
    ! calculate daily mean value (do_slow=.TRUE.)

    CALL stomate_accu (do_slow, litterhumdiag, litterhum_daily)
    CALL stomate_accu (do_slow, t2m,           t2m_daily)
    CALL stomate_accu (do_slow, temp_sol,      tsurf_daily)
    CALL stomate_accu (do_slow, stempdiag,     tsoil_daily)
    CALL stomate_accu (do_slow, shumdiag,      soilhum_daily)
    CALL stomate_accu (do_slow, precip,        precip_daily)
    CALL stomate_accu (do_slow, gpp_d,         gpp_daily)
    CALL stomate_accu (do_slow, drainage_pft, drainage_daily) 

    !! 4.2 Daily minimum temperature
    t2m_min_daily(:) = MIN( t2m(:), t2m_min_daily(:) )

    !! 4.3 Calculate maintenance respiration
    ! Note: lai is passed as output argument to overcome previous problems with 
    ! natural and agricultural vegetation types. 
    CALL maint_respiration (kjpindex, dt_sechiba, t2m, t2m_longterm, &
         stempdiag, gpp, gpp_week, lab_fac, &
         veget_cov_max, rprof, resp_maint_part_radia, &
         circ_class_n, circ_class_biomass)

    ! Aggregate maintenance respiration across the different plant parts 
    resp_maint_radia(:,:) = zero
    DO j=2,nvm
       DO k= 1, nparts
          resp_maint_radia(:,j) = resp_maint_radia(:,j) &
               & + resp_maint_part_radia(:,j,k)
       ENDDO
    ENDDO

    ! Maintenance respiration separated by plant parts
    resp_maint_part(:,:,:) = resp_maint_part(:,:,:) &
         & + resp_maint_part_radia(:,:,:)

    !! 4.4 Initialize check for mass balance closure
    !  Mass balance closure for the half-hourly (dt_sechiba)
    !  processes in stomate.f90. This test is always performed.
    !  If err_act.EQ.1 then the value of the mass balance error
    !  -if any- is written to the history file. 
!!$    printlev_loc=get_printlev('stomate')
    check_intern(:,:,:,:) = zero
    pool_start(:,:,:) = zero
    DO iele = 1,nelements

       ! atm_to_bm has as intent inout, the variable 
       ! accumulates carbon over the course of a day. 
       ! Use the difference between the start and end of 
       ! this routine to account for the change in 
       ! atm_to_bm
       check_intern(:,:,iatm2land,iele) = - un * &
            atm_to_bm(:,:,iele) * veget_cov_max(:,:) * dt_sechiba

       ! Biomass pool (gC m-2)*(m2 m-2) 
       DO ipar = 1,nparts
          pool_start(:,:,iele) = pool_start(:,:,iele) + &
               (turnover_daily(:,:,ipar,iele) + &
               bm_to_litter(:,:,ipar,iele)) * veget_cov_max(:,:) / &
               one_day * dt_sechiba
       ENDDO

       ! Litter pool (gC m-2)*(m2 m-2) 
       DO ilitt = 1,nlitt
          DO ilev = 1,nlevs
             pool_start(:,:,iele) = pool_start(:,:,iele) + &
                  litter(:,ilitt,:,ilev,iele) * veget_cov_max(:,:)
          ENDDO
       ENDDO

       ! Soil carbon (gC m-2)*(m2 m-2)
       DO icarb = 1,ncarb
          pool_start(:,:,iele) = pool_start(:,:,iele) + &
               som(:,icarb,:,iele) * veget_cov_max(:,:)
       ENDDO

    ENDDO ! # nelements

    ! Account for the N-pool in the soil 
    DO inspec = 1,nnspec
       pool_start(:,:,initrogen) = pool_start(:,:,initrogen) + &
            soil_n_min(:,:,inspec) * veget_cov_max(:,:)
    ENDDO

    ! Initialize check for area conservation
    veget_cov_max_begin(:,:) = veget_cov_max(:,:)
   
    !! 4.5 Litter dynamics and litter heterothropic respiration 
    !  Including: litter update, lignin content, PFT parts, litter decay,
    !  litter heterotrophic respiration, dead leaf soil cover.
    !  Note: there is no vertical discretisation in the soil for litter decay.
    n_mineralisation(:,:) = zero
    turnover_littercalc(:,:,:,:) = turnover_daily(:,:,:,:) * dt_sechiba/one_day
    bm_to_littercalc(:,:,:,:) = bm_to_litter(:,:,:,:) * dt_sechiba/one_day

    CALL littercalc (kjpindex, &
         turnover_littercalc, bm_to_littercalc, &
         veget_cov_max, temp_sol, stempdiag, shumdiag, litterhumdiag, som, &
         clay, silt, soil_n_min, litter, dead_leaves, lignin_struc, &
         lignin_wood, n_mineralisation, deadleaf_cover, resp_hetero_litter, &
         som_input_inst, control_temp_inst, control_moist_inst, n_fungivores, &
         matrixA, vectorB, CN_target, CN_som_litter_longterm, tau_CN_longterm, &
         ld_redistribute, circ_class_biomass, circ_class_n)

    !--- TEMP---!
    IF(printlev_loc>=4) THEN
       WRITE(numout,*) 'between litter and soil ' 
       WRITE(numout,*) 'n_mineralisation ', n_mineralisation(:,:)
       WRITE(numout,*) 'drainage_pft ', drainage_pft(:,:)
       WRITE(numout,*) 'control_temp_inst, ',control_temp_inst(:,:)
       WRITE(numout,*) 'control_moist_inst, ',control_moist_inst(:,:)
    ENDIF

    !! 4.6 Soil carbon dynamics and soil heterotrophic respiration
    ! Note: there is no vertical discretisation in the soil for litter decay.
    CALL som_dynamics (kjpindex, clay, silt, veget_cov_max, &
         som_input_inst, control_temp_inst, control_moist_inst, drainage_pft,&
         CN_target, som, soil_n_min, resp_hetero_soil, matrixA, &
         n_mineralisation, CN_som_litter_longterm, tau_CN_longterm)

    ! Heterothropic soil respiration during time step ::dt_sechiba
    ! @tex $(gC m^{-2})$ @endtex
    IF(printlev_loc >= 4) THEN
       WRITE(numout,*)'betweem som_dynamics and nitrogen_dynamics'
       WRITE(numout,*)'n_mineralisation ', n_mineralisation(:,:)       
       WRITE(numout,*)'resp_hetero_soil',resp_hetero_soil(:,:)
       WRITE(numout,*)'dt_sechiba',dt_sechiba 
       WRITE(numout,*)'one_day',one_day 
    ENDIF

    ! Heterothropic litter respiration during time step ::dt_sechiba 
    ! @tex $(gC m^{-2})$ @endtex
    resp_hetero_litter(:,:) = resp_hetero_litter(:,:) * dt_sechiba/one_day

    !! resp_hetero_soil(:,1)=zero
    resp_hetero_soil(:,:) = resp_hetero_soil(:,:) * dt_sechiba/one_day

    ! Total heterothrophic respiration during time step ::dt_sechiba 
    ! @tex $(gC m^{-2})$ @endtex
    resp_hetero_radia(:,:) = resp_hetero_litter(:,:) + resp_hetero_soil(:,:)
    resp_hetero_d(:,:) = resp_hetero_d(:,:) + resp_hetero_radia(:,:)

    ! Sum heterotrophic and autotrophic respiration in soil 
    resp_total_soil(:,:) = resp_hetero_radia(:,:) + & 
         resp_maint_part_radia(:,:,isapbelow) + resp_maint_part_radia(:,:,iroot)

    IF(ok_ncycle) THEN

       CALL nitrogen_dynamics(kjpindex, clay, MAX(zero, un - silt - clay), & 
            stempdiag-tp_00, tmc_pft, drainage_pft, swc_pft, veget_cov_max, &
            resp_total_soil, som, &  
            n_input, soil_ph, n_mineralisation, pb, n_fungivores, &  
            plant_n_uptake, bulk, soil_n_min, p_O2, bact, atm_to_bm, &
            leaching, emission, input_dt_sechiba, ld_redistribute, circ_class_biomass, &
            circ_class_n)  

    ENDIF

    ! Accumulate over the day
    plant_n_uptake_daily(:,:,:) = plant_n_uptake_daily(:,:,:) + plant_n_uptake(:,:,:)
    n_mineralisation_d(:,:) = n_mineralisation_d(:,:) + n_mineralisation(:,:)  

    !! 4.7 Accumulate instantaneous variables (do_slow=.FALSE.) 
    ! Accumulate instantaneous variables (do_slow=.FALSE.) and eventually 
    ! calculate daily mean value (do_slow=.TRUE.) 
    CALL stomate_accu (do_slow, control_moist_inst, control_moist_daily)
    CALL stomate_accu (do_slow, control_temp_inst, control_temp_daily)
    CALL stomate_accu (do_slow, som_input_inst, som_input_daily)

    !! 4.8 Check numerical consistency of this routine
    !  These checks only check the processes that happen
    !  every half-hour (dt_radia). This test is always 
    !  performed. If err_act.EQ.1 then the value of the 
    !  mass balance error -if any- is written to the 
    !  history file.
!!$    printlev_loc=get_printlev('stomate')

    !  Check surface area
    CALL check_surface_area("stomate dt_sechiba", kjpindex, veget_cov_max_begin, &
         veget_cov_max,'pixel')

    ! 4.8.2 Mass balance closure (dt_radia)
    ! Calculate final carbon and nitrogen pools
    pool_end(:,:,:) = zero
    DO iele = 1,nelements

       ! Litter pool
       DO ilitt = 1,nlitt
          DO ilev = 1,nlevs
             pool_end(:,:,iele) = pool_end(:,:,iele) + &
                  litter(:,ilitt,:,ilev,iele) * veget_cov_max(:,:)
          ENDDO
       ENDDO

       ! Soil carbon and nitrogen
       DO icarb = 1,ncarb
          pool_end(:,:,iele) = pool_end(:,:,iele) + &
               som(:,icarb,:,iele) * veget_cov_max(:,:)
       ENDDO

    ENDDO ! # nelements

    ! The nitrogen pool in the soil may have changed
    DO inspec = 1,nnspec
       pool_end(:,:,initrogen) = pool_end(:,:,initrogen) + &
            soil_n_min(:,:,inspec) * veget_cov_max(:,:)
    ENDDO

    DO ivm = 1,nvm
       pool_end(:,ivm,initrogen) = pool_end(:,ivm,initrogen) + &
            n_fungivores(:,ivm) * veget_cov_max(:,ivm)
    ENDDO

    ! Calculate mass balance
    ! Specific processes
    check_intern(:,:,iland2atm,icarbon) = -un * (resp_hetero_litter(:,:) + &
         resp_hetero_soil(:,:)) * veget_cov_max(:,:)

    DO ininput = 1,ninput
       check_intern(:,:,iatm2land,initrogen) = &
            check_intern(:,:,iatm2land,initrogen) + &
            input_dt_sechiba(:,:,ininput) * veget_cov_max(:,:)
    ENDDO

    DO inspec= 1, nnspec
       check_intern(:,:,iland2atm,initrogen) = &
            check_intern(:,:,iland2atm,initrogen) &
            -un * (emission(:,:,inspec) * veget_cov_max(:,:))
    ENDDO

    DO inionspec = 1, nionspec
       check_intern(:,:,ilat2out,initrogen) = &
            check_intern(:,:,ilat2out,initrogen) &
            -un * ( plant_n_uptake(:,:,inionspec) + &
            leaching(:,:,inionspec) ) * veget_cov_max(:,:)
    ENDDO

    ! Common processes
    DO iele = 1,nelements
       check_intern(:,:,iatm2land,iele) = check_intern(:,:,iatm2land,iele) + &
            atm_to_bm(:,:,iele) * dt_sechiba * veget_cov_max(:,:)
       check_intern(:,:,ipoolchange,iele) = &
            -un * (pool_end(:,:,iele) - pool_start(:,:,iele))
    ENDDO

    closure_intern(:,:,:) = zero
    DO imbc = 1,nmbcomp
       DO iele = 1,nelements
          ! Debug
          IF (printlev_loc>=4) WRITE(numout,*) &
               'check_intern, ivm, imbc, iele, ', imbc, &
               iele, SUM(check_intern(:,:,imbc,iele),2)
          !-
          closure_intern(:,:,iele) = closure_intern(:,:,iele) + &
               check_intern(:,:,imbc,iele)
       ENDDO
    ENDDO

    CALL check_mass_balance("stomate dt_sechiba", closure_intern, kjpindex, &
         pool_end, pool_start, veget_cov_max, 'pft')

    ! Accumulate the mass balance error over the course of the day
    ! If all is well the mass balance error is zero (.LT. min_stomate)
    DO iele = 1,nelements
       mass_balance_closure(:,iele) = mass_balance_closure(:,iele) + &
            SUM((closure_intern(:,:,iele) * veget_cov_max(:,:)),2)
    ENDDO

    !! 5. Daily processes - performed at the end of the day

    IF (do_slow) THEN

       !+++CHECK+++
       ! No longer needed. lai is no longer passed
       ! circ_class_biomass and circ_class_n are now the
       ! prognostic variables.
!!$       !! 5.1 Update lai
!!$       ! Use lai from stomate
!!$       ! ?? check if this is the only time ok_pheno is used??
!!$       ! ?? Looks like it is the only time. But this variables probably is defined 
!!$       ! in stomate_constants or something, in which case, it is difficult to track.
!!$       IF (ok_pheno) THEN
!!$          !! 5.1.1 Update LAI 
!!$          ! Set lai of bare soil to zero
!!$          lai(:,ibare_sechiba) = zero
!!$          ! lai for all PFTs
!!$          DO ipts = 1, kjpindex
!!$             DO j = 2, nvm
!!$                lai(ipts,j) = cc_to_lai(circ_class_biomass(ipts,j,:,ileaf,icarbon),&
!!$                     circ_class_n(ipts,j,:),j)
!!$             ENDDO
!!$          ENDDO
!!$          frac_age(:,:,:) = leaf_frac(:,:,:)
!!$       ELSE 
!!$          ! 5.1.2 Use a prescribed lai
!!$          ! WARNING: code in setlai is effectively the same as the lines above
!!$          ! Update subroutine if LAI should be prescribed. This is a bit an optimistic
!!$          ! function. It is rather difficult to force an lai with a dynamic allocation 
!!$          ! and a dynamic nitrogen cycle. This will quickly result in inconsistencies. 
!!$          CALL  setlai(kjpindex, lai, circ_class_biomass,circ_class_n) 
!!$          frac_age(:,:,:) = zero
!!$       ENDIF
       !++++++++++++

       !! 5.2 Calculate long-term "meteorological" and biological parameters
       ! mainly in support of calculating phenology. If ::EndOfYear=.TRUE.
       ! annual values are update (i.e. xx_lastyear).
       CALL season &
            &          (kjpindex, dt_days, EndOfYear, &
            &           veget_cov, veget_cov_max, &
            &           humrel_daily, t2m_daily, tsoil_daily, soilhum_daily, lalo, &
            &           precip_daily, npp_daily, circ_class_biomass, circ_class_n, &
            &           turnover_daily, gpp_daily, when_growthinit, &
            &           maxhumrel_lastyear, maxhumrel_thisyear, &
            &           minhumrel_lastyear, minhumrel_thisyear, &
            &           maxgppweek_lastyear, maxgppweek_thisyear, &
            &           gdd0_lastyear, gdd0_thisyear, &
            &           precip_lastyear, precip_thisyear, &
            &           lm_lastyearmax, lm_thisyearmax, &
            &           maxfpc_lastyear, maxfpc_thisyear, &
            &           humrel_month, humrel_week, t2m_longterm, tau_longterm, &
            &           t2m_month, t2m_week, tsoil_month, soilhum_month, &
            &           npp_longterm, turnover_longterm, gpp_week, plant_status, &
            &           gdd_m5_dormance, gdd_midwinter, ncd_dormance, ngd_minus5, &
            &           time_hum_min, hum_min_dormance, gdd_init_date, &
            &           gdd_from_growthinit, herbivores, &
            &           Tseason, Tseason_length, Tseason_tmp, &
            &           Tmin_spring_time, t2m_min_daily, onset_date, &
            &           cn_leaf_min_season,nstress_season, &
            &           humrel_growingseason,rue_longterm, harvest_area, &
            &           harvest_5y_area)

       !! 5.4 Waterstress

       !  The waterstress factor varies between 0.1 and 1 and is calculated
       !  from ::humrel_growingseason. The latter is only used in the allometric 
       !  allocation and its time integral is determined by tau_sap for trees
       !  (see constantes_mtc.f90 for tau_sap and see pft_constantes.f90 for 
       !  the definition of tau_hum_growingseason). The time integral for 
       !  grasses and crops is a prescribed constant (see constantes.f90). By
       !  having ::wstress_fac working on the turnover, water stress is progated
       !  into LF.
       !  Because the calculated values for ::wstress_fac are too low for this purpose
       !  Sonke Zhaele multiply it by two in the N-branch. This approach maintains
       !  the physiological basis of KF while combining it with a simple 
       !  multiplicative factor for water stress. Clearly after multiplication with 
       !  2, wstress is closer to 1 and will thus result in a KF values closer to 
       !  the physiologically expected KF. 
       !  In this implementation we take the sqrt (this is done in stomate where 
       !  ::humrel_daily is calculated from ::stressed and ::unstressed. The
       !  transformation from the ratio between stressed an unstressed gpp into a 
       !  numerical value that is used in the allocation and turnover is arbitrairy.
       !  A more physiological approach accounting for turgor would be needed to de
       !  fundamentally better.
       !  Note that the current implementation allows for the plants to adapt to drought 
       !  by adjusting its allocation. This is a long-term effect and it is long-term
       !  because ::humrel_growingseason integrates over ::tau_sap. For the moment no
       !  short term effects to drought are implemented. Short-term effects should be 
       !  implmented on mortality (through loss of turgor, heat stress, carbon starvation).

       wstress_season(:,1) = zero
       wstress_month(:,1) = zero
!!$       vir_wstress_fac(:,1) = zero

       DO jv = 2,nvm

          ! Calculate waterstress
          ! Water stress used in stomate
          ! Set wstress to 1-humrel so that the value is consistent with its
          ! meaning. wstress=0 indicates no stress, wstress=1 indicates stress 
          wstress_season(:,jv) = un - MAX(humrel_growingseason(:,jv), min_water_stress)
          wstress_month(:,jv) = un - MAX(humrel_month(:,jv), min_water_stress)
!!$          vir_wstress_fac(:,jv) = MAX(vir_humrel_growingseason(:,jv), min_water_stress)

          !+++CHECK+++
          ! The reduction of the leaf longevity should probably depend on
          ! the leaf skin temperature that will become available through
          ! the multi-layer energy budget. For the moment we don't have
          ! water stress on the leaves. A long term adaption could be 
          ! through ::sla
          tau_eff_leaf(:,jv) = tau_leaf(jv) * un

          ! The reduction of the root longevity depends on the soil moisture
          ! stress which we believe is reasonably well captured by our 
          ! proxy for wstress. We need to produce more roots to take the water
          ! from those layers that have water.
          ! feedback to c-allocation has been switched off
          tau_eff_root(:,jv) = tau_root(jv) *  un

          ! The reduction of sapwood longevity should depend on the cavitation
          ! which is calculated in hydrol_architecture. Should be linked
          ! once the memory of cavitation is calculated
          tau_eff_sap(:,jv) = tau_sap(jv) *  un
          !+++++++++

       ENDDO

       ! Add to history file
       CALL histwrite_p (hist_id_stomate, 'WSTRESS_SEASON', itime, &
            wstress_season(:,:), kjpindex*nvm, horipft_index)
       CALL histwrite_p (hist_id_stomate, 'WSTRESS_MONTH', itime, &
            wstress_month(:,:), kjpindex*nvm, horipft_index)
       CALL xios_orchidee_send_field("WSTRESS_SEASON",wstress_season(:,:))
       CALL xios_orchidee_send_field("WSTRESS_MONTH",wstress_month(:,:))

       !! 5.3.1  Activate stomate processes 
       ! Activate stomate processes (the complete list of processes depends 
       ! on whether the DGVM is used or not). Processes include: climate constraints 
       ! for PFTs, PFT dynamics, Phenology, Allocation, NPP (based on GPP and
       ! authothropic respiration), fire, mortality, vmax, assimilation temperatures,
       ! all turnover processes, light competition, sapling establishment, lai and 
       ! land cover change.
       CALL stomate_lpj_vegetation (kjpindex, dt_days, EndOfYear, &
            &             neighbours, resolution, &
            &             herbivores, &
            &             tsurf_daily, tsoil_daily, t2m_daily, t2m_min_daily, &
            &             litterhum_daily, soilhum_daily, &
            &             maxhumrel_lastyear, minhumrel_lastyear, &
            &             gdd0_lastyear, precip_lastyear, &
            &             humrel_month, humrel_week, &
            &             t2m_longterm, t2m_month, t2m_week, &
            &             tsoil_month, soilhum_month, &
            &             gdd_m5_dormance,  gdd_from_growthinit, gdd_midwinter, ncd_dormance, ngd_minus5, &
            &             turnover_longterm, gpp_daily, &
            &             time_hum_min, hum_min_dormance, maxfpc_lastyear, resp_maint_part,&
            &             PFTpresent, age, fireindex, firelitter, &
            &             leaf_age, leaf_frac, adapted, regenerate, &
            &             plant_status, when_growthinit, litter, &
            &             dead_leaves, som, lignin_struc, lignin_wood, &
            &             veget_cov_max, veget_cov, veget_cov_max_new, npp_longterm, lm_lastyearmax, &
            &             veget_lastlight, everywhere, need_adjacent, RIP_time, &
            &             rprof, npp_daily, turnover_daily, turnover_time,&
            &             control_moist_inst, control_temp_inst, som_input_daily, &
            &             atm_to_bm, co2_fire, &
            &             resp_hetero_d, resp_maint_d, resp_growth_d, &
            &             height, deadleaf_cover, vcmax, nue, &
            &             bm_to_litter,&
            &             prod_s, prod_m, prod_l, flux_s, flux_m, flux_l, &
            &             flux_prod_s, flux_prod_m, flux_prod_l, carb_mass_total, &
            &             fpc_max, MatrixA, MatrixV, VectorB, VectorU, &
            &             Tseason, Tmin_spring_time, onset_date, KF, k_latosa_adapt,&
            &             cn_leaf_min_season, nstress_season, humrel_growingseason, soil_n_min, &
            &             rue_longterm, plant_n_uptake_daily, &
            &             circ_class_n, circ_class_biomass, forest_managed, forest_managed_lastyear, &
            &             tau_eff_leaf, tau_eff_sap, tau_eff_root, &
            &             species_change_map, fm_change_map, lpft_replant, &
            &             age_stand, rotation_n, last_cut, mai, pai, &
            &             previous_wood_volume, mai_count, coppice_dens, &
            &             lab_fac, circ_class_dist, store_sum_delta_ba, &
            &             harvest_pool_bound, harvest_pool, harvest_type, harvest_cut, harvest_area, &
            &             lai_per_level, laieff_fit, h_array_out, z_array_out, &
            &             max_height_store, &
            &             wstress_month, wstress_season, st_dist, litter_demand,&
            &             mass_balance_closure, light_tran_to_level_season, p_O2, bact, &
            &             CN_som_litter_longterm, nbp_pool_start,&
            &             wind_speed_daily, soil_temp_daily, harvest_5y_area)

       ! Write the mass balance error accumulated over the course of day
       ! to the history file
       ! If err_act = 3  the model should stop as soon as there is an 
       ! imbalance. Hence, when this block of code is reached it means 
       ! there is no mass balanc error. In case err_act is 1 or 2, the
       ! model will not be stopped. It is thus possible to end up here
       ! and to write a mass balance error.
       DO iele = 1,nelements
          IF (iele == icarbon) THEN
             element_str(iele) = '_c'
          ELSEIF (iele == initrogen) THEN
             element_str(iele) = '_n'
          ELSE
             CALL ipslerr_p(3,'stomate.f90','Define label for iele','','')
          ENDIF

          CALL histwrite_p (hist_id_stomate,'MB_CLOSURE'//TRIM(element_str(iele)), itime, &
               mass_balance_closure(:,iele), kjpindex, horipft_index)
          CALL xios_orchidee_send_field("MB_CLOSURE"//TRIM(element_str(iele)), &
               mass_balance_closure(:,iele)) 
       ENDDO

       !! 5.5.2 Long term adaptation of allocation to water stress
       !  ::wstress season is calculated as the seasonal mean of the
       !  ratio between the stressed and unstressed GPP. If the plant
       !  experiences a short spell of drought, leaves will be killed
       !  (see stomate_turnover). However, when the drought stress is 
       !  maintained during the season, it is assumed that the plant 
       !  will economise its canopy and therefore adjust its allocation
       ! factors to grow less leaves. To avoid that the canopy can only
       ! shrink the plants will try to grow more leaves when they do
       ! not experience any water stress. Extending the LAI, however,
       ! increases the chances that the plant will experience drough 
       ! stress in the future. These feedbacks should stabilize the LAI.
       wstress_adapt(:,:) = zero
       WHERE (wstress_season(:,:) .LT. 0.01)

          ! Increase the leaf allocation by 5% over the whole
          ! year. If there was no water stress during the whole 
          ! year, the following year more C will be allocated
          ! to the leaves.
          wstress_adapt(:,:) = 1.05

       ELSEWHERE

          wstress_adapt(:,:) = wstress_season(:,:)

       ENDWHERE

       !+++CHECK+++
       ! Don't use this adaptation
!!$       k_latosa_adapt(:,:) = k_latosa_adapt(:,:) * &
!!$            (wstress_adapt(:,:)**(un/365))
       !+++++++++++

       DO j = 2,nvm

          WHERE ( k_latosa_adapt(:,j) .GE. k_latosa_max(j) )

             k_latosa_adapt(:,j) = k_latosa_max(j)

          ENDWHERE

       ENDDO

       CALL histwrite_p (hist_id_stomate, 'K_LATOSA_ADAPT', itime, &
            k_latosa_adapt(:,:), kjpindex*nvm, horipft_index)
       CALL xios_orchidee_send_field("K_LATOSA_ADAPT",k_latosa_adapt)

       !+++CHECK+++
       ! check units and variable names for new lcchange and product use
       !! 5.3.2 Calculate the total CO2 flux from land use change
       !fco2_lu(:) = convflux(:) &
       !     &             + flux_prod10(:)  &
       !     &             + flux_prod100(:) &
       !     &             + harvest_above(:)
       ! It is an output variable so we just gave it a value
       ! when looking at the history files the value zero should
       ! ring a bell that there is something wrong.
       !fco2_lu(:) = 0
       !+++++++++++

       !! 5.4 Calculate veget_max
       veget_max(:,:) = zero 
       DO j = 1, nvm
          veget_max(:,j) = veget_max(:,j) + &
               & veget_cov_max(:,j) * ( 1.-totfrac_nobio(:) )
       ENDDO

       !! 5.5 Photosynthesis parameters
       assim_param(:,:,ivcmax) = zero
       assim_param(:,:,inue)   = zero
       assim_param(:,:,ileafn) = zero
       DO j = 2,nvm
          assim_param(:,j,ivcmax) = vcmax(:,j)
          assim_param(:,j,inue)   = nue(:,j)
          assim_param(:,j,ileafn) = SUM(circ_class_biomass(:,j,:,ileaf,initrogen)*circ_class_n(:,j,:),2)
       ENDDO

       !! 5.6 Update forcing variables for soil carbon
       IF (TRIM(Cforcing_name) /= 'NONE') THEN
          npp_tot(:) = 0
          DO j=2,nvm
             npp_tot(:) = npp_tot(:) + npp_daily(:,j)
          ENDDO
          ! ::nbyear Number of years saved for carbon spinup
          sf_time = MODULO(REAL(date,r_std)-1,one_year*REAL(nbyear,r_std))
          iatt=FLOOR(sf_time/dt_forcesoil) + 1
          IF (iatt == 0) iatt = iatt_old + 1
          IF ((iatt<iatt_old) .and. (.not. cumul_Cforcing)) THEN
             nforce(:)=0
             carbon_input(:,:,:,:) = zero
             nitrogen_input(:,:,:,:) = zero
             control_moist(:,:,:) = zero
             control_temp(:,:,:) = zero
             npp_equil(:,:) = zero
             drainage(:,:,:) = zero
          ENDIF
          iatt_old = iatt
          ! Update forcing
          nforce(iatt) = nforce(iatt)+1
          carbon_input(:,:,:,iatt) = carbon_input(:,:,:,iatt) + som_input_daily(:,:,:,icarbon)
          nitrogen_input(:,:,:,iatt) = nitrogen_input(:,:,:,iatt) + som_input_daily(:,:,:,initrogen)
          control_moist(:,:,iatt) = control_moist(:,:,iatt) + control_moist_daily(:,:)
          control_temp(:,:,iatt) = control_temp(:,:,iatt) + control_temp_daily(:,:)
          npp_equil(:,iatt) = npp_equil(:,iatt) + npp_tot(:)
          drainage(:,:,iatt) = drainage(:,:,iatt) + drainage_daily(:,:)
       ENDIF

       !! 5.8 Write forcing file
       ! Note: if STOMATE is run in coupled mode the forcing file is written
       ! If run in stand-alone mode, the forcing file is read!
       IF ( TRIM(forcing_name) /= 'NONE' ) THEN

          !! 5.8.1 Convert GPP to sechiba time steps
          ! GPP is multiplied by coverage to obtain forcing @tex $(gC m^{-2} dt_stomate^{-1})$\f \end@tex $(m^2 m^{-2})$ @endtexonly
          ! @tex$ m^{-2}$ @endtex remains in the units because ::veget_cov_max is a fraction, not a 
          ! surface area. In sechiba values are ponderated by surface and frac_no_bio. 
          ! At the beginning of stomate, the units are converted. 
          ! When we use forcesoil we call sechiba_main and so we need the have the same units as in sechiba.
          gpp_daily_x(:,:) = zero
          DO j = 2, nvm             
             gpp_daily_x(:,j) = gpp_daily_x(:,j) + &
                  & gpp_daily(:,j) * dt_stomate / one_day * veget_cov_max(:,j)
          ENDDO

          ! Bare soil moisture availability has not been treated
          ! in STOMATE, update it here
          humrel_daily(:,ibare_sechiba) = humrel(:,ibare_sechiba)   

          ! Update index to store the next forcing step in memory
          iisf = iisf+1

          ! How many times have we treated this forcing state
          xn = REAL(nf_cumul(isf(iisf)),r_std)

          !! 5.8.2 Cumulate forcing variables
          ! Cumulate forcing variables (calculate average)
          ! Note: precipitation is multiplied by dt_stomate/one_day to be consistent with 
          ! the units in sechiba
          IF (cumul_forcing) THEN
             clay_fm(:,iisf) = (xn*clay_fm(:,iisf)+clay(:))/(xn+1.)
             silt_fm(:,iisf) = (xn*silt_fm(:,iisf)+silt(:))/(xn+1.) 
             bulk_fm(:,iisf) = (xn*bulk_fm(:,iisf)+bulk(:))/(xn+1.) 
             humrel_daily_fm(:,:,iisf) = &
                  & (xn*humrel_daily_fm(:,:,iisf) + humrel_daily(:,:))/(xn+1.)
             litterhum_daily_fm(:,iisf) = &
                  & (xn*litterhum_daily_fm(:,iisf)+litterhum_daily(:))/(xn+1.)
             t2m_daily_fm(:,iisf) = &
                  & (xn*t2m_daily_fm(:,iisf)+t2m_daily(:))/(xn+1.)
             t2m_min_daily_fm(:,iisf) = &
                  & (xn*t2m_min_daily_fm(:,iisf)+t2m_min_daily(:))/(xn+1.)
             tsurf_daily_fm(:,iisf) = &
                  & (xn*tsurf_daily_fm(:,iisf)+tsurf_daily(:))/(xn+1.)
             tsoil_daily_fm(:,:,iisf) = &
                  & (xn*tsoil_daily_fm(:,:,iisf)+tsoil_daily(:,:))/(xn+1.)
             soilhum_daily_fm(:,:,iisf) = &
                  & (xn*soilhum_daily_fm(:,:,iisf)+soilhum_daily(:,:))/(xn+1.)
             precip_fm(:,iisf) = &
                  & (xn*precip_fm(:,iisf)+precip_daily(:)*dt_stomate/one_day)/(xn+1.)
             gpp_daily_fm(:,:,iisf) = &
                  & (xn*gpp_daily_fm(:,:,iisf) + gpp_daily_x(:,:))/(xn+1.)
             veget_fm(:,:,iisf) = &
                  & (xn*veget_fm(:,:,iisf) + veget(:,:) )/(xn+1.)
             veget_max_fm(:,:,iisf) = &
                  & (xn*veget_max_fm(:,:,iisf) + veget_max(:,:) )/(xn+1.)
!!$             lai_fm(:,:,iisf) = &
!!$                  & (xn*lai_fm(:,:,iisf) + lai(:,:) )/(xn+1.)
             drainage_fm(:,:,iisf) = & 
                  & (xn*drainage_fm(:,:,iisf) + drainage_daily(:,:) )/(xn+1.) 
          ELSE
             ! Here we just calculate the values
             clay_fm(:,iisf) = clay(:)
             silt_fm(:,iisf) = silt(:) 
             bulk_fm(:,iisf) = bulk(:) 
             humrel_daily_fm(:,:,iisf) = humrel_daily(:,:)
             litterhum_daily_fm(:,iisf) = litterhum_daily(:)
             t2m_daily_fm(:,iisf) = t2m_daily(:)
             t2m_min_daily_fm(:,iisf) =t2m_min_daily(:)
             tsurf_daily_fm(:,iisf) = tsurf_daily(:)
             tsoil_daily_fm(:,:,iisf) =tsoil_daily(:,:)
             soilhum_daily_fm(:,:,iisf) =soilhum_daily(:,:)
             precip_fm(:,iisf) = precip_daily(:)
             gpp_daily_fm(:,:,iisf) =gpp_daily_x(:,:)
             veget_fm(:,:,iisf) = veget(:,:)
             veget_max_fm(:,:,iisf) =veget_max(:,:)
!!$             lai_fm(:,:,iisf) =lai(:,:)
             drainage_fm(:,:,iisf) =drainage_daily(:,:) 
          ENDIF
          nf_cumul(isf(iisf)) = nf_cumul(isf(iisf))+1

          ! 5.8.3 Do we have to write the forcing states?
          IF (iisf == nsfm) THEN

             !! 5.8.3.1 Write these forcing states
             CALL forcing_write(forcing_id,1,nsfm)
             ! determine which forcing states must be read
             isf(1) = isf(nsfm)+1
             IF ( isf(1) > nsft ) isf(1) = 1
             DO iisf = 2, nsfm
                isf(iisf) = isf(iisf-1)+1
                IF (isf(iisf) > nsft)  isf(iisf) = 1
             ENDDO

             ! Read forcing variables - for debug use only
             ! CALL forcing_read(forcing_id,nsfm)
             iisf = 0

          ENDIF

       ENDIF


!!$       !! 5.9 Compute daily CO2 flux (AR5 output - not essential)
!!$       ! CO2 flux in @tex $gC m^{-2} s^{-1}$ @endtex (positive towards the atmosphere) is sum of:
!!$       ! (1) heterotrophic respiration from ground + (2) maintenance respiration 
!!$       ! from the plants + (3) growth respiration from the plants + (4) co2 
!!$       ! emissions from fire - (5) co2 taken up in the DGVM to establish 
!!$       ! saplings - (6) co2 taken up by photosyntyhesis
!!$       co2_flux_daily(:,:)=   &
!!$            & resp_maint_d(:,:) + resp_growth_d(:,:) + resp_hetero_d(:,:) + &
!!$            & co2_fire(:,:) - atm_to_bm(:,:,icarbon) - gpp_daily(:,:)
!!$
!!$     CALL xios_orchidee_send_field("nep",SUM(co2_flux_daily*veget_cov_max,dim=2)/1e3/one_day)
!!$
!!$       IF ( hist_id_stom_IPCC > 0 ) THEN
!!$          vartmp(:) = SUM(co2_flux_daily*veget_cov_max,dim=2)/1e3/one_day*contfrac
!!$          CALL histwrite_p (hist_id_stom_IPCC, "nep", itime, &
!!$               vartmp, kjpindex, hori_index)
!!$       ENDIF
!!$
!!$       ! See 5.9 for details on NEP + fire. At the monthly time step also 
!!$       ! harvest and land use change are calculated
!!$       co2_flux_monthly(:,:) = co2_flux_monthly(:,:) + co2_flux_daily(:,:)
!!$       harvest_above_monthly(:) = harvest_above_monthly(:) + harvest_above(:)
!!$       flux_prod_monthly(:) = flux_prod_monthly(:) + convflux(:) + & 
!!$        & flux_prod10(:) + flux_prod100(:)
!!$      
!!$       !! 5.10 Compute monthly CO2 fluxes 
!!$       IF ( EndOfMonth ) THEN
!!$          !! 5.10.1 Write history file for monthly fluxes
!!$          CALL histwrite_p (hist_id_stomate, 'CO2FLUX', itime, &
!!$               co2_flux_monthly, kjpindex*nvm, horipft_index)
!!$          
!!$          !?? I (=VB) translated the French, but the whole stuff does not make sense to me.
!!$          ! If one deletes the montly cumulation,
!!$          ! one should not forget this change in resolution(:,1)*resolution(:,2)*contfrac(:)
!!$          ! Si on supprimer le cumul par mois, 
!!$          ! il ne faut pas oublier cette modif resolution(:,1)*resolution(:,2)*contfrac(:)
!!$          ! Should be supressed, this is post-processing 
!!$          DO j=2, nvm
!!$             co2_flux_monthly(:,j) = co2_flux_monthly(:,j)* &
!!$                  resolution(:,1)*resolution(:,2)*contfrac(:)
!!$          ENDDO
!!$
!!$          ! Should be supressed, this is post-processing
!!$          ! ?? How does it differ from co2_flux_monthly??
!!$          net_co2_flux_monthly = zero
!!$          DO ji=1,kjpindex
!!$             DO j=2,nvm
!!$                net_co2_flux_monthly = net_co2_flux_monthly + &
!!$                     &  co2_flux_monthly(ji,j)*veget_cov_max(ji,j)
!!$             ENDDO
!!$          ENDDO
!!$
!!$     
!!$          !! 5.10.2 Cumulative fluxes of land use cover change, harvest and net biosphere production
!!$          ! Parallel processing, gather the information from different processors. first argument is the lo
!!$          ! local variable, the second argument is the global variable. bcast send it to all processors.
!!$          net_flux_prod_monthly_sum = &
!!$              &  SUM(flux_prod_monthly(:)*resolution(:,1)*resolution(:,2)*contfrac(:))*1e-15
!!$          CALL reduce_sum(net_flux_prod_monthly_sum,net_flux_prod_monthly_tot)
!!$          CALL bcast(net_flux_prod_monthly_tot)
!!$          net_harvest_above_monthly_sum = &
!!$             &   SUM(harvest_above_monthly(:)*resolution(:,1)*resolution(:,2)*contfrac(:))*1e-15
!!$          CALL reduce_sum(net_harvest_above_monthly_sum,net_harvest_above_monthly_tot)
!!$          CALL bcast(net_harvest_above_monthly_tot)
!!$          net_co2_flux_monthly = net_co2_flux_monthly*1e-15
!!$          CALL reduce_sum(net_co2_flux_monthly,net_co2_flux_monthly_sum)
!!$          CALL bcast(net_co2_flux_monthly_sum)
!!$          net_biosp_prod_monthly_tot =  &
!!$             & ( net_co2_flux_monthly_sum + net_flux_prod_monthly_tot + &
!!$             & net_harvest_above_monthly_tot )
!!$          
!!$          WRITE(numout,9010) 'GLOBAL net_flux_prod_monthly    (Peta gC/month)  = ',net_flux_prod_monthly_tot
!!$          WRITE(numout,9010) 'GLOBAL net_harvest_above_monthly (Peta gC/month)  = ',net_harvest_above_monthly_tot
!!$          WRITE(numout,9010) 'GLOBAL net_co2_flux_monthly      (Peta gC/month)  = ',net_co2_flux_monthly_sum
!!$          WRITE(numout,9010) 'GLOBAL net_biosp_prod_monthly    (Peta gC/month)  = ',net_biosp_prod_monthly_tot
!!$
!!$9010  FORMAT(A52,F17.14)
!!$
!!$          ! Reset Monthly values
!!$          co2_flux_monthly(:,:) = zero
!!$          harvest_above_monthly(:) = zero
!!$          flux_prod_monthly(:)    = zero
!!$
!!$       ENDIF ! Monthly processes - at the end of the month

    !   IF (spinup_analytic) THEN
    !   I (ASLA) Do not think we have to include the below calculations of
    !   nbp_accu. It is only used to calculate nbp_flux, but this variable is
    !   use for nothing except being written to the restart/being read from the
    !   restart. It is still included on CN, but there I do not see the use of
    !   it either.  
    !  
    !      WRITE(numout,*) 'ERROR:check the code. There was nothing defined yet'
    !      CALL ipslerr_p(3,'ERROR:check the code', &
    !           'there was nothing defined yet','','')

!!$       CALL xios_orchidee_send_field("nep",SUM(co2_flux_daily*veget_cov_max,dim=2)/1e3/one_day)

          !+++CHECK+++
    !      nbp_accu(:) = nbp_accu(:) + (-SUM(co2_flux_daily(:,:) * &
    !           veget_cov_max(:,:),dim=2) - (convflux(:) + flux_prod10(:) + &
    !           flux_prod100(:))  - harvest_above(:))/1e3 

          ! Daily NBP is used in the spin-up. Rather than starting from 
          ! all the initial components as done previously we prefer to 
          ! make use of the aggregated components calculated to check the 
          ! mass balance closure. NBP is then the change in all
          ! biomass pools.

          !+++CHECK+++
          ! pool_end and pool_begin need to be defined
!!$          nbp_accu(:) = nbp_accu(:) + &
!!$               SUM((pool_end(:,:,icarbon) - pool_start(:,:,icarbon)),2)
          !+++++++++++

      ! ENDIF (spinup_analytic)
       !! 5.11 Reset daily variables
       humrel_daily(:,:) = zero
       litterhum_daily(:) = zero
       t2m_daily(:) = zero
       t2m_min_daily(:) = large_value
       tsurf_daily(:) = zero
       tsoil_daily(:,:) = zero
       soilhum_daily(:,:) = zero
       precip_daily(:) = zero
       gpp_daily(:,:) = zero
       resp_maint_part(:,:,:)=zero
       resp_hetero_d=zero
       drainage_daily(:,:) = zero 
       plant_n_uptake_daily(:,:,:)=zero 
       n_mineralisation_d(:,:)=zero 
       IF (printlev_loc >= 3) THEN
          WRITE(numout,*) 'stomate_main: daily processes done'
       ENDIF


       ! 5.12 pool-based NBP. 
       ! Calculate the end pool for the pool-based NBP estimate
       IF( SPINUP_ANALYTIC ) THEN
          DO ipar = 1,nparts
             DO icir=1, ncirc
                nbp_pool_end(:,:) = nbp_pool_end(:,:) + &
                   (circ_class_biomass(:,:,icir,ipar,icarbon) * &
                   circ_class_n(:,:,icir)*veget_cov_max(:,:))
             ENDDO
          ENDDO
          ! To make sure we the carbon that leaves the ecosystem via heterotropic
          ! respiration 
          ! in to account, the soil and litter pools are summed
          DO icarb = 1, ncarb
             nbp_pool_end(:,:) = nbp_pool_end(:,:) + &
               som(:,icarb,:,icarbon)*veget_cov_max(:,:)
          ENDDO

          DO ilitt = 1,nlitt
             DO ilev =1,nlevs
                nbp_pool_end(:,:) = nbp_pool_end(:,:) + &
                   litter(:,ilitt,:,ilev,icarbon)*veget_cov_max(:,:)
             ENDDO
          ENDDO

          ! Calculate nbp for each time step
          nbp(:,:)=nbp_pool_end(:,:) - nbp_pool_start(:,:)
          ! Calculate an accumulated pool-based NBP
          nbp_accu_pool(:,:) = nbp_accu_pool(:,:) + nbp(:,:)
    
          ! reset the nbp_pool_start and nbp_pool_end
          nbp_pool_start(:,:) = nbp_pool_end
          nbp_pool_end(:,:) = zero 

          ! Write to output
          CALL xios_orchidee_send_field("NBP_pool",nbp)
       ENDIF ! SPINUP_ANALYTIC
 
    ENDIF  ! Daily processes - at the end of the day

    !! 6. Outputs from Stomate

    ! co2_flux receives a value from STOMATE only if STOMATE is activated.
    ! Otherwise, the calling hydrological module must do this itself.

    !! 6.1 Respiration and fluxes
    resp_maint(:,:) = resp_maint_radia(:,:)*veget_cov_max(:,:)
    resp_maint(:,ibare_sechiba) = zero
    resp_growth(:,:)= resp_growth_d(:,:)*veget_cov_max(:,:)*dt_sechiba/one_day
    resp_hetero(:,:) = resp_hetero_radia(:,:)*veget_cov_max(:,:)

    !! 6.2 Derived CO2 fluxes
    ! CO2 flux in gC m^{-2} s^{-1} (positive towards the atmosphere) is sum of:
    ! (1) heterotrophic respiration from ground + (2) maintenance respiration 
    ! from the plants + (3) growth respiration from the plants + (4) co2 
    ! emissions from fire - (5) co2 taken up in the DGVM to establish 
    ! saplings - (6) co2 taken up by photosyntyhesis
    co2_flux(:,:) = resp_hetero(:,:) + resp_maint(:,:) + resp_growth(:,:) &
         & + (co2_fire(:,:)-atm_to_bm(:,:,icarbon))*veget_cov_max(:,:)/one_day &
         & - gpp(:,:)

    temp_growth(:)=t2m_month(:)-tp_00 

    !! 7. Analytical spinup

    IF (spinup_analytic) THEN

       tau_CN_longterm = tau_CN_longterm + dt_sechiba/one_day

       !! 7.1. Update V and U at sechiba time step
       DO m = 2,nvm
          DO j = 1,kjpindex 
             ! V <- A * V
             matrixV(j,m,:,:) = MATMUL(matrixA(j,m,:,:),matrixV(j,m,:,:))
             ! U <- A*U + B
             vectorU(j,m,:) = MATMUL(matrixA(j,m,:,:),vectorU(j,m,:)) + vectorB(j,m,:)
          ENDDO ! loop pixels
       ENDDO ! loop PFTS


       !! 7.2. What happened at the end of the year ?
       IF (EndOfYear) THEN

          !
          ! 7.2.1 Increase the years counter every EndOfyear
          !
          global_years = global_years + 1 


          !
          ! 7.2.3 Is global_years is a multiple of the period time ?
          !

          !
          ! 3.2.1 When global_years is a multiple of the spinup_period, we calculate :
          !       1) the mean nbp flux over the period. This value is restarted
          !       2) we solve the matrix system by Gauss Jordan method
          !       3) We test if a point is at equilibrium : if yes, we mark the point (ok_equilibrium array)
          !       4) Then we reset the matrix 
          !       5) We erase the carbon_stock calculated by ORCHIDEE by the one found by the method
          IF( MOD(global_years, spinup_period) == 0 ) THEN
             WRITE(numout,*) 'Spinup analytic : Calculate if system is in equlibrium. global_years=',global_years
             ! The number total of days during the forcing period is given by :
             !    spinup_period*365 (we consider only the noleap calendar)
             nbp_flux(:) = nbp_accu(:) / ( spinup_period * 365.)
             nbp_pool(:,:) = nbp_accu_pool(:,:) / ( spinup_period * 365.)
             ! write to output
             CALL xios_orchidee_send_field("NBP_pool_spinup",nbp_pool)

             ! Reset the values
             nbp_accu(:) = zero
             nbp_accu_pool(:,:) = zero

             carbon_stock(:,ibare_sechiba,:) = zero
             ! Prepare the matrix for the resolution
             ! Add a temporary matrix W which contains I-matrixV
             ! we should take the opposite of matrixV and add the identitiy : we solve (I-matrixV)*C = vectorU
             matrixW(:,:,:,:) = moins_un * matrixV(:,:,:,:)
             DO jv = 1,nbpools
                matrixW(:,:,jv,jv) =  matrixW(:,:,jv,jv) + un
             ENDDO
             carbon_stock(:,:,:) = vectorU(:,:,:)

             !
             !  Solve the linear system
             !
             DO m = 2,nvm
                DO j = 1,kjpindex
                   ! the solution will be stored in vectorU : so it should be restarted before
                   ! loop over kjpindex and nvm, so we solved kjpindex*(nvm-1) (7,7) linear systems
                   CALL gauss_jordan_method(nbpools,matrixW(j,m,:,:),carbon_stock(j,m,:))
                ENDDO ! loop pixels
             ENDDO ! loop PFTS

             ! Reset temporary matrixW
             matrixW(:,:,:,:) = zero 


             previous_stock(:,:,:) = current_stock(:,:,:)
             current_stock(:,:,:) = carbon_stock(:,:,:)
  
             ! The relative error is calculated over the passive carbon pool 
             ! (sum over the pfts) over the pixel.
             CALL error_L1_passive(kjpindex,nvm, nbpools, current_stock, &
                  previous_stock, veget_cov_max, eps_carbon, carbon_eq)   

             !! ok_equilibrium is saved,
             WHERE( carbon_eq(:) .AND. .NOT.(ok_equilibrium(:)) )
                ok_equilibrium(:) = .TRUE.  
             ENDWHERE

             WRITE(numout,*) 'current_stock actif:',current_stock(test_grid,test_pft,iactive)
             WRITE(numout,*) 'current_stock slow:',current_stock(test_grid,test_pft,islow)
             WRITE(numout,*) 'current_stock passif:',current_stock(test_grid,test_pft,ipassive)
             WRITE(numout,*) 'current_stock surface:',current_stock(test_grid,test_pft,isurface)

             ! Reset matrixV for the pixel to the identity matrix and vectorU to zero
             matrixV(:,:,:,:) = zero
             vectorU(:,:,:) = zero
             DO jv = 1,nbpools
                matrixV(:,:,jv,jv) = un
             END DO
             WRITE(numout,*) 'Reset for matrixV and vectorU done'    

             !! Write the values found in the standard outputs of ORCHIDEE
             litter(:,istructural,:,iabove,icarbon) = carbon_stock(:,:,istructural_above)
             litter(:,istructural,:,ibelow,icarbon) = carbon_stock(:,:,istructural_below)
             litter(:,imetabolic,:,iabove,icarbon) = carbon_stock(:,:,imetabolic_above)
             litter(:,imetabolic,:,ibelow,icarbon) = carbon_stock(:,:,imetabolic_below)
             litter(:,iwoody,:,iabove,icarbon) = carbon_stock(:,:,iwoody_above)
             litter(:,iwoody,:,ibelow,icarbon) = carbon_stock(:,:,iwoody_below)
             som(:,iactive,:,icarbon) = carbon_stock(:,:,iactive_pool)
             som(:,isurface,:,icarbon) = carbon_stock(:,:,isurface_pool)
             som(:,islow,:,icarbon) = carbon_stock(:,:,islow_pool)
             som(:,ipassive,:,icarbon) = carbon_stock(:,:,ipassive_pool) 

             litter(:,:,:,:,initrogen) = zero
             som(:,:,:,initrogen) = zero

             WHERE( CN_som_litter_longterm(:,:,istructural_above) .GT. min_stomate)
                litter(:,istructural,:,iabove,initrogen) = carbon_stock(:,:,istructural_above) &
                     / CN_som_litter_longterm(:,:,istructural_above)
             ENDWHERE

             WHERE( CN_som_litter_longterm(:,:,istructural_below) .GT. min_stomate)
                litter(:,istructural,:,ibelow,initrogen) = carbon_stock(:,:,istructural_below) &
                     / CN_som_litter_longterm(:,:,istructural_below)
             ENDWHERE

             WHERE( CN_som_litter_longterm(:,:,imetabolic_above) .GT. min_stomate)
                litter(:,imetabolic,:,iabove,initrogen)  = carbon_stock(:,:,imetabolic_above)  &
                     / CN_som_litter_longterm(:,:,imetabolic_above)
             ENDWHERE

             WHERE( CN_som_litter_longterm(:,:,imetabolic_below) .GT. min_stomate)
                litter(:,imetabolic,:,ibelow,initrogen)  = carbon_stock(:,:,imetabolic_below)  &
                     / CN_som_litter_longterm(:,:,imetabolic_below)
             ENDWHERE

             WHERE( CN_som_litter_longterm(:,:,iwoody_above) .GT. min_stomate)
                litter(:,iwoody,:,iabove,initrogen)      = carbon_stock(:,:,iwoody_above)      &
                     / CN_som_litter_longterm(:,:,iwoody_above)    
             ENDWHERE

             WHERE( CN_som_litter_longterm(:,:,iwoody_below) .GT. min_stomate)
                litter(:,iwoody,:,ibelow,initrogen)      = carbon_stock(:,:,iwoody_below)      &
                     / CN_som_litter_longterm(:,:,iwoody_below)    
             ENDWHERE

             WHERE(CN_som_litter_longterm(:,:,iactive_pool) .GT. min_stomate)
                som(:,iactive,:,initrogen)               = carbon_stock(:,:,iactive_pool)      &
                     / CN_som_litter_longterm(:,:,iactive_pool)    
             ENDWHERE

             WHERE(CN_som_litter_longterm(:,:,isurface_pool) .GT. min_stomate)
                som(:,isurface,:,initrogen)              = carbon_stock(:,:,isurface_pool)     &
                     / CN_som_litter_longterm(:,:,isurface_pool)   
             ENDWHERE

             WHERE(CN_som_litter_longterm(:,:,islow_pool) .GT. min_stomate)
                som(:,islow,:,initrogen)                 = carbon_stock(:,:,islow_pool)        &
                     / CN_som_litter_longterm(:,:,islow_pool)      
             ENDWHERE

             WHERE(CN_som_litter_longterm(:,:,ipassive_pool) .GT. min_stomate)
                som(:,ipassive,:,initrogen)              = carbon_stock(:,:,ipassive_pool)     &
                     / CN_som_litter_longterm(:,:,ipassive_pool)     
             ENDWHERE

             CN_som_litter_longterm(:,:,:) = zero
             tau_CN_longterm = dt_sechiba/one_day
             ! Final step, test if all points at the local domain are at equilibrium
             ! The simulation can be stopped when all local domains have reached the equilibrium
             IF(ALL(ok_equilibrium)) THEN
                WRITE(numout,*) 'Spinup analytic : Equilibrium for carbon pools is reached for current local domain'
             ELSE
                WRITE(numout,*) 'Spinup analytic : Equilibrium for carbon pools is not yet reached for current local domain'
             END IF
          ENDIF ! ( MOD(global_years,spinup_period) == 0)
       ENDIF ! (EndOfYear)

    ENDIF !(spinup_analytic)

    IF (printlev >= 4) WRITE(numout,*) 'Leaving stomate_main'

  END SUBROUTINE stomate_main

!! ================================================================================================================================
!! SUBROUTINE 	: stomate_finalize
!!
!>\BRIEF        Write variables to restart file
!!
!! DESCRIPTION  : Write variables to restart file
!! RECENT CHANGE(S) : None
!!
!! MAIN OUTPUT VARIABLE(S): 
!!
!! REFERENCES	: 
!!
!! \n
!_ ================================================================================================================================

  SUBROUTINE stomate_finalize (kjit, kjpindex, index, clay, assim_param, silt, bulk, &
       circ_class_biomass, circ_class_n, lai_per_level, laieff_fit) 
    
    IMPLICIT NONE
    
    !! 0. Variable and parameter declaration
    !! 0.1 Input variables
    INTEGER(i_std),INTENT(in)                       :: kjit              !! Time step number (unitless)
    INTEGER(i_std),INTENT(in)                       :: kjpindex          !! Domain size - terrestrial pixels only (unitless)
    INTEGER(i_std),DIMENSION(kjpindex),INTENT(in)   :: index             !! Indices of the terrestrial pixels only (unitless)
    REAL(r_std),DIMENSION(kjpindex),INTENT(in)      :: clay              !! Clay fraction of soil (0-1, unitless)
    REAL(r_std),DIMENSION(kjpindex),INTENT(in)      :: silt              !! Silt fraction of soil (0-1, unitless) 
    REAL(r_std),DIMENSION(kjpindex),INTENT(in)      :: bulk              !! Bulk density (kg/m**3)
    REAL(r_std),DIMENSION(kjpindex,nvm,npco2),INTENT(in) :: assim_param  !! min+max+opt temperatures (K) & vmax for 
                                                                         !! photosynthesis   
    REAL(r_std), DIMENSION(:,:,:,:,:), INTENT(in)   :: circ_class_biomass !!  Biomass components of the model tree  
                                                                         !! within a circumference class
                                                                         !! class @tex $(g C ind^{-1})$ @endtex
    REAL(r_std), DIMENSION(:,:,:), INTENT(in)       :: circ_class_n      !! Number of trees within each circumference
                                                                         !! class @tex $(m^{-2})$ @endtex
    REAL(r_std), DIMENSION(:,:,:), INTENT(in)       :: lai_per_level     !! This is the LAI per vertical level
                                                                         !! @tex $(m^{2} m^{-2})$
    TYPE(laieff_type),DIMENSION (:,:,:),INTENT(in) &
                                                   :: laieff_fit         !! Fitted parameters for the effective LAI

!! 0.2 Output variables 

    !! 0.4 Local variables
    REAL(r_std)                                   :: dt_days_read             !! STOMATE time step read in restart file (days)
    INTEGER(i_std)                                :: l,k,ji, jv, i, j, m      !! indices    
    REAL(r_std),PARAMETER                         :: max_dt_days = 5.         !! Maximum STOMATE time step (days)
    REAL(r_std)                                   :: hist_days                !! Writing frequency for history file (days)
    REAL(r_std),DIMENSION(0:nbdl)                 :: z_soil                   !! Variable to store depth of the different soil layers (m)
    REAL(r_std),DIMENSION(kjpindex)               :: cvegtot                  !! Total "vegetation" cover (unitless)
    REAL(r_std),DIMENSION(kjpindex)               :: precip                   !! Total liquid and solid precipitation  
                                                                              !! @tex $(??mm dt_stomate^{-1})$ @endtex 
    REAL(r_std),DIMENSION(kjpindex,nvm)           :: gpp_d                    !! Gross primary productivity per ground area 
                                                                              !! @tex $(??gC m^{-2} dt_stomate^{-1})$ @endtex  
    REAL(r_std),DIMENSION(kjpindex,nvm)           :: gpp_daily_x              !! "Daily" gpp for teststomate  
                                                                              !! @tex $(??gC m^{-2} dt_stomate^{-1})$ @endtex 
    REAL(r_std),DIMENSION(kjpindex,nvm)           :: resp_hetero_litter       !! Litter heterotrophic respiration per ground area 
                                                                              !! @tex $(gC m^{-2} day^{-1})$ @endtex  
                                                                              !! ??Same variable is also used to 
                                                                              !! store heterotrophic respiration per ground area 
                                                                              !! over ::dt_sechiba?? 
    REAL(r_std),DIMENSION(kjpindex,nvm)           :: resp_hetero_soil         !! soil heterotrophic respiration  
                                                                              !! @tex $(gC m^{-2} day^{-1})$ @endtex
    REAL(r_std),DIMENSION(kjpindex,nvm)           :: veget_cov                !! Fractional coverage: actually share of the pixel 
                                                                              !! covered by a PFT (fraction of ground area), 
                                                                              !! taking into account LAI ??(= grid scale fpc)?? 
    REAL(r_std),DIMENSION(kjpindex,nvm)           :: vcmax                    !! Maximum rate of carboxylation
                                                                              !! @tex $(\mumol m^{-2} s^{-1})$ @endtex
    REAL(r_std),DIMENSION(kjpindex,nlevs)         :: control_moist_inst       !! Moisture control of heterotrophic respiration 
                                                                              !! (0-1, unitless) 
    REAL(r_std),DIMENSION(kjpindex,nlevs)         :: control_temp_inst        !! Temperature control of heterotrophic 
                                                                              !! respiration, above and below (0-1, unitless) 
    REAL(r_std),DIMENSION(kjpindex,ncarb,nvm,nelements)   :: som_input_inst   !! Quantity of carbon going into carbon pools from 
                                                                              !! litter decomposition 
                                                                              !! @tex $(gC m^{-2} day^{-1})$ @endtex 
    
    INTEGER(i_std)                                :: ier                      !! Check errors in netcdf call (unitless)
    REAL(r_std)                                   :: sf_time                  !! Intermediate variable to calculate current time 
                                                                              !! step 
    INTEGER(i_std)                                :: max_totsize              !! Memory management - maximum memory size (Mb)
    INTEGER(i_std)                                :: totsize_1step            !! Memory management - memory required to store one 
                                                                              !! time step on one processor (Mb) 
    INTEGER(i_std)                                :: totsize_tmp              !! Memory management - memory required to store one 
                                                                              !! time step on all processors(Mb) 
    REAL(r_std)                                   :: xn                       !! How many times have we treated in this forcing 
    REAL(r_std), DIMENSION(kjpindex)              :: vartmp                   !! Temporary variable
    INTEGER(i_std)                                :: vid                      !! Variable identifer of netCDF (unitless)
    INTEGER(i_std)                                :: nneigh                   !! Number of neighbouring pixels
    INTEGER(i_std)                                :: direct                   !! ??
    INTEGER(i_std),DIMENSION(ndm)                 :: d_id                     !! ??
    REAL(r_std),DIMENSION(nbp_glo)                :: clay_g                   !! Clay fraction of soil (0-1, unitless), parallel 
                                                                              !! computing 
    REAL(r_std),DIMENSION(nbp_glo)                :: silt_g                   !! Silt fraction of soil (0-1, unitless), parallel  
                                                                              !! computing 
    REAL(r_std),DIMENSION(nbp_glo)                :: bulk_g                   !! Bulk density (kg/m**3), parallel                                                                                                                                                                 !! computing 
    REAL(r_std),ALLOCATABLE,DIMENSION(:,:,:,:)    :: carbon_input_g           !! Quantity of carbon going into carbon pools from 
                                                                              !! litter decomposition  
                                                                              !! @tex $(gC m^{-2} dt_sechiba^{-1})$ @endtex, parallel 
                                                                              !! computing 
    REAL(r_std),ALLOCATABLE,DIMENSION(:,:,:,:)    :: nitrogen_input_g         !! Quantity of nitrogen going into nitrogen pools from  
                                                                              !! litter decomposition   
                                                                              !! @tex $(gC m^{-2} dtradia^{-1})$ @endtex, parallel  
    REAL(r_std),ALLOCATABLE,DIMENSION(:,:,:)      :: control_moist_g          !! Moisture control of heterotrophic respiration 
                                                                              !! (0-1, unitless), parallel computing 
    REAL(r_std),ALLOCATABLE,DIMENSION(:,:,:)      :: control_temp_g           !! Temperature control of heterotrophic respiration 
                                                                              !! (0-1, unitless), parallel computing 
    REAL(r_std),ALLOCATABLE,DIMENSION(:,:)        :: npp_equil_g              !! Equilibrium NPP written to forcesoil 
                                                                              !! @tex $(gC m^{-2} year^{-1})$ @endtex, parallel 
                                                                              !! computing 

!!$    REAL(r_std)                                   :: net_flux_prod_monthly_sum    !! AR5 output?? gC m2 month-1 (one variable for 
!!$                                                                                   !! reduce_sum and one for bcast??), parallel 
!!$                                                                                   !! computing 
!!$    REAL(r_std)                                   :: net_flux_prod_monthly_tot    !! AR5 output?? gC m2 month-1 (one variable for 
!!$                                                                                   !! reduce_sum and one for bcast??), parallel 
!!$                                                                                   !! computing 
!!$    REAL(r_std)                                   :: net_harvest_above_monthly_sum !! AR5 output?? gC m2 month-1 (one variable for 
!!$                                                                                   !! reduce_sum and one for bcast??), parallel 
!!$                                                                                   !! computing 
!!$    REAL(r_std)                                   :: net_harvest_above_monthly_tot !! AR5 output?? gC m2 month-1 (one variable for 
!!$                                                                                   !! reduce_sum and one for bcast??), parallel 
!!$                                                                                   !! computing 
!!$    REAL(r_std)                                   :: net_biosp_prod_monthly_sum    !! AR5 output?? gC m2 month-1 (one variable for 
!!$                                                                                   !! reduce_sum and one for bcast??), parallel 
!!$                                                                                   !! computing 
!!$    REAL(r_std)                                   :: net_biosp_prod_monthly_tot    !! AR5 output?? gC m2 month-1 (one variable for 
!!$                                                                                   !! reduce_sum and one for bcast??), parallel 
                                                                                   !! computing 
    REAL(r_std), DIMENSION(kjpindex,nvm,nbpools)  :: carbon_stock                  !! Array containing the carbon stock for each pool
                                                                                   !! used by ORCHIDEE
    

!_ ================================================================================================================================
    
    !! 1. Write restart file for stomate
    IF (printlev>=3) WRITE (numout,*) 'Write restart file for STOMATE' 
   
    CALL writerestart &
         (kjpindex, index, &
         dt_days, date, &
         adapted, regenerate, &
         humrel_daily, gdd_init_date, litterhum_daily, &
         t2m_daily, t2m_min_daily, tsurf_daily, tsoil_daily, &
         soilhum_daily, precip_daily, &
         gpp_daily, npp_daily, turnover_daily, &
         humrel_month, humrel_week, humrel_growingseason, &
         t2m_longterm, tau_longterm, t2m_month, t2m_week, &
         tsoil_month, soilhum_month, fireindex, firelitter, &
         maxhumrel_lastyear, maxhumrel_thisyear, &
         minhumrel_lastyear, minhumrel_thisyear, &
         maxgppweek_lastyear, maxgppweek_thisyear, &
         gdd0_lastyear, gdd0_thisyear, &
         precip_lastyear, precip_thisyear, &
         gdd_m5_dormance, gdd_from_growthinit, gdd_midwinter, ncd_dormance, ngd_minus5, &
         PFTpresent, npp_longterm, lm_lastyearmax, lm_thisyearmax, &
         maxfpc_lastyear, maxfpc_thisyear, &
         turnover_longterm, gpp_week, resp_maint_part, &
         leaf_age, leaf_frac, &
         plant_status, when_growthinit, age, &
         resp_hetero_d, resp_maint_d, resp_growth_d, co2_fire, atm_to_bm, &
         veget_lastlight, everywhere, need_adjacent, &
         RIP_time, &
         time_hum_min, hum_min_dormance, &
         litter, dead_leaves, &
         som, lignin_struc, lignin_wood,turnover_time,&
         prod_s, prod_m, prod_l, &
         flux_s, flux_m, flux_l, flux_prod_s, flux_prod_m, &
         flux_prod_l, bm_to_litter, carb_mass_total, &
         Tseason, Tseason_length, Tseason_tmp, &
         Tmin_spring_time, onset_date, &
         global_years, ok_equilibrium, nbp_accu, nbp_flux, &
         nbp_accu_pool, nbp_pool, nbp_pool_start, &
         matrixV, vectorU, previous_stock, current_stock, &
         assim_param, CN_som_litter_longterm, &
         tau_CN_longterm, KF, k_latosa_adapt, rue_longterm, &
         cn_leaf_min_season, &
         nstress_season, soil_n_min, p_O2, bact, &
         circ_class_biomass, circ_class_n, store_sum_delta_ba, &
         forest_managed, forest_managed_lastyear, &
         species_change_map, fm_change_map, lpft_replant, lai_per_level, &
         laieff_fit, wstress_season, wstress_month, &
         age_stand, rotation_n, last_cut, mai, pai, &
         previous_wood_volume, mai_count, coppice_dens, &
         light_tran_to_level_season, harvest_5y_area)
    
    !! 2. Write file with variables that force general processes in stomate
    IF ( allow_forcing_write ) THEN
       IF ( TRIM(forcing_name) /= 'NONE' ) THEN  
          CALL forcing_write(forcing_id,1,iisf)
          ! Close forcing file
          IF (is_root_prc) ier = NF90_CLOSE (forcing_id)
          forcing_id=-1
       END IF
    END IF
    
    !! 3. Collect variables that force the soil processes in stomate
    IF (TRIM(Cforcing_name) /= 'NONE' ) THEN 
       
       !! Collet variables 
       WRITE(numout,*) &
            &      'stomate: writing the forcing file for carbon spinup'
       DO iatt = 1, nparan*nbyear
          IF ( nforce(iatt) > 0 ) THEN
             carbon_input(:,:,:,iatt) = &
                  & carbon_input(:,:,:,iatt)/REAL(nforce(iatt),r_std)
             nitrogen_input(:,:,:,iatt) = &
                  & nitrogen_input(:,:,:,iatt)/REAL(nforce(iatt),r_std)
             control_moist(:,:,iatt) = &
                  & control_moist(:,:,iatt)/REAL(nforce(iatt),r_std)
             control_temp(:,:,iatt) = &
                  & control_temp(:,:,iatt)/REAL(nforce(iatt),r_std)
             npp_equil(:,iatt) = &
                  & npp_equil(:,iatt)/REAL(nforce(iatt),r_std)
             drainage(:,:,iatt) = & 
                  &  drainage(:,:,iatt)/REAL(nforce(iatt),r_std) 
          ELSE
             WRITE(numout,*) &
                  &         'We have no soil carbon forcing data for this time step:', &
                  &         iatt
             WRITE(numout,*) ' -> we set them to zero'
             carbon_input(:,:,:,iatt) = zero
             nitrogen_input(:,:,:,iatt) = zero
             control_moist(:,:,iatt) = zero
             control_temp(:,:,iatt) = zero
             npp_equil(:,iatt) = zero
          ENDIF
       ENDDO
       
       ! Allocate memory for parallel computing
       IF (is_root_prc) THEN
          ALLOCATE(carbon_input_g(nbp_glo,ncarb,nvm,nparan*nbyear))
          ALLOCATE(nitrogen_input_g(nbp_glo,ncarb,nvm,nparan*nbyear))
          ALLOCATE(control_moist_g(nbp_glo,nlevs,nparan*nbyear))
          ALLOCATE(control_temp_g(nbp_glo,nlevs,nparan*nbyear))
          ALLOCATE(npp_equil_g(nbp_glo,nparan*nbyear))
       ENDIF
       
       ! Gather distributed variables

       CALL gather(clay,clay_g)
       CALL gather(silt,silt_g) 
       CALL gather(bulk,bulk_g) 
       CALL gather(carbon_input,carbon_input_g)
       CALL gather(nitrogen_input,nitrogen_input_g)
       CALL gather(control_moist,control_moist_g)
       CALL gather(control_temp,control_temp_g)
       CALL gather(npp_equil,npp_equil_g)
       
       !! Create netcdf
       ! Create, define and populate a netcdf file containing the forcing data.
       ! For the root processor only (parallel computing). NF90_ are functions
       ! from and external library.  
       IF (is_root_prc) THEN
          WRITE (numout,*) 'Create Cforcing file : ',TRIM(Cforcing_name)
          ! Create new netCDF dataset
          ier = NF90_CREATE (TRIM(Cforcing_name),NF90_64BIT_OFFSET ,Cforcing_id)
          IF (ier /= NF90_NOERR) THEN
             WRITE (numout,*) 'Error in creating Cforcing file : ',TRIM(Cforcing_name)
             CALL ipslerr_p (3,'stomate_finalize', &
                  &        'PROBLEM creating Cforcing file', &
                  &        NF90_STRERROR(ier),'')
          END IF
          
          ! Add variable attribute
          ! Note ::nbp_glo is the number of global continental points
          ier = NF90_PUT_ATT (Cforcing_id,NF90_GLOBAL, &
               &                        'kjpindex',REAL(nbp_glo,r_std))
          ier = NF90_PUT_ATT (Cforcing_id,NF90_GLOBAL, &
               &                        'nparan',REAL(nparan,r_std))
          ier = NF90_PUT_ATT (Cforcing_id,NF90_GLOBAL, &
               &                        'nbyear',REAL(nbyear,r_std))
          
          ! Add new dimension
          ier = NF90_DEF_DIM (Cforcing_id,'points',nbp_glo,d_id(1))
          ier = NF90_DEF_DIM (Cforcing_id,'carbtype',ncarb,d_id(2))
          ier = NF90_DEF_DIM (Cforcing_id,'vegtype',nvm,d_id(3))
          ier = NF90_DEF_DIM (Cforcing_id,'level',nlevs,d_id(4))
          ier = NF90_DEF_DIM (Cforcing_id,'time_step',NF90_UNLIMITED,d_id(5))
          
          ! Add new variable
          ier = NF90_DEF_VAR (Cforcing_id,'points',    r_typ,d_id(1),vid)
          ier = NF90_DEF_VAR (Cforcing_id,'carbtype',  r_typ,d_id(2),vid)
          ier = NF90_DEF_VAR (Cforcing_id,'vegtype',   r_typ,d_id(3),vid)
          ier = NF90_DEF_VAR (Cforcing_id,'level',     r_typ,d_id(4),vid)
          ier = NF90_DEF_VAR (Cforcing_id,'time_step', r_typ,d_id(5),vid)
          ier = NF90_DEF_VAR (Cforcing_id,'index',     r_typ,d_id(1),vid)
          ier = NF90_DEF_VAR (Cforcing_id,'clay',      r_typ,d_id(1),vid)
          ier = NF90_DEF_VAR (Cforcing_id,'silt',      r_typ,d_id(1),vid) 
          ier = NF90_DEF_VAR (Cforcing_id,'bulk',      r_typ,d_id(1),vid) 
          ier = NF90_DEF_VAR (Cforcing_id,'carbon_input',r_typ, &
               &                        (/ d_id(1),d_id(2),d_id(3),d_id(5) /),vid)
          ier = NF90_DEF_VAR (Cforcing_id,'nitrogen_input',r_typ, &
               &                        (/ d_id(1),d_id(2),d_id(3),d_id(5) /),vid)
          ier = NF90_DEF_VAR (Cforcing_id,'control_moist',r_typ, &
               &                        (/ d_id(1),d_id(4),d_id(5) /),vid)
          ier = NF90_DEF_VAR (Cforcing_id,'control_temp',r_typ, &
               &                        (/ d_id(1),d_id(4),d_id(5) /),vid)
          ier = NF90_DEF_VAR (Cforcing_id,'npp_equil',r_typ, &
               &                        (/ d_id(1),d_id(5) /),vid)

          ier = NF90_DEF_VAR (Cforcing_id,'drainage',r_typ, & 
               &                        (/ d_id(1),d_id(6),d_id(6) /),vid) 
          ier = NF90_ENDDEF (Cforcing_id)
          
          ! Given the name of a varaible, nf90_inq_varid finds the variable 
          ! ID (::vid). Put data value(s) into variable ::vid 
          ier = NF90_INQ_VARID (Cforcing_id,'points',vid)
          ier = NF90_PUT_VAR (Cforcing_id,vid, &
               &                          (/(REAL(i,r_std),i=1,nbp_glo)/))
          ier = NF90_INQ_VARID (Cforcing_id,'carbtype',vid)
          ier = NF90_PUT_VAR (Cforcing_id,vid, &
               &                        (/(REAL(i,r_std),i=1,ncarb)/))
          ier = NF90_INQ_VARID (Cforcing_id,'vegtype',vid)
          ier = NF90_PUT_VAR (Cforcing_id,vid, &
               &                            (/(REAL(i,r_std),i=1,nvm)/))
          ier = NF90_INQ_VARID (Cforcing_id,'level',vid)
          ier = NF90_PUT_VAR (Cforcing_id,vid, &
               &                          (/(REAL(i,r_std),i=1,nlevs)/))
          ier = NF90_INQ_VARID (Cforcing_id,'time_step',vid)
          ier = NF90_PUT_VAR (Cforcing_id,vid, &
               &                          (/(REAL(i,r_std),i=1,nparan*nbyear)/))
          ier = NF90_INQ_VARID (Cforcing_id,'index',vid)
          ier = NF90_PUT_VAR (Cforcing_id,vid, REAL(index_g,r_std) )
          ier = NF90_INQ_VARID (Cforcing_id,'clay',vid)
          ier = NF90_PUT_VAR (Cforcing_id,vid, clay_g )
          ier = NF90_INQ_VARID (Cforcing_id,'silt',vid) 
          ier = NF90_PUT_VAR (Cforcing_id,vid, silt_g ) 
          ier = NF90_INQ_VARID (Cforcing_id,'bulk',vid) 
          ier = NF90_PUT_VAR (Cforcing_id,vid, bulk_g ) 
          ier = NF90_INQ_VARID (Cforcing_id,'carbon_input',vid)
          ier = NF90_PUT_VAR (Cforcing_id,vid, carbon_input_g )
          ier = NF90_INQ_VARID (Cforcing_id,'nitrogen_input',vid)
          ier = NF90_PUT_VAR (Cforcing_id,vid, nitrogen_input_g )
          ier = NF90_INQ_VARID (Cforcing_id,'control_moist',vid)
          ier = NF90_PUT_VAR (Cforcing_id,vid, control_moist_g )
          ier = NF90_INQ_VARID (Cforcing_id,'control_temp',vid)
          ier = NF90_PUT_VAR (Cforcing_id,vid, control_temp_g )
          ier = NF90_INQ_VARID (Cforcing_id,'npp_equil',vid)
          ier = NF90_PUT_VAR (Cforcing_id,vid, npp_equil_g )
          
          ! Close netCDF
          ier = NF90_CLOSE (Cforcing_id)
          IF (ier /= NF90_NOERR) THEN
             CALL ipslerr_p (3,'stomate_finalize', &
                  &        'PROBLEM in closing Cforcing file', &
                  &        NF90_STRERROR(ier),'')
          END IF
          
          Cforcing_id = -1
       ENDIF

       ! Clear memory
       IF (is_root_prc) THEN
          DEALLOCATE(carbon_input_g)
          DEALLOCATE(nitrogen_input_g)
          DEALLOCATE(control_moist_g)
          DEALLOCATE(control_temp_g)
          DEALLOCATE(npp_equil_g)
       ENDIF
       
    ENDIF
  
  END SUBROUTINE stomate_finalize


!! ================================================================================================================================
!! SUBROUTINE 	: stomate_init
!!
!>\BRIEF        The routine is called only at the first simulation. At that 
!! time settings and flags are read and checked for internal consistency and 
!! memory is allocated for the variables in stomate.
!!
!! DESCRIPTION  : The routine reads the 
!! following flags from the run definition file:
!! -ipd (index of grid point for online diagnostics)\n
!! -ok_herbivores (flag to activate herbivores)\n
!! -treat_expansion (flag to activate PFT expansion across a pixel\n
!! -harvest_agri (flag to harvest aboveground biomass from agricultural PFTs)\n
!! \n
!! Check for inconsistent setting between the following flags:
!! -ok_stomate\n
!! -ok_dgvm\n
!! \n
!! Memory is allocated for all the variables of stomate and new indexing tables 
!! are build. New indexing tables are needed because a single pixel can conatin 
!! several PFTs. The new indexing tables have separate indices for the different 
!! PFTs. Similar index tables are build for land use cover change.\n
!! \n
!! Several global variables and land cover change variables are initialized to 
!! zero.\n
!!
!! RECENT CHANGE(S) : None
!!
!! MAIN OUTPUT VARIABLE(S): Strictly speaking the subroutine has no output 
!! variables. However, the routine allocates memory and builds new indexing 
!! variables for later use.\n 
!!
!! REFERENCE(S)	: None
!!
!! FLOWCHART    : None
!! \n
!_ ================================================================================================================================

  SUBROUTINE stomate_init &
       &  (kjpij, kjpindex, index, lalo, &
       &   rest_id_stom, hist_id_stom, hist_id_stom_IPCC)

  !! 0. Variable and parameter declaration

    !! 0.1 Input variables

    INTEGER(i_std),INTENT(in)                    :: kjpij             !! Total size of the un-compressed grid, including 
                                                                      !! oceans (unitless) 
    INTEGER(i_std),INTENT(in)                    :: kjpindex          !! Domain size - number of terrestrial pixels 
                                                                      !! (unitless) 
    INTEGER(i_std),INTENT(in)                    :: rest_id_stom      !! STOMATE's _Restart_ file identifier
    INTEGER(i_std),INTENT(in)                    :: hist_id_stom      !! STOMATE's _history_ file identifier
    INTEGER(i_std),INTENT(in)                    :: hist_id_stom_IPCC !! STOMATE's IPCC _history_ file identifier 
    INTEGER(i_std),DIMENSION(kjpindex),INTENT(in):: index             !! Indices of the terrestrial pixels on the global 
                                                                      !! map 
    REAL(r_std),DIMENSION(kjpindex,2),INTENT(in) :: lalo              !! Geogr. coordinates (latitude,longitude) (degrees)
   
    !! 0.2 Output variables

    !! 0.3 Modified variables

    !! 0.4 Local variables

    LOGICAL                                      :: l_error           !! Check errors in netcdf call
    INTEGER(i_std)                               :: ier               !! Check errors in netcdf call
    INTEGER(i_std)                               :: ji,j,ipd,l        !! Indices
    INTEGER(i_std)                               :: idia              !! indices
!_ ================================================================================================================================
    
  !! 1. Online diagnostics

    IF ( kjpindex > 0 ) THEN
       !Config  Key  = STOMATE_DIAGPT
       !Config  Desc = Index of grid point for online diagnostics
       !Config If    = OK_STOMATE
       !Config  Def  = 1
       !Config  Help = This is the index of the grid point which
       !               will be used for online diagnostics.
       !Config Units = [-]
       ! By default ::ipd is set to 1
       ipd = 1
       ! Get ::ipd from run definition file
       CALL getin_p('STOMATE_DIAGPT',ipd)
       ipd = MIN( ipd, kjpindex )
       WRITE(numout,*) 'Stomate: '
       WRITE(numout,*) '  Index of grid point for online diagnostics: ',ipd
       WRITE(numout,*) '  Lon, lat:',lalo(ipd,2),lalo(ipd,1)
       WRITE(numout,*) '  Index of this point on GCM grid: ',index(ipd)
       !
    ENDIF
    
  !! 2. Check consistency of flags

    IF ( ( .NOT. ok_stomate ) .AND. ok_dgvm ) THEN
       WRITE(numout,*) 'Cannot do dynamical vegetation without STOMATE.'
       WRITE(numout,*) 'Inconsistency between ::ok_stomate and ::ok_dgvm'
       WRITE(numout,*) 'Stop: fatal error'
       STOP
    ENDIF

  !! 3. Communicate settings

    WRITE(numout,*) 'stomate first call - overview of the activated flags:'
    WRITE(numout,*) '  STOMATE: ', ok_stomate
    WRITE(numout,*) '  LPJ: ', ok_dgvm
    
  !! 4. Allocate memory for STOMATE's variables

    l_error = .FALSE.

    ALLOCATE(veget_cov_max(kjpindex,nvm),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for veget_cov_max. We stop. We need kjpindex*nvm words',kjpindex,nvm
       STOP 'stomate_init'
    ENDIF

    ALLOCATE(adapted(kjpindex,nvm),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for adapted. We stop. We need kjpindex*nvm words',kjpindex,nvm
       STOP 'stomate_init'
    ENDIF

    ALLOCATE(regenerate(kjpindex,nvm),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for regenerate. We stop. We need kjpindex*nvm words',kjpindex,nvm
       STOP 'stomate_init'
    ENDIF

    ALLOCATE(humrel_daily(kjpindex,nvm),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for humrel_daily. We stop. We need kjpindex*nvm words',kjpindex,nvm
       STOP 'stomate_init'
    ENDIF

    ALLOCATE(vir_humrel_daily(kjpindex,nvm),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for vir_humrel_daily. We stop. We need kjpindex*nvm words',kjpindex,nvm
       CALL ipslerr_p (3,'stomate_init', 'Memory allocation issue','','')
    ENDIF
    vir_humrel_daily(:,:)=zero

    ALLOCATE(stressed_daily(kjpindex,nvm),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for stressed_daily. We stop. We need kjpindex*nvm words',kjpindex,nvm
       CALL ipslerr_p (3,'stomate_init', 'Memory allocation issue','','')
    ENDIF
    stressed_daily = zero

    ALLOCATE(unstressed_daily(kjpindex,nvm),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for unstressed_daily. We stop. We need kjpindex*nvm words',kjpindex,nvm
       CALL ipslerr_p (3,'stomate_init', 'Memory allocation issue','','')
    ENDIF
    unstressed_daily(:,:) = zero

    ALLOCATE(daylight(kjpindex,nvm),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for daylight. We stop. We need kjpindex*nvm words',kjpindex,nvm
       CALL ipslerr_p (3,'stomate_init', 'Memory allocation issue','','')
    ENDIF
    daylight(:,:) = zero

    ALLOCATE(light_tran_to_level_season(kjpindex,nvm,nlevels_tot),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for light_tran_to_level_season. We stop. We need kjpindex*nvm*nlevels_tot words', &
            kjpindex,nvm,nlevels_tot
       CALL ipslerr_p (3,'stomate_init', 'Memory allocation issue','','')
    ENDIF
    light_tran_to_level_season = zero

    ALLOCATE(daylight_count(kjpindex,nvm),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for daylight_count. We stop. We need kjpindex*nvm words',kjpindex,nvm
       CALL ipslerr_p (3,'stomate_init', 'Memory allocation issue','','')
    ENDIF
    daylight_count(:,:) = zero

    ALLOCATE(transpir_supply_daily(kjpindex,nvm),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for transpir_supply_daily. We stop. We need kjpindex*nvm words',kjpindex,nvm
       CALL ipslerr_p (3,'stomate_init', 'Memory allocation issue','','')
    ENDIF
    transpir_supply_daily=zero

    ALLOCATE(vir_transpir_supply_daily(kjpindex,nvm),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for vir_transpir_supply_daily. We stop. We need kjpindex*nvm words',kjpindex,nvm
       CALL ipslerr_p (3,'stomate_init', 'Memory allocation issue','','')
    ENDIF
    vir_transpir_supply_daily=zero

    ALLOCATE(transpir_daily(kjpindex,nvm),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for transpir_daily. We stop. We need kjpindex*nvm words',kjpindex,nvm
       CALL ipslerr_p (3,'stomate_init', 'Memory allocation issue','','')
    ENDIF
    transpir_daily=zero


    ALLOCATE(litterhum_daily(kjpindex),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for litterhum_daily. We stop. We need kjpindex words',kjpindex
       STOP 'stomate_init'
    ENDIF

    ALLOCATE(t2m_daily(kjpindex),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for t2m_daily. We stop. We need kjpindex words',kjpindex
       STOP 'stomate_init'
    ENDIF

    ALLOCATE(t2m_min_daily(kjpindex),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for t2m_min_daily. We stop. We need kjpindex words',kjpindex
       STOP 'stomate_init'
    ENDIF

    ALLOCATE(wind_speed_daily(kjpindex),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for wind_speed_daily. We stop. We need kjpindex words',kjpindex
       STOP 'stomate_init'
    ENDIF

    ALLOCATE(wind_max_daily(kjpindex),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for wind_max_daily. We stop. We need kjpindex words',kjpindex
       STOP 'stomate_init'
    ENDIF

    ALLOCATE(soil_temp_daily(kjpindex),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for soil_temp_daily. We stop. We need kjpindex words',kjpindex
       STOP 'stomate_init'
    ENDIF

    ALLOCATE(soil_max_daily(kjpindex),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for soil_max_daily. We stop. We need kjpindex words',kjpindex
       STOP 'stomate_init'
    ENDIF

    ALLOCATE(tsurf_daily(kjpindex),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for tsurf_daily. We stop. We need kjpindex words',kjpindex
       STOP 'stomate_init'
    ENDIF

    ALLOCATE(tsoil_daily(kjpindex,nbdl),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for tsoil_daily. We stop. We need kjpindex*nbdl words',kjpindex,nbdl
       STOP 'stomate_init'
    ENDIF

    ALLOCATE(soilhum_daily(kjpindex,nbdl),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for soilhum_daily. We stop. We need kjpindex*nbdl words',kjpindex,nbdl
       STOP 'stomate_init'
    ENDIF

    ALLOCATE(precip_daily(kjpindex),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for precip_daily. We stop. We need kjpindex words',kjpindex,nvm
       STOP 'stomate_init'
    ENDIF

    ALLOCATE(gpp_daily(kjpindex,nvm),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for gpp_daily. We stop. We need kjpindex*nvm words',kjpindex,nvm
       STOP 'stomate_init'
    ENDIF

    ALLOCATE(npp_daily(kjpindex,nvm),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for npp_daily. We stop. We need kjpindex*nvm words',kjpindex,nvm
       STOP 'stomate_init'
    ENDIF

    ALLOCATE(turnover_daily(kjpindex,nvm,nparts,nelements),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for turnover_daily. We stop. We need kjpindex*nvm*nparts*nelements words', &
       &   kjpindex,nvm,nparts,nelements
       STOP 'stomate_init'
    ENDIF

    ALLOCATE(turnover_littercalc(kjpindex,nvm,nparts,nelements),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for turnover_littercalc. We stop. We need kjpindex*nvm*nparts*nelements words', & 
        &  kjpindex,nvm,nparts,nelements
       STOP 'stomate_init'
    ENDIF

    ALLOCATE(humrel_month(kjpindex,nvm),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for humrel_month. We stop. We need kjpindex*nvm words',kjpindex,nvm
       STOP 'stomate_init'
    ENDIF

    ALLOCATE(humrel_week(kjpindex,nvm),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for humrel_week. We stop. We need kjpindex*nvm words',kjpindex,nvm
       STOP 'stomate_init'
    ENDIF

    ALLOCATE(humrel_growingseason(kjpindex,nvm),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for humrel_growingseason. We stop. We need kjpindex*nvm words',kjpindex,nvm
       STOP 'stomate_init'
    ENDIF

    ALLOCATE(vir_humrel_growingseason(kjpindex,nvm),stat=ier)
    l_error = l_error .OR. (ier /= 0)

    ALLOCATE(t2m_longterm(kjpindex),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for t2m_longterm. We stop. We need kjpindex*nvm words',kjpindex,nvm
       STOP 'stomate_init'
    ENDIF

    ALLOCATE(t2m_month(kjpindex),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for t2m_month. We stop. We need kjpindex words',kjpindex
       STOP 'stomate_init'
    ENDIF

    ALLOCATE(Tseason(kjpindex),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for Tseason. We stop. We need kjpindex words',kjpindex
       STOP 'stomate_init'
    ENDIF

    ALLOCATE(Tseason_length(kjpindex),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for Tseason_length. We stop. We need kjpindex words',kjpindex
       STOP 'stomate_init'
    ENDIF

    ALLOCATE(Tseason_tmp(kjpindex),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for Tseason_tmp. We stop. We need kjpindex words',kjpindex
       STOP 'stomate_init'
    ENDIF

    ALLOCATE(Tmin_spring_time(kjpindex,nvm),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for Tmin_spring_time. We stop. We need kjpindex*nvm words',kjpindex,nvm
       STOP 'stomate_init'
    ENDIF

    ALLOCATE(onset_date(kjpindex,nvm,2),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for onset_date. We stop. We need kjpindex*nvm*nparts words',kjpindex,nvm,2
       STOP 'stomate_init'
    ENDIF

    ALLOCATE(t2m_week(kjpindex),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for t2m_week. We stop. We need kjpindex words',kjpindex
       STOP 'stomate_init'
    ENDIF

    ALLOCATE(tsoil_month(kjpindex,nbdl),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for tsoil_month. We stop. We need kjpindex*nbdl words',kjpindex,nbdl
       STOP 'stomate_init'
    ENDIF

    ALLOCATE(soilhum_month(kjpindex,nbdl),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for soilhum_month. We stop. We need kjpindex*nbdl words',kjpindex,nbdl
       STOP 'stomate_init'
    ENDIF

    ALLOCATE(fireindex(kjpindex,nvm),stat=ier) 
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for fireindex. We stop. We need kjpindex*nvm words',kjpindex,nvm
       STOP 'stomate_init'
    ENDIF

    ALLOCATE(firelitter(kjpindex,nvm),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for firelitter. We stop. We need kjpindex*nvm words',kjpindex,nvm
       STOP 'stomate_init'
    ENDIF

    ALLOCATE(maxhumrel_lastyear(kjpindex,nvm),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for maxhumrel_lastyear. We stop. We need kjpindex*nvm words',kjpindex,nvm
       STOP 'stomate_init'
    ENDIF

    ALLOCATE(maxhumrel_thisyear(kjpindex,nvm),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for maxhumrel_thisyear. We stop. We need kjpindex*nvm words',kjpindex,nvm
       STOP 'stomate_init'
    ENDIF

    ALLOCATE(minhumrel_lastyear(kjpindex,nvm),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for minhumrel_lastyear. We stop. We need kjpindex*nvm words',kjpindex,nvm
       STOP 'stomate_init'
    ENDIF

    ALLOCATE(minhumrel_thisyear(kjpindex,nvm),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for minhumrel_thisyear. We stop. We need kjpindex*nvm words',kjpindex,nvm
       STOP 'stomate_init'
    ENDIF

    ALLOCATE(maxgppweek_lastyear(kjpindex,nvm),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for maxgppweek_lastyear. We stop. We need kjpindex*nvm words',kjpindex,nvm
       STOP 'stomate_init'
    ENDIF

    ALLOCATE(maxgppweek_thisyear(kjpindex,nvm),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for maxgppweek_thisyear. We stop. We need kjpindex*nvm words',kjpindex,nvm
       STOP 'stomate_init'
    ENDIF

    ALLOCATE(gdd0_lastyear(kjpindex),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for gdd0_lastyear. We stop. We need kjpindex words',kjpindex
       STOP 'stomate_init'
    ENDIF

    ALLOCATE(gdd0_thisyear(kjpindex),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for gdd0_thisyear. We stop. We need kjpindex words',kjpindex
       STOP 'stomate_init'
    ENDIF

    ALLOCATE(gdd_init_date(kjpindex,2),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for gdd_init_date. We stop. We need kjpindex*2 words',kjpindex,2
       STOP 'stomate_init'
    ENDIF

    ALLOCATE(gdd_from_growthinit(kjpindex,nvm),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for gdd_from_growthinit. We stop. We need kjpindex*nvm words',kjpindex,nvm
       STOP 'stomate_init'
    ENDIF

    ALLOCATE(precip_lastyear(kjpindex),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for precip_lastyear. We stop. We need kjpindex*nvm words',kjpindex
       STOP 'stomate_init'
    ENDIF

    ALLOCATE(precip_thisyear(kjpindex),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for precip_thisyear. We stop. We need kjpindex words',kjpindex
       STOP 'stomate_init'
    ENDIF

    ALLOCATE(gdd_m5_dormance(kjpindex,nvm),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for gdd_m5_dormance. We stop. We need kjpindex*nvm words',kjpindex,nvm
       STOP 'stomate_init'
    ENDIF

    ALLOCATE(gdd_midwinter(kjpindex,nvm),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for gdd_midwinter. We stop. We need kjpindex*nvm words',kjpindex,nvm
       STOP 'stomate_init'
    ENDIF

    ALLOCATE(ncd_dormance(kjpindex,nvm),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for ncd_dormance. We stop. We need kjpindex*nvm words',kjpindex,nvm
       STOP 'stomate_init'
    ENDIF

    ALLOCATE(ngd_minus5(kjpindex,nvm),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for ngd_minus5. We stop. We need kjpindex*nvm words',kjpindex,nvm
       STOP 'stomate_init'
    ENDIF

    ALLOCATE(PFTpresent(kjpindex,nvm),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for PFTpresent. We stop. We need kjpindex*nvm words',kjpindex,nvm
       STOP 'stomate_init'
    ENDIF

    ALLOCATE(npp_longterm(kjpindex,nvm),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for npp_longterm. We stop. We need kjpindex*nvm words',kjpindex,nvm
       STOP 'stomate_init'
    ENDIF

    ALLOCATE(lm_lastyearmax(kjpindex,nvm),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for lm_lastyearmax. We stop. We need kjpindex*nvm words',kjpindex,nvm
       STOP 'stomate_init'
    ENDIF

    ALLOCATE(lm_thisyearmax(kjpindex,nvm),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for lm_thisyearmax. We stop. We need kjpindex*nvm words',kjpindex,nvm
       STOP 'stomate_init'
    ENDIF

    ALLOCATE(maxfpc_lastyear(kjpindex,nvm),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for maxfpc_lastyear. We stop. We need kjpindex*nvm words',kjpindex,nvm
       STOP 'stomate_init'
    ENDIF

    ALLOCATE(maxfpc_thisyear(kjpindex,nvm),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for maxfpc_thisyear. We stop. We need kjpindex*nvm words',kjpindex,nvm
       STOP 'stomate_init'
    ENDIF

    ALLOCATE(turnover_longterm(kjpindex,nvm,nparts,nelements),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for turnover_longterm. We stop. We need kjpindex*nvm*nparts*nelements words', & 
       &    kjpindex,nvm,nparts,nelements
       STOP 'stomate_init'
    ENDIF

    ALLOCATE(gpp_week(kjpindex,nvm),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for gpp_week. We stop. We need kjpindex*nvm words',kjpindex,nvm
       STOP 'stomate_init'
    ENDIF

    ALLOCATE(plant_status(kjpindex,nvm),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for plant_status. We stop. We need kjpindex*nvm words',kjpindex,nvm
       STOP 'stomate_init'
    ENDIF

    ALLOCATE(when_growthinit(kjpindex,nvm),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for when_growthinit. We stop. We need kjpindex*nvm words',kjpindex,nvm
       STOP 'stomate_init'
    ENDIF

    ALLOCATE(age(kjpindex,nvm),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for age. We stop. We need kjpindex*nvm words',kjpindex,nvm
       STOP 'stomate_init'
    ENDIF

    ALLOCATE(resp_hetero_d(kjpindex,nvm),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for resp_hetero_d. We stop. We need kjpindex*nvm words',kjpindex,nvm
       STOP 'stomate_init'
    ENDIF

    ALLOCATE(resp_hetero_radia(kjpindex,nvm),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for resp_hetero_radia. We stop. We need kjpindex*nvm words',kjpindex,nvm
       STOP 'stomate_init'
    ENDIF

    ALLOCATE(resp_maint_d(kjpindex,nvm),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for resp_maint_d. We stop. We need kjpindex*nvm words',kjpindex,nvm
       STOP 'stomate_init'
    ENDIF

    ALLOCATE(resp_growth_d(kjpindex,nvm),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for resp_growth_d. We stop. We need kjpindex*nvm words',kjpindex,nvm
       STOP 'stomate_init'
    ENDIF

    ALLOCATE(co2_fire(kjpindex,nvm),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for co2_fire. We stop. We need kjpindex*nvm words',kjpindex,nvm
       STOP 'stomate_init'
    ENDIF

    ALLOCATE(atm_to_bm(kjpindex,nvm,nelements),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for atm_to_bm. We stop. We need kjpindex*nvm words',kjpindex,nvm,nelements
       STOP 'stomate_init'
    ENDIF

    ALLOCATE(veget_lastlight(kjpindex,nvm),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for veget_lastlight. We stop. We need kjpindex*nvm words',kjpindex,nvm
       STOP 'stomate_init'
    ENDIF

    ALLOCATE(everywhere(kjpindex,nvm),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for everywhere. We stop. We need kjpindex*nvm words',kjpindex,nvm
       STOP 'stomate_init'
    ENDIF

    ALLOCATE(need_adjacent(kjpindex,nvm),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for need_adjacent. We stop. We need kjpindex*nvm words',kjpindex,nvm
       STOP 'stomate_init'
    ENDIF

    ALLOCATE(leaf_age(kjpindex,nvm,nleafages),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for leaf_age. We stop. We need kjpindex*nvm*nleafages words', & 
       &      kjpindex,nvm,nleafages
       STOP 'stomate_init'
    ENDIF

    ALLOCATE(leaf_frac(kjpindex,nvm,nleafages),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for leaf_frac. We stop. We need kjpindex*nvm*nleafages words', & 
       &      kjpindex,nvm,nleafages
       STOP 'stomate_init'
    ENDIF

    ALLOCATE(RIP_time(kjpindex,nvm),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for RIP_time. We stop. We need kjpindex*nvm words',kjpindex,nvm
       STOP 'stomate_init'
    ENDIF

    ALLOCATE(time_hum_min(kjpindex,nvm),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for time_hum_min. We stop. We need kjpindex*nvm words',kjpindex,nvm
       STOP 'stomate_init'
    ENDIF

    ALLOCATE(hum_min_dormance(kjpindex,nvm),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for hum_min_dormance. We stop. We need kjpindex*nvm words',kjpindex,nvm
       STOP 'stomate_init'
    ENDIF


    ALLOCATE(litter(kjpindex,nlitt,nvm,nlevs,nelements),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for litter. We stop. We need kjpindex*nlitt*nvm*nlevs*nelements words', & 
       &    kjpindex,nlitt,nvm,nlevs,nelements
       STOP 'stomate_init'
    ENDIF

    ALLOCATE(dead_leaves(kjpindex,nvm,nlitt),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for dead_leaves. We stop. We need kjpindex*nvm*nlitt words', & 
       &   kjpindex,nvm,nlitt
       STOP 'stomate_init'
    ENDIF

    ALLOCATE(som(kjpindex,ncarb,nvm,nelements),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for som. We stop. We need kjpindex*ncarb*nvm*nelements words', &
            kjpindex,ncarb,nvm,nelements
       STOP 'stomate_init'
    ENDIF

    ALLOCATE(lignin_struc(kjpindex,nvm,nlevs),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for lignin_struc. We stop. We need kjpindex*nvm*nlevs words', &
            kjpindex,nvm,nlevs
       STOP 'stomate_init'
    ENDIF

    ALLOCATE(lignin_wood(kjpindex,nvm,nlevs),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for lignin_wood. We stop. We need kjpindex*nvm*nlevs words', &
            kjpindex,nvm,nlevs
       STOP 'stomate_init'
    ENDIF

    ALLOCATE(turnover_time(kjpindex,nvm,nparts),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for turnover_time. We stop. We need kjpindex*nvm words', &
            kjpindex,nvm
       STOP 'stomate_init'
    ENDIF

!!$    ALLOCATE(co2_flux_daily(kjpindex,nvm),stat=ier)
!!$    l_error = l_error .OR. (ier /= 0)
!!$    IF (l_error) THEN
!!$       WRITE(numout,*) 'Memory allocation error for co2_flux_daily. We stop. We need kjpindex*nvm words',kjpindex,nvm
!!$       STOP 'stomate_init'
!!$    ENDIF

!!$    ALLOCATE(co2_flux_monthly(kjpindex,nvm),stat=ier)
!!$    l_error = l_error .OR. (ier /= 0)
!!$    IF (l_error) THEN
!!$       WRITE(numout,*) 'Memory allocation error for co2_flux_monthly. We stop. We need kjpindex*nvm words',kjpindex,nvm
!!$       STOP 'stomate_init'
!!$    ENDIF
!!$
!!$    ALLOCATE (flux_prod_monthly(kjpindex), stat=ier)
!!$    l_error = l_error .OR. (ier /= 0)
!!$    IF (l_error) THEN
!!$       WRITE(numout,*) 'Memory allocation error for flux_prod_monthly. We stop. We need kjpindex words',kjpindex
!!$       STOP 'stomate_init'
!!$    ENDIF
!!$ 
!!$    ALLOCATE (harvest_above_monthly(kjpindex), stat=ier)
!!$    l_error = l_error .OR. (ier /= 0)
!!$    IF (l_error) THEN
!!$       WRITE(numout,*) 'Memory allocation error for harvest_above_monthly. We stop. We need kjpindex words',kjpindex
!!$       STOP 'stomate_init'
!!$    ENDIF

    ALLOCATE(bm_to_litter(kjpindex,nvm,nparts,nelements),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for bm_to_litter. We stop. We need kjpindex*nvm*nparts*nelements words', & 
       &    kjpindex,nvm,nparts,nelements
       STOP 'stomate_init'
    ENDIF

    ALLOCATE(bm_to_littercalc(kjpindex,nvm,nparts,nelements),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for bm_to_littercalc. We stop. We need kjpindex*nvm*nparts*nelements words', &
       &   kjpindex,nvm,nparts,nelements
       STOP 'stomate_init'
    ENDIF

    ALLOCATE(herbivores(kjpindex,nvm),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for herbivores. We stop. We need kjpindex*nvm words',kjpindex,nvm
       STOP 'stomate_init'
    ENDIF

        ALLOCATE(resp_maint_part_radia(kjpindex,nvm,nparts),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for resp_maint_part_radia. We stop. We need kjpindex*nvm*nparts words', &
       &  kjpindex,nvm,nparts
       STOP 'stomate_init'
    ENDIF

    ALLOCATE(resp_maint_radia(kjpindex,nvm),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for resp_maint_radia. We stop. We need kjpindex*nvm words',kjpindex,nvm
       STOP 'stomate_init'
    ENDIF

    ALLOCATE(resp_maint_part(kjpindex,nvm,nparts),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for resp_maint_part. We stop. We need kjpindex*nvm*nparts words', &
       &    kjpindex,nvm,nparts
       STOP 'stomate_init'
    ENDIF
    resp_maint_part(:,:,:) = zero

    ALLOCATE(hori_index(kjpindex),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for hori_index. We stop. We need kjpindex words',kjpindex
       STOP 'stomate_init'
    ENDIF

    ALLOCATE(horipft_index(kjpindex*nvm),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for horipft_index. We stop. We need kjpindex*nvm words',kjpindex*nvm
       STOP 'stomate_init'
    ENDIF

    ALLOCATE(horican_index(kjpindex*nlevels_tot),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for horican_index. We stop. We need kjpindex*nlevels_tot words',&
            kjpindex*nlevels_tot
       CALL ipslerr_p (3,'stomate_init', 'Memory allocation issue','','')
    ENDIF

    ALLOCATE(horicut_index(kjpindex*ncut_times),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for horicut_index. We stop. We need kjpindex*ncut_times words',&
            kjpindex*ncut_times
       CALL ipslerr_p (3,'stomate_init', 'Memory allocation issue','','')
    ENDIF

    ALLOCATE (horip_s_index(kjpindex*nshort), stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for horip_s_index. We stop. We need kjpindex*10 words',kjpindex,nshort
       CALL ipslerr_p (3,'stomate_init', 'Memory allocation issue','','')
    ENDIF

    ALLOCATE (horip_m_index(kjpindex*nmedium), stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for horip_m_index. We stop. We need kjpindex*10 words',kjpindex,nmedium
       CALL ipslerr_p (3,'stomate_init', 'Memory allocation issue','','')
    ENDIF

    ALLOCATE (horip_l_index(kjpindex*100), stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for horip_l_index. We stop. We need kjpindex*100 words',kjpindex,nlong
       CALL ipslerr_p (3,'stomate_init', 'Memory allocation issue','','')
    ENDIF
 
    ALLOCATE (horip_ss_index(kjpindex*(nshort+1)), stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for horip_ss_index. We stop. We need kjpindex*11 words',kjpindex,nshort+1
       CALL ipslerr_p (3,'stomate_init', 'Memory allocation issue','','')
    ENDIF 

    ALLOCATE (horip_mm_index(kjpindex*(nmedium+1)), stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for horip_mm_index. We stop. We need kjpindex*11 words',kjpindex,nmedium+1
       CALL ipslerr_p (3,'stomate_init', 'Memory allocation issue','','')
    ENDIF

    ALLOCATE (horip_ll_index(kjpindex*(nlong+1)), stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for horip_ll_index. We stop. We need kjpindex*101 words',kjpindex,nlong+1
       CALL ipslerr_p (3,'stomate_init', 'Memory allocation issue','','')
    ENDIF

    ALLOCATE (prod_s(kjpindex,0:nshort,nelements), stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for prod_s. We stop. We need kjpindex*(nshort+1)*nelements words', &
            kjpindex,nshort+1,nelements
       CALL ipslerr_p (3,'stomate_init', 'Memory allocation issue','','')
    ENDIF  

    ALLOCATE (prod_m(kjpindex,0:nmedium,nelements), stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for prod_m. We stop. We need kjpindex*(nmedium+1)*nelements words', &
            kjpindex,nmedium+1,nelements
       CALL ipslerr_p (3,'stomate_init', 'Memory allocation issue','','')
    ENDIF 

    ALLOCATE (prod_l(kjpindex,0:nlong,nelements), stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for prod_l. We stop. We need kjpindex*(nlong+1)*nelements words', &
            kjpindex,nlong+1,nelements
       CALL ipslerr_p (3,'stomate_init', 'Memory allocation issue','','')
    ENDIF

    ALLOCATE (flux_s(kjpindex,nshort,nelements), stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for flux_s. We stop. We need kjpindex*nshort*nelements words', &
            kjpindex,nshort,nelements
       CALL ipslerr_p (3,'stomate_init', 'Memory allocation issue','','')
    ENDIF

    ALLOCATE (flux_m(kjpindex,nmedium,nelements), stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for flux_m. We stop. We need kjpindex*nmedium words', &
            kjpindex,nmedium,nelements
       CALL ipslerr_p (3,'stomate_init', 'Memory allocation issue','','')
    ENDIF

    ALLOCATE (flux_l(kjpindex,nlong,nelements), stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for flux_l. We stop. We need kjpindex*nlong*nelements words', &
            kjpindex,nlong,nelements
       CALL ipslerr_p (3,'stomate_init', 'Memory allocation issue','','')
    ENDIF

    ALLOCATE (flux_prod_s(kjpindex,nelements), stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for flux_prod_s. We stop. We need kjpindex*nelements words',kjpindex,nelements
       CALL ipslerr_p (3,'stomate_init', 'Memory allocation issue','','')
    ENDIF

    ALLOCATE (flux_prod_m(kjpindex,nelements), stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for flux_prod_m. We stop. We need kjpindex*nelements words',kjpindex,nelements
       CALL ipslerr_p (3,'stomate_init', 'Memory allocation issue','','')
    ENDIF

    ALLOCATE (flux_prod_l(kjpindex,nelements), stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for flux_prod_l. We stop. We need kjpindex*nelements words',kjpindex,nelements
       CALL ipslerr_p (3,'stomate_init', 'Memory allocation issue','','')
    ENDIF

    ALLOCATE (carb_mass_total(kjpindex), stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for carb_mass_total. We stop. We need kjpindex words',kjpindex
       STOP 'stomate_init'
    ENDIF

    ALLOCATE (som_input_daily(kjpindex,ncarb,nvm,nelements), stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for som_input_daily. We stop. We need kjpindex*ncarb*nvm*nelements words', & 
       &    kjpindex,ncarb,nvm,nelements
       STOP 'stomate_init'
    ENDIF

    ALLOCATE (control_temp_daily(kjpindex,nlevs), stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for control_temp_daily. We stop. We need kjpindex*nlevs words',kjpindex,nlevs
       STOP 'stomate_init'
    ENDIF

    ALLOCATE (control_moist_daily(kjpindex,nlevs), stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for control_moist_daily. We stop. We need kjpindex*nlevs words',kjpindex,nlevs
       STOP 'stomate_init'
    ENDIF

    ALLOCATE (fpc_max(kjpindex,nvm), stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for fpc_max. We stop. We need kjpindex*nvm words',kjpindex,nvm
       STOP 'stomate_init'
    ENDIF

    ALLOCATE(cn_leaf_min_season(kjpindex,nvm),stat=ier) 
    l_error = l_error .OR. (ier /= 0) 
    IF (l_error) THEN 
       WRITE(numout,*) 'Memory allocation error for cn_leaf_min_season. We stop. We need kjpindex*nvm words',kjpindex,nvm 
       STOP 'stomate_init' 
    ENDIF 
 
    ALLOCATE(nstress_season(kjpindex,nvm),stat=ier) 
    l_error = l_error .OR. (ier /= 0) 
    IF (l_error) THEN 
       WRITE(numout,*) 'Memory allocation error for nstress_season. We stop. We need kjpindex*nvm words',kjpindex,nvm 
       STOP 'stomate_init' 
    ENDIF 
 
    ALLOCATE(soil_n_min(kjpindex,nvm,nnspec),stat=ier) 
    l_error = l_error .OR. (ier /= 0) 
    IF (l_error) THEN 
       WRITE(numout,*) 'Memory allocation error for soil_n_min. We stop. We need kjpindex*nvm words',kjpindex,nvm,nnspec 
       STOP 'stomate_init' 
    ENDIF 

    ALLOCATE(p_O2(kjpindex,nvm),stat=ier) 
    l_error = l_error .OR. (ier /= 0) 
    IF (l_error) THEN 
       WRITE(numout,*) 'Memory allocation error for p_O2. We stop. We need kjpindex*nvm words',kjpindex,nvm 
       STOP 'stomate_init' 
    ENDIF 

    ALLOCATE(bact(kjpindex,nvm),stat=ier) 
    l_error = l_error .OR. (ier /= 0) 
    IF (l_error) THEN 
       WRITE(numout,*) 'Memory allocation error for bact. We stop. We need kjpindex*nvm words',kjpindex,nvm 
       STOP 'stomate_init' 
    ENDIF 

    ALLOCATE(ok_equilibrium(kjpindex),stat=ier)
    l_error = l_error .OR. (ier /= 0) 
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for ok_equilibrium. We stop. We need kjpindex words',kjpindex
       STOP 'stomate_init'
    ENDIF

    ALLOCATE(drainage_daily(kjpindex,nvm),stat=ier) 
    l_error = l_error .OR. (ier /= 0) 
    IF (l_error) THEN 
       WRITE(numout,*) ' Memory allocation error for drainage_daily. We stop. We need kjpindex words = ',kjpindex,nvm 
       STOP 'drainage_daily' 
    ENDIF 
 
    ALLOCATE (plant_n_uptake_daily(kjpindex,nvm,nionspec), stat=ier) 
    l_error = l_error .OR. (ier.NE.0) 
    IF (l_error) THEN 
       WRITE(numout,*) ' Memory allocation error for plant_n_uptake_daily. We stop. We need kjpindex words = ',kjpindex*nvm*nionspec 
       STOP 'plant_n_uptake_daily' 
    ENDIF 
 
    ALLOCATE (n_mineralisation_d(kjpindex,nvm), stat=ier) 
    l_error = l_error .OR. (ier.NE.0) 
    IF (l_error) THEN 
       WRITE(numout,*) ' Memory allocation error for n_mineralisation_d. We stop. We need kjpindex words = ',kjpindex*nvm 
       STOP 'n_mineralisation_d' 
    ENDIF 

    ALLOCATE(carbon_eq(kjpindex),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for carbon_eq. We stop. We need kjpindex words',kjpindex
       STOP 'stomate_init'
    ENDIF

    ALLOCATE(nbp_accu(kjpindex),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for nbp_accu. We stop. We need kjpindex words',kjpindex
       STOP 'stomate_init'
    ENDIF

    ALLOCATE(nbp_accu_pool(kjpindex,nvm),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for nbp_accu_pool. We stop. We need kjpindex*nvm words',kjpindex*nvm
       STOP 'stomate_init'
    ENDIF


    ALLOCATE(nbp_flux(kjpindex),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for nbp_flux. We stop. We need kjpindex words',kjpindex
       STOP 'stomate_init'
    ENDIF

    ALLOCATE(nbp_pool(kjpindex,nvm),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for nbp_pool. We stop. We need kjpindex*nvm words',kjpindex*nvm
       STOP 'stomate_init'
    ENDIF

    ALLOCATE(nbp_pool_start(kjpindex,nvm),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for nbp_pool_start. We stop. We need kjpindex*nvm words',kjpindex*nvm
       STOP 'stomate_init'
    ENDIF

    ALLOCATE(nbp_pool_end(kjpindex,nvm),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for nbp_pool_end. We stop. We need kjpindex*nvm words',kjpindex*nvm
       STOP 'stomate_init'
    ENDIF

    ALLOCATE(matrixA(kjpindex,nvm,nbpools,nbpools),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for matrixA. We stop. We need kjpindex*nvm*nbpools*nbpools words',  & 
       &     kjpindex, nvm, nbpools, nbpools
       STOP 'stomate_init'
    ENDIF

    ALLOCATE(vectorB(kjpindex,nvm,nbpools),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for vectorB. We stop. We need kjpindex*nvm*nbpools words',  & 
       &     kjpindex, nvm, nbpools
       STOP 'stomate_init'
    ENDIF

    ALLOCATE(vectorU(kjpindex,nvm,nbpools),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for vectorU. We stop. We need kjpindex*nvm*nbpools words',  & 
       &     kjpindex, nvm, nbpools
       STOP 'stomate_init'
    ENDIF

    ALLOCATE(matrixV(kjpindex,nvm,nbpools,nbpools),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for matrixV. We stop. We need kjpindex*nvm*nbpools*nbpools words',  & 
       &     kjpindex, nvm, nbpools, nbpools
       STOP 'stomate_init'
    ENDIF

    ALLOCATE(matrixW(kjpindex,nvm,nbpools,nbpools),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for matrixW. We stop. We need kjpindex*nvm*nbpools*nbpools words',  & 
       &     kjpindex, nvm, nbpools, nbpools
       STOP 'stomate_init'
    ENDIF

    ALLOCATE(previous_stock(kjpindex,nvm,nbpools),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for previous_stock. We stop. We need kjpindex*nvm*nbpools words',  & 
       &     kjpindex, nvm, nbpools
       STOP 'stomate_init'
    ENDIF

    ALLOCATE(current_stock(kjpindex,nvm,nbpools),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for current_stock. We stop. We need kjpindex*nvm*nbpools words',  & 
       &     kjpindex, nvm, nbpools
       STOP 'stomate_init'
    ENDIF

    ALLOCATE(CN_som_litter_longterm(kjpindex,nvm,nbpools),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for CN_som_litter_longterm. We stop. We need kjpindex*nvm*nbpools words',  & 
       &     kjpindex, nvm, nbpools
       STOP 'stomate_init'
    ENDIF
    
    ALLOCATE(KF(kjpindex,nvm),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) ' Memory allocation error for KF. We stop. We need nvm words = ',kjpindex*nvm
       CALL ipslerr_p (3,'stomate_init', 'Memory allocation issue','','')
    ENDIF
    KF(:,:) = zero ! Is there a better place in the code for this?

    ALLOCATE(k_latosa_adapt(kjpindex,nvm),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) ' Memory allocation error for k_latosa_adapt. We stop. We need nvm words = ',kjpindex*nvm
       CALL ipslerr_p (3,'stomate_init', 'Memory allocation issue','','')
    ENDIF

    ALLOCATE(harvest_pool(kjpindex,nvm,ndia_harvest+1,nelements),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) ' Memory allocation error for harvest_pool. We stop. We need many words = ',&
            kjpindex*nvm*(ndia_harvest+1)*nelements
       CALL ipslerr_p (3,'stomate_init', 'Memory allocation issue','','')
    ENDIF
    harvest_pool(:,:,:,:) = zero

    ALLOCATE(harvest_type(kjpindex,nvm),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) ' Memory allocation error for harvest_type. We stop. We need many words = ',&
            kjpindex*nvm
       CALL ipslerr_p (3,'stomate_init', 'Memory allocation issue','','')
    ENDIF
    harvest_type(:,:) = zero

    ALLOCATE(harvest_cut(kjpindex,nvm),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) ' Memory allocation error for harvest_cut. We stop. We need many words = ',&
            kjpindex*nvm
       CALL ipslerr_p (3,'stomate_init', 'Memory allocation issue','','')
    ENDIF
    harvest_cut(:,:) = zero

    ALLOCATE(harvest_area(kjpindex,nvm),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) ' Memory allocation error for harvest_area. We stop. We need many words = ',&
            kjpindex*nvm
       CALL ipslerr_p (3,'stomate_init', 'Memory allocation issue','','')
    ENDIF
    harvest_area(:,:) = zero

    ALLOCATE(harvest_5y_area(kjpindex,nvm,wind_years),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) ' Memory allocation error for harvest_5y_area. We stop. We need many words = ',&
            kjpindex*nvm*wind_years
       CALL ipslerr_p (3,'stomate_init', 'Memory allocation issue','','')
    ENDIF
    harvest_5y_area(:,:,:)=zero

    ALLOCATE(harvest_pool_bound(0:ndia_harvest+1),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) ' Memory allocation error for harvest_pool_bound. ' // &
            'We stop. We need ndia_harvest+2 words = ',&
            ndia_harvest+2
       CALL ipslerr_p (3,'stomate_init', 'Memory allocation issue','','')
    ENDIF

    ! Here we can initialize the values of this array, too. They 
    ! should never change over the course of the simulation.
    harvest_pool_bound(ndia_harvest+1) = val_exp
    DO idia = 0,ndia_harvest
       harvest_pool_bound(idia) = max_harvest_dia * &
            REAL(idia,r_std) / REAL(ndia_harvest,r_std)
    ENDDO

    ALLOCATE(mai(kjpindex,nvm),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) ' Memory allocation error for mai. ' // &
            'We stop. We need kjpindex*nvm words = ',&
            kjpindex*nvm
       CALL ipslerr_p (3,'stomate_init', 'Memory allocation issue','','')
    ENDIF

    ALLOCATE(pai(kjpindex,nvm),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) ' Memory allocation error for pai. ' // &
            'We stop. We need kjpindex*nvm words = ',&
            kjpindex*nvm
       CALL ipslerr_p (3,'stomate_init', 'Memory allocation issue','','')
    ENDIF

    ALLOCATE(previous_wood_volume(kjpindex,nvm),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) ' Memory allocation error for previous_wood_volume. ' // &
            'We stop. We need kjpindex*nvm words = ',&
            kjpindex*nvm
       CALL ipslerr_p (3,'stomate_init', 'Memory allocation issue','','')
    ENDIF
    
    ALLOCATE(mai_count(kjpindex,nvm),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for mai_count. We stop. We need kjpindex*nvm words',  & 
       &     kjpindex, nvm
       CALL ipslerr_p (3,'stomate_init', 'Memory allocation issue','','')
    ENDIF

    ALLOCATE(coppice_dens(kjpindex,nvm),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) ' Memory allocation error for coppice_dens. ' // &
            'We stop. We need kjpindex*nvm words = ',&
            kjpindex*nvm
       CALL ipslerr_p (3,'stomate_init', 'Memory allocation issue','','')
    ENDIF

    ALLOCATE (rue_longterm(kjpindex,nvm), stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for rue_longterm. We stop. We need kjpindex*nlevs words',kjpindex,nvm
       CALL ipslerr_p (3,'stomate_init', 'Memory allocation issue','','')
    ENDIF
    rue_longterm(:,:) = un

    ALLOCATE (lab_fac(kjpindex,nvm), stat=ier)
    ! temp fix, initializing it here...the values get set in stomate_growth_fun_all,
    ! but they get used in stomate_resp beforehand, so maybe a code flow problem?
    lab_fac(:,:)=zero
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for lab_fac. We stop. We need kjpindex*nlevs words',kjpindex,nvm
       CALL ipslerr_p (3,'stomate_init', 'Memory allocation issue','','')
    ENDIF

    ALLOCATE(forest_managed(kjpindex,nvm),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for forest_managed. We stop. We need kjpindex*nvm words',  & 
       &     kjpindex, nvm
       CALL ipslerr_p (3,'stomate_init', 'Memory allocation issue','','')
    ENDIF

    ALLOCATE(forest_managed_lastyear(kjpindex,nvm),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for forest_managed_lastyear. We stop. We need kjpindex*nvm words',  & 
       &     kjpindex, nvm
       CALL ipslerr_p (3,'stomate_init', 'Memory allocation issue','','')
    ENDIF

    ALLOCATE(species_change_map(kjpindex,nvm),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for species_change_map. We stop. We need kjpindex*nvm words',  & 
       &     kjpindex, nvm
       CALL ipslerr_p (3,'stomate_init', 'Memory allocation issue','','')
    ENDIF
    species_change_map(:,:)=0

    ALLOCATE(fm_change_map(kjpindex,nvm),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for fm_change_map. We stop. We need kjpindex*nvm words',  & 
       &     kjpindex, nvm
       CALL ipslerr_p (3,'stomate_init', 'Memory allocation issue','','')
    ENDIF
    fm_change_map(:,:)=0

    ALLOCATE(lpft_replant(kjpindex,nvm),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for lpft_replant. We stop. We need kjpindex*nvm words',  & 
       &     kjpindex, nvm
       CALL ipslerr_p (3,'stomate_init', 'Memory allocation issue','','')
    ENDIF
    lpft_replant(:,:)=.FALSE.

        ALLOCATE(age_stand(kjpindex,nvm),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for age_stand. We stop. We need kjpindex*nvm words',  & 
       &     kjpindex, nvm
       CALL ipslerr_p (3,'stomate_init', 'Memory allocation issue','','')
    ENDIF

    ALLOCATE(rotation_n(kjpindex,nvm),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for rotation_n. We stop. We need kjpindex*nvm words',  & 
       &     kjpindex, nvm
       CALL ipslerr_p (3,'stomate_init', 'Memory allocation issue','','')
    ENDIF

    ALLOCATE(last_cut(kjpindex,nvm),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for last_cut. We stop. We need kjpindex*nvm words',  & 
       &     kjpindex, nvm
       CALL ipslerr_p (3,'stomate_init', 'Memory allocation issue','','')
    ENDIF

    ALLOCATE(sigma(kjpindex,nvm),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Memory allocation error for sigma. We stop. We need kjpindex*nvm words',  & 
       &     kjpindex, nvm
       CALL ipslerr_p (3,'stomate_init', 'Memory allocation issue','','')
    ENDIF

    ALLOCATE(litter_demand(kjpindex),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) ' Memory allocation error for litter_demand. ' // &
            'We stop. We need kjpindex words = ',&
            kjpindex
       CALL ipslerr_p (3,'stomate_init', 'Memory allocation issue','','')
    ENDIF

    ALLOCATE(wstress_season(kjpindex,nvm),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) ' Memory allocation error for wstress_season. ' // &
            'We stop. We need kjpindex*nvm words = ',&
            kjpindex, nvm
       CALL ipslerr_p (3,'stomate_init', 'Memory allocation issue','','')
    ENDIF

    ALLOCATE(wstress_month(kjpindex,nvm),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) ' Memory allocation error for wstress_month. ' // &
            'We stop. We need kjpindex*nvm words = ',&
            kjpindex, nvm
       CALL ipslerr_p (3,'stomate_init', 'Memory allocation issue','','')
    ENDIF

    ALLOCATE(store_sum_delta_ba(kjpindex,nvm,ncirc),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) ' Memory allocation error for st_dist. ' // &
            'We need kjpindex*nvm*ncirc words = ',kjpindex,nvm,ncirc
       CALL ipslerr_p (3,'stomate_main', &
            ' Memory allocation error for store_sum_delta_ba.','','')
    END IF

  !! 5. File definitions

    ! Store history and restart files in common variables
    hist_id_stomate = hist_id_stom
    hist_id_stomate_IPCC = hist_id_stom_IPCC
    rest_id_stomate = rest_id_stom
    
    ! In STOMATE reduced grids are used containing only terrestrial pixels.
    ! Build a new indexing table for the vegetation fields separating 
    ! between the different PFTs. Note that ::index has dimension (kjpindex) 
    ! wheras ::indexpft has dimension (kjpindex*nvm). 

    hori_index(:) = index(:)

    DO j = 1, nvm
       DO ji = 1, kjpindex
          horipft_index((j-1)*kjpindex+ji) = index(ji)+(j-1)*kjpij + offset_omp - offset_mpi
       ENDDO
    ENDDO

     DO j = 1, nlevels_tot
       DO ji = 1, kjpindex
          horican_index((j-1)*kjpindex+ji) = index(ji)+(j-1)*kjpij + offset_omp - offset_mpi
       ENDDO
    ENDDO

    DO j = 1, ncut_times
       DO ji = 1, kjpindex
          horicut_index((j-1)*kjpindex+ji) = index(ji)+(j-1)*kjpij + offset_omp - offset_mpi
       ENDDO
    ENDDO

    ! Similar index tables are build for the wood use
    DO j = 1, nshort
       DO ji = 1, kjpindex
          horip_s_index((j-1)*kjpindex+ji) = &
               index(ji)+(j-1)*kjpij + offset_omp - offset_mpi
       ENDDO
    ENDDO

    DO j = 1, nmedium
       DO ji = 1, kjpindex
          horip_m_index((j-1)*kjpindex+ji) = &
               index(ji)+(j-1)*kjpij + offset_omp - offset_mpi
       ENDDO
    ENDDO
    
    DO j = 1, nlong
       DO ji = 1, kjpindex
          horip_l_index((j-1)*kjpindex+ji) = &
               index(ji)+(j-1)*kjpij + offset_omp - offset_mpi
       ENDDO
    ENDDO

    DO j = 1, nshort+1
       DO ji = 1, kjpindex
          horip_ss_index((j-1)*kjpindex+ji) = &
               index(ji)+(j-1)*kjpij + offset_omp - offset_mpi
       ENDDO
    ENDDO

    DO j = 1, nmedium+1
       DO ji = 1, kjpindex
          horip_mm_index((j-1)*kjpindex+ji) = &
               index(ji)+(j-1)*kjpij + offset_omp - offset_mpi
       ENDDO
    ENDDO

    DO j = 1, nlong+1
       DO ji = 1, kjpindex
          horip_ll_index((j-1)*kjpindex+ji) = &
               index(ji)+(j-1)*kjpij + offset_omp - offset_mpi
       ENDDO
    ENDDO
  

  !! 6. Initialization of global and land cover change variables. 

    ! All variables are cumulative variables. bm_to_litter is not and is therefore
    ! excluded
    ! bm_to_litter(:,:,:) = zero
    turnover_daily(:,:,:,:) = zero
    resp_hetero_d(:,:) = zero
!!$    co2_flux_daily(:,:) = zero
!!$    co2_flux_monthly(:,:) = zero
!!$    flux_prod_monthly(:) = zero
!!$    harvest_above_monthly(:) = zero
    control_moist_daily(:,:) = zero
    control_temp_daily(:,:) = zero
    som_input_daily(:,:,:,:) = zero
    drainage_daily(:,:) = zero
    fpc_max(:,:)=zero

    ! Land cover change variables
    prod_s(:,:,:)  = zero
    prod_m(:,:,:)  = zero
    prod_l(:,:,:) = zero
    flux_s(:,:,:)  = zero
    flux_m(:,:,:)  = zero
    flux_l(:,:,:) = zero
    flux_prod_s(:,:)  = zero
    flux_prod_m(:,:) = zero
    flux_prod_l(:,:) = zero
   
    ! n variables
    nstress_season(:,:) = zero 
    soil_n_min(:,:,:) = zero
    plant_n_uptake_daily(:,:,:)=zero 
    n_mineralisation_d(:,:)=zero 

  END SUBROUTINE stomate_init


!! ================================================================================================================================
!! SUBROUTINE 	: stomate_clear
!!
!>\BRIEF        Deallocate memory of the stomate variables.
!!
!! DESCRIPTION  : None
!!
!! RECENT CHANGE(S) : None
!!
!! MAIN OUTPUT VARIABLE(S): None
!!
!! REFERENCES	: None
!!
!! FLOWCHART    : None
!! \n
!_ ================================================================================================================================
  
  SUBROUTINE stomate_clear

  !! 1. Deallocate all dynamics variables

    IF (ALLOCATED(veget_cov_max)) DEALLOCATE(veget_cov_max)
    IF (ALLOCATED(adapted)) DEALLOCATE(adapted)
    IF (ALLOCATED(regenerate)) DEALLOCATE(regenerate)
    IF (ALLOCATED(humrel_daily)) DEALLOCATE(humrel_daily)
    IF (ALLOCATED(vir_humrel_daily)) DEALLOCATE(vir_humrel_daily)
    IF (ALLOCATED(transpir_supply_daily)) DEALLOCATE(transpir_supply_daily)
    IF (ALLOCATED(vir_transpir_supply_daily)) DEALLOCATE(vir_transpir_supply_daily)
    IF (ALLOCATED(transpir_daily)) DEALLOCATE(transpir_daily)
    IF (ALLOCATED(gdd_init_date)) DEALLOCATE(gdd_init_date)
    IF (ALLOCATED(litterhum_daily)) DEALLOCATE(litterhum_daily)
    IF (ALLOCATED(t2m_daily))  DEALLOCATE(t2m_daily)
    IF (ALLOCATED(t2m_min_daily))  DEALLOCATE(t2m_min_daily)
    IF (ALLOCATED(tsurf_daily))  DEALLOCATE(tsurf_daily)
    IF (ALLOCATED(tsoil_daily)) DEALLOCATE(tsoil_daily)
    IF (ALLOCATED(soilhum_daily)) DEALLOCATE(soilhum_daily)
    IF (ALLOCATED(precip_daily)) DEALLOCATE(precip_daily)
    IF (ALLOCATED(gpp_daily)) DEALLOCATE(gpp_daily)
    IF (ALLOCATED(npp_daily)) DEALLOCATE(npp_daily)
    IF (ALLOCATED(turnover_daily)) DEALLOCATE(turnover_daily)
    IF (ALLOCATED(turnover_littercalc)) DEALLOCATE(turnover_littercalc)
    IF (ALLOCATED(humrel_month)) DEALLOCATE(humrel_month)
    IF (ALLOCATED(humrel_week)) DEALLOCATE(humrel_week)
    IF (ALLOCATED(humrel_growingseason)) DEALLOCATE(humrel_growingseason)
    IF (ALLOCATED(vir_humrel_growingseason)) DEALLOCATE(vir_humrel_growingseason)
    IF (ALLOCATED(t2m_longterm)) DEALLOCATE(t2m_longterm)
    IF (ALLOCATED(t2m_month)) DEALLOCATE(t2m_month)
    IF (ALLOCATED(Tseason)) DEALLOCATE(Tseason)
    IF (ALLOCATED(Tseason_length)) DEALLOCATE(Tseason_length)
    IF (ALLOCATED(Tseason_tmp)) DEALLOCATE(Tseason_tmp)
    IF (ALLOCATED(Tmin_spring_time)) DEALLOCATE(Tmin_spring_time)
    IF (ALLOCATED(onset_date)) DEALLOCATE(onset_date)
!!$    IF (ALLOCATED(begin_leaves)) DEALLOCATE(begin_leaves)
    IF (ALLOCATED(t2m_week)) DEALLOCATE(t2m_week)
    IF (ALLOCATED(tsoil_month)) DEALLOCATE(tsoil_month)
    IF (ALLOCATED(soilhum_month)) DEALLOCATE(soilhum_month)
    IF (ALLOCATED(fireindex)) DEALLOCATE(fireindex)
    IF (ALLOCATED(firelitter)) DEALLOCATE(firelitter)
    IF (ALLOCATED(maxhumrel_lastyear)) DEALLOCATE(maxhumrel_lastyear)
    IF (ALLOCATED(maxhumrel_thisyear)) DEALLOCATE(maxhumrel_thisyear)
    IF (ALLOCATED(minhumrel_lastyear)) DEALLOCATE(minhumrel_lastyear)
    IF (ALLOCATED(minhumrel_thisyear)) DEALLOCATE(minhumrel_thisyear)
    IF (ALLOCATED(maxgppweek_lastyear)) DEALLOCATE(maxgppweek_lastyear)
    IF (ALLOCATED(maxgppweek_thisyear)) DEALLOCATE(maxgppweek_thisyear)
    IF (ALLOCATED(gdd0_lastyear)) DEALLOCATE(gdd0_lastyear)
    IF (ALLOCATED(gdd0_thisyear)) DEALLOCATE(gdd0_thisyear)
    IF (ALLOCATED(precip_lastyear)) DEALLOCATE(precip_lastyear)
    IF (ALLOCATED(precip_thisyear)) DEALLOCATE(precip_thisyear)
    IF (ALLOCATED(gdd_m5_dormance)) DEALLOCATE(gdd_m5_dormance)
    IF (ALLOCATED(gdd_from_growthinit)) DEALLOCATE(gdd_from_growthinit)
    IF (ALLOCATED(gdd_midwinter)) DEALLOCATE(gdd_midwinter)
    IF (ALLOCATED(ncd_dormance)) DEALLOCATE(ncd_dormance)
    IF (ALLOCATED(ngd_minus5))  DEALLOCATE(ngd_minus5)
    IF (ALLOCATED(PFTpresent)) DEALLOCATE(PFTpresent)
    IF (ALLOCATED(npp_longterm)) DEALLOCATE(npp_longterm)
    IF (ALLOCATED(lm_lastyearmax)) DEALLOCATE(lm_lastyearmax)
    IF (ALLOCATED(lm_thisyearmax)) DEALLOCATE(lm_thisyearmax)
    IF (ALLOCATED(maxfpc_lastyear)) DEALLOCATE(maxfpc_lastyear)
    IF (ALLOCATED(maxfpc_thisyear)) DEALLOCATE(maxfpc_thisyear)
    IF (ALLOCATED(turnover_longterm)) DEALLOCATE(turnover_longterm)
    IF (ALLOCATED(gpp_week)) DEALLOCATE(gpp_week)
    IF (ALLOCATED(plant_status)) DEALLOCATE(plant_status)
    IF (ALLOCATED(when_growthinit)) DEALLOCATE(when_growthinit)
    IF (ALLOCATED(age))  DEALLOCATE(age)
    IF (ALLOCATED(resp_hetero_d)) DEALLOCATE(resp_hetero_d)
    IF (ALLOCATED(resp_hetero_radia)) DEALLOCATE(resp_hetero_radia)
    IF (ALLOCATED(resp_maint_d)) DEALLOCATE(resp_maint_d)
    IF (ALLOCATED(resp_growth_d)) DEALLOCATE(resp_growth_d)
    IF (ALLOCATED(co2_fire)) DEALLOCATE(co2_fire)
!!$    IF (ALLOCATED(co2_to_bm_dgvm)) DEALLOCATE(co2_to_bm_dgvm)
!!$    IF (ALLOCATED(n_to_bm)) DEALLOCATE(n_to_bm)
    IF (ALLOCATED(atm_to_bm)) DEALLOCATE(atm_to_bm)
    IF (ALLOCATED(veget_lastlight)) DEALLOCATE(veget_lastlight)
    IF (ALLOCATED(everywhere)) DEALLOCATE(everywhere)
    IF (ALLOCATED(need_adjacent)) DEALLOCATE(need_adjacent)
    IF (ALLOCATED(leaf_age)) DEALLOCATE(leaf_age)
    IF (ALLOCATED(leaf_frac)) DEALLOCATE(leaf_frac)
    IF (ALLOCATED(RIP_time)) DEALLOCATE(RIP_time)
    IF (ALLOCATED(time_hum_min)) DEALLOCATE(time_hum_min)
    IF (ALLOCATED(hum_min_dormance)) DEALLOCATE(hum_min_dormance)
    IF (ALLOCATED(litter)) DEALLOCATE(litter)
    IF (ALLOCATED(dead_leaves)) DEALLOCATE(dead_leaves)
    IF (ALLOCATED(som)) DEALLOCATE(som)
    IF (ALLOCATED(lignin_struc)) DEALLOCATE(lignin_struc)
    IF (ALLOCATED(lignin_wood)) DEALLOCATE(lignin_wood)
    IF (ALLOCATED(turnover_time)) DEALLOCATE(turnover_time)
!!$    IF (ALLOCATED(co2_flux_daily)) DEALLOCATE(co2_flux_daily)
!!$    IF (ALLOCATED(co2_flux_monthly)) DEALLOCATE(co2_flux_monthly)
!!$    IF (ALLOCATED(harvest_above_monthly)) DEALLOCATE (harvest_above_monthly)
!!$    IF (ALLOCATED(flux_prod_monthly)) DEALLOCATE (flux_prod_monthly)
    IF (ALLOCATED(bm_to_litter)) DEALLOCATE(bm_to_litter)
    IF (ALLOCATED(bm_to_littercalc)) DEALLOCATE(bm_to_littercalc)
    IF (ALLOCATED(herbivores)) DEALLOCATE(herbivores)
    IF (ALLOCATED(resp_maint_part_radia)) DEALLOCATE(resp_maint_part_radia)
    IF (ALLOCATED(resp_maint_radia)) DEALLOCATE(resp_maint_radia)
    IF (ALLOCATED(resp_maint_part)) DEALLOCATE(resp_maint_part)
    IF (ALLOCATED(hori_index)) DEALLOCATE(hori_index)
    IF (ALLOCATED(horipft_index)) DEALLOCATE(horipft_index)
    IF (ALLOCATED(horican_index)) DEALLOCATE(horican_index)
    IF (ALLOCATED(horicut_index)) DEALLOCATE(horicut_index)
    IF (ALLOCATED(horip_s_index)) DEALLOCATE (horip_s_index)
    IF (ALLOCATED(horip_m_index)) DEALLOCATE (horip_m_index)
    IF (ALLOCATED(horip_l_index)) DEALLOCATE (horip_l_index)
    IF (ALLOCATED(horip_ss_index)) DEALLOCATE (horip_ss_index)
    IF (ALLOCATED(horip_mm_index)) DEALLOCATE (horip_mm_index)
    IF (ALLOCATED(horip_ll_index)) DEALLOCATE (horip_ll_index)
    IF (ALLOCATED(clay_fm)) DEALLOCATE(clay_fm)
    IF (ALLOCATED(silt_fm)) DEALLOCATE(silt_fm) 
    IF (ALLOCATED(bulk_fm)) DEALLOCATE(bulk_fm) 
    IF (ALLOCATED(humrel_daily_fm)) DEALLOCATE(humrel_daily_fm)
    IF (ALLOCATED(litterhum_daily_fm))  DEALLOCATE(litterhum_daily_fm)
    IF (ALLOCATED(t2m_daily_fm))  DEALLOCATE(t2m_daily_fm)
    IF (ALLOCATED(t2m_min_daily_fm))  DEALLOCATE(t2m_min_daily_fm)
    IF (ALLOCATED(tsurf_daily_fm)) DEALLOCATE(tsurf_daily_fm)
    IF (ALLOCATED(tsoil_daily_fm)) DEALLOCATE(tsoil_daily_fm)
    IF (ALLOCATED(soilhum_daily_fm))  DEALLOCATE(soilhum_daily_fm)
    IF (ALLOCATED(precip_fm)) DEALLOCATE(precip_fm)
    IF (ALLOCATED(gpp_daily_fm))  DEALLOCATE(gpp_daily_fm)
    IF (ALLOCATED(veget_fm)) DEALLOCATE(veget_fm)
    IF (ALLOCATED(veget_max_fm)) DEALLOCATE(veget_max_fm)
    IF (ALLOCATED(drainage_fm)) DEALLOCATE(drainage_fm) 
    !
    IF (ALLOCATED(ok_equilibrium)) DEALLOCATE(ok_equilibrium)
    IF (ALLOCATED(carbon_eq)) DEALLOCATE(carbon_eq)
    IF (ALLOCATED(matrixA)) DEALLOCATE(matrixA)
    IF (ALLOCATED(vectorB)) DEALLOCATE(vectorB)
    IF (ALLOCATED(matrixV)) DEALLOCATE(matrixV)
    IF (ALLOCATED(vectorU)) DEALLOCATE(vectorU)
    IF (ALLOCATED(matrixW)) DEALLOCATE(matrixW)
    IF (ALLOCATED(previous_stock)) DEALLOCATE(previous_stock)
    IF (ALLOCATED(current_stock)) DEALLOCATE(current_stock)
    IF (ALLOCATED(sigma)) DEALLOCATE (sigma) 
    IF (ALLOCATED(age_stand)) DEALLOCATE (age_stand)
    IF (ALLOCATED(rotation_n)) DEALLOCATE (rotation_n)
    IF (ALLOCATED(last_cut)) DEALLOCATE (last_cut)
    IF (ALLOCATED(CN_som_litter_longterm)) DEALLOCATE(CN_som_litter_longterm) 
    IF (ALLOCATED(KF)) DEALLOCATE (KF)
    IF (ALLOCATED(k_latosa_adapt)) DEALLOCATE (k_latosa_adapt)
    IF (ALLOCATED(harvest_pool)) DEALLOCATE (harvest_pool)
    IF (ALLOCATED(harvest_type)) DEALLOCATE (harvest_type)
    IF (ALLOCATED(harvest_cut)) DEALLOCATE (harvest_cut)
    IF (ALLOCATED(harvest_area)) DEALLOCATE (harvest_area)
    IF (ALLOCATED(harvest_5y_area)) DEALLOCATE (harvest_5y_area)
    IF (ALLOCATED(harvest_pool_bound)) DEALLOCATE (harvest_pool_bound)
    IF (ALLOCATED(prod_s)) DEALLOCATE (prod_s)
    IF (ALLOCATED(prod_m)) DEALLOCATE (prod_m)
    IF (ALLOCATED(prod_l)) DEALLOCATE (prod_l)
    IF (ALLOCATED(flux_s)) DEALLOCATE (flux_s)
    IF (ALLOCATED(flux_m)) DEALLOCATE (flux_m)
    IF (ALLOCATED(flux_l)) DEALLOCATE (flux_l)
    IF (ALLOCATED(flux_prod_s)) DEALLOCATE (flux_prod_s)
    IF (ALLOCATED(flux_prod_m)) DEALLOCATE (flux_prod_m)
    IF (ALLOCATED(flux_prod_l)) DEALLOCATE (flux_prod_l)

    IF (ALLOCATED(mai)) DEALLOCATE (mai)
    IF (ALLOCATED(pai)) DEALLOCATE (pai)
    IF (ALLOCATED(previous_wood_volume)) DEALLOCATE (previous_wood_volume)
    IF (ALLOCATED(mai_count)) DEALLOCATE (mai_count)
    IF (ALLOCATED(coppice_dens)) DEALLOCATE (coppice_dens)
    IF (ALLOCATED(litter_demand)) DEALLOCATE (litter_demand)
    IF (ALLOCATED(wstress_season)) DEALLOCATE (wstress_season)
    IF (ALLOCATED(wstress_month)) DEALLOCATE (wstress_month)
    IF (ALLOCATED(rue_longterm)) DEALLOCATE (rue_longterm)
    IF (ALLOCATED(nbp_accu)) DEALLOCATE(nbp_accu)
    IF (ALLOCATED(nbp_accu_pool)) DEALLOCATE(nbp_accu_pool)
    IF (ALLOCATED(nbp_flux)) DEALLOCATE(nbp_flux)
    IF (ALLOCATED(nbp_pool)) DEALLOCATE(nbp_pool)
    IF (ALLOCATED(nbp_pool_start)) DEALLOCATE(nbp_pool_start)
    IF (ALLOCATED(nbp_pool_end)) DEALLOCATE(nbp_pool_end)

    IF (ALLOCATED(clay_fm_g)) DEALLOCATE(clay_fm_g)
    IF (ALLOCATED(silt_fm_g)) DEALLOCATE(silt_fm_g) 
    IF (ALLOCATED(bulk_fm_g)) DEALLOCATE(bulk_fm_g) 
    IF (ALLOCATED(humrel_daily_fm_g)) DEALLOCATE(humrel_daily_fm_g)
    IF (ALLOCATED(litterhum_daily_fm_g))  DEALLOCATE(litterhum_daily_fm_g)
    IF (ALLOCATED(t2m_daily_fm_g))  DEALLOCATE(t2m_daily_fm_g)
    IF (ALLOCATED(t2m_min_daily_fm_g))  DEALLOCATE(t2m_min_daily_fm_g)
    IF (ALLOCATED(tsurf_daily_fm_g)) DEALLOCATE(tsurf_daily_fm_g)
    IF (ALLOCATED(tsoil_daily_fm_g)) DEALLOCATE(tsoil_daily_fm_g)
    IF (ALLOCATED(soilhum_daily_fm_g))  DEALLOCATE(soilhum_daily_fm_g)
    IF (ALLOCATED(precip_fm_g)) DEALLOCATE(precip_fm_g)
    IF (ALLOCATED(gpp_daily_fm_g))  DEALLOCATE(gpp_daily_fm_g)
    IF (ALLOCATED(veget_fm_g)) DEALLOCATE(veget_fm_g)
    IF (ALLOCATED(veget_max_fm_g)) DEALLOCATE(veget_max_fm_g)
!!$    IF (ALLOCATED(lai_fm_g))  DEALLOCATE(lai_fm_g)
    IF (ALLOCATED(drainage_fm_g))  DEALLOCATE(drainage_fm_g)    
 
    IF (ALLOCATED(isf)) DEALLOCATE(isf)
    IF (ALLOCATED(nf_written)) DEALLOCATE(nf_written)
    IF (ALLOCATED(nf_cumul)) DEALLOCATE(nf_cumul)
    IF (ALLOCATED(nforce)) DEALLOCATE(nforce)
    IF (ALLOCATED(control_moist)) DEALLOCATE(control_moist)
    IF (ALLOCATED(control_temp)) DEALLOCATE(control_temp)
    IF (ALLOCATED(carbon_input)) DEALLOCATE(carbon_input)
    IF (ALLOCATED(nitrogen_input)) DEALLOCATE(nitrogen_input)
    IF ( ALLOCATED (som_input_daily)) DEALLOCATE (som_input_daily)
    IF ( ALLOCATED (control_temp_daily)) DEALLOCATE (control_temp_daily)
    IF ( ALLOCATED (control_moist_daily)) DEALLOCATE (control_moist_daily)

    IF ( ALLOCATED (drainage_daily)) DEALLOCATE(drainage_daily) 
    IF ( ALLOCATED (plant_n_uptake_daily)) DEALLOCATE(plant_n_uptake_daily) 
    IF ( ALLOCATED (n_mineralisation_d)) DEALLOCATE(n_mineralisation_d) 
    IF ( ALLOCATED (cn_leaf_min_season)) DEALLOCATE (cn_leaf_min_season) 
    IF ( ALLOCATED (nstress_season)) DEALLOCATE (nstress_season) 
    IF ( ALLOCATED (soil_n_min)) DEALLOCATE (soil_n_min) 
    IF ( ALLOCATED (p_O2)) DEALLOCATE (p_O2) 
    IF ( ALLOCATED (bact)) DEALLOCATE (bact) 
    IF ( ALLOCATED (fpc_max)) DEALLOCATE (fpc_max)

    IF (ALLOCATED(forest_managed)) DEALLOCATE (forest_managed)
    IF (ALLOCATED(forest_managed_lastyear)) DEALLOCATE (forest_managed_lastyear)
    IF (ALLOCATED(species_change_map)) DEALLOCATE (species_change_map)
    IF (ALLOCATED(fm_change_map)) DEALLOCATE (fm_change_map) 
    IF (ALLOCATED(lpft_replant)) DEALLOCATE (lpft_replant)

 !! 2. reset l_first

    l_first_stomate=.TRUE.

 !! 3. call to clear functions

!!$    CALL get_reftemp_clear
    CALL season_clear
    CALL stomate_lpj_clear
    CALL littercalc_clear
    CALL vmax_clear
 
  END SUBROUTINE stomate_clear


!! ================================================================================================================================
!! SUBROUTINE 	: stomate_var_init
!!
!>\BRIEF        Initialize variables of stomate with a none-zero initial value.
!! Subroutine is called only if ::ok_stomate = .TRUE. STOMATE diagnoses some 
!! variables for SECHIBA : assim_param, deadleaf_cover, etc. These variables can 
!! be recalculated from STOMATE's prognostic variables. Note that height is
!! saved in SECHIBA.
!!
!! DESCRIPTION  : None
!!
!! RECENT CHANGE(S) : None
!!
!! MAIN OUTPUT VARIABLE(S): leaf age (::leaf_age) and fraction of leaves in leaf 
!! age class (::leaf_frac). The maximum water on vegetation available for 
!! interception, fraction of soil covered by dead leaves
!! (::deadleaf_cover) and assimilation parameters (:: assim_param).
!!
!! REFERENCE(S)	: None
!!
!! FLOWCHART    : None
!! \n
!_ ================================================================================================================================
  
  SUBROUTINE stomate_var_init &
       &  (kjpindex, veget_cov_max, leaf_age, leaf_frac, &
       &   dead_leaves, &
       &   veget_cov, deadleaf_cover, assim_param, &
       &   circ_class_biomass, circ_class_n)


  !! 0. Variable and parameter declaration

    !! 0.1 Input variables

    INTEGER(i_std),INTENT(in)                             :: kjpindex        !! Domain size - terrestrial pixels only
    REAL(r_std),DIMENSION(kjpindex,nvm),INTENT(in)        :: veget_cov           !! Fraction of pixel covered by PFT. Fraction 
                                                                             !! accounts for none-biological land covers 
                                                                             !! (unitless) 
    REAL(r_std),DIMENSION(kjpindex,nvm),INTENT(in)        :: veget_cov_max   !! Fractional coverage: maximum share of the pixel 
                                                                             !! covered by a PFT (unitless) 
    REAL(r_std),DIMENSION(kjpindex,nvm,nlitt),INTENT(in)  :: dead_leaves     !! Metabolic and structural fraction of dead leaves 
                                                                             !! per ground area 
                                                                             !! @tex $(gC m^{-2})$ @endtex  
    REAL(r_std), DIMENSION(kjpindex,nvm,ncirc,nparts,nelements),INTENT(in)       :: circ_class_biomass         !! @tex $(gC m^{-2})$ @endtex

    REAL(r_std), DIMENSION(kjpindex,nvm,ncirc),INTENT(in)        :: circ_class_n !! @tex $(gC m^{-2})$ @endtex


    !! 0.2 Modified variables

    REAL(r_std),DIMENSION(kjpindex,nvm,nleafages),INTENT(inout) :: leaf_age  !! Age of different leaf classes per PFT (days)
    REAL(r_std),DIMENSION(kjpindex,nvm,nleafages),INTENT(inout) :: leaf_frac !! Fraction of leaves in leaf age class per PFT 
                                                                             !! (unitless; 1) 
    
    !! 0.3 Output variables

    REAL(r_std),DIMENSION(kjpindex), INTENT (out)         :: deadleaf_cover  !! Fraction of soil covered by dead leaves 
                                                                             !! (unitless) 
    REAL(r_std),DIMENSION(kjpindex,nvm,npco2),INTENT(out) :: assim_param     !! min+max+opt temperatures (K) & vmax for 
                                                                             !! photosynthesis  
                                                                             !! @tex $(\mumol m^{-2} s^{-1})$ @endtex
    ! 0.4 Local variables
   
    REAL(r_std),PARAMETER                                 :: dt_0 = zero     !! Dummy time step, must be zero
    REAL(r_std),DIMENSION(kjpindex,nvm)                   :: vcmax           !! Dummy vcmax 
                                                                             !! @tex $(\mu mol m^{-2} s^{-1})$ @endtex

    REAL(r_std),DIMENSION(kjpindex,nvm)                   :: nue             !! Nitrogen use Efficiency with impact of leaf age (umol CO2 (gN)-1 s-1) 
                                                                             !! @tex $(\mu mol m^{-2} s^{-1})$ @endtex 
    INTEGER(i_std)                                        :: j               !! Index (untiless)
    
!_ ================================================================================================================================   

    ! Only if stomate is activated
    IF (printlev>=4) WRITE(numout,*) 'Entering stomate_var_init'
 
    !! 1. photosynthesis parameters

    !! 1.1 vcmax (stomate_vmax.f90)
    CALL vmax (kjpindex, dt_0, leaf_age, leaf_frac, vcmax, nue )

    IF (printlev_loc>=4) THEN
       WRITE(numout,*) 'vcmax, ', vcmax
       WRITE(numout,*) 'nue, ', nue
    ENDIF
    
    !! 1.3 transform into nvm vegetation types
    assim_param(:,:,ivcmax) = zero
    assim_param(:,:,inue) = zero 
    assim_param(:,:,ileafN) = zero 
    DO j = 2, nvm
       assim_param(:,j,ivcmax)=vcmax(:,j)
       assim_param(:,j,inue)=nue(:,j) 
       assim_param(:,j,ileafN)=SUM(circ_class_biomass(:,j,:,ileaf,initrogen)*circ_class_n(:,j,:),2)
    ENDDO
    
    !! 2. Dead leaf cover (stomate_litter.f90)
    
    CALL deadleaf (kjpindex, veget_cov_max, dead_leaves, deadleaf_cover)     
    
  END SUBROUTINE stomate_var_init


!! ================================================================================================================================
!! INTERFACE  	: stomate_accu
!!
!>\BRIEF        Accumulate a variable for the time period specified by 
!! dt_sechiba or calculate the mean value over the period of dt_stomate.
!! 
!! DESCRIPTION  : Accumulate a variable for the time period specified by  
!! dt_sechiba or calculate the mean value over the period of dt_stomate. 
!! stomate_accu interface can be used for variables having 1, 2 or 3 dimensions. 
!! The corresponding subruoutine stomate_accu_r1d, stomate_accu_r2d or 
!! stomate_accu_r3d will be selected through the interface depending on the number of dimensions.
!!
!! RECENT CHANGE(S) : None
!!
!! MAIN OUTPUT VARIABLE(S): accumulated or mean variable ::field_out:: 
!!
!! REFERENCE(S)	: None
!!
!! FLOWCHART    : None
!! \n
!_ ================================================================================================================================
  
  SUBROUTINE stomate_accu_r1d (ldmean, field_in, field_out)
    
  !! 0. Variable and parameter declaration

    !! 0.1 Input variables
    LOGICAL,INTENT(in)                     :: ldmean    !! Flag to calculate the mean over
    REAL(r_std),DIMENSION(:),INTENT(in)    :: field_in  !! Field that needs to be accumulated
    
    !! 0.2 Modified variables
    REAL(r_std),DIMENSION(:),INTENT(inout) :: field_out !! Accumulated or mean field

!_ ================================================================================================================================

  !! 1. Accumulate field

    field_out(:) = field_out(:)+field_in(:)*dt_sechiba
   
  !! 2. Mean fields
    IF (ldmean) THEN
       field_out(:) = field_out(:)/dt_stomate
    ENDIF

  END SUBROUTINE stomate_accu_r1d

  SUBROUTINE stomate_accu_r2d (ldmean, field_in, field_out)
    
  !! 0. Variable and parameter declaration

    !! 0.1 Input variables
    LOGICAL,INTENT(in)                       :: ldmean    !! Flag to calculate the mean over
    REAL(r_std),DIMENSION(:,:),INTENT(in)    :: field_in  !! Field that needs to be accumulated
    
    !! 0.2 Modified variables
    REAL(r_std),DIMENSION(:,:),INTENT(inout) :: field_out !! Accumulated or mean field

!_ ================================================================================================================================

  !! 1. Accumulate field
  
    field_out(:,:) = field_out(:,:)+field_in(:,:)*dt_sechiba
   
  !! 2. Mean fields
    IF (ldmean) THEN
       field_out(:,:) = field_out(:,:)/dt_stomate
    ENDIF
  
  END SUBROUTINE stomate_accu_r2d

  SUBROUTINE stomate_accu_r3d (ldmean, field_in, field_out)
   
  !! 0. Variable and parameter declaration

    !! 0.1 Input variables
    LOGICAL,INTENT(in)                         :: ldmean    !! Flag to calculate the mean over
    REAL(r_std),DIMENSION(:,:,:),INTENT(in)    :: field_in  !! Field that needs to be accumulated
   
    !! 0.2 Modified variables
    REAL(r_std),DIMENSION(:,:,:),INTENT(inout) :: field_out !! Accumulated or mean field

!_ ================================================================================================================================

  !! 1. Accumulate field

    field_out(:,:,:) = field_out(:,:,:)+field_in(:,:,:)*dt_sechiba
   
  !! 2. Mean fields

    IF (ldmean) THEN
       field_out(:,:,:) = field_out(:,:,:)/dt_stomate
    ENDIF

  END SUBROUTINE stomate_accu_r3d

  SUBROUTINE stomate_accu_r4d (ldmean, field_in, field_out)

  !! 0. Variable and parameter declaration

    !! 0.1 Input variables
    LOGICAL,INTENT(in)                           :: ldmean    !! Flag to calculate the mean over  
    REAL(r_std),DIMENSION(:,:,:,:),INTENT(in)    :: field_in  !! Field that needs to be accumulated

    !! 0.2 Modified variables
    REAL(r_std),DIMENSION(:,:,:,:),INTENT(inout) :: field_out !! Accumulated or mean field

!_ ================================================================================================================================

  !! 1. Accumulate field

    field_out(:,:,:,:) = field_out(:,:,:,:)+field_in(:,:,:,:)*dt_sechiba

  !! 2. Mean fields

    IF (ldmean) THEN
       field_out(:,:,:,:) = field_out(:,:,:,:)/dt_stomate
    ENDIF

  END SUBROUTINE stomate_accu_r4d

!! ================================================================================================================================
!! SUBROUTINE 	: stomate_veget_update
!!
!>\BRIEF        After the vegetation has been updated some variables needs to be reinitialized for too small fractions
!! 
!! DESCRIPTION :  After the vegetation has been updated some variables needs to be reinitialized for too small fractions.
!!                This subroutine is called from slowproc_veget after removing to small fractions.
!!
!! RECENT CHANGE(S) : These calculations were previously done in stomate_lcchange
!!
!! MAIN OUTPUT VARIABLE(S): None
!!
!! REFERENCE(S)	: None
!!
!! FLOWCHART    : None
!! \n
!_ ================================================================================================================================

  SUBROUTINE stomate_veget_update(kjpindex, veget_max)

    !! 0. Variable and parameter declaration
    !! 0.1 Input variables
    INTEGER(i_std),INTENT(in)                       :: kjpindex          !! Local domain size - terrestrial pixels only (unitless)
    REAL(r_std),DIMENSION(kjpindex,nvm),INTENT(in)  :: veget_max         !! Maximum fraction of vegetation type including
                                                                         !! non-biological fraction (unitless) 
    !! 0.3 Local variables
    INTEGER(i_std)                                  :: i, j              !! Indices

    ! Only proceed if the initalization of stomate has been done
    IF (.NOT. l_first_stomate) THEN

       ! Loop over all points on all pft's
       ! If the fraction is too small, reset the depending variables
       ! veget_max will be changed in slowproc_veget after this subroutine has been done. 
       DO i = 1, kjpindex
          DO j=1, nvm
             IF ( veget_max(i,j) < min_vegfrac ) THEN 
                PFTpresent(i,j) = .FALSE.
                plant_status(i,j) = idormant
                age(i,j) = zero
                when_growthinit(i,j) = undef
                everywhere(i,j) = zero
                som(i,:,j,:) = zero
                litter(i,:,j,:,:) = zero
                bm_to_litter(i,j,:,:) = zero
                turnover_daily(i,j,:,:) = zero
             ENDIF
          END DO
       END DO
    END IF

  END SUBROUTINE stomate_veget_update

!! ================================================================================================================================
!! SUBROUTINE 	: init_forcing
!!
!>\BRIEF        Allocate memory for the variables containing the forcing data.
!! The maximum size of the allocated memory is specified in run definition file
!! (::max_totsize) and needs to be a compromise between charging the memory and 
!! accessing disks to get the forcing data.
!!
!! DESCRIPTION : None
!!
!! RECENT CHANGE(S) : None
!!
!! MAIN OUTPUT VARIABLE(S): Strictly speaking the subroutine has no output 
!! variables. However, the routine allocates memory for later use. 
!!
!! REFERENCE(S)	: None
!!
!! FLOWCHART    : None
!! \n
!_ ================================================================================================================================
  
  SUBROUTINE init_forcing (kjpindex,nsfm,nsft_loc)
    
  !! 0. Variable and parameter declaration

    !! 0.1 Input variables
    INTEGER(i_std),INTENT(in) :: kjpindex !! Domain size - terrestrial pixels only (unitless)
    INTEGER(i_std),INTENT(in) :: nsfm     !! Number of time steps that can be stored in memory (unitless)
    INTEGER(i_std),INTENT(in) :: nsft_loc !! Number of time steps in a year (unitless)

   !! 0.2 Output variables

   !! 0.3 Modified variables

   !! 0.4 Local variables

    LOGICAL                   :: l_error  !! Check errors in netcdf call
    INTEGER(i_std)            :: ier      !! Check errors in netcdf call
!_ ================================================================================================================================
    
  !! 1. Allocate memory

    ! Note ::nvm is number of PFTs and ::nbdl is number of soil layers
    l_error = .FALSE.
    ALLOCATE(clay_fm(kjpindex,nsfm),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Problem with memory allocation: forcing variables clay_fm ',kjpindex,nsfm
       STOP 'init_forcing'
    ENDIF

    ALLOCATE(silt_fm(kjpindex,nsfm),stat=ier) 
    l_error = l_error .OR. (ier /= 0) 
    IF (l_error) THEN 
       WRITE(numout,*) 'Problem with memory allocation: forcing variables silt_fm ',kjpindex,nsfm 
       STOP 'init_forcing' 
    ENDIF 
    ALLOCATE(bulk_fm(kjpindex,nsfm),stat=ier) 
    l_error = l_error .OR. (ier /= 0) 
    IF (l_error) THEN 
       WRITE(numout,*) 'Problem with memory allocation: forcing variables bulk_fm ',kjpindex,nsfm 
       STOP 'init_forcing' 
    ENDIF 

    ALLOCATE(humrel_daily_fm(kjpindex,nvm,nsfm),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Problem with memory allocation: forcing variables humrel_daily_fm ',kjpindex,nvm,nsfm
       STOP 'init_forcing'
    ENDIF
    ALLOCATE(litterhum_daily_fm(kjpindex,nsfm),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Problem with memory allocation: forcing variables litterhum_daily_fm ',kjpindex,nsfm
       STOP 'init_forcing'
    ENDIF
    ALLOCATE(t2m_daily_fm(kjpindex,nsfm),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Problem with memory allocation: forcing variables t2m_daily_fm ',kjpindex,nsfm
       STOP 'init_forcing'
    ENDIF
    ALLOCATE(t2m_min_daily_fm(kjpindex,nsfm),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Problem with memory allocation: forcing variables t2m_min_daily_fm ',kjpindex,nsfm
       STOP 'init_forcing'
    ENDIF
    ALLOCATE(tsurf_daily_fm(kjpindex,nsfm),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Problem with memory allocation: forcing variables tsurf_daily_fm ',kjpindex,nsfm
       STOP 'init_forcing'
    ENDIF
    ALLOCATE(tsoil_daily_fm(kjpindex,nbdl,nsfm),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Problem with memory allocation: forcing variables tsoil_daily_fm ',kjpindex,nbdl,nsfm
       STOP 'init_forcing'
    ENDIF
    ALLOCATE(soilhum_daily_fm(kjpindex,nbdl,nsfm),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Problem with memory allocation: forcing variables soilhum_daily_fm ',kjpindex,nbdl,nsfm
       STOP 'init_forcing'
    ENDIF
    ALLOCATE(precip_fm(kjpindex,nsfm),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Problem with memory allocation: forcing variables precip_fm ',kjpindex,nsfm
       STOP 'init_forcing'
    ENDIF
    ALLOCATE(gpp_daily_fm(kjpindex,nvm,nsfm),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Problem with memory allocation: forcing variables gpp_daily_fm ',kjpindex,nvm,nsfm
       STOP 'init_forcing'
    ENDIF
    ALLOCATE(veget_fm(kjpindex,nvm,nsfm),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Problem with memory allocation: forcing variables veget_fm ',kjpindex,nvm,nsfm
       STOP 'init_forcing'
    ENDIF
    ALLOCATE(veget_max_fm(kjpindex,nvm,nsfm),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Problem with memory allocation: forcing variables veget_max_fm ',kjpindex,nvm,nsfm
       STOP 'init_forcing'
    ENDIF
!!$    ALLOCATE(lai_fm(kjpindex,nvm,nsfm),stat=ier)
!!$    l_error = l_error .OR. (ier /= 0)
!!$    IF (l_error) THEN
!!$       WRITE(numout,*) 'Problem with memory allocation: forcing variables lai_fm ',kjpindex,nvm,nsfm
!!$       STOP 'init_forcing'
!!$    ENDIF
    ALLOCATE(drainage_fm(kjpindex,nvm,nsfm),stat=ier) 
    l_error = l_error .OR. (ier /= 0) 
    IF (l_error) THEN 
       WRITE(numout,*) 'Problem with memory allocation: forcing variables drainage_fm ',kjpindex,nvm,nsfm 
       STOP 'init_forcing' 
    ENDIF 
    ALLOCATE(isf(nsfm),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Problem with memory allocation: forcing variables isf ',nsfm
       STOP 'init_forcing'
    ENDIF
    ALLOCATE(nf_written(nsft_loc),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Problem with memory allocation: forcing variables nf_written ',nsft_loc
       STOP 'init_forcing'
    ENDIF
    ALLOCATE(nf_cumul(nsft_loc),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) 'Problem with memory allocation: forcing variables nf_cumul ',nsft_loc
       STOP 'init_forcing'
    ENDIF
    
  !! 2. Allocate memory for the root processor only (parallel computing)

    ! Where, ::nbp_glo is the number of global continental points
    IF (is_root_prc) THEN
       ALLOCATE(clay_fm_g(nbp_glo,nsfm),stat=ier)
       l_error = l_error .OR. (ier /= 0)
       IF (l_error) THEN
          WRITE(numout,*) 'Problem with memory allocation: forcing variables clay_fm_g ',nbp_glo,nsfm
          STOP 'init_forcing'
       ENDIF
       ALLOCATE(silt_fm_g(nbp_glo,nsfm),stat=ier) 
       l_error = l_error .OR. (ier /= 0) 
       IF (l_error) THEN 
          WRITE(numout,*) 'Problem with memory allocation: forcing variables silt_fm_g ',nbp_glo,nsfm 
          STOP 'init_forcing' 
       ENDIF
       ALLOCATE(bulk_fm_g(nbp_glo,nsfm),stat=ier) 
       l_error = l_error .OR. (ier /= 0) 
       IF (l_error) THEN 
          WRITE(numout,*) 'Problem with memory allocation: forcing variables bulk_fm_g ',nbp_glo,nsfm 
          STOP 'init_forcing' 
       ENDIF
       ALLOCATE(humrel_daily_fm_g(nbp_glo,nvm,nsfm),stat=ier)
       l_error = l_error .OR. (ier /= 0)
       IF (l_error) THEN
          WRITE(numout,*) 'Problem with memory allocation: forcing variables humrel_daily_fm_g ',nbp_glo,nvm,nsfm
          STOP 'init_forcing'
       ENDIF
       ALLOCATE(litterhum_daily_fm_g(nbp_glo,nsfm),stat=ier)
       l_error = l_error .OR. (ier /= 0)
       IF (l_error) THEN
          WRITE(numout,*) 'Problem with memory allocation: forcing variables litterhum_daily_fm_g ',nbp_glo,nsfm
          STOP 'init_forcing'
       ENDIF
       ALLOCATE(t2m_daily_fm_g(nbp_glo,nsfm),stat=ier)
       l_error = l_error .OR. (ier /= 0)
       IF (l_error) THEN
          WRITE(numout,*) 'Problem with memory allocation: forcing variables t2m_daily_fm_g ',nbp_glo,nsfm
          STOP 'init_forcing'
       ENDIF
       ALLOCATE(t2m_min_daily_fm_g(nbp_glo,nsfm),stat=ier)
       l_error = l_error .OR. (ier /= 0)
       IF (l_error) THEN
          WRITE(numout,*) 'Problem with memory allocation: forcing variables t2m_min_daily_fm_g ',nbp_glo,nsfm
          STOP 'init_forcing'
       ENDIF
       ALLOCATE(tsurf_daily_fm_g(nbp_glo,nsfm),stat=ier)
       l_error = l_error .OR. (ier /= 0)
       IF (l_error) THEN
          WRITE(numout,*) 'Problem with memory allocation: forcing variables tsurf_daily_fm_g ',nbp_glo,nsfm
          STOP 'init_forcing'
       ENDIF
       ALLOCATE(tsoil_daily_fm_g(nbp_glo,nbdl,nsfm),stat=ier)
       l_error = l_error .OR. (ier /= 0)
       IF (l_error) THEN
          WRITE(numout,*) 'Problem with memory allocation: forcing variables tsoil_daily_fm_g ',nbp_glo,nbdl,nsfm
          STOP 'init_forcing'
       ENDIF
       ALLOCATE(soilhum_daily_fm_g(nbp_glo,nbdl,nsfm),stat=ier)
       l_error = l_error .OR. (ier /= 0)
       IF (l_error) THEN
          WRITE(numout,*) 'Problem with memory allocation: forcing variables soilhum_daily_fm_g ',nbp_glo,nbdl,nsfm
          STOP 'init_forcing'
       ENDIF
       ALLOCATE(precip_fm_g(nbp_glo,nsfm),stat=ier)
       l_error = l_error .OR. (ier /= 0)
       IF (l_error) THEN
          WRITE(numout,*) 'Problem with memory allocation: forcing variables precip_fm_g ',nbp_glo,nsfm
          STOP 'init_forcing'
       ENDIF
       ALLOCATE(gpp_daily_fm_g(nbp_glo,nvm,nsfm),stat=ier)
       l_error = l_error .OR. (ier /= 0)
       IF (l_error) THEN
          WRITE(numout,*) 'Problem with memory allocation: forcing variables gpp_daily_fm_g ',nbp_glo,nvm,nsfm
          STOP 'init_forcing'
       ENDIF
       ALLOCATE(veget_fm_g(nbp_glo,nvm,nsfm),stat=ier)
       l_error = l_error .OR. (ier /= 0)
       IF (l_error) THEN
          WRITE(numout,*) 'Problem with memory allocation: forcing variables veget_fm_g ',nbp_glo,nvm,nsfm
          STOP 'init_forcing'
       ENDIF
       ALLOCATE(veget_max_fm_g(nbp_glo,nvm,nsfm),stat=ier)
       l_error = l_error .OR. (ier /= 0)
       IF (l_error) THEN
          WRITE(numout,*) 'Problem with memory allocation: forcing variables veget_max_fm_g ',nbp_glo,nvm,nsfm
          STOP 'init_forcing'
       ENDIF
!!$       ALLOCATE(lai_fm_g(nbp_glo,nvm,nsfm),stat=ier)
!!$       l_error = l_error .OR. (ier /= 0)
!!$       IF (l_error) THEN
!!$          WRITE(numout,*) 'Problem with memory allocation: forcing variables lai_fm_g ',nbp_glo,nvm,nsfm
!!$          STOP 'init_forcing'
!!$       ENDIF
       ALLOCATE(drainage_fm_g(nbp_glo,nvm,nsfm),stat=ier) 
       l_error = l_error .OR. (ier /= 0) 
       IF (l_error) THEN 
          WRITE(numout,*) 'Problem with memory allocation: forcing variables drainage_fm_g ',nbp_glo,nvm,nsfm 
          STOP 'init_forcing' 
       ENDIF
    ELSE
       ! Allocate memory for co-processors
       ALLOCATE(clay_fm_g(0,nsfm),stat=ier)
       ALLOCATE(silt_fm_g(0,nsfm),stat=ier) 
       ALLOCATE(bulk_fm_g(0,nsfm),stat=ier) 
       ALLOCATE(humrel_daily_fm_g(0,nvm,nsfm),stat=ier)
       ALLOCATE(litterhum_daily_fm_g(0,nsfm),stat=ier)
       ALLOCATE(t2m_daily_fm_g(0,nsfm),stat=ier)
       ALLOCATE(t2m_min_daily_fm_g(0,nsfm),stat=ier)
       ALLOCATE(tsurf_daily_fm_g(0,nsfm),stat=ier)
       ALLOCATE(tsoil_daily_fm_g(0,nbdl,nsfm),stat=ier)
       ALLOCATE(soilhum_daily_fm_g(0,nbdl,nsfm),stat=ier)
       ALLOCATE(precip_fm_g(0,nsfm),stat=ier)
       ALLOCATE(gpp_daily_fm_g(0,nvm,nsfm),stat=ier)
       ALLOCATE(veget_fm_g(0,nvm,nsfm),stat=ier)
       ALLOCATE(veget_max_fm_g(0,nvm,nsfm),stat=ier)
!!$       ALLOCATE(lai_fm_g(0,nvm,nsfm),stat=ier)
       ALLOCATE(drainage_fm_g(0,nvm,nsfm),stat=ier) 
    ENDIF ! is_root_proc
    
    IF (l_error) THEN
       WRITE(numout,*) 'Problem with memory allocation: forcing variables'
       STOP 'init_forcing'
    ENDIF

  !! 3. Initialize variables

    CALL forcing_zero
    
  END SUBROUTINE init_forcing


!! ================================================================================================================================
!! SUBROUTINE 	: forcing_zero
!!
!>\BRIEF        Initialize variables containing the forcing data; variables are 
!! set to zero.
!!
!! DESCRIPTION  : None
!!
!! RECENT CHANGE(S) : None
!!
!! MAIN OUTPUT VARIABLE(S): None
!!
!! REFERENCES	: None
!!
!! FLOWCHART    : None
!! \n
!_ ================================================================================================================================
  
  SUBROUTINE forcing_zero
    
    clay_fm(:,:) = zero
    silt_fm(:,:) = zero 
    bulk_fm(:,:) = zero 
    humrel_daily_fm(:,:,:) = zero
    litterhum_daily_fm(:,:) = zero
    t2m_daily_fm(:,:) = zero
    t2m_min_daily_fm(:,:) = zero
    tsurf_daily_fm(:,:) = zero
    tsoil_daily_fm(:,:,:) = zero
    soilhum_daily_fm(:,:,:) = zero
    precip_fm(:,:) = zero
    gpp_daily_fm(:,:,:) = zero
    veget_fm(:,:,:) = zero
    veget_max_fm(:,:,:) = zero
!!$    lai_fm(:,:,:) = zero
    drainage_fm(:,:,:) = zero     
  END SUBROUTINE forcing_zero


!! ================================================================================================================================
!! SUBROUTINE 	: forcing_write
!!
!>\BRIEF        Appends data values to a netCDF file containing the forcing 
!! variables of the general processes in stomate.
!!
!! DESCRIPTION  : None
!!
!! RECENT CHANGE(S) : None
!!
!! MAIN OUTPUT VARIABLE(S): netCDF file
!!
!! REFERENCES	: None
!!
!! FLOWCHART    : None
!! \n
!_ ================================================================================================================================
  
  SUBROUTINE forcing_write(forcing_id,ibeg,iend)
    
  !! 0. Variable and parameter declaration

    !! 0.1 Input variables

    INTEGER(i_std),INTENT(in)      :: forcing_id  !! File identifer of forcing file, assigned when netcdf is created
    INTEGER(i_std),INTENT(in)      :: ibeg, iend  !! First and last time step to be written

    !! 0.2 Output variables

    !! 0.3 Modified variables

    !! 0.4 Local variables

    INTEGER(i_std)                 :: ii          !! Index of isf where isf is the number of time steps that can be 
                                                  !! stored in memory 
    INTEGER(i_std)                 :: iblocks     !! Index of block that is written
    INTEGER(i_std)                 :: nblocks     !! Number of blocks that needs to be written
    INTEGER(i_std)                 :: ier         !! Check errors in netcdf call
    INTEGER(i_std),DIMENSION(0:2)  :: ifirst      !! First block in memory - changes with iblocks
    INTEGER(i_std),DIMENSION(0:2)  :: ilast       !! Last block in memory - changes with iblocks
    INTEGER(i_std),PARAMETER       :: ndm = 10    !! Maximum number of dimensions
    INTEGER(i_std),DIMENSION(ndm)  :: start       !! First block to write
    INTEGER(i_std)                 :: ndim        !! Dimensions of forcing to be added to the netCDF
    INTEGER(i_std),DIMENSION(ndm)  :: count_force !! Number of elements in each dimension  
    INTEGER(i_std)                 :: vid         !! Variable identifer of netCDF
!_ ================================================================================================================================
    
  !! 1. Determine number of blocks of forcing variables that are stored in memory

    nblocks = 0
    ifirst(:) = 1
    ilast(:) = 1
    DO ii = ibeg, iend
       IF (     (nblocks /= 0) &
            &      .AND.(isf(ii) == isf(ilast(nblocks))+1)) THEN
          ! Last block found
          ilast(nblocks) = ii
       ELSE
          ! First block found
          nblocks = nblocks+1
          IF (nblocks > 2)  STOP 'Problem in forcing_write'
          ifirst(nblocks) = ii
          ilast(nblocks) = ii
       ENDIF
    ENDDO

  !! 2. Gather distributed variables (parallel computing)

    CALL gather(clay_fm,clay_fm_g)
    CALL gather(silt_fm,silt_fm_g) 
    CALL gather(bulk_fm,bulk_fm_g) 
    CALL gather(humrel_daily_fm,humrel_daily_fm_g)
    CALL gather(litterhum_daily_fm,litterhum_daily_fm_g)
    CALL gather(t2m_daily_fm,t2m_daily_fm_g)
    CALL gather(t2m_min_daily_fm,t2m_min_daily_fm_g)
    CALL gather(tsurf_daily_fm,tsurf_daily_fm_g)
    CALL gather(tsoil_daily_fm,tsoil_daily_fm_g)
    CALL gather(soilhum_daily_fm,soilhum_daily_fm_g)
    CALL gather(precip_fm,precip_fm_g)
    CALL gather(gpp_daily_fm,gpp_daily_fm_g)
    CALL gather(veget_fm,veget_fm_g)
    CALL gather(veget_max_fm,veget_max_fm_g)
!!$    CALL gather(lai_fm,lai_fm_g)
    CALL gather(drainage_fm,drainage_fm_g)  
 !! 3. Append data to netCDF file
   
    IF (is_root_prc) THEN
       ! The netCDF file has been created earlier in this module, a file ID is available 
       ! and variables and dimensions have already been defined
       DO iblocks = 1, nblocks
          IF (ifirst(iblocks) /= ilast(iblocks)) THEN
             ndim = 2
             start(1:ndim) = 1; start(ndim) = isf(ifirst(iblocks));
             count_force(1:ndim) = SHAPE(clay_fm_g)
             count_force(ndim) = isf(ilast(iblocks))-isf(ifirst(iblocks))+1
             ier = NF90_INQ_VARID (forcing_id,'clay',vid)
             ier = NF90_PUT_VAR (forcing_id,vid, &
                  &              clay_fm_g(:,ifirst(iblocks):ilast(iblocks)), &
                  & start=start(1:ndim), count=count_force(1:ndim))
             ndim = 2 
             start(1:ndim) = 1; start(ndim) = isf(ifirst(iblocks)); 
             count_force(1:ndim) = SHAPE(silt_fm_g) 
             count_force(ndim) = isf(ilast(iblocks))-isf(ifirst(iblocks))+1 
             ier = NF90_INQ_VARID (forcing_id,'silt',vid) 
             ier = NF90_PUT_VAR (forcing_id,vid, & 
                  &              silt_fm_g(:,ifirst(iblocks):ilast(iblocks)), & 
                  & start=start(1:ndim), count=count_force(1:ndim)) 
             ndim = 2 
             start(1:ndim) = 1; start(ndim) = isf(ifirst(iblocks)); 
             count_force(1:ndim) = SHAPE(bulk_fm_g) 
             count_force(ndim) = isf(ilast(iblocks))-isf(ifirst(iblocks))+1 
             ier = NF90_INQ_VARID (forcing_id,'bulk',vid) 
             ier = NF90_PUT_VAR (forcing_id,vid, & 
                  &              bulk_fm_g(:,ifirst(iblocks):ilast(iblocks)), & 
                  & start=start(1:ndim), count=count_force(1:ndim)) 

             ndim = 3;
             start(1:ndim) = 1; start(ndim) = isf(ifirst(iblocks));
             count_force(1:ndim) = SHAPE(humrel_daily_fm_g)
             count_force(ndim) = isf(ilast(iblocks))-isf(ifirst(iblocks))+1
             ier = NF90_INQ_VARID (forcing_id,'humrel',vid)
             ier = NF90_PUT_VAR (forcing_id, vid, &
                  &            humrel_daily_fm_g(:,:,ifirst(iblocks):ilast(iblocks)), &
                  &            start=start(1:ndim), count=count_force(1:ndim))
             ndim = 2;
             start(1:ndim) = 1; start(ndim) = isf(ifirst(iblocks));
             count_force(1:ndim) = SHAPE(litterhum_daily_fm_g)
             count_force(ndim) = isf(ilast(iblocks))-isf(ifirst(iblocks))+1
             ier = NF90_INQ_VARID (forcing_id,'litterhum',vid)
             ier = NF90_PUT_VAR (forcing_id, vid, &
                  &            litterhum_daily_fm_g(:,ifirst(iblocks):ilast(iblocks)), &
                  & start=start(1:ndim), count=count_force(1:ndim))
             ndim = 2;
             start(1:ndim) = 1; start(ndim) = isf(ifirst(iblocks));
             count_force(1:ndim) = SHAPE(t2m_daily_fm_g)
             count_force(ndim) = isf(ilast(iblocks))-isf(ifirst(iblocks))+1
             ier = NF90_INQ_VARID (forcing_id,'t2m',vid)
             ier = NF90_PUT_VAR (forcing_id, vid, &
                  &            t2m_daily_fm_g(:,ifirst(iblocks):ilast(iblocks)), &
                  & start=start(1:ndim), count=count_force(1:ndim))
             ndim = 2;
             start(1:ndim) = 1; start(ndim) = isf(ifirst(iblocks));
             count_force(1:ndim) = SHAPE(t2m_min_daily_fm_g)
             count_force(ndim) = isf(ilast(iblocks))-isf(ifirst(iblocks))+1
             ier = NF90_INQ_VARID (forcing_id,'t2m_min',vid)
             ier = NF90_PUT_VAR (forcing_id, vid, &
                  &            t2m_min_daily_fm_g(:,ifirst(iblocks):ilast(iblocks)), &
                  & start=start(1:ndim), count=count_force(1:ndim))
             ndim = 2;
             start(1:ndim) = 1; start(ndim) = isf(ifirst(iblocks));
             count_force(1:ndim) = SHAPE(tsurf_daily_fm_g)
             count_force(ndim) = isf(ilast(iblocks))-isf(ifirst(iblocks))+1
             ier = NF90_INQ_VARID (forcing_id,'tsurf',vid)
             ier = NF90_PUT_VAR (forcing_id, vid, &
                  &            tsurf_daily_fm_g(:,ifirst(iblocks):ilast(iblocks)), &
                  & start=start(1:ndim), count=count_force(1:ndim))
             ndim = 3;
             start(1:ndim) = 1; start(ndim) = isf(ifirst(iblocks));
             count_force(1:ndim) = SHAPE(tsoil_daily_fm_g)
             count_force(ndim) = isf(ilast(iblocks))-isf(ifirst(iblocks))+1
             ier = NF90_INQ_VARID (forcing_id,'tsoil',vid)
             ier = NF90_PUT_VAR (forcing_id, vid, &
                  &            tsoil_daily_fm_g(:,:,ifirst(iblocks):ilast(iblocks)), &
                  & start=start(1:ndim), count=count_force(1:ndim))
             ndim = 3;
             start(1:ndim) = 1; start(ndim) = isf(ifirst(iblocks));
             count_force(1:ndim) = SHAPE(soilhum_daily_fm_g)
             count_force(ndim) = isf(ilast(iblocks))-isf(ifirst(iblocks))+1
             ier = NF90_INQ_VARID (forcing_id,'soilhum',vid)
             ier = NF90_PUT_VAR (forcing_id, vid, &
                  &            soilhum_daily_fm_g(:,:,ifirst(iblocks):ilast(iblocks)), &
                  & start=start(1:ndim), count=count_force(1:ndim))
             ndim = 2;
             start(1:ndim) = 1; start(ndim) = isf(ifirst(iblocks));
             count_force(1:ndim) = SHAPE(precip_fm_g)
             count_force(ndim) = isf(ilast(iblocks))-isf(ifirst(iblocks))+1
             ier = NF90_INQ_VARID (forcing_id,'precip',vid)
             ier = NF90_PUT_VAR (forcing_id, vid, &
                  &            precip_fm_g(:,ifirst(iblocks):ilast(iblocks)), &
                  & start=start(1:ndim), count=count_force(1:ndim))
             ndim = 3;
             start(1:ndim) = 1; start(ndim) = isf(ifirst(iblocks));
             count_force(1:ndim) = SHAPE(gpp_daily_fm_g)
             count_force(ndim) = isf(ilast(iblocks))-isf(ifirst(iblocks))+1
             ier = NF90_INQ_VARID (forcing_id,'gpp',vid)
             ier = NF90_PUT_VAR (forcing_id, vid, &
                  &            gpp_daily_fm_g(:,:,ifirst(iblocks):ilast(iblocks)), &
                  &            start=start(1:ndim), count=count_force(1:ndim))
             ndim = 3;
             start(1:ndim) = 1; start(ndim) = isf(ifirst(iblocks));
             count_force(1:ndim) = SHAPE(veget_fm_g)
             count_force(ndim) = isf(ilast(iblocks))-isf(ifirst(iblocks))+1
             ier = NF90_INQ_VARID (forcing_id,'veget',vid)
             ier = NF90_PUT_VAR (forcing_id, vid, &
                  &            veget_fm_g(:,:,ifirst(iblocks):ilast(iblocks)), &
                  &            start=start(1:ndim), count=count_force(1:ndim))
             ndim = 3;
             start(1:ndim) = 1; start(ndim) = isf(ifirst(iblocks));
             count_force(1:ndim) = SHAPE(veget_max_fm_g)
             count_force(ndim) = isf(ilast(iblocks))-isf(ifirst(iblocks))+1
             ier = NF90_INQ_VARID (forcing_id,'veget_max',vid)
             ier = NF90_PUT_VAR (forcing_id, vid, &
                  &            veget_max_fm_g(:,:,ifirst(iblocks):ilast(iblocks)), &
                  &            start=start(1:ndim), count=count_force(1:ndim))
!!$             ndim = 3;
!!$             start(1:ndim) = 1; start(ndim) = isf(ifirst(iblocks));
!!$             count_force(1:ndim) = SHAPE(lai_fm_g)
!!$             count_force(ndim) = isf(ilast(iblocks))-isf(ifirst(iblocks))+1
!!$             ier = NF90_INQ_VARID (forcing_id,'lai',vid)
!!$             ier = NF90_PUT_VAR (forcing_id, vid, &
!!$                  &            lai_fm_g(:,:,ifirst(iblocks):ilast(iblocks)), &
!!$                  &            start=start(1:ndim), count=count_force(1:ndim))
             ndim = 2; 
             start(1:ndim) = 1; start(ndim) = isf(ifirst(iblocks)); 
             count_force(1:ndim) = SHAPE(drainage_fm_g) 
             count_force(ndim) = isf(ilast(iblocks))-isf(ifirst(iblocks))+1 
             ier = NF90_INQ_VARID (forcing_id,'drainage',vid) 
             ier = NF90_PUT_VAR (forcing_id, vid, & 
                  &            drainage_fm_g(:,:,ifirst(iblocks):ilast(iblocks)), & 
                  &            start=start(1:ndim), count=count_force(1:ndim)) 
          ENDIF
       ENDDO
    ENDIF
    
  !! 4. Adjust flag of forcing file
    nf_written(isf(:)) = .TRUE.

  END SUBROUTINE forcing_write

  
!! ================================================================================================================================
!! SUBROUTINE 	: stomate_forcing_read
!!
!>\BRIEF        Read forcing file.
!!
!! DESCRIPTION  : None
!!
!! RECENT CHANGE(S) : None
!!
!! MAIN OUTPUT VARIABLE(S): None 
!!
!! REFERENCES	: None
!!
!! FLOWCHART    : None
!! \n
!_ ================================================================================================================================
  
  SUBROUTINE stomate_forcing_read(forcing_id,nsfm)
   
  !! 0. Variable and parameter declaration

    !! 0.1 Input variables

    INTEGER(i_std),INTENT(in)  :: forcing_id           !! File identifer of forcing file, assigned when netcdf is created
    INTEGER(i_std),INTENT(in)  :: nsfm                 !! Number of time steps stored in memory        
    
    !! 0.2 Output variables

    !! 0.3 Modified variables

    !! 0.4 Local variables

    INTEGER(i_std)                 :: ii                !! Index of isf where isf is the number of time steps that can be stored in 
                                                        !! memory 
    INTEGER(i_std)                 :: iblocks           !! Index of block that is written
    INTEGER(i_std)                 :: nblocks           !! Number of blocks that needs to be written
    INTEGER(i_std)                 :: ier               !! Check error of netcdf call
    INTEGER(i_std),DIMENSION(0:2)  :: ifirst            !! First block in memory - changes with iblocks
    INTEGER(i_std),DIMENSION(0:2)  :: ilast             !! Last block in memory - changes with iblocks
    INTEGER(i_std),PARAMETER       :: ndm = 10          !! Maximum number of dimensions
    INTEGER(i_std),DIMENSION(ndm)  :: start             !! First block to write
    INTEGER(i_std)                 :: ndim              !! Dimensions of forcing to be added to the netCDF
    INTEGER(i_std),DIMENSION(ndm)  :: count_force       !! Number of elements in each dimension
    INTEGER(i_std)                 :: vid               !! Variable identifer of netCDF
    LOGICAL, PARAMETER             :: check=.FALSE.     !! Flag for debugging 
    LOGICAL                        :: a_er=.FALSE.      !! Error catching from netcdf file
!_ ================================================================================================================================

    IF (check) WRITE(numout,*) "stomate_forcing_read "
    
  !! 1. Set to zero if the corresponding forcing state

    ! has not yet been written into the file  
    DO ii = 1, nsfm
       IF (.NOT.nf_written(isf(ii))) THEN
          clay_fm(:,ii) = zero
          silt_fm(:,iisf) = zero 
          bulk_fm(:,iisf) = zero 
          humrel_daily_fm(:,:,ii) = zero
          litterhum_daily_fm(:,ii) = zero
          t2m_daily_fm(:,ii) = zero
          t2m_min_daily_fm(:,ii) = zero
          tsurf_daily_fm(:,ii) = zero
          tsoil_daily_fm(:,:,ii) = zero
          soilhum_daily_fm(:,:,ii) = zero
          precip_fm(:,ii) = zero
          gpp_daily_fm(:,:,ii) = zero
          veget_fm(:,:,ii) = zero
          veget_max_fm(:,:,ii) = zero
!!$          lai_fm(:,:,ii) = zero
          drainage_fm(:,:,iisf) = zero 
       ENDIF
    ENDDO
    
  !! 2. determine blocks of forcing states that are contiguous in memory

    nblocks = 0
    ifirst(:) = 1
    ilast(:) = 1
    
    DO ii = 1, nsfm
       IF (nf_written(isf(ii))) THEN
          IF (     (nblocks /= 0) &
               &        .AND.(isf(ii) == isf(ilast(nblocks))+1)) THEN

             ! element is contiguous with last element found
             ilast(nblocks) = ii
          ELSE

             ! found first element of new block
             nblocks = nblocks+1
             IF (nblocks > 2)  STOP 'Problem in stomate_forcing_read'
             
             ifirst(nblocks) = ii
             ilast(nblocks) = ii
          ENDIF
       ENDIF
    ENDDO
    IF (check) WRITE(numout,*) "stomate_forcing_read nblocks, ifirst, ilast",nblocks, ifirst, ilast
    
  !! 3. Read variable values

    IF (is_root_prc) THEN
       DO iblocks = 1, nblocks
          IF (check) WRITE(numout,*) "stomate_forcing_read iblocks, ifirst(iblocks), ilast(iblocks)",iblocks, &
               ifirst(iblocks), ilast(iblocks)
          IF (ifirst(iblocks) /= ilast(iblocks)) THEN
             a_er=.FALSE.
             ndim = 2;
             start(1:ndim) = 1; start(ndim) = isf(ifirst(iblocks));
             count_force(1:ndim) = SHAPE(clay_fm_g)
             count_force(ndim) = isf(ilast(iblocks))-isf(ifirst(iblocks))+1
             ier = NF90_INQ_VARID (forcing_id,'clay',vid)
             a_er = a_er.OR.(ier /= 0)
             ier = NF90_GET_VAR (forcing_id, vid, &
                  &            clay_fm_g(:,ifirst(iblocks):ilast(iblocks)), &
                  &            start=start(1:ndim), count=count_force(1:ndim))
             a_er = a_er.OR.(ier /= 0)


             ndim = 2; 
             start(1:ndim) = 1; start(ndim) = isf(ifirst(iblocks)); 
             count_force(1:ndim) = SHAPE(silt_fm_g) 
             count_force(ndim) = isf(ilast(iblocks))-isf(ifirst(iblocks))+1 
             ier = NF90_INQ_VARID (forcing_id,'silt',vid) 
             a_er = a_er.OR.(ier /= 0) 
             ier = NF90_GET_VAR (forcing_id, vid, & 
                  &            silt_fm_g(:,ifirst(iblocks):ilast(iblocks)), & 
                  &            start=start(1:ndim), count=count_force(1:ndim)) 
             a_er = a_er.OR.(ier /= 0) 
             
             ndim = 2; 
             start(1:ndim) = 1; start(ndim) = isf(ifirst(iblocks)); 
             count_force(1:ndim) = SHAPE(bulk_fm_g) 
             count_force(ndim) = isf(ilast(iblocks))-isf(ifirst(iblocks))+1 
             ier = NF90_INQ_VARID (forcing_id,'bulk',vid) 
             a_er = a_er.OR.(ier /= 0) 
             ier = NF90_GET_VAR (forcing_id, vid, & 
                  &            bulk_fm_g(:,ifirst(iblocks):ilast(iblocks)), & 
                  &            start=start(1:ndim), count=count_force(1:ndim)) 
             a_er = a_er.OR.(ier /= 0) 
             
             ndim = 3;
             start(1:ndim) = 1; start(ndim) = isf(ifirst(iblocks));
             count_force(1:ndim) = SHAPE(humrel_daily_fm_g)
             count_force(ndim) = isf(ilast(iblocks))-isf(ifirst(iblocks))+1
             ier = NF90_INQ_VARID (forcing_id,'humrel',vid)
             a_er = a_er.OR.(ier /= 0)
             ier = NF90_GET_VAR (forcing_id, vid, &
                  &            humrel_daily_fm_g(:,:,ifirst(iblocks):ilast(iblocks)), &
                  &            start=start(1:ndim), count=count_force(1:ndim))
             a_er = a_er.OR.(ier /= 0)

             ndim = 2;
             start(1:ndim) = 1; start(ndim) = isf(ifirst(iblocks));
             count_force(1:ndim) = SHAPE(litterhum_daily_fm_g)
             count_force(ndim) = isf(ilast(iblocks))-isf(ifirst(iblocks))+1
             ier = NF90_INQ_VARID (forcing_id,'litterhum',vid)
             a_er = a_er.OR.(ier /= 0)
             ier = NF90_GET_VAR (forcing_id, vid, &
                  &              litterhum_daily_fm_g(:,ifirst(iblocks):ilast(iblocks)), &
                  &            start=start(1:ndim), count=count_force(1:ndim))
             a_er = a_er.OR.(ier /= 0)

             ndim = 2;
             start(1:ndim) = 1; start(ndim) = isf(ifirst(iblocks));
             count_force(1:ndim) = SHAPE(t2m_daily_fm_g)
             count_force(ndim) = isf(ilast(iblocks))-isf(ifirst(iblocks))+1
             ier = NF90_INQ_VARID (forcing_id,'t2m',vid)
             a_er = a_er.OR.(ier /= 0)
             ier = NF90_GET_VAR (forcing_id, vid, &
                  &              t2m_daily_fm_g(:,ifirst(iblocks):ilast(iblocks)), &
                  &            start=start(1:ndim), count=count_force(1:ndim))
             a_er = a_er.OR.(ier /= 0)

             ndim = 2;
             start(1:ndim) = 1; start(ndim) = isf(ifirst(iblocks));
             count_force(1:ndim) = SHAPE(t2m_min_daily_fm_g)
             count_force(ndim) = isf(ilast(iblocks))-isf(ifirst(iblocks))+1
             ier = NF90_INQ_VARID (forcing_id,'t2m_min',vid)
             a_er = a_er.OR.(ier /= 0)
             ier = NF90_GET_VAR (forcing_id, vid, &
                  &              t2m_min_daily_fm_g(:,ifirst(iblocks):ilast(iblocks)), &
                  &            start=start(1:ndim), count=count_force(1:ndim))
             a_er = a_er.OR.(ier /= 0)

             ndim = 2;
             start(1:ndim) = 1; start(ndim) = isf(ifirst(iblocks));
             count_force(1:ndim) = SHAPE(tsurf_daily_fm_g)
             count_force(ndim) = isf(ilast(iblocks))-isf(ifirst(iblocks))+1
             ier = NF90_INQ_VARID (forcing_id,'tsurf',vid)
             a_er = a_er.OR.(ier /= 0)
             ier = NF90_GET_VAR (forcing_id, vid, &
                  &              tsurf_daily_fm_g(:,ifirst(iblocks):ilast(iblocks)), &
                  &            start=start(1:ndim), count=count_force(1:ndim))
             a_er = a_er.OR.(ier /= 0)

             ndim = 3;
             start(1:ndim) = 1; start(ndim) = isf(ifirst(iblocks));
             count_force(1:ndim) = SHAPE(tsoil_daily_fm_g)
             count_force(ndim) = isf(ilast(iblocks))-isf(ifirst(iblocks))+1
             ier = NF90_INQ_VARID (forcing_id,'tsoil',vid)
             a_er = a_er.OR.(ier /= 0)
             ier = NF90_GET_VAR (forcing_id, vid, &
                  &              tsoil_daily_fm_g(:,:,ifirst(iblocks):ilast(iblocks)), &
                  &            start=start(1:ndim), count=count_force(1:ndim))
             a_er = a_er.OR.(ier /= 0)

             ndim = 3;
             start(1:ndim) = 1; start(ndim) = isf(ifirst(iblocks));
             count_force(1:ndim) = SHAPE(soilhum_daily_fm_g)
             count_force(ndim) = isf(ilast(iblocks))-isf(ifirst(iblocks))+1
             ier = NF90_INQ_VARID (forcing_id,'soilhum',vid)
             a_er = a_er.OR.(ier /= 0)
             ier = NF90_GET_VAR (forcing_id, vid, &
                  &              soilhum_daily_fm_g(:,:,ifirst(iblocks):ilast(iblocks)), &
                  &            start=start(1:ndim), count=count_force(1:ndim))
             a_er = a_er.OR.(ier /= 0)

             ndim = 2;
             start(1:ndim) = 1; start(ndim) = isf(ifirst(iblocks));
             count_force(1:ndim) = SHAPE(precip_fm_g)
             count_force(ndim) = isf(ilast(iblocks))-isf(ifirst(iblocks))+1
             ier = NF90_INQ_VARID (forcing_id,'precip',vid)
             a_er = a_er.OR.(ier /= 0)
             ier = NF90_GET_VAR (forcing_id, vid, &
                  &              precip_fm_g(:,ifirst(iblocks):ilast(iblocks)), &
                  &            start=start(1:ndim), count=count_force(1:ndim))
             a_er = a_er.OR.(ier /= 0)

             ndim = 3;
             start(1:ndim) = 1; start(ndim) = isf(ifirst(iblocks));
             count_force(1:ndim) = SHAPE(gpp_daily_fm_g)
             count_force(ndim) = isf(ilast(iblocks))-isf(ifirst(iblocks))+1
             ier = NF90_INQ_VARID (forcing_id,'gpp',vid)
             a_er = a_er.OR.(ier /= 0)
             ier = NF90_GET_VAR (forcing_id, vid, &
                  &            gpp_daily_fm_g(:,:,ifirst(iblocks):ilast(iblocks)), &
                  &            start=start(1:ndim), count=count_force(1:ndim))
             a_er = a_er.OR.(ier /= 0)

             ndim = 3;
             start(1:ndim) = 1; start(ndim) = isf(ifirst(iblocks));
             count_force(1:ndim) = SHAPE(veget_fm_g)
             count_force(ndim) = isf(ilast(iblocks))-isf(ifirst(iblocks))+1
             ier = NF90_INQ_VARID (forcing_id,'veget',vid)
             a_er = a_er.OR.(ier /= 0)
             ier = NF90_GET_VAR (forcing_id, vid, &
                  &            veget_fm_g(:,:,ifirst(iblocks):ilast(iblocks)), &
                  &            start=start(1:ndim), count=count_force(1:ndim))
             a_er = a_er.OR.(ier /= 0)

             ndim = 3;
             start(1:ndim) = 1; start(ndim) = isf(ifirst(iblocks));
             count_force(1:ndim) = SHAPE(veget_max_fm_g)
             count_force(ndim) = isf(ilast(iblocks))-isf(ifirst(iblocks))+1
             ier = NF90_INQ_VARID (forcing_id,'veget_max',vid)
             a_er = a_er.OR.(ier /= 0)
             ier = NF90_GET_VAR (forcing_id, vid, &
                  &            veget_max_fm_g(:,:,ifirst(iblocks):ilast(iblocks)), &
                  &            start=start(1:ndim), count=count_force(1:ndim))
             a_er = a_er.OR.(ier /= 0)

!!$             ndim = 3;
!!$             start(1:ndim) = 1; start(ndim) = isf(ifirst(iblocks));
!!$             count_force(1:ndim) = SHAPE(lai_fm_g)
!!$             count_force(ndim) = isf(ilast(iblocks))-isf(ifirst(iblocks))+1
!!$             ier = NF90_INQ_VARID (forcing_id,'lai',vid)
!!$             a_er = a_er.OR.(ier /= 0)
!!$             ier = NF90_GET_VAR (forcing_id, vid, &
!!$                  &            lai_fm_g(:,:,ifirst(iblocks):ilast(iblocks)), &
!!$                  &            start=start(1:ndim), count=count_force(1:ndim))
!!$             a_er = a_er.OR.(ier /= 0)

             ndim = 2; 
             start(1:ndim) = 1; start(ndim) = isf(ifirst(iblocks)); 
             count_force(1:ndim) = SHAPE(drainage_fm_g) 
             count_force(ndim) = isf(ilast(iblocks))-isf(ifirst(iblocks))+1 
             ier = NF90_INQ_VARID (forcing_id,'drainage',vid) 
             a_er = a_er.OR.(ier /= 0) 
             ier = NF90_GET_VAR (forcing_id, vid, & 
                  &            drainage_fm_g(:,:,ifirst(iblocks):ilast(iblocks)), & 
                  &            start=start(1:ndim), count=count_force(1:ndim)) 
             a_er = a_er.OR.(ier /= 0) 
             IF (a_er) THEN
                CALL ipslerr_p (3,'stomate_forcing_read', &
                     &        'PROBLEM when read forcing file', &
                     &        '','')
             ENDIF

          ENDIF ! (ifirst(iblocks) /= ilast(iblocks))
       ENDDO ! iblocks
    ENDIF ! is_root_prc

  !! 4. Distribute the variable over several processors

    CALL scatter(clay_fm_g,clay_fm)
    CALL scatter(silt_fm_g,silt_fm) 
    CALL scatter(bulk_fm_g,bulk_fm) 
    CALL scatter(humrel_daily_fm_g,humrel_daily_fm)
    CALL scatter(litterhum_daily_fm_g,litterhum_daily_fm)
    CALL scatter(t2m_daily_fm_g,t2m_daily_fm)
    CALL scatter(t2m_min_daily_fm_g,t2m_min_daily_fm)
    CALL scatter(tsurf_daily_fm_g,tsurf_daily_fm)
    CALL scatter(tsoil_daily_fm_g,tsoil_daily_fm)
    CALL scatter(soilhum_daily_fm_g,soilhum_daily_fm)
    CALL scatter(precip_fm_g,precip_fm)
    CALL scatter(gpp_daily_fm_g,gpp_daily_fm)
    CALL scatter(veget_fm_g,veget_fm)
    CALL scatter(veget_max_fm_g,veget_max_fm)
!!$    CALL scatter(lai_fm_g,lai_fm)
    CALL scatter(drainage_fm_g,drainage_fm) 
  END SUBROUTINE stomate_forcing_read


!! ================================================================================================================================
!! SUBROUTINE 	: setlai
!!
!>\BRIEF        Routine to force the lai in STOMATE. The code in this routine
!! simply CALCULATES lai and is therefore not functional. The routine should be 
!! rewritten if one wants to force lai. Forcing lai with the new allocation is
!! not straightforward and would quickly (after 5 years ?) result in serious
!! inconsistencies
!!
!! DESCRIPTION  : None
!!
!! RECENT CHANGE(S) : None
!!
!! MAIN OUTPUT VARIABLE(S): ::lai
!!
!! REFERENCE(S) : None
!!
!! FLOWCHART : None
!! \n
!_ ================================================================================================================================
  
!!$  SUBROUTINE setlai(kjpindex,lai, circ_class_biomass, circ_class_n)
!!$
!!$  !! 0 Variable and parameter declaration 
!!$  
!!$    !! 0.1 Input variables
!!$
!!$    INTEGER(i_std),INTENT(in)                    :: kjpindex      !! Domain size - number of pixels (unitless)
!!$    REAL(r_std), DIMENSION(:,:,:,:,:), INTENT(in)  :: circ_class_biomass   !! Biomass per PFT @tex $(gC m^{-2})$ @endtex
!!$    REAL(r_std), DIMENSION(:,:,:), INTENT(in)  :: circ_class_n   !! Biomass per PFT @tex $(gC m^{-2})$ @endtex
!!$    !! 0.2 Output variables
!!$
!!$    REAL(r_std),DIMENSION(kjpindex,nvm), INTENT(out)  :: lai      !! PFT leaf area index @tex $(m^{2} m^{-2})$ @endtex
!!$
!!$    !! 0.3 Modified variables
!!$
!!$    !! 0.4 Local variables
!!$
!!$    INTEGER(i_std)                               :: j,i         !! index (unitless)
!!$!_ ================================================================================================================================
!!$    
!!$    !! 1. Set lai for bare soil to zero
!!$
!!$    lai(:,ibare_sechiba) = zero
!!$
!!$    !! 2. Multiply foliage biomass by sla to calculate lai for all PFTs and pixels
!!$    DO i=1, kjpindex
!!$        DO j=2,nvm
!!$            lai(i,j) = cc_to_lai(circ_class_biomass(i,j,:,ileaf,icarbon),circ_class_n(i,j,:),j)
!!$        ENDDO
!!$    ENDDO
!!$  END SUBROUTINE setlai


END MODULE stomate
